const chatApp = new Vue({
    el: '#chat-app',
    data: {
        messages: [],
        users: [],
        AllUsers: [],
        search: '',
        alert_sounds: document.head.querySelector('meta[name="alert-sounds"]').content,
        receiver_id: document.head.querySelector('meta[name="default-receiver-id"]').content,
        sender_id: document.head.querySelector('meta[name="default-sender-id"]').content,
        pusher_status: document.head.querySelector('meta[name="pusher-status"]').content,
        need_scroll: false,
        last_message_id_received: 0,
        sendBtn: '<i class="fe fe-send"></i>'
    },
    mounted() {
        //
    },
    created() {
        this.fetchMessages();
        this.fetchUsers();
        if (this.pusher_status == "1") {
            Echo.private('ChatMessage').listen('ChatSentEvent', (e) => {
                if ((e.message.sender_id == this.receiver_id && e.message.receiver_id == this.sender_id) || (e.message.sender_id == this.sender_id && e.message.receiver_id == this.receiver_id)) {
                    this.messages.push(e.message);
                    // alert sound
                    if (this.alert_sounds == '1') {
                        document.getElementById('alert_sound').play();
                    }

                    if (e.message.sender_id == this.receiver_id) {
                        this.last_message_id_received = e.message.id;
                        this.readMessage(e.message.id, this.receiver_id);
                    }
                    //update my sent messages read
                    const newMessages = this.messages.map(function addOne(message) {
                        if (message.id < chatApp.last_message_id_received) {
                            message.read_status = true;
                        }
                        message.new = false;
                        return message;
                    });
                    this.messages = newMessages;
                }
            });
        } else {
            this.interval = setInterval(() => this.fetchMessages(), document.head.querySelector('meta[name="refresh-alerts-every"]').content);
        }
        this.interval = setInterval(() => this.fetchUsers(), document.head.querySelector('meta[name="refresh-users-every"]').content);

    },
    updated() {
        this.scrollToEnd();
    },
    methods: {
        usersFilter() {
            this.users = this.AllUsers.filter(user => {
                return user.name.toLowerCase().includes(this.search.toLowerCase())
            });
        },
        openChat(rec_id, rec_name, rec_job, rec_photo, rec_status) {
            if (rec_id != this.receiver_id) {
                $("#chat_loader").show();
                $("#chat_empty").hide();
                this.messages = [];
                this.receiver_id = rec_id;
                VueApp.receiver_id = rec_id;
                $("#attach_receiver_id").val(rec_id);
                $("#chat_receiver_name").html(rec_name);
                $("#chat_receiver_job").html(rec_job);
                $("#chat_receiver_photo").attr("src", rec_photo);
                if (rec_status) {
                    $("#chat_receiver_status").addClass("bg-green");
                    $("#chat_receiver_status").removeClass("bg-yellow");
                } else {
                    $("#chat_receiver_status").addClass("bg-yellow");
                    $("#chat_receiver_status").removeClass("bg-green");
                }
                this.need_scroll = true;
                this.fetchMessages();

                //update unread count
                var newMsgs = 0;
                const newUsers = this.AllUsers.map(function addOne(user) {
                    if (user.id == chatApp.receiver_id) {
                        newMsgs = user.alerts;
                        user.alerts = "";
                    }
                    return user;
                });
                this.AllUsers = newUsers;
                this.usersFilter();
                if (newMsgs > 0) {
                    VueApp.chat_alerts = VueApp.chat_alerts - newMsgs;
                    if (VueApp.chat_alerts < 1) {
                        VueApp.chat_alerts = "";
                    }
                }

                $('html, body').animate({
                    scrollTop: $("#page_top").offset().top
                }, 500);
            }
        },
        fetchMessages() {
            axios.get(app_preferences['dashboard_url'] + '/chat/messages/' + this.receiver_id).then(response => {
                this.messages = response.data;
                $("#chat_loader").hide();
                $("#chat_empty").hide();
                if (this.messages == "") {
                    $("#chat_empty").show();
                }
            });
        },
        fetchUsers() {
            axios.get(app_preferences['dashboard_url'] + '/chat/users/' + this.receiver_id).then(response => {
                this.users = response.data.users;
                this.AllUsers = response.data.users;
                if (response.data.rec_status) {
                    $("#chat_receiver_status").addClass("bg-green");
                    $("#chat_receiver_status").removeClass("bg-yellow");
                } else {
                    $("#chat_receiver_status").addClass("bg-yellow");
                    $("#chat_receiver_status").removeClass("bg-green");
                }
            });
        },
        addMessage(message) {
            axios.post(app_preferences['dashboard_url'] + '/chat/store', message).then(response => {
                message.id = response.data.id;
                this.messages.push(message);
                this.need_scroll = true;
                this.sendBtn = "<i class=\"fe fe-send\"></i>";
                $("#chat_loader").hide();
                $("#chat_empty").hide();

                if (this.pusher_status == "1") {
                    //listen for read status
                    Echo.private('ChatMessageRead.' + response.data.id).listen('ChatReadEvent', (e) => {
                        const newMessages = this.messages.map(function addOne(message) {
                            if (message.id == e.message.id || chatApp.last_message_id_received > message.id) {
                                message.read_status = true;
                            }
                            message.new = false;
                            return message;
                        });
                        this.messages = newMessages;
                    });
                }
            });
        },
        readMessage(mid, uid) {
            axios.get(app_preferences['dashboard_url'] + '/chat/read/' + mid + '/' + uid).then(response => {
                VueApp.chat_alerts--;
                if (VueApp.chat_alerts < 1) {
                    VueApp.chat_alerts = "";
                }
            });
        },
        scrollToEnd() {
            var container = this.$el.querySelector("#chat_window ul");
            if (((container.scrollTop + container.offsetHeight) >= (container.scrollHeight - container.offsetHeight)) || (container.scrollTop == 0) || this.need_scroll) {
                container.scrollTop = container.scrollHeight;
                //console.log(this.need_scroll);
                this.need_scroll = false;
            }
        }
    }
});

var delete_url = document.head.querySelector('meta[name="delete_url"]').content,
    confirm_msg = document.head.querySelector('meta[name="delete_confirm_msg"]').content,
    confirm_btn_txt = document.head.querySelector('meta[name="delete_confirm_btn_txt"]').content,
    cancel_btn_txt = document.head.querySelector('meta[name="delete_cancel_btn_txt"]').content,
    success_msg = document.head.querySelector('meta[name="delete_success_msg"]').content,
    failed_msg = document.head.querySelector('meta[name="delete_failed_msg"]').content,
    close_btn_txt = document.head.querySelector('meta[name="delete_close_btn_txt"]').content,
    delete_no_data = document.head.querySelector('meta[name="delete_no_data"]').content;
$('#chat_del_btn').click(function () {
    if (chatApp.messages != "" && chatApp.receiver_id > 0) {
        swal({
            title: confirm_msg,
            type: "error",
            showCancelButton: true,
            closeOnConfirm: false,
            animation: "slide-from-top",
            confirmButtonText: confirm_btn_txt,
            cancelButtonText: cancel_btn_txt,
            showLoaderOnConfirm: true,
            confirmButtonColor: "#F27474",
        }, function () {
            var xhr = $.ajax({
                type: "GET",
                url: delete_url + "/" + chatApp.receiver_id,
                success: function (result) {
                    var obj_result = jQuery.parseJSON(result);
                    if (obj_result.stat == 'success') {
                        chatApp.messages = [];
                        swal({
                            title: "<span class='text-success'>" + success_msg + "</span>",
                            html: true,
                            type: "success",
                            confirmButtonText: close_btn_txt,
                            confirmButtonColor: "#acacac",
                            timer: 5000,
                        }, function () {
                            swal.close();
                        });
                    } else {
                        swal({
                            title: "<span class='text-danger'>" + failed_msg + "</span>",
                            html: true,
                            type: "error",
                            confirmButtonText: close_btn_txt,
                            confirmButtonColor: "#acacac",
                        });
                    }
                    return true;
                }
            });
            //console.log(xhr);
            return false;
        });
    } else {
        swal({
            title: "<span class='text-dark'>" + delete_no_data + "</span>",
            html: true,
            type: "info",
            confirmButtonText: close_btn_txt,
            confirmButtonColor: "#46b8da",
        });
    }
});
