<?php

namespace App\Console;

use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Foundation\Console\Kernel as ConsoleKernel;

use App\Models\Currency;
use App\Helpers\History;
use App\Models\Preference;

use App\Models\User;
use App\Models\Alert;
use App\Models\Notice;
use App\Models\Message;
use App\Models\Event;
use App\Models\Attachment;
use App\Models\Activity;
use App\Models\Chat;
use Artisan;
use Auth;
use File;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Mail;
use OneSignal;
use URL;
use App\Helpers\AppSender;
use App\Helpers\Cats;
use App\Helpers\Helper;
use App\Helpers\Backup;
use App\Helpers\DeviceTrust;

class Kernel extends ConsoleKernel
{
    /**
     * The Artisan commands provided by your application.
     *
     * @var array
     */
    protected $commands = [
        //
    ];

    /**
     * Define the application's command schedule.
     *
     * @param \Illuminate\Console\Scheduling\Schedule $schedule
     * @return void
     */
    protected function schedule(Schedule $schedule)
    {

        // Auto  DB backup generate
        if (env('BACKUP_TYPE') == "daily" || env('BACKUP_TYPE') == "weakly" || env('BACKUP_TYPE') == "monthly") {
            $backup_schedule = $schedule->call(function () {
                try {
                    $Backup_file = 'backup_' . date('m_d_Y_h_i_s_a') . ".sql";
                    $remote = 0;
                    if (Backup::dumpDatabase($Backup_file)) {

                        //Save to FTP
                        if (Helper::Preferences("backup_ftp") && Helper::FileSystem() != "") {
                            $localFile = Storage::disk('local')->get("backups/" . $Backup_file);
                            $Remote_store = Storage::disk(Helper::FileSystem())->put("backups/" . $Backup_file, $localFile);
                            if ($Remote_store) {
                                $remote = 1;
                                // Delete from local
                                Storage::disk('local')->delete("backups/" . $Backup_file);
                            }
                        }

                        // storage location is project_root/storage/app/backup.json file.
                        $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                        $fileData = [
                            "title" => "Automatic Backup",
                            "file" => $Backup_file,
                            "remote" => $remote,
                            "created_by" => 0,
                            "created_ip" => \Request::ip(),
                            "created_at" => date('Y-m-d H:i:s'),
                        ];

                        array_push($backup_list, $fileData);

                        Storage::disk('local')->put('backup.json', json_encode($backup_list));
                    }
                } catch (\Exception $e) {
                }

            });
            if (env('BACKUP_TYPE') == "daily") {
                $backup_schedule->daily();
            }
            if (env('BACKUP_TYPE') == "weakly") {
                $backup_schedule->weekly();
            }
            if (env('BACKUP_TYPE') == "monthly") {
                $backup_schedule->monthly();
            }
        }


        $schedule->call(function () {

            // Alerts Sending
            $template_mail_code = 1;
            $template_sms_code = 0;
            $template_browser_code = 0;
            //List of Alerts
            $Alerts = Alert::where('status', false)->where('alert_date', '<=', date('Y-m-d H:i:59'))->get();
            foreach ($Alerts as $Alert) {

                $User = $Alert->user;
                $alert_title = $Alert->title;
                $alert_details = "";
                $alert_link = "";
                $alert_full_link = "";

                foreach (Cats::AlertsTypes() as $key => $AlertsType) {
                    if ($key === $Alert->cat) {
                        if ($Alert->cat_id > 0) {
                            $alert_link = "/" . DeviceTrust::env('BACKEND_PATH') . $AlertsType['module_url'] . encrypt($Alert->cat_id);
                            $alert_full_link = URL::to(DeviceTrust::env('BACKEND_PATH')) . $AlertsType['module_url'] . encrypt($Alert->cat_id);
                        }
                        $alert_details = trans("backLang.alertFrom") . " " . $AlertsType['title'] . " " . trans("backLang.alertOn") . " " . $Alert->title;
                    }
                }

                if (!empty($User)) {

                    // send to system
                    if ($Alert->system) {
                        AppSender::ToSystem(
                            1,
                            [
                                "type" => 0,
                                "title" => $alert_title,
                                "details" => $alert_details,
                                "url" => $alert_link,
                                "to_user_name" => $User->name,
                                "icon" => "fe fe-bell",
                            ],
                            $User->id,
                            $Alert->cat,
                            $Alert->cat_id
                        );
                    }

                    //send to mail
                    if ($Alert->mail) {
                        AppSender::ToMail(
                            $User->email,
                            $User->name,
                            $template_mail_code,
                            [
                                "title" => $alert_title,
                                "details" => $alert_details,
                                "url" => $alert_full_link,
                                "to_user_name" => $User->name,
                            ],
                            $User->id,
                            $Alert->cat,
                            $Alert->cat_id,
                            false
                        );
                    }

                    // send to sms
                    if ($Alert->sms) {
                        AppSender::ToSMS(
                            $User->phone_full,
                            $template_sms_code,
                            [
                                "title" => $alert_title,
                                "details" => $alert_details,
                                "url" => $alert_full_link,
                                "to_user_name" => $User->name,
                            ],
                            $User->id,
                            $Alert->cat,
                            $Alert->cat_id,
                            false
                        );
                    }

                    // send to browser
                    if ($Alert->browser) {
                        AppSender::ToBrowser(
                            $template_browser_code,
                            [
                                "title" => $alert_title,
                                "details" => $alert_details,
                                "url" => $alert_full_link,
                                "to_user_name" => $User->name,
                            ],
                            $User->id,
                            $Alert->cat,
                            $Alert->cat_id,
                            false
                        );
                    }

                    // update alert status
                    $Alert->status = true;
                    $Alert->sent_date = date('Y-m-d H:i:s');
                    $Alert->save();
                }

            }

            // Auto Delete
            $cron_settings = [];
            if (Helper::Preferences("cron_settings") != "") {
                try {
                    //get cron settings
                    $cron_settings = json_decode(Helper::Preferences("cron_settings"));

                    // default values
                    $update_currencies = @$cron_settings->currencies->status;
                    $update_currencies_hours = @$cron_settings->currencies->hours;

                    $del_notifications = @$cron_settings->notifications->status;
                    $del_notifications_days = @$cron_settings->notifications->days;
                    $del_activities = @$cron_settings->activities->status;
                    $del_activities_days = @$cron_settings->activities->days;
                    $del_messages = @$cron_settings->messages->status;
                    $del_messages_days = @$cron_settings->messages->days;
                    $del_events = @$cron_settings->events->status;
                    $del_events_days = @$cron_settings->events->days;
                    $del_alerts = @$cron_settings->alerts->status;
                    $del_alerts_days = @$cron_settings->alerts->days;
                    $del_attachments = @$cron_settings->attachments->status;
                    $del_attachments_days = @$cron_settings->attachments->days;
                    $del_backups = @$cron_settings->backups->status;
                    $del_backups_days = @$cron_settings->backups->days;
                    $del_chats = @$cron_settings->chats->status;
                    $del_chats_days = @$cron_settings->chats->days;

                    //update currencies
                    if ($update_currencies) {
                        $update_currencies_date = date('Y-m-d H:i:s');
                        if ($update_currencies_hours > 0) {
                            $del_notifications_date = date('Y-m-d H:i:s', strtotime('-' . $update_currencies_hours . ' hours'));
                            if ($del_notifications_date >= Helper::Preferences("currency_updated_at") || Helper::Preferences("currency_updated_at") == "" || Helper::Preferences("currency_updated_at") == null) {
                                // start currency update
                                $Currencies = Currency::all();
                                $Default = Currency::find(Helper::Preferences("default_currency"));
                                if (!empty($Default)) {
                                    foreach ($Currencies as $Currency) {
                                        if ($Currency->code == $Default->code) {
                                            $rate = 1;
                                        } else {

                                            $query = strtoupper($Default->code) . '_' . strtoupper($Currency->code);

                                            $url = 'https://api.currconv.com/api/v7/convert?apiKey=' . env("CURRENCY_CONVERTER_API_KEY") . '&q=' . $query . '&compact=ultra';
                                            $ch = curl_init();
                                            // Disable SSL verification
                                            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                                            // Will return the response, if false it print the response
                                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                                            // Set the url
                                            curl_setopt($ch, CURLOPT_URL, $url);
                                            // Execute
                                            $result = curl_exec($ch);
                                            // Closing
                                            curl_close($ch);

                                            $obj = json_decode($result, true);
                                            $rate = number_format(floatval($obj[$query]), 2, '.', '');
                                        }
                                        if ($rate > 0) {
                                            $Currency->rate = $rate;
                                            $Currency->update();
                                        }
                                        // Log to history
                                        History::Log(
                                            $module = 6,
                                            $cat = 600,
                                            $window = 6004,
                                            $record_id = $Currency->id,
                                            $operation = 3,
                                            $title = $Currency->title_ar . "<br>" . trans("backLang.currencyRate") . ": " . $rate
                                        );
                                    }
                                    // save currency update date/time
                                    $Preference = Preference::find(1);
                                    $Preference->currency_updated_at = date('Y-m-d H:i:s');
                                    $Preference->save();

                                }
                            }

                        }
                    }

                    // delete notifications
                    if ($del_notifications) {
                        $del_notifications_date = date('Y-m-d 23:59:59');
                        if ($del_notifications_days > 0) {
                            $del_notifications_date = date('Y-m-d 23:59:59', strtotime('-' . $del_notifications_days . ' day'));
                        }
                        Notice::where("created_at", "<=", $del_notifications_date)->delete();

                    }

                    // delete activities
                    if ($del_activities) {
                        $del_activities_date = date('Y-m-d 23:59:59');
                        if ($del_activities_days > 0) {
                            $del_activities_date = date('Y-m-d 23:59:59', strtotime('-' . $del_activities_days . ' day'));
                        }
                        Activity::where("created_at", "<=", $del_activities_date)->delete();

                    }

                    // delete messages
                    if ($del_messages) {
                        $del_messages_date = date('Y-m-d 23:59:59');
                        if ($del_messages_days > 0) {
                            $del_messages_date = date('Y-m-d 23:59:59', strtotime('-' . $del_messages_days . ' day'));
                        }
                        Message::where("created_at", "<=", $del_messages_date)->delete();

                    }

                    // delete events
                    if ($del_events) {
                        $del_events_date = date('Y-m-d 23:59:59');
                        if ($del_events_days > 0) {
                            $del_events_date = date('Y-m-d 23:59:59', strtotime('-' . $del_events_days . ' day'));
                        }
                        $Events = Event::where("created_at", "<=", $del_events_date)->get();
                        if (count($Events) > 0) {
                            foreach ($Events as $Event) {
                                if ($Event->photo != "") {
                                    if (file_exists("uploads/events/" . $Event->photo)) {
                                        File::delete("uploads/events/" . $Event->photo);
                                    }
                                }
                                $Attachments = Attachment::where('cat', 1)->where('cat_id', $Event->id)->get();
                                foreach ($Attachments as $Attachment) {
                                    if ($Attachment->file != "") {
                                        if ($Attachment->remote && Helper::FileSystem() != "") {
                                            Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                        } else {
                                            if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                                File::delete("uploads/attachments/" . $Attachment->file);
                                            }
                                        }
                                    }
                                    $Attachment->shares()->delete();
                                    $Attachment->comments()->delete();
                                    $Attachment->alerts()->delete();
                                    $Attachment->delete();
                                }

                                $Event->shares()->delete();
                                $Event->comments()->delete();
                                $Event->alerts()->delete();
                                $Event->delete();
                            }
                        }

                    }

                    // delete alerts
                    if ($del_alerts) {
                        $del_alerts_date = date('Y-m-d 23:59:59');
                        if ($del_alerts_days > 0) {
                            $del_alerts_date = date('Y-m-d 23:59:59', strtotime('-' . $del_alerts_days . ' day'));
                        }
                        Alert::where("created_at", "<=", $del_alerts_date)->delete();
                    }

                    // delete attachments
                    if ($del_attachments) {
                        $del_attachments_date = date('Y-m-d 23:59:59');
                        if ($del_attachments_days > 0) {
                            $del_attachments_date = date('Y-m-d 23:59:59', strtotime('-' . $del_attachments_days . ' day'));
                        }
                        $Attachments = Attachment::where("created_at", "<=", $del_attachments_date)->get();
                        if (count($Attachments) > 0) {
                            foreach ($Attachments as $Attachment) {
                                function run_delete($item, $path)
                                {
                                    if ($item->file != "") {
                                        if ($item->remote && Helper::FileSystem() != "") {
                                            Storage::disk(Helper::FileSystem())->delete($item->file);
                                        } else {
                                            if (file_exists($path . $item->file)) {
                                                File::delete($path . $item->file);
                                            }
                                        }
                                    }
                                    $item->shares()->delete();
                                    $item->comments()->delete();
                                    $item->alerts()->delete();
                                    $item->delete();
                                }

                                // delete all sub
                                $FolderSubItems = Helper::FolderSubItems($Attachment->id);
                                if (count($FolderSubItems) > 0) {
                                    foreach ($FolderSubItems as $FolderSubItem) {
                                        run_delete(Attachment::find($FolderSubItem['id']), "uploads/attachments/");
                                    }
                                }

                                // Delete main folder
                                run_delete($Attachment, "uploads/attachments/");

                            }
                        }
                    }

                    // delete backups
                    if ($del_backups) {
                        $del_backups_date = date('Y-m-d 23:59:59');
                        if ($del_backups_days > 0) {
                            $del_backups_date = date('Y-m-d 23:59:59', strtotime('-' . $del_backups_days . ' day'));
                        }
                        // storage location is project_root/storage/app/backup.json file.
                        $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                        $i = 0;
                        $new_backup_list = [];
                        foreach ($backup_list as $backup_row) {
                            //check the property of every element
                            if ($backup_row->created_at <= $del_backups_date) {
                                unset($backup_list[$i]);

                                if ($backup_row->remote && Helper::FileSystem() != "") {
                                    Storage::disk(Helper::FileSystem())->delete("backups/" . $backup_row->file);
                                } else {
                                    Storage::disk('local')->delete("backups/" . $backup_row->file);
                                }

                            } else {
                                $new_bu = [
                                    "title" => $backup_row->title,
                                    "file" => $backup_row->file,
                                    "remote" => $backup_row->remote,
                                    "created_by" => $backup_row->created_by,
                                    "created_ip" => $backup_row->created_ip,
                                    "created_at" => $backup_row->created_at,
                                ];
                                array_push($new_backup_list, $new_bu);
                            }
                            $i++;
                        }
                        Storage::disk('local')->put('backup.json', json_encode($new_backup_list));

                    }

                    // delete chats
                    if ($del_chats) {
                        $del_chats_date = date('Y-m-d 23:59:59');
                        if ($del_chats_days > 0) {
                            $del_chats_date = date('Y-m-d 23:59:59', strtotime('-' . $del_chats_days . ' day'));
                        }

                        $Chats = Chat::where("created_at", "<=", $del_chats_date)->get();
                        if (count($Chats) > 0) {
                            foreach ($Chats as $Message) {
                                $Attachments = Attachment::where('cat', 2)->where('cat_id', $Message->id)->get();
                                foreach ($Attachments as $Attachment) {
                                    if ($Attachment->file != "") {
                                        if ($Attachment->remote && Helper::FileSystem() != "") {
                                            Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                        } else {
                                            if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                                File::delete("uploads/attachments/" . $Attachment->file);
                                            }
                                        }
                                    }
                                    $Attachment->shares()->delete();
                                    $Attachment->comments()->delete();
                                    $Attachment->alerts()->delete();
                                    $Attachment->delete();
                                }
                                $Message->delete();
                            }
                        }
                    }
                } catch (\Exception $e) {
                }
            }

        })->everyMinute();

    }

    /**
     * Register the commands for the application.
     *
     * @return void
     */
    protected function commands()
    {
        $this->load(__DIR__ . '/Commands');

        require base_path('routes/console.php');
    }
}
