<?php

namespace App\Exports;

use App\Models\Bank;
use App\Models\Currency;
use App\Helpers\Columns;
use App\Models\Journal;
use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class BalanceExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_currency;
    protected $search_user;
    protected $search_start;
    protected $search_end;
    protected $search_period;
    protected $search_center_id;
    protected $search_posted;
    protected $search_type;
    protected $search_ledger;
    protected $rows_count;


    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_currency = $request->search_currency;
        $this->search_user = $request->search_user;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;
        $this->search_center_id = $request->search_center_id;
        $this->search_period = $request->search_period;
        $this->search_posted = $request->search_posted;
        $this->search_type = $request->search_type;
        $this->search_ledger = $request->search_ledger;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;

        if (Roles::Check("view")) {
            $Journals = Journal::where("created_by", @Auth::user()->id)->where("type", $this->search_type);
        } else {
            $Journals = Journal::where("type", $this->search_type);
        }

        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($this->search_currency > 0) {
            $Journals = $Journals->where("currency_id", $this->search_currency);
        }
        if ($this->search_posted != "") {
            $Journals = $Journals->where("posted", $this->search_posted);
        }
        if ($this->search_ledger > 0) {
            $Journals = $Journals->where("ledger_id", $this->search_ledger);
        }
        if ($this->search_center_id > 0) {
            $Journals = $Journals->where("center_id", $this->search_center_id);
        }
        if ($this->search_user > 0) {
            $Journals = $Journals->where("created_by", $this->search_user);
        }
        if ($this->search_period > 0) {
            $Journals = $Journals->where("period_id", $this->search_period);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }

        $Journals = $Journals->orderBy("date", "asc")->get();

        $this->rows_count = count($Journals);
        if (count($Journals) > 0) {
            foreach ($Journals as $Journal) {
                $nestedData = [];
                $nestedData[] = date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M"));
                $nestedData[] = @$Journal->ledger->code;
                $nestedData[] = @$Journal->ledger->{"title_" . trans("backLang.code")};
                $nestedData[] = (($Journal->dept > 0) ? $Journal->dept : "-");
                $nestedData[] = (($Journal->credit > 0) ? $Journal->credit : "-");
                $nestedData[] = @$Journal->currency->code;
                $nestedData[] = $Journal->details;
                $nestedData[] = ((($Journal->posted) ? trans("backLang.posted") : trans("backLang.notPosted")));
                $nestedData[] = strip_tags(Helper::Category($Journal->center_id));
                $nestedData[] = $Journal->user->name;
                $collections[] = $nestedData;
            }
        }
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.date"),
            trans("backLang.ledgerCode"),
            trans("backLang.ledger"),
            trans("backLang.debtor"),
            trans("backLang.creditor"),
            trans("backLang.currency"),
            trans("backLang.desc"),
            trans("backLang.recordStatus"),
            trans("backLang.costPosition"),
            trans("backLang.createdBy")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:J1';
                $AllCellRange = 'A1:J' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('I')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('J')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
