<?php

namespace App\Exports;

use App\Models\Currency;
use App\Models\Installment;
use App\Models\Loan;
use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class InstallmentsExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_cat;
    protected $search_start;
    protected $search_end;
    protected $search_paid;
    protected $rows_count;

    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_cat = $request->search_cat;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;;
        $this->search_paid = $request->search_paid;;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;



        $loans_ids = Loan::select("id")->where("status", true);
        $Installments = Installment::wherein("loan_id", $loans_ids);
        if ($this->search_paid != "") {
            $Installments = $Installments->where("paid_status", $this->search_paid);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Installments = $Installments->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Installments = $Installments->where('date', "<=", $search_end_date);
        }
        if ($this->search_cat > 0) {
            $loans_ids = Loan::select("id")->where("cat_id", $this->search_cat);
            if (!empty($loans_ids)) {
                $Installments = $Installments->wherein("loan_id", $loans_ids);
            }
        }

        if ($search_word != "") {
            $n_search_q = $search_word;
            if ($n_search_q != "") {
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "0") {
                    $n_search_q = substr($n_search_q, 1);
                }
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "0") {
                    $n_search_q = substr($n_search_q, 1);
                }
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "+") {
                    $n_search_q = substr($n_search_q, 1);
                }
            }

            $Installments = $Installments->where(function ($query) use ($search_word, $n_search_q) {
                $query->where("notes", "like", "%" . $search_word . "%");
            });
        }
        $Installments = $Installments->orderby("date", "asc")->orderby("id", "asc")->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

        $this->rows_count = count($Installments);
        if (count($Installments) > 0) {
            foreach ($Installments as $Installment) {
                if (@$Installment->loan->pay_method == 0) {
                    $i_date_note = trans("backLang.LoanPayMethods.0");
                } else {
                    $i_date_note = "";
                }
                $i_date =  strip_tags(date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M")));
                $Installment_status_title = trans("backLang.unpaidInstallment");
                $Installment_status_date = "";
                if ($Installment->paid_status) {
                    $Installment_status_title = trans("backLang.paidInstallment");
                    $Installment_status_date = strip_tags(Helper::DateView($Installment->paid_at, 1));
                }

                $nestedData = [];
                $nestedData[] =  $Installment->amount;
                $nestedData[] = @$DefaultCurrency->symbol;
                $nestedData[] = $i_date;
                $nestedData[] = $i_date_note;
                $nestedData[] = $Installment_status_title;
                $nestedData[] = $Installment_status_date;
                $nestedData[] = @$Installment->loan->name;
                $nestedData[] = @$Installment->loan->id_number;
                $nestedData[] = trans('backLang.identityTypes')[@$Installment->loan->id_type];
                $nestedData[] = @$Installment->loan->phone;
                $nestedData[] = (($Installment->notes != "") ? $Installment->notes : "-");
                $nestedData[] = $Installment->user->name;
                $collections[] = $nestedData;
            }
        }
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.installmentAmount"),
            trans("backLang.currency"),
            trans("backLang.dueDate"),
            trans("backLang.notes"),
            trans("backLang.status"),
            trans("backLang.payDate"),
            trans("backLang.name"),
            trans("backLang.idNumber"),
            trans("backLang.identityType"),
            trans("backLang.phone"),
            trans("backLang.notes"),
            trans("backLang.createdBy")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:L1';
                $AllCellRange = 'A1:L' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('I')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('J')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('K')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('L')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
