<?php

namespace App\Exports;

use App\Models\Bank;
use App\Models\Currency;
use App\Helpers\Columns;
use App\Models\Journal;
use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class JournalExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_currency;
    protected $search_user;
    protected $search_start;
    protected $search_end;
    protected $search_period;
    protected $search_center_id;
    protected $search_posted;
    protected $search_ledger;
    protected $rows_count;


    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_currency = $request->search_currency;
        $this->search_user = $request->search_user;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;
        $this->search_center_id = $request->search_center_id;
        $this->search_period = $request->search_period;
        $this->search_posted = $request->search_posted;
        $this->search_ledger = $request->search_ledger;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($this->search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $this->search_currency);
        }
        if ($this->search_posted != "") {
            $Receipts = $Receipts->where("posted", $this->search_posted);
        }

        if ($this->search_ledger > 0) {
            $receipt_ids = Journal::select("receipt_id")->where('ledger_id', $this->search_ledger);
            $Receipts = $Receipts->wherein("id", $receipt_ids);
        }
        if ($this->search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $this->search_center_id);
        }
        if ($this->search_user > 0) {
            $Receipts = $Receipts->where("created_by", $this->search_user);
        }
        if ($this->search_period > 0) {
            $Receipts = $Receipts->where("period_id", $this->search_period);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $Receipts = $Receipts->orderBy("date", "asc")->get();

        $a_count = 0;
        if (count($Receipts) > 0) {
            foreach ($Receipts as $Receipt) {
                $j_count = $Receipt->journals()->count();
                if ($j_count > 0) {
                    foreach ($Receipt->journals as $Journal) {
                        $nestedData = [];
                        $nestedData[] = $Receipt->id;
                        $nestedData[] = date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M"));
                        $nestedData[] = (($Journal->dept > 0) ? $Journal->dept : "-");
                        $nestedData[] = (($Journal->credit > 0) ? $Journal->credit : "-");
                        $nestedData[] = @$Receipt->currency->code;
                        $nestedData[] = @$Journal->ledger->code;
                        $nestedData[] = @$Journal->ledger->{"title_" . trans("backLang.code")};
                        $nestedData[] = $Journal->details;
                        $nestedData[] = $Receipt->code;
                        $nestedData[] = ((($Receipt->posted) ? trans("backLang.posted") : trans("backLang.notPosted")));
                        $nestedData[] = strip_tags(Helper::Category($Receipt->center_id));
                        $nestedData[] = $Receipt->user->name;
                        $collections[] = $nestedData;
                        $a_count++;
                    }
                }
            }
        }

        $this->rows_count = $a_count;
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.recordId"),
            trans("backLang.date"),
            trans("backLang.debtor"),
            trans("backLang.creditor"),
            trans("backLang.currency"),
            trans("backLang.ledgerCode"),
            trans("backLang.ledger"),
            trans("backLang.desc"),
            trans("backLang.fileNo"),
            trans("backLang.recordStatus"),
            trans("backLang.costPosition"),
            trans("backLang.createdBy")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:L1';
                $AllCellRange = 'A1:L' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('I')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('J')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('K')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
