<?php

namespace App\Exports;

use App\Helpers\Columns;
use App\Models\Journal;
use App\Models\Ledger;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class LedgersExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_start;
    protected $search_end;
    protected $search_period;
    protected $search_ledger;
    protected $rows_count;

    public function __construct($request)
    {
        $this->search_start = @$request->search_start;
        $this->search_end = @$request->search_end;
        $this->search_period = @$request->search_period;
        $this->search_ledger = @$request->search_ledger;
    }

    public function array(): array
    {
        $collections = [];
        $Ledgers = Ledger::where('id', ">", 0);
        if ($this->search_ledger > 0) {
            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            $Ledger = Ledger::find($this->search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
            // End prepare ledgers ids
            $Ledgers = $Ledgers->whereIn("id", $Ledgers_Ids);
        }
        $Ledgers = $Ledgers->orderBy("code", "asc")->get();

        if (count($Ledgers) > 0) {
            $search_period = Helper::Preferences("default_period");
            if ($this->search_period != "") {
                $search_period = $this->search_period;
            }
            $search_start = $this->search_start;
            $search_end = $this->search_end;
            $Ledgers = $Ledgers->map(function ($Ledger) use ($search_period, $search_start, $search_end) {
                $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_dept = $sum_of_dept->sum("cost_dept");
                $sum_of_credit = $sum_of_credit->sum("cost_credit");

                if ($search_start != "") {
                    $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                    $sum_of_dept = $sum_of_dept->where('date', ">=", $search_start_date);
                    $sum_of_credit = $sum_of_credit->where('date', ">=", $search_start_date);
                }
                if ($search_end != "") {
                    $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                    $sum_of_dept = $sum_of_dept->where('date', "<=", $search_end_date);
                    $sum_of_credit = $sum_of_credit->where('date', "<=", $search_end_date);
                }

                $Ledger->dept = $sum_of_dept;
                $Ledger->credit = $sum_of_credit;
                return $Ledger;
            });
        }
        $this->rows_count = count($Ledgers);
        foreach ($Ledgers as $Ledger) {
            $nestedData = [];
            $nestedData[] = $Ledger->code;
            $nestedData[] = $Ledger->{"title_" . trans("backLang.code")};
            $nestedData[] = ($Ledger->group) ? trans("backLang.group") : trans("backLang.account");
            if ($Ledger->group) {
                $nestedData[] = "-";
                $nestedData[] = "-";
            } else {
                $nestedData[] = $Ledger->dept;
                $nestedData[] = $Ledger->credit;
            }
            $collections[] = $nestedData;
        }

        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.ledgerCode"),
            trans("backLang.ledgerName"),
            trans("backLang.ledgerType"),
            trans("backLang.creditor"),
            trans("backLang.debtor")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:E1';
                $AllCellRange = 'A1:E' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
