<?php

namespace App\Exports;

use App\Models\Loan;
use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class LoansExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_cat_id;
    protected $search_user;
    protected $search_start;
    protected $search_end;
    protected $search_status;
    protected $rows_count;

    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_cat_id = $request->search_cat_id;
        $this->search_user = $request->search_user;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;;
        $this->search_status = $request->search_status;;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;


        if (Roles::Check("view")) {
            $Loans = Loan::where("created_by", @Auth::user()->id);
        } else {
            $Loans = Loan::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Loans = $Loans->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere("phone", $search_word)
                    ->orwhere("email", $search_word)
                    ->orwhere("id_number", $search_word)
                    ->orwhere("notes", "like", "%" . $search_word . "%")
                    ->orwhere("guarantor_name", "like", "%" . $search_word . "%")
                    ->orwhere("witness_1_name", "like", "%" . $search_word . "%")
                    ->orwhere("witness_2_name", "like", "%" . $search_word . "%");
            });
        }
        if ($this->search_cat_id > 0) {
            $Loans = $Loans->where("cat_id", $this->search_cat_id);
        }
        if ($this->search_user > 0) {
            $Loans = $Loans->where("created_by", $this->search_user);
        }
        if ($this->search_status != "") {
            $Loans = $Loans->where("status", $this->search_status);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Loans = $Loans->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Loans = $Loans->where('date', "<=", $search_end_date);
        }
        $Loans = $Loans->orderby("id", "asc")->get();

        $this->rows_count = count($Loans);
        if (count($Loans) > 0) {
            foreach ($Loans as $Loan) {
                $paid_amount = $Loan->installments()->where("paid_status", true)->sum("amount");
                $nestedData = [];
                $nestedData[] = $Loan->id;
                $nestedData[] = date(Helper::Preferences("date_format"), strtotime($Loan->date)) . " " . ((@$Loan->date_type) ? trans("backLang.H") : trans("backLang.M"));
                $nestedData[] = $Loan->name;
                $nestedData[] = trans('backLang.identityTypes')[$Loan->id_type];
                $nestedData[] = $Loan->id_number;
                $nestedData[] = $Loan->phone;
                $nestedData[] = $Loan->email;
                $nestedData[] = strip_tags(Helper::Category($Loan->cat_id));
                $nestedData[] = $Loan->amount;
                $nestedData[] = $Loan->total;
                $nestedData[] = $Loan->instalments_count;
                $nestedData[] = $Loan->instalments_amount;
                $nestedData[] = $Loan->instalments_last;
                $nestedData[] = date(Helper::Preferences("date_format"), strtotime($Loan->instalments_date)) . " " . ((@$Loan->date_type) ? trans("backLang.H") : trans("backLang.M"));
                $nestedData[] = trans("backLang.LoanPayMethods." . $Loan->pay_method);
                $nestedData[] = $paid_amount;
                $nestedData[] = round($Loan->total - $paid_amount, 2);
                $nestedData[] = $Loan->user->name;
                $collections[] = $nestedData;
            }
        }
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.recordId"),
            trans("backLang.date"),
            trans("backLang.beneficiaryName"),
            trans("backLang.identityType"),
            trans("backLang.idNumber"),
            trans("backLang.phone"),
            trans("backLang.email"),
            trans("backLang.loansCategory"),
            trans("backLang.loanAmount"),
            trans("backLang.loanTotal"),
            trans("backLang.loanInstalmentCount"),
            trans("backLang.loanInstalmentAmount"),
            trans("backLang.loanLastInstalmentAmount"),
            trans("backLang.loanFirstInstalmentDate"),
            trans("backLang.LoanPayMethod"),
            trans("backLang.paid"),
            trans("backLang.unpaid"),
            trans("backLang.createdBy")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:R1';
                $AllCellRange = 'A1:R' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('I')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('J')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('K')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('L')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('M')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('N')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('O')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('P')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('Q')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('R')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
