<?php

namespace App\Exports;

use App\Models\Bank;
use App\Models\Currency;
use App\Helpers\Columns;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Events\AfterSheet;
use Helper;
use Roles;

class PositionsReportExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_user;
    protected $search_start;
    protected $search_end;
    protected $search_period;
    protected $search_center_id;
    protected $rows_count;


    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_user = $request->search_user;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;
        $this->search_center_id = $request->search_center_id;
        $this->search_period = $request->search_period;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;


        $Journals = Journal::where('posted', true)->where("center_id", $this->search_center_id);

        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($this->search_user > 0) {
            $Journals = $Journals->where("created_by", $this->search_user);
        }
        if ($this->search_period > 0) {
            $Journals = $Journals->where("period_id", $this->search_period);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }

        $Journals = $Journals->orderBy("date", "asc")->orderBy("id", "asc")->get();
        $this->rows_count = count($Journals);
        $i = 0;
        $balance = 0;
        $total_of_dept = 0;
        $total_of_credit = 0;
        if (count($Journals) > 0) {
            foreach ($Journals as $Journal) {
                $nestedData = [];

                if ($Journal->cost_dept > 0) {
                    $total_of_dept += $Journal->cost_dept;
                }
                if ($Journal->cost_credit > 0) {
                    $total_of_credit += $Journal->cost_credit;
                }
                $cur_details = "";
                if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                    if ($Journal->dept > 0) {
                        $cur_details = " ( " . $Journal->dept . " " . @$Journal->currency->symbol . " )";
                    } else {
                        $cur_details = " ( " . $Journal->credit . " " . @$Journal->currency->symbol . " )";
                    }
                }

                $type = trans("backLang.dailyEntry");
                if ($Journal->type == 2) {
                    $type = trans("backLang.reconciliationEntry");
                } elseif ($Journal->type == 1) {
                    $type = trans("backLang.startBalance");
                } else {
                    if (@$Journal->receipt->type == 1) {
                        $type = trans("backLang.cashReceipt");
                    } elseif (@$Journal->receipt->type == 0) {
                        $type = trans("backLang.payReceipt");
                    }
                }

                $nestedData[] = $type;
                $nestedData[] = (($Journal->dept > 0) ? $Journal->cost_dept : "");
                $nestedData[] = (($Journal->credit > 0) ? $Journal->cost_credit : "");
                $nestedData[] = $Journal->details . $cur_details;
                $nestedData[] = (date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")));
                $nestedData[] = $Journal->id;
                $nestedData[] = @$Journal->receipt->id;
                $nestedData[] = @$Journal->receipt->code;
                $collections[] = $nestedData;

                $i++;
            }
        }
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.ledgerType"),
            trans("backLang.debtor"),
            trans("backLang.creditor"),
            trans("backLang.desc"),
            trans("backLang.date"),
            trans("backLang.entryNo"),
            trans("backLang.dailyEntry"),
            trans("backLang.fileNo")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:H1';
                $AllCellRange = 'A1:H' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
