<?php

namespace App\Exports;

use App\Models\Receipt;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\WithEvents;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Helper;
use Maatwebsite\Excel\Events\AfterSheet;
use Roles;

class ReceiptExport implements FromArray, WithHeadings, WithEvents
{
    use Exportable;

    protected $search_word;
    protected $search_type;
    protected $search_fund;
    protected $search_pay_method;
    protected $search_currency;
    protected $search_cat;
    protected $search_user;
    protected $search_start;
    protected $search_end;
    protected $search_period;
    protected $search_center_id;
    protected $search_ledger;
    protected $rows_count;

    public function __construct($request)
    {
        $this->search_word = $request->search_word;
        $this->search_type = $request->search_type;
        $this->search_fund = $request->search_fund;
        $this->search_pay_method = $request->search_pay_method;
        $this->search_currency = $request->search_currency;
        $this->search_cat = $request->search_cat;
        $this->search_user = $request->search_user;
        $this->search_start = $request->search_start;
        $this->search_end = $request->search_end;
        $this->search_center_id = $request->search_center_id;
        $this->search_period = $request->search_period;
        $this->search_ledger = $request->search_ledger;
    }

    public function array(): array
    {
        $collections = [];
        $search_word = $this->search_word;

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", '>', 0);
        }
        if ($this->search_type) {
            $Receipts = $Receipts->where("type", true);
        } else {
            $Receipts = $Receipts->where("type", false);
        }
        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($this->search_fund > 0) {
            $Receipts = $Receipts->where("bank_id", $this->search_fund);
        }
        if ($this->search_ledger > 0) {
            $Receipts = $Receipts->where("ledger_id", $this->search_ledger);
        }
        if ($this->search_period > 0) {
            $Receipts = $Receipts->where("period_id", $this->search_period);
        }
        if ($this->search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $this->search_pay_method);
        }
        if ($this->search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $this->search_currency);
        }
        if ($this->search_cat > 0) {
            $Receipts = $Receipts->where("cat_id", $this->search_cat);
        }

        if ($this->search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $this->search_center_id);
        }
        if ($this->search_user > 0) {
            $Receipts = $Receipts->where("created_by", $this->search_user);
        }
        if ($this->search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($this->search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($this->search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }
        $Receipts = $Receipts->orderBy("date", "asc")->get();

        $this->rows_count = count($Receipts);
        if (count($Receipts) > 0) {
            foreach ($Receipts as $Receipt) {
                $nestedData = [];
                $nestedData[] = $Receipt->id;
                $nestedData[] = date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M"));
                $nestedData[] = $Receipt->amount;
                $nestedData[] = @$Receipt->currency->code;
                $nestedData[] = $Receipt->details;
                $nestedData[] =  @$Receipt->ledger->code;
                $nestedData[] = @$Receipt->ledger->{"title_" . trans("backLang.code")};
                $nestedData[] = strip_tags(Helper::Category($Receipt->cat_id));
                $nestedData[] = strip_tags(Helper::Category($Receipt->center_id));
                $nestedData[] = $Receipt->user->name;
                $collections[] = $nestedData;
            }
        }
        return $collections;
    }

    public function headings(): array
    {
        $heading = [
            trans("backLang.recordId"),
            trans("backLang.date"),
            trans("backLang.amount"),
            trans("backLang.currency"),
            trans("backLang.desc"),
            trans("backLang.accountId"),
            trans("backLang.nameOfCustomer"),
            trans("backLang.ledgerCat"),
            trans("backLang.costPosition"),
            trans("backLang.createdBy")
        ];
        return $heading;
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {

                // All headers - set font size to 14
                $HeadCellRange = 'A1:J1';
                $AllCellRange = 'A1:J' . ($this->rows_count + 1);

                // Set first row to height 25
                $event->sheet->getDelegate()->getRowDimension(1)->setRowHeight(25);

                $event->sheet->getDelegate()->getColumnDimension('A')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('B')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('C')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('D')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('E')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('F')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('G')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('H')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('I')->setAutoSize(true);
                $event->sheet->getDelegate()->getColumnDimension('J')->setAutoSize(true);

                $HeadStyleArray = [
                    'font' => [
                        'size' => 14,
                        'bold' => true,
                        'color' => ['argb' => 'ffffffff'],
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'color' => ['argb' => '00e0060'],
                    ],
                ];
                $AllStyleArray = [
                    'font' => [
                        'size' => 13,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '00e0060'],
                        ],
                    ]
                ];
                $event->sheet->getDelegate()->getStyle($HeadCellRange)->applyFromArray($HeadStyleArray);
                $event->sheet->getDelegate()->getStyle($AllCellRange)->applyFromArray($AllStyleArray);

                if (trans("backLang.code") == "ar")
                    $event->getDelegate()->setRightToLeft(true);
            },
        ];
    }
}
