<?php

namespace App\Helpers;

use Helper;
use App\Models\Template;
use App\Models\Message;
use App\Models\Notice;
use App\Models\Preference;
use App\Jobs\SendEmails;
use App\Jobs\SendSMS;
use App\Jobs\SendMessage;
use Mail;
use Auth;
use OneSignal;
use Cats;

class AppSender
{
    static function Notify($CaseId, $Data = ['subject' => '', 'message' => '', 'url' => ''], $Cat = '', $CatId = '', $Hidden = false, $IsCustomer = false)
    {
        /*
         * Data Basic Example:
         * -------------
         [
        "subject"=>"",
        "message"=>"",
        "subject_en"=>"",
        "message_en"=>"",
        "url"=>"",
        "type"=>"",
        "to_user_id"=>"",
        "to_name"=>"",
        "to_email"=>"",
        "to_phone"=>"",
        ]
         */

        $Case = [];
        $Result = [
            "system" => 0,
            "mail" => 0,
            "sms" => 0,
            "browser" => 0
        ];
        $Preference = Preference::find(1);
        if (!empty($Preference)) {
            if ($Preference->notification_cases != "") {
                try {
                    $Cases_list = json_decode($Preference->notification_cases);
                    $Case = $Cases_list->$CaseId;
                } catch (\Exception $e) {

                }
            }
        }
        if (!empty($Case)) {
            $TemplateId = 0;
            foreach (Notifications::Cases() as $key => $NotificationCase) {
                if ($CaseId == $key) {
                    $TemplateId = $NotificationCase['template'];
                }
            }
            if ($Case->system && $TemplateId > 0) {
                // send on system
                $Result["system"] = AppSender::ToSystem($TemplateId, $Data, @$Data['to_user_id'], $Cat, $CatId, $IsCustomer);
            }
            if ($Case->mail && $TemplateId > 0) {
                // send via mail
                $Result["mail"] = AppSender::ToMail(@$Data['to_email'], @$Data['to_name'], $TemplateId, $Data, @$Data['to_user_id'], $Cat, $CatId, $Hidden, $IsCustomer);
            }
            if ($Case->sms && $TemplateId > 0) {
                // send via sms
                $Result["sms"] = AppSender::ToSMS(@$Data['to_phone'], $TemplateId, $Data, @$Data['to_user_id'], $Cat, $CatId, $Hidden, $IsCustomer);
            }
            if ($Case->browser && $TemplateId > 0) {
                // send via onesignal
                $Result["browser"] = AppSender::ToBrowser($TemplateId, $Data, @$Data['to_user_id'], $Cat, $CatId, $Hidden, $IsCustomer);
            }
        }

        return $Result;
    }


    static function ToSystem($TemplateId = '', $Data = ['type' => '', 'subject' => '', 'message' => '', 'url' => ''], $ToUserId = 0, $Cat = '', $CatId = '', $IsCustomer = false)
    {
        $subject_ar = @$Data['subject'];
        $body_ar = @$Data['message'];
        $subject_en = @$Data['subject_en'];
        $body_en = @$Data['message_en'];
        $attached_url = @$Data['url'];
        $Template = [];
        if ($TemplateId > 0) {
            $Template = Template::where('cat', 3)->where('stat', $TemplateId)->first();
        }
        if (!empty($Template)) {
            $subject_ar = $Template->ParseSubject($Data);
            $subject_en = $subject_ar;
            $body_ar = nl2br($Template->ParseContent($Data));
            $body_en = $body_ar;
            $attached_url = $Template->ParseURL($Data);
        }

        // send on system
        $Notice = new Notice;
        $Notice->type = @$Data['type'];
        $Notice->title_ar = $subject_ar;
        $Notice->details_ar = $body_ar;
        $Notice->title_en = $subject_en;
        $Notice->details_en = $body_en;
        $Notice->url = $attached_url;
        $Notice->icon = @$Data['icon'];
        $Notice->status = 0;
        $Notice->cat = $Cat;
        $Notice->cat_id = $CatId;
        if ($IsCustomer) {
            $Notice->to_customer_id = $ToUserId;
        } else {
            $Notice->to_user_id = $ToUserId;
        }
        $Notice->created_ip = \Request::ip();
        $Notice->created_by = @Auth::user()->id;
        $Notice->save();

        return true;
    }

    static function ToMail($ToEmail, $ToName, $TemplateId = '', $MessageData = ['subject' => '', 'message' => ''], $ToUserId = 0, $Cat = '', $CatId = '', $Hidden = false, $IsCustomer = false)
    {
        if (Helper::CheckMailStatus()) {
            try {
                $Template = [];
                if ($TemplateId > 0) {
                    $Template = Template::where('cat', 0)->where('stat', $TemplateId)->first();
                }
                $MAIL_FROM_ADDRESS = env('MAIL_FROM_ADDRESS');
                $MAIL_FROM_NAME = env('MAIL_FROM_NAME');
                $subject = @$MessageData['subject'];
                $body = @$MessageData['message'];
                if (!empty($Template)) {
                    $subject = $Template->ParseSubject($MessageData);
                    $body = $Template->ParseEmailContent($MessageData);
                }
                // save to messages history
                $Message = new Message;
                $Message->type = 0;
                $Message->title = $subject;
                $Message->details = $body;
                $Message->url = "";
                $Message->status = 0;
                $Message->cat = $Cat;
                $Message->cat_id = $CatId;
                $Message->hidden = $Hidden;
                if ($IsCustomer) {
                    $Message->to_customer_id = $ToUserId;
                } else {
                    $Message->to_user_id = $ToUserId;
                }
                $Message->created_ip = \Request::ip();
                $Message->created_by = @Auth::user()->id;
                $Message->to_email = $ToEmail;
                $Message->to_phone = "";
                $Message->save();

                if (env("QUEUE_MAIL_STATUS", false)) {
                    $mail_details = [
                        "from_email" => $MAIL_FROM_ADDRESS,
                        "from_name" => $MAIL_FROM_NAME,
                        "to_email" => $ToEmail,
                        "to_name" => $ToName,
                        "subject" => $subject,
                        "body" => $body,
                        "message_id" => $Message->id
                    ];
                    SendEmails::dispatch($mail_details)->delay(now()->addSeconds(env("QUEUE_DISPATCH_DELAY", 2)));
                } else {
                    Mail::send([], [], function ($message) use ($subject, $body, $ToEmail, $ToName, $MAIL_FROM_ADDRESS, $MAIL_FROM_NAME) {
                        $message->from($MAIL_FROM_ADDRESS, $MAIL_FROM_NAME)
                            ->to($ToEmail, $ToName)
                            ->subject($subject)
                            ->setBody($body, 'text/html');
                    });
                    $Message->status = 1;
                    $Message->save();
                }
                return true;
            } catch (\Exception $e) {

                $Message->status = 2;
                $Message->save();

                return false;
            }
        }
        return false;
    }

    static function ToSMS($ToPhone, $TemplateId = '', $MessageData = ['message' => ''], $ToUserId = 0, $Cat = '', $CatId = '', $Hidden = false, $IsCustomer = false)
    {
        if (Helper::CheckSMSStatus()) {
            try {
                $Template = [];
                if ($TemplateId > 0) {
                    $Template = Template::where('cat', 1)->where('stat', $TemplateId)->first();
                }
                $body = @$MessageData['message'];
                if (!empty($Template)) {
                    $body = nl2br($Template->ParseContent($MessageData));
                }
                // save to messages history
                $Message = new Message;
                $Message->type = 1;
                $Message->title = "";
                $Message->details = $body;
                $Message->url = "";
                $Message->status = 0;
                $Message->cat = $Cat;
                $Message->cat_id = $CatId;
                $Message->hidden = $Hidden;
                if ($IsCustomer) {
                    $Message->to_customer_id = $ToUserId;
                } else {
                    $Message->to_user_id = $ToUserId;
                }
                $Message->created_ip = \Request::ip();
                $Message->created_by = @Auth::user()->id;
                $Message->to_email = "";
                $Message->to_phone = $ToPhone;
                $Message->save();
                if (env("QUEUE_SMS_STATUS", false)) {
                    $sms_details = [
                        "phone" => $ToPhone,
                        "message" => $body,
                        "message_id" => $Message->id
                    ];
                    SendSMS::dispatch($sms_details)->delay(now()->addSeconds(env("QUEUE_DISPATCH_DELAY", 2)));
                } else {
                    $send_sms = Helper::SMS_Send($ToPhone, $body);
                    $Message->status = $send_sms;
                    $Message->save();
                }
                return true;
            } catch (\Exception $e) {

                $Message->status = 2;
                $Message->save();

                return false;
            }
        }
    }

    static function ToBrowser($TemplateId = '', $MessageData = ['message' => '', 'url' => ''], $ToUserId = 0, $Cat = '', $CatId = '', $Hidden = false, $IsCustomer = false)
    {
        if (Helper::CheckOneSignalStatus()) {
            try {
                $Template = [];
                if ($TemplateId > 0) {
                    $Template = Template::where('cat', 2)->where('stat', $TemplateId)->first();
                }
                $body = @$MessageData['message'];
                $attached_url = (@$MessageData['url'] != "") ? @$MessageData['url'] : null;
                if (!empty($Template)) {
                    $body = $Template->ParseContent($MessageData);
                    $attached_url = $Template->ParseURL($MessageData);
                }
                // save to messages history
                $Message = new Message;
                $Message->type = 2;
                $Message->title = "";
                $Message->details = $body;
                $Message->url = $attached_url;
                $Message->status = 0;
                $Message->cat = $Cat;
                $Message->cat_id = $CatId;
                $Message->hidden = $Hidden;
                if ($IsCustomer) {
                    $Message->to_customer_id = $ToUserId;
                } else {
                    $Message->to_user_id = $ToUserId;
                }
                $Message->created_ip = \Request::ip();
                $Message->created_by = @Auth::user()->id;
                $Message->to_email = "";
                $Message->to_phone = "";
                $Message->save();
                if (env("QUEUE_MESSAGE_STATUS", false)) {
                    $sms_details = [
                        "attached_url" => $attached_url,
                        "message" => $body,
                        "message_id" => $Message->id
                    ];
                    SendMessage::dispatch($sms_details)->delay(now()->addSeconds(env("QUEUE_DISPATCH_DELAY", 2)));
                } else {
                    //Old syntax
                    //OneSignal::sendNotificationUsingTags($body, array(["field" => "tag", "key" => "userId", "relation" => "=", "value" => "$ToUserId"]), $attached_url, $data = null, $buttons = null, $schedule = null);

                    if ($IsCustomer) {
                        OneSignal::sendNotificationUsingTags($body, array(["key" => "customerId", "relation" => "=", "value" => "$ToUserId"]), $attached_url, $data = null, $buttons = null, $schedule = null);
                    } else {
                        OneSignal::sendNotificationUsingTags($body, array(["key" => "userId", "relation" => "=", "value" => "$ToUserId"]), $attached_url, $data = null, $buttons = null, $schedule = null);
                    }
                    $Message->status = true;
                    $Message->save();
                }
                return true;
            } catch (\Exception $e) {

                $Message->status = 2;
                $Message->save();

                return false;
            }
        }
        return false;
    }

}

?>
