<?php

namespace App\Helpers;

use Illuminate\Support\Facades\Storage;

class Backup
{

    static function dumpDatabase($sql_filename)
    {
        if (is_array($connectionData = config("database.connections.mysql"))) {
            if ($connectionData['driver'] == 'mysql') {

                $hostname = escapeshellarg($connectionData['host']);
                $port = $connectionData['port'];
                $database = $connectionData['database'];
                $username = escapeshellarg($connectionData['username']);
                $password = $connectionData['password'];
                $mysqldumpPath = env("MYSQLDUMP_PATH");

                $databaseArg = escapeshellarg($database);
                $portArg = !empty($port) ? '-P ' . escapeshellarg($port) : '';
                $passwordArg = !empty($password) ? '-p' . escapeshellarg($password) : '';

                $dumpCommand = "{$mysqldumpPath} -C -h {$hostname} {$portArg} -u{$username} {$passwordArg} --single-transaction --skip-lock-tables --quick {$databaseArg}";

                exec($dumpCommand, $dumpResult, $result);

                if ($result == 0) {
                    $dumpResult = implode(PHP_EOL, $dumpResult);
                    Backup::storeDumpFile($dumpResult, $sql_filename);
                    return true;
                }
            }
        }
        return false;
    }

    static function restoreDatabase($sql_filename)
    {
        if (is_array($connectionData = config("database.connections.mysql"))) {
            if ($connectionData['driver'] == 'mysql') {

                $hostname = escapeshellarg($connectionData['host']);
                $port = $connectionData['port'];
                $database = $connectionData['database'];
                $username = escapeshellarg($connectionData['username']);
                $password = $connectionData['password'];
                $mysqlPath = env("MYSQL_PATH");

                $databaseArg = escapeshellarg($database);
                $portArg = !empty($port) ? '-P ' . escapeshellarg($port) : '';
                $passwordArg = !empty($password) ? '-p' . escapeshellarg($password) : '';

                $localFilename = $sql_filename;
                $filename = $localFilename;
                $localDisk = 'local';
                $localPath = 'backups';


                $restoreCommand = "{$mysqlPath} -h {$hostname} {$portArg} -u{$username} {$passwordArg} {$databaseArg} < " . Backup::getAbsFilePath($filename, $localDisk, $localPath);

                exec($restoreCommand, $restoreResult, $result);

                if ($result == 0) {
                    return true;
                }
            }
        }
        return false;
    }

    static function storeDumpFile($data, $sql_filename)
    {
        $localDisk = 'local';
        Storage::disk($localDisk)->put(Backup::getFilePath($sql_filename), $data);
    }

    static function getFilePath($sql_filename)
    {
        $localPath = 'backups';
        $localPath = Backup::cleanPath($localPath);

        return $localPath . DIRECTORY_SEPARATOR . $sql_filename;
    }

    static function cleanPath($path)
    {
        return ltrim(rtrim($path, DIRECTORY_SEPARATOR), DIRECTORY_SEPARATOR);
    }

    static function getAbsFilePath($filename, $disk = null, $path = null)
    {
        $path = Backup::cleanPath($path);
        $final_path = Storage::disk($disk)->getDriver()->getAdapter()->getPathPrefix() . $path . DIRECTORY_SEPARATOR . $filename;
        $final_path = str_replace(" ", "\ ", $final_path);
        return $final_path;
    }

}
