<?php

namespace App\Helpers;

class Menus
{
    static function DashboardQuickMenu()
    {
        $QuickMenu = [];
        if (Roles::Check("role_calendar")) {
            // Calendar
            $QuickMenu[] = [
                "id" => "calendar",
                "title" => trans('backLang.calendar'),
                "icon" => "icon-calendar",
                "link" => route("AppEvents"),
            ];
        }

        if (Roles::Check("role_archive")) {
            // Attachments
            $QuickMenu[] = [
                "id" => "fileArchive",
                "title" => trans('backLang.fileArchive'),
                "icon" => "icon-folder-alt",
                "link" => route("AppAttachments"),
            ];
        }

        if (count(Helper::users("active")) > 1) {
            if (Roles::Check("role_chat")) {
                // Chat
                $QuickMenu[] = [
                    "id" => "chat",
                    "title" => trans('backLang.chat'),
                    "icon" => "icon-speech",
                    "link" => route("AppChat"),
                ];
            }
        }


        return $QuickMenu;
    }

    static function DashboardMainMenu()
    {
        $MainMenu = [];

        // example 1
//        $MainMenu[] = [
//            "id" => "link1",
//            "title" => ((trans('backLang.code') == "ar") ? "مثال رابط 2" : " Example 1"),
//            "icon" => "icon-present",
//            "link" => "#",
//            "sub" => []
//        ];


        // loans separated
        if (!Roles::Check("role_accountant")) {
            if (Roles::Check("role_loans")) {
                $MainMenu[] = [
                    "id" => "loansAndSolaf",
                    "title" => trans('backLang.loans'),
                    "icon" => "icon-briefcase",
                    "link" => route("Loans"),
                    "sub" => []
                ];
            }
        }

        // accountant
        if (Roles::Check("role_accountant")) {
            $SubMenu = [];


            // sub menu roles check
            if (Roles::Check("accountant_revenues")) {
                $SubMenu[] = [
                    "id" => "accountantRevenues",
                    "title" => trans('backLang.revenues'),
                    "link" => route("AppReceipts", "revenues"),
                ];
            }
            if (Roles::Check("accountant_expenses")) {
                $SubMenu[] = [
                    "id" => "accountantExpenses",
                    "title" => trans('backLang.expenses'),
                    "link" => route("AppReceipts", "expenses"),
                ];
            }
            if (Roles::Check("accountant_fund")) {
                $SubMenu[] = [
                    "id" => "accountantFunds",
                    "title" => trans('backLang.fundAccount'),
                    "link" => route("AppSafeBalance"),
                ];
            }
            if (Roles::Check("accountant_bank")) {
                $SubMenu[] = [
                    "id" => "accountantBank",
                    "title" => trans('backLang.bankAccount'),
                    "link" => route("AppBankBalance"),
                ];
            }
            if (Roles::Check("role_loans")) {
                $SubMenu[] = [
                    "id" => "accountantLoans",
                    "title" => trans('backLang.loans'),
                    "link" => route("Loans"),
                ];
            }

            if (Roles::Check("accountant_journals")) {
                $SubMenu[] = [
                    "id" => "dailyEntries",
                    "title" => trans('backLang.dailyEntries'),
                    "link" => route("AppJournals"),
                ];
            }
            if (Roles::Check("accountant_post_entries")) {
                $SubMenu[] = [
                    "id" => "postingToLedger",
                    "title" => trans('backLang.postingToLedger'),
                    "link" => route("AppAccountantPosting"),
                ];
            }
            if (Roles::Check("accountant_reconciliation")) {
                $SubMenu[] = [
                    "id" => "reconciliation",
                    "title" => trans('backLang.reconciliation'),
                    "link" => route("AppReconciliation"),
                ];
            }
            if (Roles::Check("accountant_opening_balances")) {
                $SubMenu[] = [
                    "id" => "openingBalances",
                    "title" => trans('backLang.openingBalances'),
                    "link" => route("AppOpeningBalances"),
                ];
            }

            if (Roles::Check("accountant_tree")) {
                $SubMenu[] = [
                    "id" => "accountantTree",
                    "title" => trans('backLang.accountantTree'),
                    "link" => route("AppLedgers"),
                ];
            }
            if (Roles::Check("app_preferences_accounting")) {
                $SubMenu[] = [
                    "id" => "accountingSettings",
                    "title" => trans('backLang.accountantSettings'),
                    "link" => route("appAccountingSettings"),
                ];
            }

            //prepare main menu
            $MainMenu[] = [
                "id" => "accountant",
                "title" => trans('backLang.accountant'),
                "icon" => "icon-wallet",
                "link" => route("AppAccountingHome"),
                "sub" => $SubMenu
            ];
        }
        return $MainMenu;
    }

    static function DashboardReportsMenu()
    {
        $ReportsMenu = [];

        /*
        // example 1
        $ReportsMenu[] = [
            "id" => "reports1",
            "title" => ((trans('backLang.code') == "ar") ? "مثال تقارير 1" : " Example Reports 1"),
            "description" => trans('backLang.generalSettingsDesc'),
            "icon" => "icon-puzzle",
            "link" => "#",
            "sub" => [
                [
                    "id" => "reports1-1",
                    "title" => ((trans('backLang.code') == "ar") ? "مثال تقارير 1.1" : " Example Reports 1.1"),
                    "description" => trans('backLang.generalSettingsDesc'),
                    "link" => "#",
                ],
                [
                    "id" => "reports1-2",
                    "title" => ((trans('backLang.code') == "ar") ? "مثال تقارير 1.2" : " Example Reports 1.2"),
                    "description" => trans('backLang.generalSettingsDesc'),
                    "link" => "#",
                ],
                [
                    "id" => "reports1-3",
                    "title" => ((trans('backLang.code') == "ar") ? "مثال تقارير 1.3" : " Example Reports 1.3"),
                    "description" => trans('backLang.generalSettingsDesc'),
                    "link" => "#",
                ],
            ]
        ];
        */

        if (Roles::Check("accountant_report_1")) {
            $ReportsMenu[] = [
                "id" => "cashReceipts",
                "title" => trans('backLang.cashReceipts'),
                "description" => trans('backLang.cashReceiptsDesc'),
                "icon" => "icon-folder",
                "link" => route("AppRevenuesReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_2")) {
            $ReportsMenu[] = [
                "id" => "payReceipts",
                "title" => trans('backLang.payReceipts'),
                "description" => trans('backLang.payReceiptsDesc'),
                "icon" => "icon-folder",
                "link" => route("AppExpensesReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_3")) {
            $ReportsMenu[] = [
                "id" => "revenuesAndExpenses",
                "title" => trans('backLang.revenuesAndExpenses'),
                "description" => trans('backLang.revenuesAndExpensesDesc'),
                "icon" => "icon-folder",
                "link" => route("AppProfitReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_4")) {
            $ReportsMenu[] = [
                "id" => "statementReport",
                "title" => trans('backLang.statementReport'),
                "description" => trans('backLang.statementReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppStatementReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_12")) {
            $ReportsMenu[] = [
                "id" => "entriesReport",
                "title" => trans('backLang.entriesReport'),
                "description" => trans('backLang.entriesReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppEntriesReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_5")) {
            $ReportsMenu[] = [
                "id" => "masterReport",
                "title" => trans('backLang.masterReport'),
                "description" => trans('backLang.masterReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppLedgerReport"),
                "sub" => []
            ];
        }


        if (Roles::Check("accountant_report_10")) {
            $ReportsMenu[] = [
                "id" => "fundsAndBanksReport",
                "title" => trans('backLang.fundsAndBanksReport'),
                "description" => trans('backLang.fundsAndBanksReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppBanksReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_11")) {
            $ReportsMenu[] = [
                "id" => "costPositionsReport",
                "title" => trans('backLang.costPositionsReport'),
                "description" => trans('backLang.costPositionsReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppPositionsReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_8")) {
            $ReportsMenu[] = [
                "id" => "IncomeReport",
                "title" => trans('backLang.IncomeReport'),
                "description" => trans('backLang.IncomeReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppIncomeReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_9")) {
            $ReportsMenu[] = [
                "id" => "cashFlowReport",
                "title" => trans('backLang.cashFlowReport'),
                "description" => trans('backLang.cashFlowReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppCashFlowReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_6")) {
            $ReportsMenu[] = [
                "id" => "trialBalance",
                "title" => trans('backLang.trialBalance'),
                "description" => trans('backLang.trialBalanceDesc'),
                "icon" => "icon-folder",
                "link" => route("AppTrialReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_13")) {
            $ReportsMenu[] = [
                "id" => "incomeListReport",
                "title" => trans('backLang.incomeListReport'),
                "description" => trans('backLang.incomeListReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppIncomeListReport"),
                "sub" => []
            ];
        }

        if (Roles::Check("accountant_report_7")) {
            $ReportsMenu[] = [
                "id" => "financialCenter",
                "title" => trans('backLang.financialCenter'),
                "description" => trans('backLang.financialCenterDesc'),
                "icon" => "icon-folder",
                "link" => route("AppFinancialReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_14")) {
            $ReportsMenu[] = [
                "id" => "balanceSheetReport",
                "title" => trans('backLang.balanceSheetReport'),
                "description" => trans('backLang.balanceSheetReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppBalanceSheetReport"),
                "sub" => []
            ];
        }
        if (Roles::Check("accountant_report_15")) {
            $ReportsMenu[] = [
                "id" => "installmentsReport",
                "title" => trans('backLang.installmentsReport'),
                "description" => trans('backLang.installmentsReportDesc'),
                "icon" => "icon-folder",
                "link" => route("AppInstallmentsReport"),
                "sub" => []
            ];
        }

        return $ReportsMenu;
    }

    static function DashboardSettingsMenu()
    {
        $SettingsMenu = [];

        if (Roles::Check("app_preferences_general")) {
            $SettingsMenu[] = [
                "id" => "generalSettings",
                "title" => trans('backLang.generalSettings'),
                "description" => trans('backLang.generalSettingsDesc'),
                "icon" => "icon-equalizer",
                "link" => route("appGeneralSettings"),
            ];
        }
        if (Roles::Check("role_accountant")) {
            if (Roles::Check("app_preferences_accounting")) {
                $SettingsMenu[] = [
                    "id" => "accountingSettings2",
                    "title" => trans('backLang.accountantSettings'),
                    "description" => trans('backLang.accountantSettingsDesc'),
                    "icon" => "icon-wallet",
                    "link" => route("appAccountingSettings"),
                ];
            }
        }
        if (Roles::Check("app_preferences_roles")) {
            $SettingsMenu[] = [
                "id" => "roles",
                "title" => trans('backLang.roles'),
                "description" => trans('backLang.rolesDesc'),
                "icon" => "icon-lock",
                "link" => route("AppRoles"),
            ];
        }
        if (Roles::Check("app_preferences_users")) {
            $SettingsMenu[] = [
                "id" => "users",
                "title" => trans('backLang.users'),
                "description" => trans('backLang.usersDesc'),
                "icon" => "icon-users",
                "link" => route("AppUsers"),
            ];
        }
        if (Roles::Check("app_preferences_security")) {
            $SettingsMenu[] = [
                "id" => "cybersecurity",
                "title" => trans('backLang.cybersecurity'),
                "description" => trans('backLang.cybersecurityDesc'),
                "icon" => "icon-key",
                "link" => route("appSecurity"),
            ];
        }
        if (Roles::Check("app_preferences_categories")) {
            $SettingsMenu[] = [
                "id" => "categories",
                "title" => trans('backLang.categories'),
                "description" => trans('backLang.categoriesDesc'),
                "icon" => "icon-directions",
                "link" => route("AppCategories"),
            ];
        }
        if (Roles::Check("app_text_pages")) {
            $SettingsMenu[] = [
                "id" => "pages",
                "title" => trans('backLang.textPages'),
                "description" => trans('backLang.textPagesDesc'),
                "icon" => "icon-notebook",
                "link" => route("AppPages"),
            ];
        }
        if (Roles::Check("app_preferences_ftp")) {
            $SettingsMenu[] = [
                "id" => "ftpSettings",
                "title" => trans('backLang.ftpSettings'),
                "description" => trans('backLang.ftpSettingsDesc'),
                "icon" => "icon-cloud-upload",
                "link" => route("appFtpSettings"),
            ];
        }
        if (Roles::Check("app_preferences_notifications")) {
            $SettingsMenu[] = [
                "id" => "notificationSettings",
                "title" => trans('backLang.notificationSettings'),
                "description" => trans('backLang.notificationSettingsDesc'),
                "icon" => "icon-speech",
                "link" => route("appNotificationsSettings"),
            ];
        }
        if (Roles::Check("app_preferences_templates")) {
            $SettingsMenu[] = [
                "id" => "notificationTemplates",
                "title" => trans('backLang.notificationTemplates'),
                "description" => trans('backLang.notificationTemplatesDesc'),
                "icon" => "icon-bubbles",
                "link" => route("AppTemplates"),
            ];
        }
        if (Roles::Check("app_preferences_print")) {
            $SettingsMenu[] = [
                "id" => "printSettings",
                "title" => trans('backLang.printSettings'),
                "description" => trans('backLang.printSettingsDesc'),
                "icon" => "icon-printer",
                "link" => route("appPrintSettings"),
            ];
        }
        if (Roles::Check("app_preferences_language")) {
            $SettingsMenu[] = [
                "id" => "languageSettings",
                "title" => trans('backLang.languageAndDateSettings'),
                "description" => trans('backLang.languageSettingsDesc'),
                "icon" => "icon-flag",
                "link" => route("appLanguagesSettings"),
            ];
        }
        if (Roles::Check("app_preferences_cron")) {
            $SettingsMenu[] = [
                "id" => "cronJob",
                "title" => trans('backLang.cronJobs'),
                "description" => trans('backLang.cronJobsDesc'),
                "icon" => "icon-clock",
                "link" => route("appCronSettings"),
            ];
        }
        if (Roles::Check("app_preferences_backup")) {
            $SettingsMenu[] = [
                "id" => "backup",
                "title" => trans('backLang.DBBackup'),
                "description" => trans('backLang.DBBackupDesc'),
                "icon" => "icon-layers",
                "link" => route("AppBackup"),
            ];
        }

        return $SettingsMenu;
    }
}

?>
