<?php

namespace App\Helpers;

use Auth;
use App\Models\Chat;
use App\Models\Notice;

/*
 * Here at the top you will see some comments/notes to remember
 *
 * ---------------------------
 * Message Model
 * ---------------------------
 * type : 0 = email message
 * type : 1 = sms message
 * type : 2 = browser message
 *
 *
 *
 */

class Notifications
{
    static function Cases()
    {
        return [

            // on add new user
            "1" => [
                "title" => trans('backLang.addingNewUser'),
                "template" => 3,
            ],

            // on share
            "2" => [
                "title" => trans('backLang.shareEventOrTopic'),
                "template" => 4,
            ],

            // on delete sharing
            "3" => [
                "title" => trans('backLang.shareEventDelete'),
                "template" => 5,
            ],

            // on change person status
            "4" => [
                "title" => trans('backLang.changeAccountStatus'),
                "template" => 6,
            ],

            // on change loan status
            "5" => [
                "title" => trans('backLang.changeLoanStatus'),
                "template" => 7,
            ],

            // on change fear status
            "6" => [
                "title" => trans('backLang.changeHelpStatus'),
                "template" => 8,
            ],

        ];
    }

    static function Templates()
    {
        return [

            // ...
            "1" => [
                "title" => trans('backLang.general_mail_template'),
                "system" => false,
                "mail" => true,
                "sms" => false,
                "browser" => false,
                "vars" => [
                    [
                        "title" => trans('backLang.subject'),
                        "var" => "subject",
                    ],
                    [
                        "title" => trans('backLang.message'),
                        "var" => "message",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                ]
            ],
            "2" => [
                "title" => trans('backLang.alerts_template'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.alertTitle'),
                        "var" => "title",
                    ],
                    [
                        "title" => trans('backLang.alertDetails'),
                        "var" => "details",
                    ],
                    [
                        "title" => trans('backLang.detailsLink'),
                        "var" => "url",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                ]
            ],
            "3" => [
                "title" => trans('backLang.addingNewUser'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.fullName'),
                        "var" => "name",
                    ],
                    [
                        "title" => trans('backLang.jobTitle'),
                        "var" => "job",
                    ],
                    [
                        "title" => trans('backLang.email'),
                        "var" => "email",
                    ],
                    [
                        "title" => trans('backLang.phone'),
                        "var" => "phone",
                    ],
                    [
                        "title" => trans('backLang.loginPassword'),
                        "var" => "password",
                    ],
                    [
                        "title" => trans('backLang.country'),
                        "var" => "country",
                    ],
                    [
                        "title" => trans('backLang.dateAndTime'),
                        "var" => "date",
                    ],
                    [
                        "title" => "IP",
                        "var" => "ip",
                    ],
                ]
            ],
            "4" => [
                "title" => trans('backLang.shareEventOrTopic'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.title'),
                        "var" => "title",
                    ],
                    [
                        "title" => trans('backLang.detailsLink'),
                        "var" => "url",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                ]
            ],
            "5" => [
                "title" => trans('backLang.shareEventDelete'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.title'),
                        "var" => "title",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                ]
            ],
            "6" => [
                "title" => trans('backLang.changeAccountStatus'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.accountStatus'),
                        "var" => "status",
                    ],
                    [
                        "title" => trans('backLang.reasonOfRejection'),
                        "var" => "reason",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                ]
            ],
            "7" => [
                "title" => trans('backLang.changeLoanStatus'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.loanStatus'),
                        "var" => "status",
                    ],
                    [
                        "title" => trans('backLang.reasonOfRejection'),
                        "var" => "reason",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                ]
            ],
            "8" => [
                "title" => trans('backLang.changeHelpStatus'),
                "system" => true,
                "mail" => true,
                "sms" => true,
                "browser" => true,
                "vars" => [
                    [
                        "title" => trans('backLang.loanStatus'),
                        "var" => "status",
                    ],
                    [
                        "title" => trans('backLang.reasonOfRejection'),
                        "var" => "reason",
                    ],
                    [
                        "title" => trans('backLang.mailReceiver'),
                        "var" => "to_user_name",
                    ],
                    [
                        "title" => trans('backLang.mailSender'),
                        "var" => "from_user_name",
                    ],
                ]
            ],
            // ...

        ];
    }

    static function ChatAlerts()
    {
        // General chat alerts count
        $alerts = "";
        if (@Auth::check()) {
            $alerts = Chat::where("receiver_id", @Auth::user()->id)->where("read_status", false)->count();
            if ($alerts == 0) {
                $alerts = "";
            }
        }
        return $alerts;
    }

    static function unreadCount()
    {
        $notifications_count = "";
        if (@Auth::check()) {
            $notifications_count = Notice::where("to_user_id", Auth::user()->id)->where("status", false)->count();
            if ($notifications_count == 0) {
                $notifications_count = "";
            }
        }
        return $notifications_count;
    }
}

?>
