<?php


namespace App\Http\Controllers\Accounting;

use App;
use App\Exports\LedgersExport;
use App\Http\Controllers\Controller;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\Period;
use App\Models\Preference;
use App\Models\Authorize;
use App\Models\Currency;
use App\Models\Bank;
use App\Models\Receipt;
use App\Models\User;
use Auth;
use Cookie;
use GeniusTS\HijriDate\Translations\Arabic;
use GeniusTS\HijriDate\Translations\English;
use Helper;
use Cats;
use Notifications;
use Roles;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use App\Helpers\History;
use App\Exports\LedgerReportExport;

class AccountingHomeController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        return view('back.accounting.home.index');
    }

    public function bank()
    {
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $bank_ledger_ids = [];
        foreach ($Funds as $fund) {
            if ($fund->ledger_id > 0) {
                $bank_ledger_ids[] = $fund->ledger_id;
            }
        }
        $Balances = [];
        $Ledgers = Ledger::whereIn("id", $bank_ledger_ids)->orderBy("code", "asc")->get();
        if (count($Ledgers) > 0) {
            $search_period = Helper::Preferences("default_period");
            foreach ($Ledgers as $Ledger) {
                $sum_of_dept = Journal::where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('ledger_id', $Ledger->id)->where("period_id", $search_period);

                $sum_of_dept = round($sum_of_dept->sum("cost_dept"), Helper::Preferences("decimal_round"));
                $sum_of_credit = round($sum_of_credit->sum("cost_credit"), Helper::Preferences("decimal_round"));

                $Balances[$Ledger->id] = [
                    "dept" => $sum_of_dept,
                    "credit" => $sum_of_credit,
                    "balance" => round($sum_of_dept - $sum_of_credit, Helper::Preferences("decimal_round"))
                ];
            }
        }

        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view('back.accounting.home.bank', compact("Funds", "Balances", "DefaultCurrency"));
    }

    public function summary()
    {
        $Balances = [];
        $Ledgers = Ledger::where("father_id", 0)->orderBy("code", "asc")->get();
        if (count($Ledgers) > 0) {
            $search_period = Helper::Preferences("default_period");
            $Ledgers = $Ledgers->map(function ($Ledger) use ($search_period) {
                $sub_ledgers_ids = Ledger::select("id")->where("id", $Ledger->id)->orwhere("group_id", $Ledger->group_id)->get();
                $sum_of_dept = Journal::wherein('ledger_id', $sub_ledgers_ids)->where("period_id", $search_period);
                $sum_of_credit = Journal::wherein('ledger_id', $sub_ledgers_ids)->where("period_id", $search_period);

                $sum_of_dept = round($sum_of_dept->sum("cost_dept"), Helper::Preferences("decimal_round"));
                $sum_of_credit = round($sum_of_credit->sum("cost_credit"), Helper::Preferences("decimal_round"));

                $Ledger->dept = $sum_of_dept;
                $Ledger->credit = $sum_of_credit;
                $Ledger->balance = round($sum_of_dept - $sum_of_credit, Helper::Preferences("decimal_round"));
                return $Ledger;
            });
        }

        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view('back.accounting.home.summary', compact("Ledgers", "DefaultCurrency"));
    }

    public function cash($stat = "day")
    {

        $search_period = Helper::Preferences("default_period");

        $CashData = [];
        if ($stat == "year") {
            for ($i = 1; $i <= 12; $i++) {

                if (Helper::Preferences("date_type")) {
                    if (trans('backLang.boxCode') == "ar") {
                        \GeniusTS\HijriDate\Date::setTranslation(new Arabic);
                    } else {
                        \GeniusTS\HijriDate\Date::setTranslation(new English);
                    }

                    $s = date('Y-m-d');

                    if (trans('backLang.boxCode') == "ar") {
                        \GeniusTS\HijriDate\Date::setTranslation(new Arabic);
                    } else {
                        \GeniusTS\HijriDate\Date::setTranslation(new English);
                    }
                    $s = \GeniusTS\HijriDate\Hijri::convertToHijri($s);
                    $Year = $s->format("Y");
                    $start = $Year . "-" . str_pad($i, 2, '0', STR_PAD_LEFT) . "-01";
                    $end_dd = date("t", strtotime($start));
                    $end = $Year . "-" . str_pad($i, 2, '0', STR_PAD_LEFT) . "-" . $end_dd;

                    $st = \GeniusTS\HijriDate\Hijri::convertToGregorian(1, $i, $Year);
                    $st = $st->format("Y-m-d");
                    $date = \GeniusTS\HijriDate\Hijri::convertToHijri($st);
                    $MM = $date->format("F") . " " . $Year;

                } else {
                    $start = date('Y') . "-" . str_pad($i, 2, '0', STR_PAD_LEFT) . "-01";
                    $end_dd = date("t", strtotime($start));
                    $end = date('Y') . "-" . str_pad($i, 2, '0', STR_PAD_LEFT) . "-" . $end_dd;
                    $MM = trans("backLang.month_names.$i") . " " . date('Y');
                }

                $revenues = Receipt::where('type', 1)->where("date", ">=", $start)->where("date", "<=", $end)->sum("cost");
                $expenses = Receipt::where('type', 0)->where("date", ">=", $start)->where("date", "<=", $end)->sum("cost");
                $credit = Receipt::where('type', 1)->where("date", ">=", $start)->where("date", "<=", $end)->count();
                $dept = Receipt::where('type', 0)->where("date", ">=", $start)->where("date", "<=", $end)->count();

                $CashData[] = [
                    "date" => $MM,
                    "revenues" => $revenues,
                    "expenses" => $expenses,
                    "dept" => $dept,
                    "credit" => $credit,
                ];
            }
        } elseif ($stat == "month") {
            for ($i = 1; $i <= 30; $i++) {
                $days = 31 - $i;
                if (Helper::Preferences("date_type")) {

                    $s = date('Y-m-d', strtotime("-$days day"));
                    $e = date('Y-m-d', strtotime("-$days day"));

                    if (trans('backLang.boxCode') == "ar") {
                        \GeniusTS\HijriDate\Date::setTranslation(new Arabic);
                    } else {
                        \GeniusTS\HijriDate\Date::setTranslation(new English);
                    }
                    $start = \GeniusTS\HijriDate\Hijri::convertToHijri($s);
                    $end = \GeniusTS\HijriDate\Hijri::convertToHijri($e);

                    $date_d = $start;

                } else {
                    $start = date('Y-m-d', strtotime("-$days day"));
                    $end = date('Y-m-d', strtotime("-$days day"));

                    $date_d = date('d', strtotime($start)) . " " . date('M', strtotime($start));
                }

                $revenues = Receipt::where('type', 1)->where("date", ">=", $start)->where("date", "<=", $end)->sum("cost");
                $expenses = Receipt::where('type', 0)->where("date", ">=", $start)->where("date", "<=", $end)->sum("cost");
                $credit = Receipt::where('type', 1)->where("date", ">=", $start)->where("date", "<=", $end)->count();
                $dept = Receipt::where('type', 0)->where("date", ">=", $start)->where("date", "<=", $end)->count();

                $CashData[] = [
                    "date" => $date_d,
                    "revenues" => $revenues,
                    "expenses" => $expenses,
                    "dept" => $dept,
                    "credit" => $credit,
                ];
            }
        } else {
            for ($i = 0; $i < 24; $i = $i + 2) {
                $am_pm = "AM";
                $time = $i;
                if ($i > 12) {
                    $time = $i - 12;
                    $am_pm = "PM";
                }
                $start = date('Y-m-d') . " " . str_pad($i, 2, '0', STR_PAD_LEFT) . ":00:00";
                $end = date('Y-m-d') . " " . str_pad(($i + 1), 2, '0', STR_PAD_LEFT) . ":59:59";

                $revenues = Receipt::where('type', 1)->where("created_at", ">=", $start)->where("created_at", "<=", $end)->sum("cost");
                $expenses = Receipt::where('type', 0)->where("created_at", ">=", $start)->where("created_at", "<=", $end)->sum("cost");
                $credit = Receipt::where('type', 1)->where("created_at", ">=", $start)->where("created_at", "<=", $end)->count();
                $dept = Receipt::where('type', 0)->where("created_at", ">=", $start)->where("created_at", "<=", $end)->count();

                $CashData[] = [
                    "date" => str_pad($time, 2, '0', STR_PAD_LEFT) . ":00 " . $am_pm,
                    "revenues" => $revenues,
                    "expenses" => $expenses,
                    "dept" => $dept,
                    "credit" => $credit,
                ];
            }
        }
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view('back.accounting.home.cash', compact("DefaultCurrency", "stat", "CashData"));
    }

    public function currencies_exchange(Request $request)
    {
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $FromCurrency = Currency::find($request->currency);
        $amount = $request->amount;
        $Amounts = [];
        if ($amount > 0 && !empty($FromCurrency)) {
            foreach (Helper::CurrenciesList() as $_Currency) {
                $new_amount = 0;
                if ($_Currency->rate != 0) {
                    if ($FromCurrency->id == $_Currency->id) {
                        $new_rate = 1;
                        $new_amount = $amount;
                    } else {
                        $new_rate = round($_Currency->rate / $FromCurrency->rate, 3);
                        $new_amount = round($amount * $new_rate, 2);
                    }
                }
                $Amounts[$_Currency->id] = [
                    "rate" => number_format($new_rate, 3),
                    "amount" => number_format($new_amount, 2)
                ];
            }
        }
        return view('back.accounting.home.exchange', compact("Amounts"));
    }
}
