<?php

namespace App\Http\Controllers\Accounting;


use App;
use App\Http\Controllers\Controller;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\Period;
use App\Models\Preference;
use App\Models\Authorize;
use App\Models\Currency;
use App\Models\Bank;
use Auth;
use Cookie;
use Helper;
use Cats;
use Notifications;
use Roles;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Validator;
use App\Helpers\History;

class AccountingPreferencesController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        //Role Check
        Roles::Check("app_preferences_accounting", true);

        // confirm password
        Helper::confirmPassword("appAccountingSettings");


        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->where("type", true)->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        $Preferences = Preference::find(1);
        $accounting_settings = [];
        if ($Preferences->accounting_settings != "") {
            try {
                $accounting_settings = json_decode($Preferences->accounting_settings);
            } catch (\Exception $e) {
                $accounting_settings = [];
            }
        }
        foreach ($accounting_settings as $key => $accounting_setting) {
            $Preferences->$key = $accounting_setting;
        }
        $DefaultPeriod = Period::find($Preferences->default_period);
        return view("back.accounting.preferences.general", compact("Preferences", "Currencies", "Funds", "ledgers_array", "Periods", "DefaultPeriod"));
    }

    public function load($whid = "")
    {
        //Role Check
        Roles::Check("app_preferences_accounting", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->where("type", true)->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        $Preferences = Preference::find(1);
        $accounting_settings = [];
        if ($Preferences->accounting_settings != "") {
            try {
                $accounting_settings = json_decode($Preferences->accounting_settings);
            } catch (\Exception $e) {
                $accounting_settings = [];
            }
        }
        foreach ($accounting_settings as $key => $accounting_setting) {
            $Preferences->$key = $accounting_setting;
        }
        $DefaultPeriod = Period::find($Preferences->default_period);
        return view("back.accounting.preferences.settings", compact("Preferences", "Currencies", "Funds", "ledgers_array", "Periods", "DefaultPeriod", "whid"));
    }

    public function update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_accounting", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $old_settings = [];
            if ($Preference->accounting_settings != "") {
                try {
                    $old_settings = json_decode($Preference->accounting_settings);
                } catch (\Exception $e) {
                    $old_settings = [];
                }
            }


            $accounting_settings = [];
            foreach ($request->all() as $index => $accounting_setting) {
                if ($index != "_token") {
                    $accounting_settings[$index] = $request->$index;
                }
            }
            try {
                $loaded_pos = $request->loaded_pos;
                $loaded_pos_array = [];
                if ($loaded_pos != "") {
                    $loaded_pos_array = explode(",", $loaded_pos);
                }
                $warehouses = \DB::select('SELECT * FROM sma_warehouses');
                if (count(@$warehouses) > 0) {
                    foreach ($warehouses as $warehouse) {
                        if (!in_array($warehouse->id, $loaded_pos_array)) {
                            if (\Lang::has('backLang.ledgerSettingsByBranch', 'ar')) {
                                foreach (trans('backLang.ledgerSettingsByBranch') as $key => $ledgerSettings) {
                                    $accounting_settings[$key . "_" . @$warehouse->id] = $old_settings->{$key . "_" . @$warehouse->id};
                                }
                            }
                        }

                    }

                }
            } catch (\Exception $e) {

            }
            $accounting_settings = json_encode($accounting_settings);
            $Preference->accounting_settings = $accounting_settings;
            $Preference->save();

            // Update .env file
            $env_update = Helper::changeEnv([
                'CURRENCY_CONVERTER_API_KEY' => $request->CURRENCY_CONVERTER_API_KEY,
            ]);

            // Log to history
            History::Log(
                $module = 6,
                $cat = 600,
                $window = 6000,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Accounting\AccountingPreferencesController@index')
                ->with('doneMessage', trans('backLang.saveDone'));
        }
    }

    public function close(Request $request)
    {
        //Role Check
        Roles::Check("accountant_period_close", true);

        $Preference = Preference::find(1);
        $OldPeriod = Period::find(Helper::Preferences("default_period"));
        if (!empty($Preference) && !empty($OldPeriod)) {
            if ($request->new_period == "-1") {
                $validator = Validator::make($request->all(), [
                    'new_period' => 'required',
                    'code' => 'required',
                    'start' => 'required',
                    'end' => 'required'
                ]);
            } else {
                $validator = Validator::make($request->all(), [
                    'new_period' => 'required',
                ]);
            }
            if ($validator->passes()) {
                try {
                    $un_posted_Journals = Journal::where('posted', false)->count();
                    if ($un_posted_Journals > 0) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.postAllToClosePeriod')]]);
                    }

                    if ($request->new_period == "-1") {
                        $NewPeriod = new Period;
                        $NewPeriod->code = $request->code;
                        $NewPeriod->start = date('Y-m-d', strtotime($request->start));
                        $NewPeriod->end = date('Y-m-d', strtotime($request->end));
                        $NewPeriod->date_type = Helper::Preferences("date_type");
                        $NewPeriod->status = true;
                        $NewPeriod->created_ip = \Request::ip();
                        $NewPeriod->created_by = Auth::user()->id;
                        $NewPeriod->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6005,
                            $record_id = $NewPeriod->id,
                            $operation = 2,
                            $title = $NewPeriod->title_ar . "<br>" . $NewPeriod->title_en
                        );
                    } else {
                        $NewPeriod = Period::find($request->new_period);
                    }
                    if (!empty($NewPeriod)) {

                        //close old period
                        $OldPeriod->status = false;
                        $OldPeriod->save();

                        // change default period
                        $acc_settings = json_decode($Preference->accounting_settings);
                        $accounting_settings = [];
                        foreach ($acc_settings as $index => $accounting_setting) {
                            if ($index == "default_period") {
                                $accounting_settings[$index] = $NewPeriod->id;
                            } else {
                                $accounting_settings[$index] = $accounting_setting;
                            }
                        }
                        $accounting_settings = json_encode($accounting_settings);
                        $Preference->accounting_settings = $accounting_settings;
                        $Preference->save();

                        //calc start balances
                        $AllLedgers = Ledger::get();
                        $default_currency = Helper::Preferences("default_currency");
                        if (count($AllLedgers) > 0) {
                            foreach ($AllLedgers as $Ledger) {
                                $sum_of_dept = Journal::where('ledger_id', $Ledger->id)->where("period_id", $OldPeriod->id)->sum("cost_dept");
                                $sum_of_credit = Journal::where('ledger_id', $Ledger->id)->where("period_id", $OldPeriod->id)->sum("cost_credit");
                                $balance = round($sum_of_dept - $sum_of_credit, 2);
                                if ($balance != 0) {
                                    $Journal = new Journal;
                                    $Journal->type = 1;
                                    $Journal->period_id = $NewPeriod->id;
                                    $Journal->receipt_id = 0;
                                    $Journal->ledger_id = $Ledger->id;
                                    $Journal->center_id = 0;
                                    $Journal->date = $NewPeriod->start;
                                    $Journal->date_type = $NewPeriod->date_type;
                                    if ($balance > 0) {
                                        $Journal->dept = $balance;
                                        $Journal->credit = 0;
                                        $Journal->currency_id = $default_currency;
                                        $Journal->cost_dept = $balance;
                                        $Journal->cost_credit = 0;
                                    } else {
                                        $balance = ($balance * -1);
                                        $Journal->dept = 0;
                                        $Journal->credit = $balance;
                                        $Journal->currency_id = $default_currency;
                                        $Journal->cost_dept = 0;
                                        $Journal->cost_credit = $balance;
                                    }
                                    $Journal->details = trans("backLang.startBalance");
                                    $Journal->posted = true;
                                    $Journal->created_by = Auth::user()->id;
                                    $Journal->save();
                                }
                            }
                        }

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6000,
                            $record_id = $Preference->id,
                            $operation = 3,
                            $title = trans("backLang.accountantPeriodClose") . " : " . @$OldPeriod->code
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Preference->id)]);
                    }
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }
}
