<?php

namespace App\Http\Controllers\Accounting;

use App;
use App\Exports\LedgersExport;
use App\Http\Controllers\Controller;
use App\Models\Installment;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\Period;
use App\Models\Preference;
use App\Models\Authorize;
use App\Models\Loan;
use App\Models\Currency;
use App\Models\Bank;
use App\Models\Receipt;
use App\Models\User;
use Auth;
use Cookie;
use Helper;
use Cats;
use Notifications;
use Roles;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use App\Helpers\History;
use App\Exports\LedgerReportExport;
use App\Exports\MasterReportExport;
use App\Exports\PositionsReportExport;
use App\Exports\InstallmentsExport;

class AccountingReportsController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function revenues()
    {
        //Role Check
        Roles::Check("accountant_report_1", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $stat = "revenues";

        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        return view("back.accounting.reports.receipts.list", compact("Currencies", "Funds", "UsersList", "stat", "Periods","ledgers_array"));
    }

    public function expenses()
    {
        //Role Check
        Roles::Check("accountant_report_2", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $stat = "expenses";

        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        return view("back.accounting.reports.receipts.list", compact("Currencies", "Funds", "UsersList", "stat", "Periods","ledgers_array"));
    }

    public function receipts_list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'date',
            2 => 'amount',
            3 => 'details',
            4 => 'ledger_id',
            5 => 'created_by',
            6 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_type = $request->input('search_type');
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_cat = $request->input('search_cat');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_ledger = $request->input('search_ledger');

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", '>', 0);
        }
        if ($search_type) {
            $Receipts = $Receipts->where("type", true);
        } else {
            $Receipts = $Receipts->where("type", false);
        }

        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($search_fund > 0) {
            $Receipts = $Receipts->where("bank_id", $search_fund);
        }

        if ($search_ledger > 0) {
            $Receipts = $Receipts->where("ledger_id", $search_ledger);
        }
        if ($search_period > 0) {
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $search_pay_method);
        }
        if ($search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_cat > 0) {
            $Receipts = $Receipts->where("cat_id", $search_cat);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $totalData = $Receipts->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Receipts = $Receipts->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Receipts as $Receipt) {
                $nestedData['id'] = "<div class='text-center'>" . $Receipt->id . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['amount'] = "<div class='text-center'><strong>" . number_format($Receipt->amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Receipt->currency->symbol . "</strong></div>";
                $nestedData['details'] = $Receipt->details;
                $nestedData['ledger_id'] = "<span class=\"badge badge-light font-13\">" . @$Receipt->ledger->code . "</span> " . @$Receipt->ledger->{"title_" . trans("backLang.code")};
                $nestedData['created_by'] = @$Receipt->user->name;
                $options = "<div class=\"text-center\">";

                if (Roles::Check("accountant_print")) {
                    $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppReceiptPrint", encrypt($Receipt->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function statement()
    {
        //Role Check
        Roles::Check("accountant_report_4", true);

        $Currencies = Currency::where("status", true)->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.reports.statement.list", compact("Currencies", "UsersList", "Periods", "ledgers_array", "DefaultCurrency"));
    }

    public function statement_list(Request $request)
    {
        $limit = $request->input('length');
        $start = $request->input('start');
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_ledger = $request->input('search_ledger');


        // prepare ledgers ids
        function getSub($items, &$Ledgers_Ids)
        {
            while ($item = array_shift($items)) {
                $Ledgers_Ids[] = $item['id'];
                $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                $SubLedgers = $SubLedgers->toArray();
                getSub($SubLedgers, $Ledgers_Ids);
            }
        }

        $Ledgers_Ids = [];
        if ($search_ledger > 0) {
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
        }
        // End prepare ledgers ids

        if (count($Ledgers_Ids) > 0) {
            $Journals = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids);
        } else {
            $Journals = Journal::where('posted', true);
        }
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }


        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy("date", "asc")
            ->orderBy("id", "asc")
            ->get();

        $data = array();
        $balance = 0;
        $total_of_dept = 0;
        $total_of_credit = 0;
        if ($totalFiltered > 0) {
            $i = 0;
            foreach ($Journals as $Journal) {
                if ($i == 0) {
                    $sum_of_dept = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids)->where("period_id", $search_period)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_dept");
                    $sum_of_credit = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids)->where("period_id", $search_period)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_credit");
                    $balance = $sum_of_dept - $sum_of_credit;
                    $total_of_dept += $sum_of_dept;
                    $total_of_credit += $sum_of_credit;
                } else {
                    if ($Journal->cost_dept > 0) {
                        $balance = $balance + $Journal->cost_dept;
                        $total_of_dept += $Journal->cost_dept;
                    }
                    if ($Journal->cost_credit > 0) {
                        $balance =$balance - $Journal->cost_credit;
                        $total_of_credit += $Journal->cost_credit;
                    }
                }
                $cur_details = "";
                if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                    if ($Journal->dept > 0) {
                        $cur_details = " ( " . number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    } else {
                        $cur_details = " ( " . number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    }
                }

                $type = trans("backLang.dailyEntry");
                if ($Journal->type == 2) {
                    $type = trans("backLang.reconciliationEntry");
                } elseif ($Journal->type == 1) {
                    $type = trans("backLang.startBalance");
                } else {
                    if (@$Journal->receipt->type == 1) {
                        $type = trans("backLang.cashReceipt");
                    } elseif (@$Journal->receipt->type == 0) {
                        $type = trans("backLang.payReceipt");
                    }
                }
                $nestedData['type'] = "<div class='text-center'>" . $type . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";
                $nestedData['dept'] = "<div class='text-center'>" . (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";

                $nestedData['balance'] = "<div class='text-center " . (($balance < 0) ? "text-danger" : "text-success") . "'><strong>" . number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . "</strong></div>";
                $nestedData['details'] = $Journal->details . $cur_details;
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['id'] = "<div class='text-center'>" . $Journal->id . "</div>";
                $nestedData['receipt'] = "<div class='text-center'>" . @$Journal->receipt->id . "</div>";
                $file_url = "";
                if (@$Journal->receipt->file != "") {
                    $file_url = route("AppReceiptsFilePreview", encrypt(@$Journal->receipt->id));
                    $nestedData['code'] = "<div class='text-center'><a href='" . $file_url . "' target='_blank'>" . @$Journal->receipt->code . " <i class='fa fa-paperclip'></i></a></div>";
                } else {
                    $nestedData['code'] = "<div class='text-center'>" . @$Journal->receipt->code . "</div>";
                }

                $receipt_link = "";
                if (!empty(@$Journal->receipt)) {
                    $receipt_link = " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt(@$Journal->receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                $nestedData['options'] = "<div class='text-center'>" . $receipt_link . "</div>";
                $data[] = $nestedData;
                $i++;
            }
        }

        // sum of total start
        if (count($Ledgers_Ids) > 0) {
            $Journals = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids);
        } else {
            $Journals = Journal::where('posted', true);
        }
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }
        $total_of_dept = $Journals->sum("dept");
        $total_of_credit = $Journals->sum("credit");
        $balance = $total_of_dept- $total_of_credit;

        $TBalance = [
            "dept" => number_format($total_of_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "credit" => number_format($total_of_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "total" => number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol"))
        ];

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);

    }

    public function statement_print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_currency = $request->input('search_currency');
            $search_ledger = $request->input('search_ledger');

            $Currency = [];
            $Period = [];

            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
            // End prepare ledgers ids

            $Journals = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids);
            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Journals = $Journals->where("currency_id", $search_currency);
            }
            if ($search_center_id > 0) {
                $Journals = $Journals->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }
            $Journals = $Journals->orderBy("date", "asc")->orderBy("id", "asc")->get();

            $Ledger = Ledger::find($search_ledger);
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

            $Journal = $Journals->first();
            $sum_of_dept = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_dept");
            $sum_of_credit = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_credit");

            return view("back.accounting.reports.statement.print", compact("Journals", "request", "Currency", "Period", "Ledger", "Ledgers_Ids", "sum_of_dept", "sum_of_credit", "DefaultCurrency"));
        }
        return "";
    }

    public function statement_export(Request $request)
    {
        if (Roles::Check("accountant_export")) {

            return (new LedgerReportExport($request))->download(trans('backLang.statementReport') . '.xlsx');
        }
        return "";
    }

    public function profit()
    {
        //Role Check
        Roles::Check("accountant_report_3", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.profit.list", compact("Currencies", "Funds", "UsersList", "Periods"));
    }

    public function profit_list(Request $request)
    {
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $stat = $request->input('stat');
        $Currency = [];
        $Fund = [];
        $Receipts = Receipt::where("id", '>', 0);
        if ($search_fund > 0) {
            $Fund = Bank::find($search_fund);
            $Receipts = $Receipts->where("bank_id", $search_fund);
        }
        if ($search_period > 0) {
            $Period = Period::find($search_period);
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $search_pay_method);
        }
        if ($search_currency > 0) {
            $Currency = Currency::find($search_currency);
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }
        $Receipts = $Receipts->get();
        $Revenues = [];
        $RevenuesTotal = $Receipts->where("type", 1)->sum("cost");
        foreach (Helper::Categories(8) as $CatItem) {
            $cost = $Receipts->where("type", 1)->where("cat_id", $CatItem->id)->sum("cost");
            $Revenues[$CatItem->id] = $cost;
        }

        $Expenses = [];
        $ExpensesTotal = $Receipts->where("type", 0)->sum("cost");
        foreach (Helper::Categories(9) as $CatItem) {
            $cost = $Receipts->where("type", 0)->where("cat_id", $CatItem->id)->sum("cost");
            $Expenses[$CatItem->id] = $cost;
        }
        $NetProfit = $RevenuesTotal - $ExpensesTotal;

        $view = "totals";
        if ($stat == "print") {
            $view = "print";
        }
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view("back.accounting.reports.profit." . $view, compact("Revenues", "Expenses", "RevenuesTotal", "ExpensesTotal", "NetProfit", "DefaultCurrency", "request", "Period", "Currency", "Fund"));

    }

    public function trial()
    {
        //Role Check
        Roles::Check("accountant_report_6", true);

        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        return view("back.accounting.reports.trial.list", compact("DefaultCurrency", "Periods", "ledgers_array"));
    }

    public function trial_list(Request $request)
    {
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_period = $request->input('search_period');
        $search_ledger = $request->input('search_ledger');
        $stat = $request->input('stat');

        $Ledgers = Ledger::where('id', ">", 0);
        if ($search_ledger > 0) {
            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
            // End prepare ledgers ids
            $Ledgers = $Ledgers->whereIn("id", $Ledgers_Ids);
        }
        $Ledgers = $Ledgers->orderBy("code", "asc")->get();

        $Period = [];
        if ($search_period > 0) {
            $Period = Period::find($search_period);
        }

        if (count($Ledgers) > 0) {
            $Ledgers = $Ledgers->map(function ($Ledger) use ($search_period, $search_start, $search_end) {

                //Start Balance
                if ($search_start != "") {
                    $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                    $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                    $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                    $sum_of_dept = $sum_of_dept->where('date', "<", $search_start_date);
                    $sum_of_credit = $sum_of_credit->where('date', "<", $search_start_date);
                } else {
                    $sum_of_dept = Journal::where('type', 1)->where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                    $sum_of_credit = Journal::where('type', 1)->where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                }
                $Ledger->start_dept = $sum_of_dept->sum("cost_dept");
                $Ledger->start_credit = $sum_of_credit->sum("cost_credit");

                //Total Balance
                $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);

                if ($search_start != "") {
                    $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                    $sum_of_dept = $sum_of_dept->where('date', ">=", $search_start_date);
                    $sum_of_credit = $sum_of_credit->where('date', ">=", $search_start_date);
                } else {
                    $sum_of_dept = $sum_of_dept->where('type', '!=', 1);
                    $sum_of_credit = $sum_of_credit->where('type', '!=', 1);
                }
                if ($search_end != "") {
                    $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                    $sum_of_dept = $sum_of_dept->where('date', "<=", $search_end_date);
                    $sum_of_credit = $sum_of_credit->where('date', "<=", $search_end_date);
                }
                $Ledger->dept = $sum_of_dept->sum("cost_dept");
                $Ledger->credit = $sum_of_credit->sum("cost_credit");


                return $Ledger;
            });
        }

        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

        $view = "ledgers";
        if ($stat == "print") {
            $view = "print";
        }
        return view("back.accounting.reports.trial." . $view, compact("Ledgers", "DefaultCurrency", "request", "Period"));

    }

    public function financial()
    {
        //Role Check
        Roles::Check("accountant_report_7", true);

        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.financial.list", compact("Periods"));
    }

    public function financial_list(Request $request)
    {
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_period = $request->input('search_period');
        $search_center_id = $request->input('search_center_id');
        $report_style = $request->input('report_style');

        $AssetLedger = Ledger::where('code', 1)->first();
        $ObligationLedger = Ledger::where('code', 2)->first();
        $PropertyLedger = Ledger::where('code', 3)->first();

        $Balances = [];
        $Ledgers = Ledger::orderBy("code", "asc")->get();
        if (count($Ledgers) > 0) {
            foreach ($Ledgers as $Ledger) {
                $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);

                if ($search_center_id > 0) {
                    $sum_of_dept = $sum_of_dept->where("center_id", $search_center_id);
                    $sum_of_credit = $sum_of_credit->where("center_id", $search_center_id);
                }
                if ($search_start != "") {
                    $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                    $sum_of_dept = $sum_of_dept->where('date', ">=", $search_start_date);
                    $sum_of_credit = $sum_of_credit->where('date', ">=", $search_start_date);
                }
                if ($search_end != "") {
                    $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                    $sum_of_dept = $sum_of_dept->where('date', "<=", $search_end_date);
                    $sum_of_credit = $sum_of_credit->where('date', "<=", $search_end_date);
                }
                $sum_of_dept = $sum_of_dept->get();
                $sum_of_credit = $sum_of_credit->get();

                $start_of_dept = $sum_of_dept->where("type", 1)->sum("cost_dept");
                $start_of_credit = $sum_of_credit->where("type", 1)->sum("cost_credit");

                $period_of_dept = $sum_of_dept->where("type", "!=", 1)->sum("cost_dept");
                $period_of_credit = $sum_of_credit->where("type", "!=", 1)->sum("cost_credit");

                $sum_of_dept = $sum_of_dept->sum("cost_dept");
                $sum_of_credit = $sum_of_credit->sum("cost_credit");

                $Balances[$Ledger->id] = [
                    "start" => $start_of_dept - $start_of_credit,
                    "period" =>$period_of_dept - $period_of_credit,
                    "balance" => $sum_of_dept - $sum_of_credit
                ];
            }
        }
        if ($request->stat == "print") {
            $view = "print";
            if ($report_style) {
                $view = "print_report";
            }
            return view("back.accounting.reports.financial.$view", compact("search_end", "AssetLedger", "ObligationLedger", "PropertyLedger", "Balances", "request"));
        } else {
            $view = "totals";
            if ($report_style) {
                $view = "report";
            }
            return view("back.accounting.reports.financial.$view", compact("search_end", "AssetLedger", "ObligationLedger", "PropertyLedger", "Balances"));
        }
    }

    public function income()
    {
        //Role Check
        Roles::Check("accountant_report_8", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.income.list", compact("Currencies", "Funds", "UsersList", "Periods"));
    }

    public function income_list(Request $request)
    {
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_user = $request->input('search_user');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $stat = $request->input('stat');
        $Currency = [];
        $Period = [];
        $Fund = [];
        $Receipts = Receipt::where("id", '>', 0);
        if ($search_fund > 0) {
            $Fund = Bank::find($search_fund);
            $Receipts = $Receipts->where("bank_id", $search_fund);
        }
        if ($search_period > 0) {
            $Period = Period::find($search_period);
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $search_pay_method);
        }
        if ($search_currency > 0) {
            $Currency = Currency::find($search_currency);
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        $Receipts = $Receipts->get();

        $Period = Period::find($search_period);
        if (!empty($Period)) {
            $s_date = $Period->start;
            $e_date = $Period->end;

            $s_m = date('m', strtotime($s_date));
            $s_y = date('Y', strtotime($e_date));
            $Months = [];

            $start_date = date("Y-m-d", strtotime($s_y . "-" . $s_m . "-01"));
            $end_date = date("Y-m-d", strtotime($s_y . "-" . $s_m . "-31"));

            $i = 0;
            while ($start_date <= $e_date) {

                $revenues = $Receipts->where('type', 1)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");
                $expenses = $Receipts->where('type', 0)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");

                $s_m = date('m', strtotime($start_date));
                $s_y = date('Y', strtotime($start_date));

                $Months[round($s_m)] = [
                    "title" => ($Period->date_type) ? trans("backLang.month_hijri." . round($s_m)) : trans("backLang.month_names." . round($s_m)) . " " . $s_y,
                    "revenues" => $revenues,
                    "expenses" => $expenses,
                ];

                if ($i != 0) {
                    $sub_date = date("Y-m", strtotime("+1 month", strtotime($start_date)));
                    $start_date = $sub_date . "-01";
                    $end_date = $sub_date . "-31";
                }
                $i++;
            }

            $view = "totals";
            if ($stat == "print") {
                $view = "print";
            }
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            return view("back.accounting.reports.income." . $view, compact("Months", "DefaultCurrency", "request", "Period", "Currency", "Fund"));
        }
    }

    public function cash()
    {
        //Role Check
        Roles::Check("accountant_report_9", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.cash.list", compact("Currencies", "Funds", "UsersList", "Periods"));
    }

    public function cash_list(Request $request)
    {
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_user = $request->input('search_user');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $stat = $request->input('stat');
        $Fund = [];
        $Period = [];
        $Currency = [];

        $Receipts = Receipt::where("id", '>', 0);
        if ($search_fund > 0) {
            $Fund = Bank::find($search_fund);
            $Receipts = $Receipts->where("bank_id", $search_fund);
        }
        if ($search_period > 0) {
            $Period = Period::find($search_period);
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $search_pay_method);
        }
        if ($search_currency > 0) {
            $Currency = Currency::find($search_currency);
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        $Receipts = $Receipts->get();

        $Period = Period::find($search_period);
        if (!empty($Period)) {
            $s_date = $Period->start;
            $e_date = $Period->end;

            $s_m = date('m', strtotime($s_date));
            $s_y = date('Y', strtotime($e_date));
            $Months = [];

            $start_date = date("Y-m-d", strtotime($s_y . "-" . $s_m . "-01"));
            $end_date = date("Y-m-d", strtotime($s_y . "-" . $s_m . "-31"));

            $i = 0;
            while ($start_date <= $e_date) {

                $revenues = [];
                $revenues_total = 0;
                foreach (Helper::Categories(8) as $CatItem) {
                    $cost = $Receipts->where('type', 1)->where("cat_id", $CatItem->id)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");
                    $revenues[$CatItem->id] = $cost;
                    $revenues_total += $cost;
                }
                $revenues_other = $Receipts->where('type', 1)->where("cat_id", 0)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");
                $revenues_total += $revenues_other;

                $expenses = [];
                $expenses_total = 0;
                foreach (Helper::Categories(9) as $CatItem) {
                    $cost = $Receipts->where('type', 0)->where("cat_id", $CatItem->id)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");
                    $expenses[$CatItem->id] = $cost;
                    $expenses_total += $cost;
                }
                $expenses_other = $Receipts->where('type', 0)->where("cat_id", 0)->where("date", ">=", $start_date)->where("date", "<=", $end_date)->sum("cost");
                $expenses_total += $expenses_other;

                $s_m = date('m', strtotime($start_date));
                $s_y = date('Y', strtotime($start_date));

                $total = $revenues_total - $expenses_total;
                $Months[round($s_m)] = [
                    "title" => ($Period->date_type) ? trans("backLang.month_hijri." . round($s_m)) : trans("backLang.month_names." . round($s_m)),
                    "revenues" => $revenues,
                    "revenues_other" => $revenues_other,
                    "revenues_total" => $revenues_total,
                    "expenses" => $expenses,
                    "expenses_other" => $expenses_other,
                    "expenses_total" => $expenses_total,
                    "total" => $total,
                ];

                if ($i != 0) {
                    $sub_date = date("Y-m", strtotime("+1 month", strtotime($start_date)));
                    $start_date = $sub_date . "-01";
                    $end_date = $sub_date . "-31";
                }
                $i++;
            }

            $view = "totals";
            if ($stat == "print") {
                $view = "print";
            }
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            return view("back.accounting.reports.cash." . $view, compact("Months", "DefaultCurrency", "request", "Period", "Currency", "Fund"));
        }

    }

    public function ledger()
    {
        //Role Check
        Roles::Check("accountant_report_4", true);

        $Currencies = Currency::where("status", true)->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.reports.ledger.list", compact("Currencies", "UsersList", "Periods", "ledgers_array", "DefaultCurrency"));
    }

    public function ledger_list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'type',
            2 => 'date',
            3 => 'credit',
            4 => 'dept',
            5 => 'ledger_id',
            6 => 'details',
            7 => 'center_id',
            8 => 'receipt',
            9 => 'code',
            10 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_ledger = $request->input('search_ledger');


        // prepare ledgers ids
        function getSub($items, &$Ledgers_Ids)
        {
            while ($item = array_shift($items)) {
                $Ledgers_Ids[] = $item['id'];
                $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                $SubLedgers = $SubLedgers->toArray();
                getSub($SubLedgers, $Ledgers_Ids);
            }
        }

        $Ledgers_Ids = [];
        if ($search_ledger > 0) {
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
        }
        // End prepare ledgers ids

        if (count($Ledgers_Ids) > 0) {
            $Journals = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids);
        } else {
            $Journals = Journal::where('posted', true);
        }
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }


        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            $i = 0;
            foreach ($Journals as $Journal) {
                $cur_details = "";
                if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                    if ($Journal->dept > 0) {
                        $cur_details = " ( " . number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    } else {
                        $cur_details = " ( " . number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    }
                }

                $type = trans("backLang.dailyEntry");
                if ($Journal->type == 2) {
                    $type = trans("backLang.reconciliationEntry");
                } elseif ($Journal->type == 1) {
                    $type = trans("backLang.startBalance");
                } else {
                    if (@$Journal->receipt->type == 1) {
                        $type = trans("backLang.cashReceipt");
                    } elseif (@$Journal->receipt->type == 0) {
                        $type = trans("backLang.payReceipt");
                    }
                }
                $nestedData['id'] = "<div class='text-center small'>" . $Journal->id . "</div>";
                $nestedData['type'] = "<div class='text-center small'>" . $type . "</div>";
                $nestedData['date'] = "<div class='text-center small'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['dept'] = "<div class='text-center'>" . (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";

                $nestedData['ledger_id'] = "<div class='small'>" . @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "</div>";
                $nestedData['details'] = "<div class='small'>" . $Journal->details . $cur_details . "</div>";
                $nestedData['center_id'] = "<div class='small'>" . @$Journal->center->{"title_" . trans('backLang.boxCode')} . "</div>";

                $nestedData['receipt'] = "<div class='text-center small'>" . @$Journal->receipt->id . "</div>";
                $file_url = "";
                if (@$Journal->receipt->file != "") {
                    $file_url = route("AppReceiptsFilePreview", encrypt(@$Journal->receipt->id));
                    $nestedData['code'] = "<div class='text-center small'><a href='" . $file_url . "' target='_blank'>" . @$Journal->receipt->code . " <i class='fa fa-paperclip'></i></a></div>";
                } else {
                    $nestedData['code'] = "<div class='text-center small'>" . @$Journal->receipt->code . "</div>";
                }
                $data[] = $nestedData;
                $i++;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function ledger_print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_currency = $request->input('search_currency');
            $search_ledger = $request->input('search_ledger');

            $Currency = [];
            $Period = [];

            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            if ($search_ledger > 0) {
                $Ledger = Ledger::find($search_ledger);
                if (!empty($Ledger)) {
                    $father_id = $Ledger->id;
                    $Ledgers_Ids[] = $father_id;
                    $SubLedgers = Ledger::where("father_id", $father_id)->get();
                    $SubLedgers = $SubLedgers->toArray();
                    if (count($SubLedgers) > 0) {
                        getSub($SubLedgers, $Ledgers_Ids);
                    }
                }
            }
            // End prepare ledgers ids

            if (count($Ledgers_Ids) > 0) {
                $Journals = Journal::where('posted', true)->wherein('ledger_id', $Ledgers_Ids);
            } else {
                $Journals = Journal::where('posted', true);
            }
            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Journals = $Journals->where("currency_id", $search_currency);
            }
            if ($search_center_id > 0) {
                $Journals = $Journals->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }
            $Journals = $Journals->orderBy("date", "asc")->orderBy("id", "asc")->get();

            $Ledger = Ledger::find($search_ledger);
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));


            return view("back.accounting.reports.ledger.print", compact("Journals", "request", "Currency", "Period", "Ledger", "Ledgers_Ids", "DefaultCurrency"));
        }
        return "";
    }

    public function ledger_export(Request $request)
    {
        if (Roles::Check("accountant_export")) {

            return (new MasterReportExport($request))->download(trans('backLang.masterReport') . '.xlsx');
        }
        return "";
    }

    public function entries()
    {
        $Currencies = Currency::where("status", true)->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.reports.entries.list", compact("Currencies", "UsersList", "Periods", "ledgers_array"));
    }

    public function entries_list(Request $request)
    {
        $columns = array(
            0 => 'type',
            1 => 'id',
            2 => 'amount',
            3 => 'amount',
            4 => 'details',
            5 => 'id',
            6 => 'code',
            7 => 'posted',
            8 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_posted = $request->input('search_posted');
        $search_ledger = $request->input('search_ledger');

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_posted != "") {
            $Receipts = $Receipts->where("posted", $search_posted);
        }
        if ($search_ledger > 0) {
            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
            // End prepare ledgers ids

            $receipt_ids = Journal::select("receipt_id")->whereIn('ledger_id', $Ledgers_Ids);
            $Receipts = $Receipts->wherein("id", $receipt_ids);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $totalData = $Receipts->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Receipts = $Receipts->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Receipts as $Receipt) {

                // details prepare
                $dept = "";
                $credit = "";
                $details_from = "";
                $details_from_count = 0;
                $details_from_title = trans("backLang.fromAccount") . ": ";
                $details_to = "";
                $details_to_count = 0;
                $details_to_title = trans("backLang.toAccount") . ": ";
                foreach ($Receipt->journals->where("dept", ">", 0) as $Journal) {
                    $details_from_count++;
                    $details_from .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $dept .= number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                    $credit .= "<br>";
                }
                foreach ($Receipt->journals->where("credit", ">", 0) as $Journal) {
                    $details_to_count++;
                    $details_to .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $credit .= number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                }
                if ($details_from_count > 1) {
                    $details_from_title = trans("backLang.fromThoseMentioned") . ":<br>";
                    $dept = "<br>" . $dept;
                    $credit = "<br>" . $credit;
                }
                if ($details_to_count > 1) {
                    $credit = "<br>" . $credit;
                    $details_to_title = trans("backLang.toThoseMentioned") . ":<br>";
                }
                $details = "<span class='text-info'>" . $details_from_title . "</span>" . $details_from . "<div class='ml-3'>" . "<span class='text-info'>" . $details_to_title . "</span>" . $details_to . "</div><div> " . $Receipt->details . "</div>";


                $type = trans("backLang.dailyEntry");
                if (@$Journal->receipt->type == 1) {
                    $type = trans("backLang.cashReceipt");
                } elseif (@$Journal->receipt->type == 0) {
                    $type = trans("backLang.payReceipt");
                }
                $nestedData['type'] = "<div class='text-center'>" . $type . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $options = "<div class=\"text-center\">";
                $nestedData['dept'] = "<div class='text-center'>" . $dept . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . $credit . "</div>";
                $nestedData['details'] = $details;
                $nestedData['id'] = "<div class='text-center'>" . $Receipt->id . "</div>";
                $file_url = "";
                if ($Receipt->file != "") {
                    $file_url = route("AppReceiptsFilePreview", encrypt($Receipt->id));
                    $nestedData['code'] = "<div class='text-center'><a href='" . $file_url . "' target='_blank'>" . $Receipt->code . " <i class='fa fa-paperclip'></i></a></div>";
                } else {
                    $nestedData['code'] = "<div class='text-center'>" . $Receipt->code . "</div>";
                }
                $nestedData['posted'] = "<div class='text-center'>" . (($Receipt->posted) ? "<span>" . trans("backLang.posted") . "</span>" : "<span class='text-danger'>" . trans("backLang.notPosted") . "</span>") . "</div>";

                $receipt_link = "";
                if (!empty(@$Journal->receipt)) {
                    $receipt_link = " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt(@$Journal->receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                $nestedData['options'] = "<div class='text-center'>" . $receipt_link . "</div>";

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function banks()
    {
        //Role Check
        Roles::Check("accountant_report_4", true);

        $Currencies = Currency::where("status", true)->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        return view("back.accounting.reports.banks.list", compact("Currencies", "UsersList", "Periods", "Funds", "DefaultCurrency"));
    }

    public function banks_list(Request $request)
    {
        $limit = $request->input('length');
        $start = $request->input('start');
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_fund = $request->input('search_fund');

        $ledger_id = "";
        if ($search_fund > 0) {
            $Bank = Bank::find($search_fund);
            if (!empty($Bank)) {
                $ledger_id = $Bank->ledger_id;
            }
        }

        if ($ledger_id > 0) {
            $Journals = Journal::where('posted', true)->where('ledger_id', $ledger_id);
        } else {
            $Journals = Journal::where('posted', true);
        }
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }


        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy("date", "asc")
            ->orderBy("id", "asc")
            ->get();

        $data = array();
        $balance = 0;
        $total_of_dept = 0;
        $total_of_credit = 0;
        if ($totalFiltered > 0) {
            $i = 0;
            foreach ($Journals as $Journal) {
                if ($i == 0) {
                    $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $ledger_id)->where("period_id", $search_period)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_dept");
                    $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $ledger_id)->where("period_id", $search_period)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_credit");
                    $balance = $sum_of_dept - $sum_of_credit;
                    $total_of_dept += $sum_of_dept;
                    $total_of_credit += $sum_of_credit;
                } else {
                    if ($Journal->cost_dept > 0) {
                        $balance = $balance + $Journal->cost_dept;
                        $total_of_dept += $Journal->cost_dept;
                    }
                    if ($Journal->cost_credit > 0) {
                        $balance = $balance - $Journal->cost_credit;
                        $total_of_credit += $Journal->cost_credit;
                    }
                }
                $cur_details = "";
                if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                    if ($Journal->dept > 0) {
                        $cur_details = " ( " . number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    } else {
                        $cur_details = " ( " . number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    }
                }

                $type = trans("backLang.dailyEntry");
                if ($Journal->type == 2) {
                    $type = trans("backLang.reconciliationEntry");
                } elseif ($Journal->type == 1) {
                    $type = trans("backLang.startBalance");
                } else {
                    if (@$Journal->receipt->type == 1) {
                        $type = trans("backLang.cashReceipt");
                    } elseif (@$Journal->receipt->type == 0) {
                        $type = trans("backLang.payReceipt");
                    }
                }
                $nestedData['type'] = "<div class='text-center'>" . $type . "</div>";
                $nestedData['dept'] = "<div class='text-center'>" . (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";

                $nestedData['balance'] = "<div class='text-center " . (($balance < 0) ? "text-danger" : "text-success") . "'><strong>" . number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . "</strong></div>";
                $nestedData['details'] = $Journal->details . $cur_details;
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['id'] = "<div class='text-center'>" . $Journal->id . "</div>";
                $nestedData['receipt'] = "<div class='text-center'>" . @$Journal->receipt->id . "</div>";
                $receipt_link = "";
                if (!empty(@$Journal->receipt)) {
                    $receipt_link = " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt(@$Journal->receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                $nestedData['options'] = "<div class='text-center'>" . $receipt_link . "</div>";
                $file_url = "";
                if (@$Journal->receipt->file != "") {
                    $file_url = route("AppReceiptsFilePreview", encrypt(@$Journal->receipt->id));
                    $nestedData['code'] = "<div class='text-center'><a href='" . $file_url . "' target='_blank'>" . @$Journal->receipt->code . " <i class='fa fa-paperclip'></i></a></div>";
                } else {
                    $nestedData['code'] = "<div class='text-center'>" . @$Journal->receipt->code . "</div>";
                }
                $data[] = $nestedData;
                $i++;
            }
        }
        $TBalance = [
            "dept" => number_format($total_of_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "credit" => number_format($total_of_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "total" => number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol"))
        ];

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);

    }

    public function banks_print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_currency = $request->input('search_currency');
            $search_fund = $request->input('search_fund');

            $Currency = [];
            $Period = [];

            $ledger_id = "";
            if ($search_fund > 0) {
                $Bank = Bank::find($search_fund);
                if (!empty($Bank)) {
                    $ledger_id = $Bank->ledger_id;
                }
            }
            $request->search_ledger = $ledger_id;

            $Journals = Journal::where('posted', true)->where('ledger_id', $ledger_id);
            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Journals = $Journals->where("currency_id", $search_currency);
            }
            if ($search_center_id > 0) {
                $Journals = $Journals->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }
            $Journals = $Journals->orderBy("date", "asc")->orderBy("id", "asc")->get();

            $Ledger = Ledger::find($ledger_id);
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

            $Journal = $Journals->first();
            $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $ledger_id)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_dept");
            $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $ledger_id)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_credit");

            return view("back.accounting.reports.banks.print", compact("Journals", "request", "Currency", "Period", "Ledger", "sum_of_dept", "sum_of_credit", "DefaultCurrency"));
        }
        return "";
    }

    public function banks_export(Request $request)
    {
        if (Roles::Check("accountant_export")) {
            $search_fund = $request->input('search_fund');
            $ledger_id = "";
            if ($search_fund > 0) {
                $Bank = Bank::find($search_fund);
                if (!empty($Bank)) {
                    $ledger_id = $Bank->ledger_id;
                }
            }
            $request->search_ledger = $ledger_id;
            return (new LedgerReportExport($request))->download(trans('backLang.statementReport') . '.xlsx');
        }
        return "";
    }

    public function receipt($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Receipt = Receipt::find(decrypt($id));
                    if (!empty($Receipt)) {
                        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
                        return view("back.accounting.reports.receipts.receipt", compact("Receipt", "DefaultCurrency"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function positions()
    {
        //Role Check
        Roles::Check("accountant_report_4", true);

        $Currencies = Currency::where("status", true)->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.reports.positions.list", compact("Currencies", "UsersList", "Periods", "ledgers_array", "DefaultCurrency"));
    }

    public function positions_list(Request $request)
    {
        $limit = $request->input('length');
        $start = $request->input('start');
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');


        $Journals = Journal::where('posted', true)->where("center_id", $search_center_id);
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }


        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy("date", "asc")
            ->orderBy("id", "asc")
            ->get();

        $data = array();
        $balance = 0;
        $total_of_dept = 0;
        $total_of_credit = 0;
        if ($totalFiltered > 0) {
            $i = 0;
            foreach ($Journals as $Journal) {
                if ($Journal->cost_dept > 0) {
                    $total_of_dept += $Journal->cost_dept;
                }
                if ($Journal->cost_credit > 0) {
                    $total_of_credit += $Journal->cost_credit;
                }
                $cur_details = "";
                if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                    if ($Journal->dept > 0) {
                        $cur_details = " ( " . number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    } else {
                        $cur_details = " ( " . number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                    }
                }

                $type = trans("backLang.dailyEntry");
                if ($Journal->type == 2) {
                    $type = trans("backLang.reconciliationEntry");
                } elseif ($Journal->type == 1) {
                    $type = trans("backLang.startBalance");
                } else {
                    if (@$Journal->receipt->type == 1) {
                        $type = trans("backLang.cashReceipt");
                    } elseif (@$Journal->receipt->type == 0) {
                        $type = trans("backLang.payReceipt");
                    }
                }
                $nestedData['type'] = "<div class='text-center'>" . $type . "</div>";
                $nestedData['dept'] = "<div class='text-center'>" . (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") . "</div>";
                $nestedData['details'] = $Journal->details . $cur_details;
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['id'] = "<div class='text-center'>" . $Journal->id . "</div>";
                $nestedData['receipt'] = "<div class='text-center'>" . @$Journal->receipt->id . "</div>";
                $file_url = "";
                if (@$Journal->receipt->file != "") {
                    $file_url = route("AppReceiptsFilePreview", encrypt(@$Journal->receipt->id));
                    $nestedData['code'] = "<div class='text-center'><a href='" . $file_url . "' target='_blank'>" . @$Journal->receipt->code . " <i class='fa fa-paperclip'></i></a></div>";
                } else {
                    $nestedData['code'] = "<div class='text-center'>" . @$Journal->receipt->code . "</div>";
                }

                $receipt_link = "";
                if (!empty(@$Journal->receipt)) {
                    $receipt_link = " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt(@$Journal->receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                $nestedData['options'] = "<div class='text-center'>" . $receipt_link . "</div>";
                $data[] = $nestedData;
                $i++;
            }
        }
        $balance = $total_of_credit - $total_of_dept;
        $TBalance = [
            "dept" => number_format($total_of_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "credit" => number_format($total_of_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "total" => number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol"))
        ];

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);

    }

    public function positions_print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');

            $Currency = [];
            $Period = [];

            $Journals = Journal::where('posted', true)->where("center_id", $search_center_id);
            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }
            $Journals = $Journals->orderBy("date", "asc")->orderBy("id", "asc")->get();

            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

            $Journal = $Journals->first();
            $sum_of_dept = Journal::where('posted', true)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_dept");
            $sum_of_credit = Journal::where('posted', true)->where("date", "<=", $Journal->date)->where("id", "<=", $Journal->id)->orderby("date", "asc")->orderBy("id", "asc")->sum("cost_credit");

            return view("back.accounting.reports.positions.print", compact("Journals", "request", "Currency", "Period", "sum_of_dept", "sum_of_credit", "DefaultCurrency"));
        }
        return "";
    }

    public function positions_export(Request $request)
    {
        if (Roles::Check("accountant_export")) {

            return (new PositionsReportExport($request))->download(trans('backLang.costPositionsReport') . '.xlsx');
        }
        return "";
    }

    public function incomelist()
    {
        //Role Check
        Roles::Check("accountant_report_3", true);

        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.incomelist.list", compact("Currencies", "Funds", "UsersList", "Periods"));
    }

    public function incomelist_list(Request $request)
    {
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $stat = $request->input('stat');
        $Period = [];
        $Currency = [];
        $Fund = [];

        $Receipts = Receipt::where("id", '>', 0);
        if ($search_period > 0) {
            $Period = Period::find($search_period);
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }
        $Receipts = $Receipts->get();
        $Revenues = [];
        $RevenuesTotal = $Receipts->where("type", 1)->sum("cost");
        foreach (Helper::Categories(8) as $CatItem) {
            $cost = $Receipts->where("type", 1)->where("cat_id", $CatItem->id)->sum("cost");
            $Revenues[$CatItem->id] = $cost;
        }

        $Expenses = [];
        $ExpensesTotal = $Receipts->where("type", 0)->sum("cost");
        foreach (Helper::Categories(9) as $CatItem) {
            $cost = $Receipts->where("type", 0)->where("cat_id", $CatItem->id)->sum("cost");
            $Expenses[$CatItem->id] = $cost;
        }
        $NetProfit = $RevenuesTotal - $ExpensesTotal;

        $view = "totals";
        if ($stat == "print") {
            $view = "print";
        }
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view("back.accounting.reports.incomelist." . $view, compact("Revenues", "Expenses", "RevenuesTotal", "ExpensesTotal", "NetProfit", "DefaultCurrency", "request", "Period", "Currency", "Fund"));

    }

    public function balance()
    {
        //Role Check
        Roles::Check("accountant_report_7", true);

        $Periods = Period::orderby("code", "asc")->get();
        return view("back.accounting.reports.balance.list", compact("Periods"));
    }

    public function balance_list(Request $request)
    {
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_period = $request->input('search_period');
        $search_center_id = $request->input('search_center_id');
        $report_style = $request->input('report_style');


        $AssetLedger = Ledger::where('code', 1)->first();
        $ObligationLedger = Ledger::where('code', 2)->first();
        $PropertyLedger = Ledger::where('code', 3)->first();

        $Balances = [];
        $Ledgers = Ledger::orderBy("code", "asc")->get();
        if (count($Ledgers) > 0) {
            foreach ($Ledgers as $Ledger) {
                $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);

                if ($search_center_id > 0) {
                    $sum_of_dept = $sum_of_dept->where("center_id", $search_center_id);
                    $sum_of_credit = $sum_of_credit->where("center_id", $search_center_id);
                }
                if ($search_start != "") {
                    $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                    $sum_of_dept = $sum_of_dept->where('date', ">=", $search_start_date);
                    $sum_of_credit = $sum_of_credit->where('date', ">=", $search_start_date);
                }
                if ($search_end != "") {
                    $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                    $sum_of_dept = $sum_of_dept->where('date', "<=", $search_end_date);
                    $sum_of_credit = $sum_of_credit->where('date', "<=", $search_end_date);
                }
                $sum_of_dept = $sum_of_dept->get();
                $sum_of_credit = $sum_of_credit->get();

                $start_of_dept = $sum_of_dept->where("type", 1)->sum("cost_dept");
                $start_of_credit = $sum_of_credit->where("type", 1)->sum("cost_credit");

                $period_of_dept = $sum_of_dept->where("type", "!=", 1)->sum("cost_dept");
                $period_of_credit = $sum_of_credit->where("type", "!=", 1)->sum("cost_credit");

                $sum_of_dept = $sum_of_dept->sum("cost_dept");
                $sum_of_credit = $sum_of_credit->sum("cost_credit");

                $Balances[$Ledger->id] = [
                    "start" => $start_of_dept - $start_of_credit,
                    "period" => $period_of_dept - $period_of_credit,
                    "balance" => $sum_of_dept - $sum_of_credit
                ];
            }
        }
        if ($request->stat == "print") {
            $view = "print";
            if ($report_style) {
                $view = "print_report";
            }
            return view("back.accounting.reports.balance.$view", compact("search_end", "AssetLedger", "ObligationLedger", "PropertyLedger", "Balances", "request"));
        } else {
            $view = "totals";
            if ($report_style) {
                $view = "report";
            }
            return view("back.accounting.reports.balance.$view", compact("search_end", "AssetLedger", "ObligationLedger", "PropertyLedger", "Balances"));
        }
    }

    public function installments()
    {
        //Role Check
        Roles::Check("accountant_report_15", true);
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view("back.accounting.reports.installments.list", compact("DefaultCurrency"));
    }

    public function installments_list(Request $request)
    {
        $columns = array(
            0 => 'amount',
            1 => 'date',
            2 => 'status',
            3 => 'person_id',
            4 => 'phone',
            5 => 'notes',
            6 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $search_word = $request->input('search_word');
        $search_cat = $request->input('search_cat');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_paid = $request->input('search_paid');

        $loans_ids = Loan::select("id")->where("status", true);
        $Installments = Installment::wherein("loan_id", $loans_ids);

        if ($search_paid != "") {
            $Installments = $Installments->where("paid_status", $search_paid);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Installments = $Installments->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Installments = $Installments->where('date', "<=", $search_end_date);
        }
        if ($search_cat > 0) {
            $loans_ids = Loan::select("id")->where("cat_id", $search_cat);
            if (!empty($loans_ids)) {
                $Installments = $Installments->wherein("loan_id", $loans_ids);
            }
        }

        if ($search_word != "") {
            $n_search_q = $search_word;
            if ($n_search_q != "") {
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "0") {
                    $n_search_q = mb_substr($n_search_q, 1);
                }
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "0") {
                    $n_search_q = mb_substr($n_search_q, 1);
                }
                $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                if ($phone_fc == "+") {
                    $n_search_q = mb_substr($n_search_q, 1);
                }
            }

            $Installments = $Installments->where(function ($query) use ($search_word, $n_search_q) {
                $query->where("notes", "like", "%" . $search_word . "%");
            });
        }

        $totalData = $Installments->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Installments = $Installments->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        $loan_total = 0;
        $loan_paid = 0;
        if ($totalFiltered > 0) {
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            foreach ($Installments as $Installment) {
                $loan_total += $Installment->amount;
                $nestedData['amount'] = "<div class='text-center font-16 font-weight-bold'>" . number_format($Installment->amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol . "</div>";
                if (@$Installment->loan->pay_method == 0) {
                    $nestedData['date'] = "<div class='text-center'><span class='small'>" . trans("backLang.LoanPayMethods.0") . "</span> <br>" . date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                } else {
                    $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                }
                $Installment_status_title = "<span class='text-danger'>" . trans("backLang.unpaidInstallment") . "</span>";
                if ($Installment->paid_status) {
                    $loan_paid += $Installment->amount;
                    $Installment_status_title = "<span class='text-success'>" . trans("backLang.paidInstallment") . "</span>" . "<div class='small'>" . Helper::DateView($Installment->paid_at, 1) . "</div>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $Installment_status_title . "</div>";

                $nestedData['person_id'] = "<div class='text-center'>" . @$Installment->loan->name . "<div class='small mt-1'>" . @$Installment->loan->id_number . " - " . trans('backLang.identityTypes')[@$Installment->loan->id_type] . "</div></div>";
                $nestedData['phone'] = "<div class='text-center'>" . @$Installment->loan->phone . "</div>";
                $nestedData['notes'] = "<div class='text-center'>" . (($Installment->notes != "") ? $Installment->notes : "-") . "</div>";

                $options = "<div class=\"text-center\">";
                $options .= "
 <a class=\"btn btn-info\" href=\"" . route("LoansView", encrypt($Installment->loan_id)) . "\" data-toggle=\"tooltip\" data-original-title='" . trans("backLang.details") . "'><i class=\"fe fe-file-text\"></i> </button>";
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "installments_total" => number_format($loan_total, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol"))
        );

        return json_encode($json_data);

    }

    public function installments_print(Request $request)
    {
        if (Roles::Check("accountant_print")) {
            $search_word = $request->input('search_word');
            $search_cat = $request->input('search_cat');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_paid = $request->input('search_paid');


            $loans_ids = Loan::select("id")->where("status", true);
            $Installments = Installment::wherein("loan_id", $loans_ids);
            if ($search_paid != "") {
                $Installments = $Installments->where("paid_status", $search_paid);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Installments = $Installments->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Installments = $Installments->where('date', "<=", $search_end_date);
            }
            if ($search_cat > 0) {
                $loans_ids = Loan::select("id")->where("cat_id", $search_cat);
                if (!empty($loans_ids)) {
                    $Installments = $Installments->wherein("loan_id", $loans_ids);
                }
            }

            if ($search_word != "") {
                $n_search_q = $search_word;
                if ($n_search_q != "") {
                    $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                    if ($phone_fc == "0") {
                        $n_search_q = mb_substr($n_search_q, 1);
                    }
                    $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                    if ($phone_fc == "0") {
                        $n_search_q = mb_substr($n_search_q, 1);
                    }
                    $phone_fc = mb_substr($n_search_q, 0, 1, 'utf-8');
                    if ($phone_fc == "+") {
                        $n_search_q = mb_substr($n_search_q, 1);
                    }
                }

                $Installments = $Installments->where(function ($query) use ($search_word, $n_search_q) {
                    $query->where("notes", "like", "%" . $search_word . "%");
                });
            }
            $Installments = $Installments->orderby("date", "asc")->orderby("id", "asc")->get();
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));

            return view("back.accounting.reports.installments.print", compact("Installments", "request", "DefaultCurrency"));
        }
        return "";
    }

    public function installments_export(Request $request)
    {
        if (Roles::Check("accountant_export")) {

            return (new InstallmentsExport($request))->download(trans('backLang.installmentsReport') . '.xlsx');
        }
        return "";
    }

}
