<?php

namespace App\Http\Controllers\Accounting;

use App\Models\Bank;
use App\Models\Currency;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Receipt;
use App\Models\Period;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;
use Illuminate\Support\Facades\Storage;
use App\Exports\BalanceExport;
use App\Imports\BalanceImport;

class BalanceController extends Controller
{
    private $uploadPath = "uploads/receipts/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function opening()
    {
        $Currencies = Currency::where("status", true)->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $stat = "opening";
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.balances.home", compact("Currencies", "UsersList", "Periods", "stat", "ledgers_array"));
    }

    public function reconciliation()
    {
        $Currencies = Currency::where("status", true)->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $stat = "reconciliation";
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.balances.home", compact("Currencies", "UsersList", "Periods", "stat", "ledgers_array"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'date',
            1 => 'ledger_id',
            2 => 'dept',
            3 => 'credit',
            4 => 'details',
            5 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_posted = $request->input('search_posted');
        $search_type = $request->input('search_type');
        $search_ledger = $request->input('search_ledger');


        if (Roles::Check("view")) {
            $Journals = Journal::where("created_by", @Auth::user()->id)->where("type", $search_type);
        } else {
            $Journals = Journal::where("type", $search_type);
        }

        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_posted != "") {
            $Journals = $Journals->where("posted", $search_posted);
        }
        if ($search_ledger > 0) {
            $Journals = $Journals->where("ledger_id", $search_ledger);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }

        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)->orderBy("id", "asc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Journals as $Journal) {
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['ledger_id'] = @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")};
                $nestedData['dept'] = "<div class='text-center'><strong>" . (($Journal->dept > 0) ? number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol : "-") . "</strong></div>";
                $nestedData['credit'] = "<div class='text-center'><strong>" . (($Journal->credit > 0) ? number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol : "-") . "</strong></div>";
                $nestedData['details'] = $Journal->details;
                $options = "<div class=\"text-center\">";

                if (Roles::Check("accountant_print")) {
                    $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppBalancePrint", encrypt($Journal->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                }
                if (Roles::Check("edit") && @$Journal->period->status) {
                    if (Roles::Check("accountant_edit")) {
                        $edit_route = "AppBalancesEdit";
                        if ($Journal->type == 2) {
                            $edit_route = "AppReconciliationEdit";
                        }
                        $options .= " <a href=\"" . route($edit_route, encrypt($Journal->id)) . "\" class=\"btn btn-sm btn-info\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></a>";
                    }
                }
                if (Roles::Check("delete") && @$Journal->period->status) {
                    if (Roles::Check("accountant_delete")) {
                        $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Journal->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function balances_create()
    {
        $Currencies = Currency::where("status", true)->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        $stat = "opening";
        return view("back.accounting.balances.create", compact("Currencies", "ledgers_array", "stat"));
    }

    public function reconciliation_create()
    {
        $Currencies = Currency::where("status", true)->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        $stat = "reconciliation";
        return view("back.accounting.balances.create", compact("Currencies", "ledgers_array", "stat"));
    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_add")) {
                $validator = Validator::make($request->all(), [
                    'currency' => 'required',
                    'date' => 'required',
                    'stat' => 'required'
                ]);
                if ($validator->passes()) {
                    try {

                        if (!Helper::Preferences("default_period")) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }

                        $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                        if (!@$CurrentPeriod->status) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }

                        $saved_count = 0;
                        if ($request->rows_max > 0) {
                            for ($i = 1; $i <= $request->rows_max; $i++) {
                                if ($request->{'ledger_' . $i} > 0 && ($request->{'debtor_' . $i} > 0 || $request->{'creditor_' . $i} > 0)) {
                                    $Journal = new Journal;
                                    $Journal->type = ($request->stat == "opening") ? 1 : 2;
                                    $Journal->period_id = Helper::Preferences("default_period");
                                    $Journal->receipt_id = 0;
                                    $Journal->ledger_id = $request->{'ledger_' . $i};
                                    $Journal->center_id = $request->center_id;
                                    $Journal->date = date('Y-m-d', strtotime($request->date));
                                    $Journal->date_type = Helper::Preferences("date_type");
                                    $Journal->dept = $request->{'debtor_' . $i};
                                    $Journal->credit = $request->{'creditor_' . $i};
                                    $Journal->currency_id = $request->currency;
                                    $Journal->cost_dept = Helper::CurrencyTransfer($request->{'debtor_' . $i}, $request->currency, Helper::Preferences("default_currency"));
                                    $Journal->cost_credit = Helper::CurrencyTransfer($request->{'creditor_' . $i}, $request->currency, Helper::Preferences("default_currency"));
                                    $Journal->details = $request->{'details_' . $i};
                                    $Journal->posted = true;
                                    $Journal->created_by = Auth::user()->id;
                                    $Journal->save();
                                    $saved_count++;
                                }
                            }
                        }

                        if ($saved_count == 0) {
                            return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                        }
                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = ($request->stat == "opening") ? 605 : 606,
                            $window = 0,
                            $record_id = $Journal->id,
                            $operation = 2,
                            $title = (($Journal->dept > 0) ? $Journal->dept : $Journal->credit) . " " . @$Journal->currency->symbol . "<br>" . @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")} . " : " . mb_substr($Journal->details, 0, 150)
                        );


                        return response()->json(['stat' => 'success', "id" => encrypt($Journal->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function balances_edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Journal = Journal::find(decrypt($id));
                    if (!empty($Journal)) {
                        $Currencies = Currency::where("status", true)->get();
                        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                        return view("back.accounting.balances.edit", compact("Journal", "Currencies", "ledgers_array"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function reconciliation_edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Journal = Journal::find(decrypt($id));
                    if (!empty($Journal)) {
                        $Currencies = Currency::where("status", true)->get();
                        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                        return view("back.accounting.balances.edit", compact("Journal", "Currencies", "ledgers_array"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                $Journal = Journal::find(decrypt($request->id));
                if (!empty($Journal)) {
                    $validator = Validator::make($request->all(), [
                        'currency' => 'required',
                        'date' => 'required',
                        'ledger' => 'required',
                        'details' => 'required'
                    ]);
                    if ($validator->passes()) {
                        try {
                            $CurrentPeriod = Period::find($Journal->period_id);
                            if (!@$CurrentPeriod->status) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youCanNotUpdateIfPeriodClosed')]]);
                            }

                            $Journal->ledger_id = $request->ledger;
                            $Journal->center_id = $request->center_id;
                            $Journal->date = date('Y-m-d', strtotime($request->date));
                            $Journal->date_type = Helper::Preferences("date_type");
                            if ($Journal->dept != $request->debtor || $Journal->credit != $request->creditor || $Journal->currency_id != $request->currency) {
                                $Journal->dept = $request->debtor;
                                $Journal->credit = $request->creditor;
                                $Journal->currency_id = $request->currency;
                                $Journal->cost_dept = Helper::CurrencyTransfer($request->debtor, $request->currency, Helper::Preferences("default_currency"));
                                $Journal->cost_credit = Helper::CurrencyTransfer($request->creditor, $request->currency, Helper::Preferences("default_currency"));
                            }
                            $Journal->details = $request->details;
                            $Journal->updated_by = Auth::user()->id;
                            $Journal->save();

                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = ($Journal->type == 2) ? 606 : 605,
                                $window = 0,
                                $record_id = $Journal->id,
                                $operation = 3,
                                $title = (($Journal->dept > 0) ? $Journal->dept : $Journal->credit) . " " . @$Journal->currency->symbol . "<br>" . @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")} . " : " . mb_substr($Journal->details, 0, 150)
                            );

                            return response()->json(['stat' => 'success', "id" => encrypt($Journal->id)]);
                        } catch (\Exception $e) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                        }
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Journal = Journal::find(decrypt($id));
                if (!empty($Journal)) {

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = ($Journal->type == 2) ? 606 : 605,
                        $window = 0,
                        $record_id = $Journal->id,
                        $operation = 4,
                        $title = (($Journal->dept > 0) ? $Journal->dept : $Journal->credit) . " " . @$Journal->currency->symbol . "<br>" . @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")} . " : " . mb_substr($Journal->details, 0, 150)
                    );

                    $Journal->delete();
                    return json_encode(array("stat" => "success", "id" => $Journal->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_currency = $request->input('search_currency');
            $search_posted = $request->input('search_posted');
            $search_type = $request->input('search_type');
            $search_ledger = $request->input('search_ledger');

            $Currency = [];
            $Period = [];
            $Ledger = [];

            if (Roles::Check("view")) {
                $Journals = Journal::where("created_by", @Auth::user()->id)->where("type", $search_type);
            } else {
                $Journals = Journal::where("type", $search_type);
            }

            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Journals = $Journals->where("currency_id", $search_currency);
            }
            if ($search_posted != "") {
                $Journals = $Journals->where("posted", $search_posted);
            }

            if ($search_ledger > 0) {
                $Ledger = Ledger::find($search_ledger);
                $Journals = $Journals->where("ledger_id", $search_ledger);
            }
            if ($search_center_id > 0) {
                $Journals = $Journals->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }

            $Journals = $Journals->orderBy("date", "asc")->get();

            return view("back.accounting.balances.print", compact("Journals", "request", "Currency", "Period", "Ledger"));
        }
        return "";
    }

    public function receipt($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Journal = Journal::find(decrypt($id));
                    if (!empty($Journal)) {
                        return view("back.accounting.balances.receipt", compact("Journal"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function export(Request $request)
    {
        if (Roles::Check("accountant_export")) {
            $title = trans('backLang.openingBalances');
            if ($request->search_type == 2) {
                $title = trans('backLang.reconciliation');
            }

            return (new BalanceExport($request))->download($title . '.xlsx');
        }
        return "";
    }

}
