<?php


namespace App\Http\Controllers\Accounting;

use App\Models\Bank;
use App\Models\Currency;
use App\Models\Journal;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Receipt;
use App\Models\Period;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;
use Illuminate\Support\Facades\Storage;
use App\Exports\BankExport;

class BankBalanceController extends Controller
{
    private $uploadPath = "uploads/receipts/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function bank()
    {
        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $stat = "bank";
        return view("back.accounting.bank.home", compact("Currencies", "UsersList", "Periods", "stat", "Funds", "DefaultCurrency"));
    }

    public function safe()
    {
        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        $stat = "safe";
        return view("back.accounting.bank.home", compact("Currencies", "UsersList", "Periods", "stat", "Funds", "DefaultCurrency"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'date',
            2 => 'amount',
            3 => 'amount',
            4 => 'details',
            5 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_type = $request->input('search_type');
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_cat = $request->input('search_cat');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');

        if (Roles::Check("view")) {
            $Journals = Journal::where("created_by", @Auth::user()->id);
        } else {
            $Journals = Journal::where("id", ">", 0);
        }
        if ($search_fund > 0) {
            $ledger_ids = Bank::select("ledger_id")->where('id', $search_fund);
        } else {
            if ($search_type) {
                $ledger_ids = Bank::select("ledger_id")->where('type', 0);
            } else {
                $ledger_ids = Bank::select("ledger_id")->where('type', 1);
            }
        }
        $Journals = $Journals->wherein('ledger_id', $ledger_ids);
        if ($search_word != "") {
            $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
        }
        if ($search_currency > 0) {
            $Journals = $Journals->where("currency_id", $search_currency);
        }
        if ($search_cat > 0) {
            $receipts_ids = Receipt::select("id")->where("cat_id", $search_cat);
            $Journals = $Journals->wherein("receipt_id", $receipts_ids);
        }
        if ($search_pay_method > 0) {
            $receipts_ids = Receipt::select("id")->where("method_id", $search_pay_method);
            $Journals = $Journals->wherein("receipt_id", $receipts_ids);
        }
        if ($search_center_id > 0) {
            $Journals = $Journals->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Journals = $Journals->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Journals = $Journals->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Journals = $Journals->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Journals = $Journals->where('date', "<=", $search_end_date);
        }

        $sum_of_dept = $Journals->sum("cost_dept");
        $sum_of_credit = $Journals->sum("cost_credit");
        $balance = $sum_of_dept - $sum_of_credit;

        $totalData = $Journals->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Journals = $Journals->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Journals as $Journal) {
                if (!empty($Journal->receipt)) {
                    $nestedData['id'] = "<div class='text-center'>" . @$Journal->receipt->id . "</div>";
                } else {
                    $nestedData['id'] = "<div class='text-center'>" . @$Journal->id . "</div>";
                }
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['ledger_id'] = @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")};
                $nestedData['dept'] = "<div class='text-center'><strong>" . (($Journal->dept > 0) ? number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol : "-") . "</strong></div>";
                $nestedData['credit'] = "<div class='text-center'><strong>" . (($Journal->credit > 0) ? number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol : "-") . "</strong></div>";
                $nestedData['details'] = $Journal->details;
                $options = "<div class=\"text-center\">";

                if (!empty($Journal->receipt)) {
                    if ($Journal->receipt->type < 2) {
                        // sanad kabd , sarf
                        if (Roles::Check("accountant_print")) {
                            $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppReceiptPrint", encrypt($Journal->receipt->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                        }
                        if (Roles::Check("edit") && @$Journal->receipt->period->status) {
                            if (Roles::Check("accountant_edit")) {
                                $options .= " <button class=\"btn btn-sm btn-info\" onclick=\"edit_row('" . encrypt($Journal->receipt->id) . "','" . $Journal->receipt->type . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></button>";
                            }
                        }
                    } else {
                        // daily entry
                        if (Roles::Check("accountant_print")) {
                            $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppJournalReceipt", encrypt($Journal->receipt->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                        }
                        if (Roles::Check("edit") && @$Journal->receipt->period->status) {
                            if (Roles::Check("accountant_edit")) {
                                $options .= " <a href=\"" . route("AppJournalsEdit", encrypt($Journal->receipt->id)) . "\" class=\"btn btn-sm btn-info\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></a>";
                            }
                        }
                    }
                    if (Roles::Check("delete") && @$Journal->receipt->period->status) {
                        if (Roles::Check("accountant_delete")) {
                            $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Journal->receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                        }
                    }
                } else {
                    // opening balance or reconciliation
                    if (Roles::Check("accountant_print")) {
                        $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppBalancePrint", encrypt($Journal->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                    }
                    if (Roles::Check("edit") && @$Journal->period->status) {
                        if (Roles::Check("accountant_edit")) {
                            $edit_route = "AppBalancesEdit";
                            if ($Journal->type == 2) {
                                $edit_route = "AppReconciliationEdit";
                            }
                            $options .= " <a href=\"" . route($edit_route, encrypt($Journal->id)) . "\" class=\"btn btn-sm btn-info\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></a>";
                        }
                    }
                    if (Roles::Check("delete") && @$Journal->period->status) {
                        if (Roles::Check("accountant_delete")) {
                            $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_journal('" . encrypt($Journal->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                        }
                    }
                }

                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }


        $TBalance = [
            "dept" => number_format($sum_of_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "credit" => number_format($sum_of_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")),
            "total" => number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol"))
        ];
        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);
    }

    public function balance($id = "")
    {
        if ($id != "") {
            $Bank = bank::find($id);
            if ($Bank->ledger_id > 0) {
                $Journals = Journal::where("ledger_id", $Bank->ledger_id);
                $sum_of_dept = $Journals->sum("cost_dept");
                $sum_of_credit = $Journals->sum("cost_credit");
                $balance = round($sum_of_credit - $sum_of_dept, 2);
                $cls = "text-success";
                if ($balance < 0) {
                    $cls = "text-danger";
                }
                return "<span class='$cls'>" . number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . "</span>";
            }
        }
    }

    public function transfer(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_add")) {
                $validator = Validator::make($request->all(), [
                    'from_fund' => 'required',
                    'to_fund' => 'required',
                    'amount' => 'required',
                    'currency' => 'required',
                    'date' => 'required',
                    'details' => 'required'
                ]);
                if ($validator->passes()) {

                    if ($request->from_fund == $request->to_fund) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.youCanNotTransferFromToSameFund')]]);
                    }
                    if (!Helper::Preferences("default_period")) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                    }

                    $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                    if (!@$CurrentPeriod->status) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                    }

                    $FromBank = bank::find($request->from_fund);
                    $ToBank = bank::find($request->to_fund);
                    if (@$FromBank->ledger_id > 0 && @$ToBank->ledger_id > 0) {
                        // Start File Upload
                        $formFileName = "file";
                        $fileFinalRand = "";
                        $fileFinalExt = "";
                        $uploaded = false;
                        $path = "";
                        if ($request->$formFileName != "") {
                            $path = $this->uploadPath;
                            $fileFinalRand = time() . rand(1111, 9999);
                            $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                            $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);
                        }

                        $Receipt = new Receipt;
                        $Receipt->type = 2;
                        $Receipt->code = $request->code;
                        $Receipt->period_id = Helper::Preferences("default_period");
                        $Receipt->date = date('Y-m-d', strtotime($request->date));
                        $Receipt->date_type = Helper::Preferences("date_type");
                        $Receipt->amount_type = 0;
                        $Receipt->amount = $request->amount;
                        $Receipt->currency_id = $request->currency;
                        $Receipt->cost = Helper::CurrencyTransfer($request->amount, $request->currency, Helper::Preferences("default_currency"));
                        $Receipt->details = $request->details;
                        if ($uploaded) {
                            $Receipt->file = $fileFinalRand;
                            $Receipt->ext = $fileFinalExt;
                            $Receipt->remote = 0;
                        }

                        if ($request->charges_status) {
                            $Receipt->bank_charges = true;
                            $Receipt->bank_charges_amount = $request->charges_amount;
                            $Receipt->bank_charges_details = $request->charges_details;
                        }

                        $Receipt->repeated = false;
                        $Receipt->posted = Helper::Preferences("posting_method");
                        $Receipt->created_ip = \Request::ip();
                        $Receipt->created_by = Auth::user()->id;
                        $Receipt->save();


                        // delete old journals
                        $Receipt->journals()->delete();

                        // save new to journal
                        $Journal = new Journal;
                        $Journal->type = 0;
                        $Journal->period_id = $Receipt->period_id;
                        $Journal->receipt_id = $Receipt->id;
                        $Journal->center_id = $Receipt->center_id;
                        $Journal->date = $Receipt->date;
                        $Journal->date_type = $Receipt->date_type;
                        $Journal->currency_id = $Receipt->currency_id;
                        $Journal->details = $Receipt->details;
                        $Journal->posted = Helper::Preferences("posting_method");
                        $Journal->created_by = Auth::user()->id;

                        // From Ledger:
                        $Journal->ledger_id = @$FromBank->ledger_id;
                        $Journal->credit = $Receipt->amount;
                        $Journal->cost_credit = $Receipt->cost;
                        $Journal->dept = 0;
                        $Journal->cost_dept = 0;
                        $Journal->save();
                        // To Ledger:
                        $SecondJournal = $Journal->replicate();
                        $SecondJournal->ledger_id = @$ToBank->ledger_id;
                        $SecondJournal->credit = 0;
                        $SecondJournal->cost_credit = 0;
                        $SecondJournal->dept = $Receipt->amount;
                        $SecondJournal->cost_dept = $Receipt->cost;
                        $SecondJournal->save();


                        //bank charges
                        if ($Receipt->bank_charges && (!empty($FromBank) || !empty($ToBank))) {
                            if ($FromBank->charges_ledger_id > 0 || $ToBank->charges_ledger_id > 0) {
                                $ThirdJournal = $Journal->replicate();
                                $ThirdJournal->details = $Receipt->bank_charges_details;
                                $ThirdJournal->ledger_id = $FromBank->ledger_id;
                                $ThirdJournal->dept = 0;
                                $ThirdJournal->cost_dept = 0;
                                $ThirdJournal->credit = $Receipt->bank_charges_amount;
                                $ThirdJournal->cost_credit = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                $ThirdJournal->save();

                                $FourthJournal = $Journal->replicate();
                                $ThirdJournal->details = $Receipt->bank_charges_details;
                                $FourthJournal->ledger_id = ($FromBank->charges_ledger_id > 0) ? $FromBank->charges_ledger_id : $ToBank->charges_ledger_id;
                                $FourthJournal->dept = $Receipt->bank_charges_amount;
                                $FourthJournal->cost_dept = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                $FourthJournal->credit = 0;
                                $FourthJournal->cost_credit = 0;
                                $FourthJournal->save();
                            }
                        }

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 608,
                            $window = 0,
                            $record_id = $Receipt->id,
                            $operation = 2,
                            $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr($Receipt->details, 0, 150)
                        );

                        //Remote storage
                        if ($uploaded) {
                            if (Helper::FileSystem() != "") {
                                $filepath = $path . $fileFinalRand;
                                $localFile = File::get($filepath);
                                $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                                if ($Remote_store) {
                                    $Receipt->remote = "1";
                                    $Receipt->save();
                                    // Delete from local
                                    File::delete($filepath);
                                }
                            }
                        }

                        return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                    } else {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }

                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_type = $request->input('search_type');
            $search_fund = $request->input('search_fund');
            $search_pay_method = $request->input('search_pay_method');
            $search_currency = $request->input('search_currency');
            $search_cat = $request->input('search_cat');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');

            $Currency = [];
            $Period = [];

            if (Roles::Check("view")) {
                $Journals = Journal::where("created_by", @Auth::user()->id);
            } else {
                $Journals = Journal::where("id", ">", 0);
            }
            if ($search_fund > 0) {
                $ledger_ids = Bank::select("ledger_id")->where('id', $search_fund);
            } else {
                if ($search_type) {
                    $ledger_ids = Bank::select("ledger_id")->where('type', 0);
                } else {
                    $ledger_ids = Bank::select("ledger_id")->where('type', 1);
                }
            }
            $Journals = $Journals->wherein('ledger_id', $ledger_ids);
            if ($search_word != "") {
                $Journals = $Journals->where("details", "like", "%" . $search_word . "%");
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Journals = $Journals->where("currency_id", $search_currency);
            }
            if ($search_cat > 0) {
                $receipts_ids = Receipt::select("id")->where("cat_id", $search_cat);
                $Journals = $Journals->wherein("receipt_id", $receipts_ids);
            }
            if ($search_pay_method > 0) {
                $receipts_ids = Receipt::select("id")->where("method_id", $search_pay_method);
                $Journals = $Journals->wherein("receipt_id", $receipts_ids);
            }
            if ($search_center_id > 0) {
                $Journals = $Journals->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Journals = $Journals->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Journals = $Journals->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Journals = $Journals->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Journals = $Journals->where('date', "<=", $search_end_date);
            }

            $Journals = $Journals->orderBy("date", "asc")->get();

            return view("back.accounting.bank.print", compact("Journals", "request", "Currency", "Period"));
        }
        return "";
    }

    public function export(Request $request)
    {
        if (Roles::Check("accountant_export")) {
            $title = trans('backLang.fundAccount');
            if ($request->search_type) {
                $title = trans('backLang.bankAccount');
            }
            return (new BankExport($request))->download($title . '.xlsx');
        }
        return "";
    }
}
