<?php

namespace App\Http\Controllers\Accounting;

use App\Helpers\History;
use App\Models\Ledger;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Bank;
use App\Models\Currency;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;

class BankController extends Controller

{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_accounting", true);
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'check',
            1 => 'order',
            2 => 'title_' . trans("backLang.code"),
            3 => 'code',
            4 => 'ledger_id',
            5 => 'status',
            6 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $stat = $request->input('stat');

        if ($stat) {
            $columns = array(
                0 => 'check',
                1 => 'order',
                2 => 'title_' . trans("backLang.code"),
                4 => 'ledger_id',
                5 => 'status',
                6 => 'options'
            );
        }

        if (Roles::Check("view")) {
            $Banks = Bank::where("created_by", @Auth::user()->id);
        } else {
            $Banks = Bank::where("id", ">", 0);
        }
        if ($stat) {
            $Banks = $Banks->where("type", 1);
        } else {
            $Banks = $Banks->where("type", 0);
        }
        $totalData = $Banks->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Banks = $Banks->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Banks as $Bank) {
                $nestedData['check'] = "<div class='text-center'><label class=\"custom-control custom-checkbox\"><input type=\"checkbox\" class=\"custom-control-input\" name=\"ids[]\" value=\"" . $Bank->id . "\"><span class=\"custom-control-label\">&nbsp;</span></label></div>";
                $nestedData['order'] = "<div class='text-center'>
<input class=\"form-control\" name=\"row_ids[]\" type=\"hidden\" value=\"" . $Bank->id . "\">
<input type='number' name='row_" . $Bank->id . "' min='0' class='form-control form-order' value='" . $Bank->order . "'></div>";

                $nestedData['title'] = $Bank->{"title_" . trans("backLang.code")};
                $nestedData['code'] = $Bank->code;
                $nestedData['ledger_id'] = "<div class='text-center'>" . @$Bank->ledger->code . " - " . @$Bank->ledger->{"title_" . trans("backLang.code")} . "</div>";
                $nestedData['charges_ledger_id'] = "<div class='text-center'>" . @$Bank->charges->code . " - " .@$Bank->charges->{"title_" . trans("backLang.code")} . "</div>";

                $nestedData['currency_id'] = "<div class='text-center'>" . ((!empty($Bank->currency)) ? $Bank->currency->{"title_" . trans("backLang.code")} : "-") . "</div>";


                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Bank->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";

                $options = "<div class=\"text-center\">";

                $BankOrFund = "bank";
                if ($Bank->type) {
                    $BankOrFund = "fund";
                }
                if (Roles::Check("edit")) {
                    $options .= "
  <button class=\"btn btn-sm btn-info\" type='button' onclick=\"edit_" . $BankOrFund . "('" . encrypt($Bank->id) . "')\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</button>
 ";
                }

                if (Roles::Check("delete")) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" type='button' onclick=\"delete_" . $BankOrFund . "('" . encrypt($Bank->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $type = ($request->type) ? 1 : 0;
            if (!$type) {
                $validator = Validator::make($request->all(), [
                    'code' => 'required',
                    'owner' => 'required',
                    'currency' => 'required',
                    'ledger_id' => 'required'
                ]);
                if (!$validator->passes()) {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }

            try {
                $next_row_order = Bank::where("type", $type)->max('order');
                if ($next_row_order < 1) {
                    $next_row_order = 1;
                } else {
                    $next_row_order++;
                }

                $Bank = new Bank;
                $Bank->order = $next_row_order;
                $Bank->title_ar = $request->title_ar;
                $Bank->title_en = $request->title_en;
                if ($type) {
                    $Bank->ledger_id = $request->ledger_id;
                    $Bank->charges_ledger_id = 0;
                    $Bank->code = "";
                    $Bank->owner = "";
                    $Bank->currency_id = 0;
                    $Bank->type = 1;
                } else {
                    $Bank->ledger_id = $request->ledger_id;
                    $Bank->charges_ledger_id = $request->charges_ledger_id;
                    $Bank->code = $request->code;
                    $Bank->bank_no = $request->bank_no;
                    $Bank->owner = $request->owner;
                    $Bank->currency_id = $request->currency;
                    $Bank->type = ($request->type) ? 1 : 0;
                }
                $Bank->status = true;
                $Bank->created_ip = \Request::ip();
                $Bank->created_by = Auth::user()->id;
                $Bank->save();


                // Log to history
                History::Log(
                    $module = 6,
                    $cat = 600,
                    $window = 6001,
                    $record_id = $Bank->id,
                    $operation = 2,
                    $title = $Bank->title_ar . "<br>" . $Bank->title_en
                );

                return response()->json(['stat' => 'success', "id" => encrypt($Bank->id)]);
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }

    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Bank = Bank::find(decrypt($id));
                if (!empty($Bank)) {
                    $Currencies = Currency::where("status", true)->get();
                    $BankOrFund = "banks";
                    if ($Bank->type) {
                        $BankOrFund = "funds";
                    }

                    $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                    $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                    return view("back.accounting.preferences." . $BankOrFund . ".edit", compact("Bank", "Currencies", "ledgers_array"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {

            $type = ($request->type) ? 1 : 0;
            if (!$type) {
                $validator = Validator::make($request->all(), [
                    'code' => 'required',
                    'owner' => 'required',
                    'currency' => 'required',
                    'ledger_id' => 'required'
                ]);
                if (!$validator->passes()) {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }
            try {
                $Bank = Bank::find(decrypt($request->id));
                if (!empty($Bank)) {
                    $Bank->title_ar = $request->title_ar;
                    $Bank->title_en = $request->title_en;
                    if ($type) {
                        $Bank->ledger_id = $request->ledger_id;
                        $Bank->charges_ledger_id = 0;
                        $Bank->code = "";
                        $Bank->owner = "";
                        $Bank->currency_id = 0;
                        $Bank->type = 1;
                    } else {
                        $Bank->ledger_id = $request->ledger_id;
                        $Bank->charges_ledger_id = $request->charges_ledger_id;
                        $Bank->code = $request->code;
                        $Bank->bank_no = $request->bank_no;
                        $Bank->owner = $request->owner;
                        $Bank->currency_id = $request->currency;
                        $Bank->type = ($request->type) ? 1 : 0;
                    }
                    $Bank->updated_ip = \Request::ip();
                    $Bank->updated_by = Auth::user()->id;
                    $Bank->save();

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6001,
                        $record_id = $Bank->id,
                        $operation = 3,
                        $title = $Bank->title_ar . "<br>" . $Bank->title_en
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Bank->id)]);
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function multi_update(Request $request)
    {

        if (Roles::Check("edit")) {
            if ($request->action == "save") {
                foreach ($request->row_ids as $rowId) {
                    $Bank = Bank::find($rowId);
                    if (!empty($Bank)) {
                        $row_no_val = "row_" . $rowId;
                        $Bank->order = $request->$row_no_val;
                        $Bank->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6001,
                            $record_id = $Bank->id,
                            $operation = 3,
                            $title = $Bank->title_ar . "<br>" . $Bank->title_en
                        );

                    }
                }
                return response()->json(['stat' => 'success']);
            }
            if ($request->ids != "") {
                if ($request->action == "activate") {
                    $Banks = Bank::wherein('id', $request->ids)->get();
                    foreach ($Banks as $Bank) {
                        $Bank->status = true;
                        $Bank->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6001,
                            $record_id = $Bank->id,
                            $operation = 9,
                            $title = $Bank->title_ar . "<br>" . $Bank->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "block") {

                    $Banks = Bank::wherein('id', $request->ids)->get();
                    foreach ($Banks as $Bank) {
                        $Bank->status = false;
                        $Bank->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6001,
                            $record_id = $Bank->id,
                            $operation = 8,
                            $title = $Bank->title_ar . "<br>" . $Bank->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "delete") {
                    $Banks = Bank::wherein('id', $request->ids)->get();
                    foreach ($Banks as $Bank) {

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6001,
                            $record_id = $Bank->id,
                            $operation = 4,
                            $title = $Bank->title_ar . "<br>" . $Bank->title_en
                        );

                        $Bank->delete();
                    }

                    return response()->json(['stat' => 'success']);
                }

            } else {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.noRowsSelected')]]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Bank = Bank::find(decrypt($id));
                if (!empty($Bank)) {

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6001,
                        $record_id = $Bank->id,
                        $operation = 4,
                        $title = $Bank->title_ar . "<br>" . $Bank->title_en
                    );

                    $Bank->delete();
                    return json_encode(array("stat" => "success", "id" => $Bank->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
