<?php

namespace App\Http\Controllers\Accounting;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Currency;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;

class CurrencyController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_accounting", true);
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'check',
            1 => 'order',
            2 => 'title_' . trans("backLang.code"),
            3 => 'code',
            4 => 'symbol',
            5 => 'rate',
            6 => 'status',
            7 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');


        if (Roles::Check("view")) {
            $Currencies = Currency::where("created_by", @Auth::user()->id);
        } else {
            $Currencies = Currency::where("id", ">", 0);
        }
        $totalData = $Currencies->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Currencies = $Currencies->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Currencies as $Currency) {
                $nestedData['check'] = "<div class='text-center'><label class=\"custom-control custom-checkbox\"><input type=\"checkbox\" class=\"custom-control-input\" name=\"ids[]\" value=\"" . $Currency->id . "\"><span class=\"custom-control-label\">&nbsp;</span></label></div>";
                $nestedData['order'] = "<div class='text-center'>
<input class=\"form-control\" name=\"row_ids[]\" type=\"hidden\" value=\"" . $Currency->id . "\">
<input type='number' name='row_" . $Currency->id . "' min='0' class='form-control form-order' value='" . $Currency->order . "'></div>";

                $nestedData['title'] = "<div class='text-center'>" . $Currency->{"title_" . trans("backLang.code")} . "</div>";
                $nestedData['code'] = "<div class='text-center'>" . $Currency->code . "</div>";
                $nestedData['symbol'] = "<div class='text-center'>" . $Currency->symbol . "</div>";
                $nestedData['rate'] = "<div class='text-center'>" . $Currency->rate . "</div>";

                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Currency->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";

                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
  <button class=\"btn btn-sm btn-info\" type='button' onclick=\"edit_currency('" . encrypt($Currency->id) . "')\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</button>
 ";
                }

                if (Roles::Check("delete")) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" type='button' onclick=\"delete_currency('" . encrypt($Currency->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $validator = Validator::make($request->all(), [
                'code' => 'required',
                'symbol' => 'required',
                'rate' => 'required'
            ]);
            if ($validator->passes()) {
                try {

                    $next_row_order = Currency::max('order');
                    if ($next_row_order < 1) {
                        $next_row_order = 1;
                    } else {
                        $next_row_order++;
                    }

                    $Currency = new Currency;
                    $Currency->order = $next_row_order;
                    $Currency->code = $request->code;
                    $Currency->title_ar = $request->title_ar;
                    $Currency->title_en = $request->title_en;
                    $Currency->symbol = $request->symbol;
                    $Currency->rate = $request->rate;
                    $Currency->status = true;
                    $Currency->created_ip = \Request::ip();
                    $Currency->created_by = Auth::user()->id;
                    $Currency->save();


                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6004,
                        $record_id = $Currency->id,
                        $operation = 2,
                        $title = $Currency->title_ar . "<br>" . $Currency->title_en
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Currency->id)]);
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Currency = Currency::find(decrypt($id));
                if (!empty($Currency)) {
                    return view("back.accounting.preferences.currencies.edit", compact("Currency"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {

            $validator = Validator::make($request->all(), [
                'code' => 'required',
                'symbol' => 'required',
                'rate' => 'required'
            ]);
            if ($validator->passes()) {
                try {
                    $Currency = Currency::find(decrypt($request->id));
                    if (!empty($Currency)) {
                        $Currency->code = $request->code;
                        $Currency->title_ar = $request->title_ar;
                        $Currency->title_en = $request->title_en;
                        $Currency->symbol = $request->symbol;
                        $Currency->rate = $request->rate;
                        $Currency->updated_ip = \Request::ip();
                        $Currency->updated_by = Auth::user()->id;
                        $Currency->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6004,
                            $record_id = $Currency->id,
                            $operation = 3,
                            $title = $Currency->title_ar . "<br>" . $Currency->title_en
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Currency->id)]);
                    }
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function multi_update(Request $request)
    {

        if (Roles::Check("edit")) {
            if ($request->action == "save") {
                foreach ($request->row_ids as $rowId) {
                    $Currency = Currency::find($rowId);
                    if (!empty($Currency)) {
                        $row_no_val = "row_" . $rowId;
                        $Currency->order = $request->$row_no_val;
                        $Currency->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6004,
                            $record_id = $Currency->id,
                            $operation = 3,
                            $title = $Currency->title_ar . "<br>" . $Currency->title_en
                        );

                    }
                }
                return response()->json(['stat' => 'success']);
            }
            if ($request->ids != "") {
                if ($request->action == "rate") {
                    $Currencies = Currency::wherein('id', $request->ids)->get();
                    $Default = Currency::find(Helper::Preferences("default_currency"));
                    if (!empty($Default)) {
                        foreach ($Currencies as $Currency) {
                            if ($Currency->code == $Default->code) {
                                $rate = 1;
                            } else {

                                $query = strtoupper($Default->code) . '_' . strtoupper($Currency->code);

                                $url = 'https://api.currconv.com/api/v7/convert?apiKey=' . env("CURRENCY_CONVERTER_API_KEY") . '&q=' . $query . '&compact=ultra';
                                $ch = curl_init();
                                // Disable SSL verification
                                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                                // Will return the response, if false it print the response
                                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                                // Set the url
                                curl_setopt($ch, CURLOPT_URL, $url);
                                // Execute
                                $result = curl_exec($ch);
                                // Closing
                                curl_close($ch);

                                $obj = json_decode($result, true);
                                $rate = number_format(floatval($obj[$query]), 2, '.', '');
                            }
                            if ($rate > 0) {
                                $Currency->rate = $rate;
                                $Currency->update();
                            }
                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = 600,
                                $window = 6004,
                                $record_id = $Currency->id,
                                $operation = 3,
                                $title = $Currency->title_ar . "<br>" . trans("backLang.currencyRate") . ": " . $rate
                            );
                        }
                        return response()->json(['stat' => 'success']);
                    } else {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                } elseif ($request->action == "activate") {
                    $Currencies = Currency::wherein('id', $request->ids)->get();
                    foreach ($Currencies as $Currency) {
                        $Currency->status = true;
                        $Currency->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6004,
                            $record_id = $Currency->id,
                            $operation = 9,
                            $title = $Currency->title_ar . "<br>" . $Currency->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "block") {

                    $Currencies = Currency::wherein('id', $request->ids)->get();
                    foreach ($Currencies as $Currency) {
                        $Currency->status = false;
                        $Currency->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6004,
                            $record_id = $Currency->id,
                            $operation = 8,
                            $title = $Currency->title_ar . "<br>" . $Currency->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "delete") {
                    $Currencies = Currency::wherein('id', $request->ids)->get();
                    foreach ($Currencies as $Currency) {

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6004,
                            $record_id = $Currency->id,
                            $operation = 4,
                            $title = $Currency->title_ar . "<br>" . $Currency->title_en
                        );

                        $Currency->delete();
                    }

                    return response()->json(['stat' => 'success']);
                }

            } else {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.noRowsSelected')]]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Currency = Currency::find(decrypt($id));
                if (!empty($Currency)) {

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6004,
                        $record_id = $Currency->id,
                        $operation = 4,
                        $title = $Currency->title_ar . "<br>" . $Currency->title_en
                    );

                    $Currency->delete();
                    return json_encode(array("stat" => "success", "id" => $Currency->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
