<?php

namespace App\Http\Controllers\Accounting;

use App;
use App\Models\Bank;
use App\Models\Category;
use App\Models\Currency;
use App\Helpers\History;
use App\Helpers\AppSender;
use App\Http\Controllers\Controller;
use App\Models\Journal;
use App\Models\Period;
use App\Models\Attachment;
use App\Models\Loan;
use App\Models\Installment;
use App\Models\Receipt;
use App\Models\User;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use Validator;

class InstallmentController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("role_loans", true);
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'amount',
            2 => 'date',
            3 => 'status',
            4 => 'notes',
            5 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
        $loan_id = $request->input('loan_id');

        $Installments = Installment::where("loan_id", $loan_id);

        $totalData = $Installments->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Installments = $Installments->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        $loan_total = 0;
        $loan_paid = 0;
        if ($totalFiltered > 0) {
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            foreach ($Installments as $Installment) {
                $loan_total += $Installment->amount;
                $nestedData['id'] = "<div class='text-center'>" . $Installment->id . "</div>";
                $nestedData['amount'] = "<div class='text-center'>" . number_format($Installment->amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol . "</div>";
                if (@$Installment->loan->pay_method == 0) {
                    $nestedData['date'] = "<div class='text-center'><span class='small'>" . trans("backLang.LoanPayMethods.0") . "</span> <br>" . date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                } else {
                    $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                }
                $Installment_status_title = "<span class='text-danger'>" . trans("backLang.unpaidInstallment") . "</span>";
                if ($Installment->paid_status) {
                    $loan_paid += $Installment->amount;
                    $Installment_status_title = "<span class='text-success'>" . trans("backLang.paidInstallment") . "</span>" . "<div class='small'>" . Helper::DateView($Installment->paid_at, 1) . "</div>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $Installment_status_title . "</div>";

                $nestedData['notes'] = "<div class='text-center'>" . (($Installment->notes != "") ? $Installment->notes : "-") . "</div>";

                $options = "<div class=\"text-center\">";
                if (Roles::Check("loan_print")) {
                    if (!empty($Installment->receipt)) {
                        $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppReceiptPrint", encrypt($Installment->receipt->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                    }
                }
                if (Roles::Check("edit")) {
                    if (Roles::Check("loans_edit")) {
                        if (@$Installment->period->status || (!@$Installment->period->status && !$Installment->paid_status)) {
                            $options .= " <button class=\"btn btn-sm btn-info\" onclick=\"edit_installment('" . encrypt($Installment->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></button>";
                        }
                    }
                }
                if (Roles::Check("delete")) {
                    if (Roles::Check("loans_delete")) {
                        if (@$Installment->period->status || (!@$Installment->period->status && !$Installment->paid_status)) {
                            $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_installment('" . encrypt($Installment->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                        }
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $unpaid_balance = $loan_total - $loan_paid;
        $TBalance = [
            "total" => number_format($loan_total, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol,
            "paid" => number_format($loan_paid, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol,
            "unpaid" => number_format($unpaid_balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol
        ];
        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);

    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loans_edit")) {
                if ($id != "") {
                    $Installment = Installment::find(decrypt($id));
                    if (!empty($Installment)) {
                        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
                        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
                        return view("back.accounting.loans.edit.installment", compact("Installment", "DefaultCurrency", "Funds"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loans_edit")) {
                $Installment = Installment::find(decrypt($request->id));
                if (!empty($Installment)) {
                    if ($request->paid_status) {
                        $validator = Validator::make($request->all(), [
                            'amount' => 'required',
                            'date' => 'required',
                            'paid_status' => 'required',
                            'fund' => 'required',
                            'pay_method' => 'required',
                        ]);
                    } else {
                        $validator = Validator::make($request->all(), [
                            'amount' => 'required',
                            'date' => 'required',
                            'paid_status' => 'required',
                        ]);
                    }
                    if ($validator->passes()) {
                        try {

                            if (!Helper::Preferences("default_period")) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                            }

                            $Installment->period_id = Helper::Preferences("default_period");
                            $Installment->amount = $request->amount;
                            $Installment->date = date('Y-m-d', strtotime($request->date));
                            $Installment->date_type = Helper::Preferences("date_type");
                            $Installment->notes = $request->notes;
                            $Installment->paid_status = 0;
                            if ($request->paid_status) {
                                $Installment->paid_status = 1;
                                $Installment->paid_at = date('Y-m-d H:i:s');
                                $Installment->paid_by = Auth::user()->id;
                            }
                            $Installment->updated_ip = \Request::ip();
                            $Installment->updated_by = Auth::user()->id;
                            $Installment->save();

                            if ($Installment->paid_status) {
                                if (!empty($Installment->receipt)) {
                                    $Receipt = $Installment->receipt;
                                } else {
                                    $Receipt = new Receipt;
                                }
                                $Receipt->type = 1;
                                $Receipt->code = $request->code;
                                $Receipt->period_id = Helper::Preferences("default_period");
                                $Receipt->date = (Helper::Preferences("date_type")) ? Helper::DateView(date('Y-m-d'), 0, 1, 'Y-m-d') : date('Y-m-d');
                                $Receipt->date_type = Helper::Preferences("date_type");
                                $Receipt->amount_type = 1;
                                $Receipt->amount = $Installment->amount;
                                $Receipt->currency_id = Helper::Preferences("default_currency");
                                $Receipt->cost = $Installment->amount;
                                $Receipt->details = trans("backLang.installmentPay") . " " . @$Installment->loan->name . " - " . trans("backLang.loanNo") . " " . $Installment->loan_id;
                                $Receipt->ledger_id = Helper::Preferences("loans_ledger");
                                $Receipt->cat_id = $request->cat_id;
                                $Receipt->bank_id = $request->fund;
                                $Receipt->method_id = $request->pay_method;
                                $Receipt->repeated = false;
                                $Receipt->posted = Helper::Preferences("posting_method");
                                $Receipt->created_ip = \Request::ip();
                                $Receipt->created_by = Auth::user()->id;
                                $Receipt->save();

                                //connect receipt
                                $Installment->receipt_id = $Receipt->id;
                                $Installment->save();

                                // delete old journals
                                $Receipt->journals()->delete();

                                // save new to journal
                                $Journal = new Journal;
                                $Journal->type = 0;
                                $Journal->period_id = $Receipt->period_id;
                                $Journal->receipt_id = $Receipt->id;
                                $Journal->center_id = $Receipt->center_id;
                                $Journal->date = $Receipt->date;
                                $Journal->date_type = $Receipt->date_type;
                                $Journal->currency_id = $Receipt->currency_id;
                                $Journal->details = $Receipt->details;
                                $Journal->posted = Helper::Preferences("posting_method");
                                $Journal->created_by = Auth::user()->id;

                                // From Ledger:
                                $Journal->ledger_id = $Receipt->ledger_id;
                                $Journal->credit = $Receipt->amount;
                                $Journal->cost_credit = $Receipt->cost;
                                $Journal->dept = 0;
                                $Journal->cost_dept = 0;
                                $Journal->save();
                                // To Ledger:
                                $SecondJournal = $Journal->replicate();
                                $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                                $SecondJournal->credit = 0;
                                $SecondJournal->cost_credit = 0;
                                $SecondJournal->dept = $Receipt->amount;
                                $SecondJournal->cost_dept = $Receipt->cost;
                                $SecondJournal->save();

                            }

                            // Log to history
                            History::Log(
                                $module = 7,
                                $cat = 702,
                                $window = 0,
                                $record_id = $Installment->id,
                                $operation = 3,
                                $title = $Installment->amount . "<br>" . (date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M"))) . " " . $Installment->notes
                            );

                            return response()->json(['stat' => 'success', "id" => encrypt($Installment->id)]);
                        } catch (\Exception $e) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                        }
                    }
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function create(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("loans_add")) {

                $validator = Validator::make($request->all(), [
                    'amount' => 'required',
                    'date' => 'required',
                    'loan_id' => 'required',
                ]);
                if ($validator->passes()) {
                    try {
                        $Loan = Loan::find(decrypt($request->loan_id));
                        if (!empty($Loan)) {
                            if (!Helper::Preferences("default_period")) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                            }
                            $Installment = new Installment;
                            $Installment->loan_id = $Loan->id;
                            $Installment->person_id = $Loan->person_id;
                            $Installment->period_id = Helper::Preferences("default_period");
                            $Installment->amount = $request->amount;
                            $Installment->date = date('Y-m-d', strtotime($request->date));
                            $Installment->date_type = Helper::Preferences("date_type");
                            $Installment->notes = $request->notes;
                            $Installment->paid_status = 0;
                            $Installment->created_ip = \Request::ip();
                            $Installment->created_by = Auth::user()->id;
                            $Installment->save();

                            // Log to history
                            History::Log(
                                $module = 7,
                                $cat = 702,
                                $window = 0,
                                $record_id = $Installment->id,
                                $operation = 2,
                                $title = $Installment->amount . "<br>" . (date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M"))) . " " . $Installment->notes
                            );
                            return response()->json(['stat' => 'success', "id" => encrypt($Installment->id)]);
                        }
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function destroy($id = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                if (Roles::Check("loans_delete")) {
                    $Installment = Installment::find(decrypt($id));
                    if (!empty($Installment)) {
                        $Installment->receipt()->delete();
                        $Installment->delete();

                        // Log to history
                        History::Log(
                            $module = 7,
                            $cat = 702,
                            $window = 0,
                            $record_id = $Installment->id,
                            $operation = 4,
                            $title = $Installment->amount . "<br>" . date(Helper::Preferences("date_format"), strtotime($Installment->date)) . " " . ((@$Installment->date_type) ? trans("backLang.H") : trans("backLang.M"))
                        );

                        return json_encode(array("stat" => "success", "id" => $Installment->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
