<?php

namespace App\Http\Controllers\Accounting;

use App\Models\Bank;
use App\Models\Currency;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Receipt;
use App\Models\Period;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;
use App\Exports\JournalExport;
use App\Imports\JournalImport;
use Illuminate\Support\Facades\Storage;

class JournalController extends Controller
{
    private $uploadPath = "uploads/receipts/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        $Currencies = Currency::where("status", true)->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.journals.home", compact("Currencies", "UsersList", "Periods", "ledgers_array"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'date',
            2 => 'amount',
            3 => 'amount',
            4 => 'details',
            5 => 'posted',
            6 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_posted = $request->input('search_posted');
        $search_ledger = $request->input('search_ledger');

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_posted != "") {
            $Receipts = $Receipts->where("posted", $search_posted);
        }
        if ($search_ledger > 0) {
            // prepare ledgers ids
            function getSub($items, &$Ledgers_Ids)
            {
                while ($item = array_shift($items)) {
                    $Ledgers_Ids[] = $item['id'];
                    $SubLedgers = Ledger::where("father_id", $item['id'])->get();
                    $SubLedgers = $SubLedgers->toArray();
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }

            $Ledgers_Ids = [];
            $Ledger = Ledger::find($search_ledger);
            if (!empty($Ledger)) {
                $father_id = $Ledger->id;
                $Ledgers_Ids[] = $father_id;
                $SubLedgers = Ledger::where("father_id", $father_id)->get();
                $SubLedgers = $SubLedgers->toArray();
                if (count($SubLedgers) > 0) {
                    getSub($SubLedgers, $Ledgers_Ids);
                }
            }
            // End prepare ledgers ids

            $receipt_ids = Journal::select("receipt_id")->whereIn('ledger_id', $Ledgers_Ids);
            $Receipts = $Receipts->wherein("id", $receipt_ids);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $totalData = $Receipts->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Receipts = $Receipts->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Receipts as $Receipt) {

                // details prepare
                $dept = "";
                $credit = "";
                $details_from = "";
                $details_from_count = 0;
                $details_from_title = trans("backLang.fromAccount") . ": ";
                $details_to = "";
                $details_to_count = 0;
                $details_to_title = trans("backLang.toAccount") . ": ";
                foreach ($Receipt->journals->where("dept", ">", 0) as $Journal) {
                    $details_from_count++;
                    $details_from .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $dept .= number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                    $credit .= "<br>";
                }
                foreach ($Receipt->journals->where("credit", ">", 0) as $Journal) {
                    $details_to_count++;
                    $details_to .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $credit .= number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                }
                if ($details_from_count > 1) {
                    $details_from_title = trans("backLang.fromThoseMentioned") . ":<br>";
                    $dept = "<br>" . $dept;
                    $credit = "<br>" . $credit;
                }
                if ($details_to_count > 1) {
                    $credit = "<br>" . $credit;
                    $details_to_title = trans("backLang.toThoseMentioned") . ":<br>";
                }
                $details = "<span class='text-info'>" . $details_from_title . "</span>" . $details_from . "<div class='ml-3'>" . "<span class='text-info'>" . $details_to_title . "</span>" . $details_to . "</div><div> " . $Receipt->details . "</div>";


                $nestedData['id'] = "<div class='text-center'>" . $Receipt->id . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $options = "<div class=\"text-center\">";
                $nestedData['dept'] = "<div class='text-center'>" . $dept . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . $credit . "</div>";
                $nestedData['details'] = $details;
                $nestedData['posted'] = "<div class='text-center'>" . (($Receipt->posted) ? "<span>" . trans("backLang.posted") . "</span>" : "<span class='text-danger'>" . trans("backLang.notPosted") . "</span>") . "</div>";

                if (Roles::Check("accountant_print")) {
                    $options .= " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                if (Roles::Check("edit") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_edit")) {
                        $options .= " <a href=\"" . route("AppJournalsEdit", encrypt($Receipt->id)) . "\" class=\"btn btn-sm btn-info\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></a>";
                    }
                }
                if (Roles::Check("delete") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_delete")) {
                        $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function create()
    {
        $Currencies = Currency::where("status", true)->get();
        $Ledgers = Ledger::orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        return view("back.accounting.journals.create", compact("Currencies", "ledgers_array"));
    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_add")) {
                $validator = Validator::make($request->all(), [
                    'currency' => 'required',
                    'date' => 'required',
                    'details' => 'required'
                ]);
                if ($validator->passes()) {
                    try {

                        if (!Helper::Preferences("default_period")) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }

                        $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                        if (!@$CurrentPeriod->status) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }

                        // Start File Upload
                        $formFileName = "file";
                        $fileFinalRand = "";
                        $fileFinalExt = "";
                        $uploaded = false;
                        $path = "";
                        if ($request->$formFileName != "") {
                            $path = $this->uploadPath;
                            $fileFinalRand = time() . rand(1111, 9999);
                            $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                            $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);
                        }

                        $Receipt = new Receipt;
                        $Receipt->type = 2;
                        $Receipt->code = $request->code;
                        $Receipt->period_id = Helper::Preferences("default_period");
                        $Receipt->date = date('Y-m-d', strtotime($request->date));
                        $Receipt->date_type = Helper::Preferences("date_type");
                        $Receipt->amount_type = 0;
                        $Receipt->amount = 0;
                        $Receipt->currency_id = $request->currency;
                        $Receipt->cost = 0;
                        $Receipt->details = $request->details;
                        $Receipt->cat_id = 0;
                        $Receipt->bank_id = 0;
                        $Receipt->method_id = 0;
                        $Receipt->center_id = $request->center_id;

                        if ($uploaded) {
                            $Receipt->file = $fileFinalRand;
                            $Receipt->ext = $fileFinalExt;
                            $Receipt->remote = 0;
                        }
                        $Receipt->repeated = false;
                        $Receipt->posted = Helper::Preferences("posting_method");
                        $Receipt->created_ip = \Request::ip();
                        $Receipt->created_by = Auth::user()->id;
                        $Receipt->save();

                        // delete old journals
                        $Receipt->journals()->delete();

                        $total_amount = 0;
                        if ($request->rows_max > 0) {
                            for ($i = 1; $i <= $request->rows_max; $i++) {
                                if ($request->{'ledger_' . $i} > 0 && ($request->{'debtor_' . $i} > 0 || $request->{'creditor_' . $i} > 0)) {
                                    $Journal = new Journal;
                                    $Journal->type = 0;
                                    $Journal->period_id = Helper::Preferences("default_period");
                                    $Journal->receipt_id = $Receipt->id;
                                    $Journal->ledger_id = $request->{'ledger_' . $i};
                                    $Journal->center_id = $Receipt->center_id;
                                    $Journal->date = date('Y-m-d', strtotime($request->date));
                                    $Journal->date_type = Helper::Preferences("date_type");
                                    $Journal->dept = $request->{'debtor_' . $i};
                                    $Journal->credit = $request->{'creditor_' . $i};
                                    $Journal->currency_id = $Receipt->currency_id;
                                    $Journal->cost_dept = Helper::CurrencyTransfer($request->{'debtor_' . $i}, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                    $Journal->cost_credit = Helper::CurrencyTransfer($request->{'creditor_' . $i}, $Receipt->currency_id, Helper::Preferences("default_currency"));

                                    $Journal->details = $request->{'details_' . $i};
                                    $Journal->posted = Helper::Preferences("posting_method");
                                    $Journal->created_by = Auth::user()->id;
                                    $Journal->save();

                                    $total_amount += $Journal->dept;
                                }
                            }
                        }

                        $Receipt->amount = $total_amount;
                        $Receipt->cost = Helper::CurrencyTransfer($total_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                        $Receipt->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 604,
                            $window = 0,
                            $record_id = $Receipt->id,
                            $operation = 2,
                            $title = $Receipt->amount . " " . @$Receipt->currency->symbol . "<br>" . mb_substr($Receipt->details, 0, 150)
                        );

                        //Remote storage
                        if ($uploaded) {
                            if (Helper::FileSystem() != "") {
                                $filepath = $path . $fileFinalRand;
                                $localFile = File::get($filepath);
                                $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                                if ($Remote_store) {
                                    $Receipt->remote = "1";
                                    $Receipt->save();
                                    // Delete from local
                                    File::delete($filepath);
                                }
                            }
                        }

                        return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Receipt = Receipt::find(decrypt($id));
                    if (!empty($Receipt)) {
                        $Currencies = Currency::where("status", true)->get();
                        $Ledgers = Ledger::orderby("code", "asc")->get();
                        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                        return view("back.accounting.journals.edit", compact("Receipt", "Currencies", "ledgers_array"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                $Receipt = Receipt::find(decrypt($request->id));
                if (!empty($Receipt)) {
                    $validator = Validator::make($request->all(), [
                        'currency' => 'required',
                        'date' => 'required',
                        'details' => 'required'
                    ]);
                    if ($validator->passes()) {
                        try {
                            $CurrentPeriod = Period::find($Receipt->period_id);
                            if (!@$CurrentPeriod->status) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youCanNotUpdateIfPeriodClosed')]]);
                            }

                            // delete old photo
                            if ($request->file_delete == 1 || $request->file != "") {
                                if ($Receipt->file != "") {
                                    if ($Receipt->remote && Helper::FileSystem() != "") {
                                        Storage::disk(Helper::FileSystem())->delete($Receipt->file);
                                    } else {
                                        if (file_exists($this->uploadPath . $Receipt->file)) {
                                            File::delete($this->uploadPath . $Receipt->file);
                                        }
                                    }
                                }

                                $Receipt->file = "";
                                $Receipt->ext = "";
                                $Receipt->remote = 0;
                            }

                            // Start File Upload
                            $formFileName = "file";
                            $fileFinalRand = "";
                            $fileFinalExt = "";
                            $uploaded = false;
                            $path = "";
                            if ($request->$formFileName != "") {
                                $path = $this->uploadPath;
                                $fileFinalRand = time() . rand(1111, 9999);
                                $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                                $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);
                            }

                            $Receipt->code = $request->code;
                            $Receipt->date = date('Y-m-d', strtotime($request->date));
                            $Receipt->date_type = Helper::Preferences("date_type");
                            $Receipt->currency_id = $request->currency;
                            $Receipt->details = $request->details;
                            $Receipt->center_id = $request->center_id;

                            if ($uploaded) {
                                $Receipt->file = $fileFinalRand;
                                $Receipt->ext = $fileFinalExt;
                                $Receipt->remote = 0;
                            }
                            $Receipt->posted = Helper::Preferences("posting_method");
                            $Receipt->repeated = false;
                            $Receipt->created_ip = \Request::ip();
                            $Receipt->created_by = Auth::user()->id;
                            $Receipt->save();


                            // delete old journals
                            $Receipt->journals()->delete();

                            $total_amount = 0;
                            if ($request->rows_max > 0) {
                                for ($i = 1; $i <= $request->rows_max; $i++) {
                                    if ($request->{'ledger_' . $i} > 0 && ($request->{'debtor_' . $i} > 0 || $request->{'creditor_' . $i} > 0)) {
                                        $Journal = new Journal;
                                        $Journal->type = 0;
                                        $Journal->period_id = Helper::Preferences("default_period");
                                        $Journal->receipt_id = $Receipt->id;
                                        $Journal->ledger_id = $request->{'ledger_' . $i};
                                        $Journal->center_id = $Receipt->center_id;
                                        $Journal->date = date('Y-m-d', strtotime($request->date));
                                        $Journal->date_type = Helper::Preferences("date_type");
                                        $Journal->dept = $request->{'debtor_' . $i};
                                        $Journal->credit = $request->{'creditor_' . $i};
                                        $Journal->currency_id = $Receipt->currency_id;
                                        $Journal->cost_dept = Helper::CurrencyTransfer($request->{'debtor_' . $i}, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                        $Journal->cost_credit = Helper::CurrencyTransfer($request->{'creditor_' . $i}, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                        $Journal->details = $request->{'details_' . $i};
                                        $Journal->posted = Helper::Preferences("posting_method");
                                        $Journal->created_by = Auth::user()->id;
                                        $Journal->save();

                                        $total_amount += $Journal->dept;
                                    }
                                }
                            }


                            $Receipt->amount = $total_amount;
                            $Receipt->cost = Helper::CurrencyTransfer($total_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                            $Receipt->save();

                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = 604,
                                $window = 0,
                                $record_id = $Receipt->id,
                                $operation = 3,
                                $title = mb_substr($Receipt->details, 0, 150)
                            );

                            //Remote storage
                            if ($uploaded) {
                                if (Helper::FileSystem() != "") {
                                    $filepath = $path . $fileFinalRand;
                                    $localFile = File::get($filepath);
                                    $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                                    if ($Remote_store) {
                                        $Receipt->remote = "1";
                                        $Receipt->save();
                                        // Delete from local
                                        File::delete($filepath);
                                    }
                                }
                            }

                            return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                        } catch (\Exception $e) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                        }
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_currency = $request->input('search_currency');
            $search_posted = $request->input('search_posted');
            $search_ledger = $request->input('search_ledger');

            $Currency = [];
            $Period = [];
            $Ledger = [];

            if (Roles::Check("view")) {
                $Receipts = Receipt::where("created_by", @Auth::user()->id);
            } else {
                $Receipts = Receipt::where("id", ">", 0);
            }
            if ($search_word != "") {
                $Receipts = $Receipts->where(function ($query) use ($search_word) {
                    $query->where("id", $search_word)
                        ->orwhere("details", "like", "%" . $search_word . "%")
                        ->orwhere("name", "like", "%" . $search_word . "%")
                        ->orwhere('code', $search_word);
                });
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Receipts = $Receipts->where("currency_id", $search_currency);
            }
            if ($search_posted != "") {
                $Receipts = $Receipts->where("posted", $search_posted);
            }
            if ($search_ledger > 0) {
                $Ledger = Ledger::find($search_ledger);
                $receipt_ids = Journal::select("receipt_id")->where('ledger_id', $search_ledger);
                $Receipts = $Receipts->wherein("id", $receipt_ids);
            }
            if ($search_center_id > 0) {
                $Receipts = $Receipts->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Receipts = $Receipts->where("created_by", $search_user);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Receipts = $Receipts->where("period_id", $search_period);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Receipts = $Receipts->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Receipts = $Receipts->where('date', "<=", $search_end_date);
            }

            $Receipts = $Receipts->orderBy("date", "asc")->get();


            return view("back.accounting.journals.print", compact("Receipts", "request", "Currency", "Period", "Ledger"));
        }
        return "";
    }

    public function export(Request $request)
    {
        if (Roles::Check("accountant_export")) {
            return (new JournalExport($request))->download(trans('backLang.dailyEntries') . '.xlsx');
        }
        return "";
    }

    public function receipt($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Receipt = Receipt::find(decrypt($id));
                    if (!empty($Receipt)) {
                        return view("back.accounting.journals.receipt", compact("Receipt"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }


    public function ledgers()
    {
        $Ledgers = Ledger::where("father_id", 0)->orderby("code", "asc")->get();
        $Balances = [];
        $AllLedgers = Ledger::get();
        if (count($AllLedgers) > 0) {
            $search_period = Helper::Preferences("default_period");
            foreach ($AllLedgers as $Ledger) {
                $sum_of_dept = Journal::where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('ledger_id', $Ledger->id)->where("period_id", $search_period);

                $sum_of_dept = $sum_of_dept->sum("cost_dept");
                $sum_of_credit = $sum_of_credit->sum("cost_credit");

                $Balances[$Ledger->id] = [
                    "dept" => $sum_of_dept,
                    "credit" => $sum_of_credit,
                    "balance" => round($sum_of_dept - $sum_of_credit, 2)
                ];
            }
        }
        return view("back.accounting.journals.ledgers", compact("Ledgers","Balances"));
    }
}
