<?php

namespace App\Http\Controllers\Accounting;

use App\Models\Currency;
use App\Models\Journal;
use App\Models\Period;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Ledger;
use Auth;
use Helper;
use File;
use Illuminate\Support\Facades\Storage;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;
use App\Exports\LedgersExport;
use App\Imports\LedgersImport;


class LedgerController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("accountant_tree", true);
    }

    public function index()
    {
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        // prepare default groups
        if (count($Ledgers) == 0) {
            foreach (trans('backLang.ledgerCats') as $key => $ledgerCat) {
                $Ledger = new Ledger;
                $Ledger->code = $key;
                $Ledger->father_id = 0;
                $Ledger->title_ar = trans('backLang.ledgerCats.' . $key, [], "ar");
                $Ledger->title_en = trans('backLang.ledgerCats.' . $key, [], "en");
                $Ledger->group_id = $key;
                $Ledger->type = ($key == 1 || $key == 5) ? 2 : 1;
                $Ledger->group = 1;
                $Ledger->status = true;
                $Ledger->created_ip = \Request::ip();
                $Ledger->created_by = Auth::user()->id;
                $Ledger->save();
            }
        }
        return view("back.accounting.ledgers.home");
    }

    public function list()
    {
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();

        if (count($Ledgers) > 0) {
            $search_period = Helper::Preferences("default_period");
            $Ledgers = $Ledgers->map(function ($Ledger) use ($search_period) {
                $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                $sum_of_dept = $sum_of_dept->sum("cost_dept");
                $sum_of_credit = $sum_of_credit->sum("cost_credit");

                $Ledger->dept = $sum_of_dept;
                $Ledger->credit = $sum_of_credit;
                return $Ledger;
            });
        }

        return view("back.accounting.ledgers.list", compact("Ledgers"));
    }

    public function create()
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_add")) {
                $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                $Ledgers->map(function ($Ledger) {
                    $LastLedger = Ledger::where("father_id", $Ledger->id)->orderby("code", "desc")->first();
                    if (!empty($LastLedger)) {
                        try {
                            $Ledger->new_code = ($LastLedger->code + 1);
                        }catch (\Exception $e){

                        }
                    } else {
                        $Ledger->new_code = $Ledger->code . "1";
                    }
                    return $Ledger;
                });

                $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
                $Currencies = Currency::where("status", true)->get();
                return view("back.accounting.ledgers.create", compact("ledgers_array", "Currencies"));
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_edit")) {
                $validator = Validator::make($request->all(), [
                    'code' => 'required',
                    'group' => 'required',
                    'father_id' => 'required'
                ]);
                if ($validator->passes()) {
                    try {
                        $CodeCheck = Ledger::where("code", $request->code)->first();
                        if (empty($CodeCheck)) {
                            $Father = Ledger::find($request->father_id);
                            if (!empty($Father)) {
                                $Ledger = new Ledger;
                                $Ledger->code = $request->code;
                                $Ledger->father_id = $Father->id;
                                $Ledger->title_ar = $request->title_ar;
                                $Ledger->title_en = $request->title_en;
                                $Ledger->group_id = $Father->group_id;
                                $Ledger->type = $Father->type;
                                $Ledger->group = $request->group;
                                $Ledger->status = true;
                                $Ledger->created_ip = \Request::ip();
                                $Ledger->created_by = Auth::user()->id;
                                $Ledger->save();

                                // Log to history
                                History::Log(
                                    $module = 6,
                                    $cat = 601,
                                    $window = 0,
                                    $record_id = $Ledger->id,
                                    $operation = 2,
                                    $title = $Ledger->title_ar . "<br>" . $Ledger->title_en
                                );

                                if ($request->amount > 0 && $request->currency > 0) {
                                    $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                                    if (@$CurrentPeriod->status) {
                                        $Journal = new Journal;
                                        $Journal->type = 1;
                                        $Journal->period_id = Helper::Preferences("default_period");
                                        $Journal->receipt_id = 0;
                                        $Journal->ledger_id = $Ledger->id;
                                        $Journal->center_id = 0;
                                        $Journal->date = date('Y-m-d');
                                        $Journal->date_type = 0;
                                        if ($request->amount_type) {
                                            $Journal->credit = $request->amount;
                                            $Journal->cost_credit = Helper::CurrencyTransfer($request->amount, $request->currency, Helper::Preferences("default_currency"));
                                        } else {
                                            $Journal->dept = $request->amount;
                                            $Journal->cost_dept = Helper::CurrencyTransfer($request->amount, $request->currency, Helper::Preferences("default_currency"));
                                        }
                                        $Journal->currency_id = $request->currency;

                                        $Journal->details = $request->details;
                                        $Journal->posted = true;
                                        $Journal->created_by = Auth::user()->id;
                                        $Journal->save();

                                        // Log to history
                                        History::Log(
                                            $module = 6,
                                            $cat = 605,
                                            $window = 0,
                                            $record_id = $Journal->id,
                                            $operation = 2,
                                            $title = (($Journal->dept > 0) ? $Journal->dept : $Journal->credit) . " " . @$Journal->currency->symbol . "<br>" . @$Journal->ledger->code . " - " . @$Journal->ledger->{"title_" . trans("backLang.code")} . " : " . mb_substr($Journal->details, 0, 150)
                                        );
                                    }
                                }

                                return response()->json(['stat' => 'success', "id" => encrypt($Ledger->id)]);
                            } else {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                            }
                        } else {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.thisCodeIsExist')]]);
                        }
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Ledgers = Ledger::where("id", "!=", decrypt($id))->orderby("code", "asc")->get();
                    $Ledgers->map(function ($CLedger) {
                        $LastLedger = Ledger::where("father_id", $CLedger->id)->orderby("code", "desc")->first();
                        if (!empty($LastLedger)) {
                            try {
                                $CLedger->new_code = ($LastLedger->code + 1);
                            }catch (\Exception $e){

                            }
                        } else {
                            $CLedger->new_code = $CLedger->code . "1";
                        }
                        return $CLedger;
                    });

                    $Ledger = Ledger::find(decrypt($id));
                    if (!empty($Ledger)) {
                        $ledgers_array = Helper::createItemArray($Ledgers, $Ledger->father_id, 0);
                        return view("back.accounting.ledgers.edit", compact("Ledger", "ledgers_array"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                $validator = Validator::make($request->all(), [
                    'code' => 'required',
                    'group' => 'required'
                ]);
                if ($validator->passes()) {
                    try {
                        $CodeCheck = Ledger::where("code", $request->code)->where('id', "!=", decrypt($request->id))->first();
                        if (empty($CodeCheck)) {
                            $Father = [];
                            if($request->father_id > 0) {
                                $Father = Ledger::find($request->father_id);
                            }
                            $Ledger = Ledger::find(decrypt($request->id));
                            if (!empty($Ledger)) {
                                $Ledger->code = $request->code;
                                if (!empty($Father)) {
                                    $Ledger->father_id = $Father->id;
                                    $Ledger->group_id = $Father->group_id;
                                    $Ledger->type = $Father->type;
                                } else {
                                    $Ledger->father_id = 0;
                                }
                                $Ledger->title_ar = $request->title_ar;
                                $Ledger->title_en = $request->title_en;
                                $Ledger->group = $request->group;
                                $Ledger->updated_ip = \Request::ip();
                                $Ledger->updated_by = Auth::user()->id;
                                $Ledger->save();

                                // Log to history
                                History::Log(
                                    $module = 6,
                                    $cat = 601,
                                    $window = 0,
                                    $record_id = $Ledger->id,
                                    $operation = 3,
                                    $title = $Ledger->title_ar . "<br>" . $Ledger->title_en
                                );

                                return response()->json(['stat' => 'success', "id" => encrypt($Ledger->id)]);
                            } else {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                            }
                        } else {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.thisCodeIsExist')]]);
                        }
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                if (Roles::Check("accountant_delete")) {
                    $Ledger = Ledger::find(decrypt($id));
                    if (!empty($Ledger)) {

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 601,
                            $window = 0,
                            $record_id = $Ledger->id,
                            $operation = 4,
                            $title = $Ledger->title_ar . "<br>" . $Ledger->title_en
                        );

                        $Ledger->delete();
                        return json_encode(array("stat" => "success", "id" => $Ledger->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function print()
    {
        if (Roles::Check("accountant_print")) {
            $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();

            if (count($Ledgers) > 0) {
                $search_period = Helper::Preferences("default_period");
                $Ledgers = $Ledgers->map(function ($Ledger) use ($search_period) {
                    $sum_of_dept = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                    $sum_of_credit = Journal::where('posted', true)->where('ledger_id', $Ledger->id)->where("period_id", $search_period);
                    $sum_of_dept = $sum_of_dept->sum("cost_dept");
                    $sum_of_credit = $sum_of_credit->sum("cost_credit");

                    $Ledger->dept = $sum_of_dept;
                    $Ledger->credit = $sum_of_credit;
                    return $Ledger;
                });
            }
            return view("back.accounting.ledgers.print", compact("Ledgers"));
        }
    }

    public function import(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_import")) {
                if ($request->file != "") {

                    //delete old data
                    Ledger::where("id", ">", 0)->delete();

                    Excel::import(new LedgersImport(), $request->file('file'));
                    return response()->json(['stat' => 'success']);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function export()
    {
        if (Roles::Check("accountant_export")) {
            return (new LedgersExport([]))->download(trans('backLang.accountantTree') . '.xlsx');
        }
    }

    public function download()
    {
        return Storage::disk('local')->download("xlsx/ledgers.xlsx", trans('backLang.accountantTree') . ".xlsx");
    }
}
