<?php

namespace App\Http\Controllers\Accounting;

use App;
use App\Models\Bank;
use App\Models\Category;
use App\Models\Currency;
use App\Exports\LoansExport;
use App\Helpers\History;
use App\Helpers\AppSender;
use App\Http\Controllers\Controller;
use App\Models\Installment;
use App\Models\Journal;
use App\Models\Period;
use App\Models\Attachment;
use App\Models\Loan;
use App\Models\Receipt;
use App\Models\User;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use Validator;


class LoanController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("role_loans", true);
    }

    public function index()
    {
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        return view("back.accounting.loans.list", compact("UsersList"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'date',
            2 => 'name',
            3 => 'cat_id',
            4 => 'amount',
            5 => 'paid',
            6 => 'unpaid',
            7 => 'status',
            8 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_cat_id = $request->input('search_cat_id');
        $search_status = $request->input('search_status');

        if (Roles::Check("view")) {
            $Loans = Loan::where("created_by", @Auth::user()->id);
        } else {
            $Loans = Loan::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Loans = $Loans->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere("phone", $search_word)
                    ->orwhere("email", $search_word)
                    ->orwhere("id_number", $search_word)
                    ->orwhere("notes", "like", "%" . $search_word . "%")
                    ->orwhere("guarantor_name", "like", "%" . $search_word . "%")
                    ->orwhere("witness_1_name", "like", "%" . $search_word . "%")
                    ->orwhere("witness_2_name", "like", "%" . $search_word . "%");
            });
        }
        if ($search_cat_id > 0) {
            $Loans = $Loans->where("cat_id", $search_cat_id);
        }
        if ($search_user > 0) {
            $Loans = $Loans->where("created_by", $search_user);
        }
        if ($search_status != "") {
            $Loans = $Loans->where("status", $search_status);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Loans = $Loans->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Loans = $Loans->where('date', "<=", $search_end_date);
        }

        $totalData = $Loans->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Loans = $Loans->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        $loans_total = 0;
        $loans_paid = 0;
        if ($totalFiltered > 0) {
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            foreach ($Loans as $Loan) {

                $paid_amount = $Loan->installments()->where("paid_status", true)->sum("amount");
                if ($Loan->status == 1) {
                    $loans_total += $Loan->total;
                    $loans_paid += $paid_amount;
                }
                $nestedData['id'] = "<div class='text-center'>" . $Loan->id . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Loan->date)) . " " . ((@$Loan->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['name'] = "<div>" . $Loan->name . "</div>";
                $nestedData['cat_id'] = "<div class='text-center'>" . Helper::Category($Loan->cat_id) . "</div>";
                $nestedData['amount'] = "<div class='text-center'>" . number_format($Loan->total, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol . "</div>";
                $nestedData['paid'] = "<div class='text-center'>" . (($paid_amount > 0) ? number_format($paid_amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol : "-") . "</div>";
                 $unpaid = $Loan->total - $paid_amount;
                $nestedData['unpaid'] = "<div class='text-center'>" . number_format($unpaid, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol . "</div>";

                if ($Loan->status == 1) {
                    $loan_status_title = "<span class=\"text-success\"><i class=\"fe fe-check-circle\"></i> " . trans('backLang.approved') . "</span>";
                } elseif ($Loan->status == -1) {
                    $loan_status_title = "<span class=\"text-danger\"><i class=\"fe fe-x-circle\"></i> " . trans('backLang.notApproved') . "</span>";
                } else {
                    $loan_status_title = "<span class=\"text-info\"><i class=\"fe fe-clock\"></i> " . trans('backLang.pending') . "</span>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $loan_status_title . "</div>";


                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
 <a class=\"btn btn-sm btn-info\" href=\"" . route("LoansView", encrypt($Loan->id)) . "\"><i class=\"fe fe-file-text\"></i> " . trans("backLang.details") . "</a>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $unpaid_balance = $loans_total - $loans_paid;
        $TBalance = [
            "total" => number_format($loans_total, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol,
            "paid" => number_format($loans_paid, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol,
            "unpaid" => number_format($unpaid_balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol
        ];
        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "balance" => $TBalance
        );

        return json_encode($json_data);

    }

    public function create($id = "")
    {
        $Person = [];
        $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
        return view("back.accounting.loans.create", compact("DefaultCurrency", "CurrentPeriod", "Person"));
    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("loans_add")) {
                $df_end_date_type = Helper::Preferences("default_" . $request->id_type . "_end_date_type");
                $var_id_end_date = "id_end_date";
                if ($df_end_date_type) {
                    $var_id_end_date = "id_end_date_hj";
                }
                $df_guarantor_date_type = Helper::Preferences("default_" . $request->guarantor_id_type . "_end_date_type");
                $var_guarantor_date = "guarantor_id_end_date";
                if ($df_guarantor_date_type) {
                    $var_guarantor_date = "guarantor_id_end_date_hj";
                }
                $df_witness_1_date_type = Helper::Preferences("default_" . $request->witness_1_id_type . "_end_date_type");
                $var_witness_1_date = "witness_1_id_end_date";
                if ($df_witness_1_date_type) {
                    $var_witness_1_date = "witness_1_id_end_date_hj";
                }
                $df_witness_2_date_type = Helper::Preferences("default_" . $request->witness_2_id_type . "_end_date_type");
                $var_witness_2_date = "witness_2_id_end_date";
                if ($df_witness_2_date_type) {
                    $var_witness_2_date = "witness_2_id_end_date_hj";
                }

                $validator = Validator::make($request->all(), [
                    'name' => 'required',
                    'phone' => 'required',
                    'id_type' => 'required',
                    'id_number' => 'required',
                    'date' => 'required',
                    'amount' => 'required',
                    'total' => 'required',
                    'instalments_amount' => 'required',
                    'instalments_count' => 'required',
                    'pay_method' => 'required',
                    'instalments_date' => 'required',
                ]);
                if ($validator->passes()) {
                    try {
                        if (!Helper::Preferences("default_period")) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }
                        if ($request->id_number == $request->guarantor_id_number) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.sponsorError')]]);
                        }
                        if ((($request->id_number == $request->witness_1_id_number || $request->witness_1_id_number == $request->guarantor_id_number) && $request->witness_1_id_number != "") || (($request->id_number == $request->witness_2_id_number || $request->witness_2_id_number == $request->guarantor_id_number) && $request->witness_2_id_number != "")) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.witnessError')]]);
                        }
                        if ($request->witness_2_id_number == $request->witness_1_id_number && $request->witness_1_id_number != "") {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.witness2Error')]]);
                        }

                        $Loan = new Loan;
                        $Loan->cat_id = $request->cat_id;
                        $Loan->period_id = Helper::Preferences("default_period");
                        if ($request->person_id != "") {
                            $Loan->person_id = decrypt($request->person_id);
                        }
                        $Loan->id_number = $request->id_number;
                        $Loan->id_type = $request->id_type;
                        $Loan->id_date_type = ($df_end_date_type) ? 1 : 0;
                        if ($request->$var_id_end_date != "") {
                            $Loan->id_end_date = date('Y-m-d', strtotime($request->$var_id_end_date));
                        }
                        $Loan->name = $request->name;
                        $Loan->phone = $request->phone;
                        $Loan->email = $request->email;
                        $Loan->date = date('Y-m-d', strtotime($request->date));
                        $Loan->date_type = Helper::Preferences("date_type");
                        $Loan->amount = $request->amount;
                        $Loan->total = $request->total;
                        $Loan->instalments_count = $request->instalments_count;
                        $Loan->instalments_amount = $request->instalments_amount;
                        $Loan->instalments_last = $request->instalments_last;
                        $Loan->instalments_date = date('Y-m-d', strtotime($request->instalments_date));

                        $Loan->guarantor_name = $request->guarantor_name;
                        $Loan->guarantor_phone = $request->guarantor_phone;
                        $Loan->guarantor_id_number = $request->guarantor_id_number;
                        $Loan->guarantor_id_type = $request->guarantor_id_type;
                        $Loan->guarantor_id_date_type = ($df_guarantor_date_type) ? 1 : 0;
                        if ($request->$var_guarantor_date != "") {
                            $Loan->guarantor_id_end_date = date('Y-m-d', strtotime($request->$var_guarantor_date));
                        }
                        $Loan->guarantor_work_place = $request->guarantor_work_place;
                        $Loan->guarantor_work_phone = $request->guarantor_work_phone;

                        $Loan->witness_1_name = $request->witness_1_name;
                        $Loan->witness_1_phone = $request->witness_1_phone;
                        $Loan->witness_1_id_number = $request->witness_1_id_number;
                        $Loan->witness_1_id_type = $request->witness_1_id_type;
                        $Loan->witness_1_id_date_type = ($df_witness_1_date_type) ? 1 : 0;
                        if ($request->$var_witness_1_date != "") {
                            $Loan->witness_1_id_end_date = date('Y-m-d', strtotime($request->$var_witness_1_date));
                        }

                        $Loan->witness_2_name = $request->witness_2_name;
                        $Loan->witness_2_phone = $request->witness_2_phone;
                        $Loan->witness_2_id_number = $request->witness_2_id_number;
                        $Loan->witness_2_id_type = $request->witness_2_id_type;
                        $Loan->witness_2_id_date_type = ($df_witness_2_date_type) ? 1 : 0;
                        if ($request->$var_witness_2_date != "") {
                            $Loan->witness_2_id_end_date = date('Y-m-d', strtotime($request->$var_witness_2_date));
                        }

                        $Loan->pay_method = $request->pay_method;
                        $Loan->notes = $request->notes;
                        $Loan->receipt_id = 0;
                        $Loan->status = (Roles::Check("loan_approve")) ? true : false;
                        $Loan->created_ip = \Request::ip();
                        $Loan->created_by = Auth::user()->id;
                        $Loan->save();

                        // add installments
                        if ($Loan->instalments_count > 0) {
                            $_date = date('Y-m-d', strtotime($Loan->instalments_date));
                            for ($i = 1; $i <= $Loan->instalments_count; $i++) {
                                if ($i > 1) {
                                    if ($Loan->pay_method > 0) {
                                        $_date = date('Y-m-d', strtotime("+" . $Loan->pay_method . " month", strtotime($_date)));
                                    } else {
                                        $_date = date('Y-m-d', strtotime("+12 month", strtotime($_date)));
                                    }
                                }
                                $Installment = new Installment;
                                $Installment->loan_id = $Loan->id;
                                $Installment->person_id = $Loan->person_id;
                                $Installment->receipt_id = 0;
                                $Installment->period_id = 0;
                                if ($i == $Loan->instalments_count) {
                                    $Installment->amount = $Loan->instalments_last;
                                } else {
                                    $Installment->amount = $Loan->instalments_amount;
                                }
                                $Installment->date = $_date;
                                $Installment->date_type = Helper::Preferences("date_type");
                                $Installment->pay_method = $Loan->pay_method;
                                $Installment->paid_status = 0;
                                $Installment->created_ip = \Request::ip();
                                $Installment->created_by = Auth::user()->id;
                                $Installment->save();
                            }
                        }

                        // Log to history
                        History::Log(
                            $module = 7,
                            $cat = 701,
                            $window = 0,
                            $record_id = $Loan->id,
                            $operation = 2,
                            $title = $Loan->amount . "<br>" . @$Loan->name
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Loan->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
                return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function view($id, $pay = 0)
    {
        $Loan = Loan::find(decrypt($id));
        if (!empty($Loan)) {
            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
            return view("back.accounting.loans.view.file", compact("Loan", "DefaultCurrency", "Funds", "pay"));
        }
        return redirect()->route("Loans");
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loans_edit")) {
                if ($id != "") {
                    $Loan = Loan::find(decrypt($id));
                    if (!empty($Loan)) {
                        $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
                        return view("back.accounting.loans.edit.details", compact("Loan", "DefaultCurrency", "CurrentPeriod"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loans_edit")) {
                $Loan = Loan::find(decrypt($request->id));
                if (!empty($Loan)) {
                    $OldLoan = $Loan;
                    $df_end_date_type = Helper::Preferences("default_" . $request->id_type . "_end_date_type");
                    $var_id_end_date = "id_end_date";
                    if ($df_end_date_type) {
                        $var_id_end_date = "id_end_date_hj";
                    }
                    $df_guarantor_date_type = Helper::Preferences("default_" . $request->guarantor_id_type . "_end_date_type");
                    $var_guarantor_date = "guarantor_id_end_date";
                    if ($df_guarantor_date_type) {
                        $var_guarantor_date = "guarantor_id_end_date_hj";
                    }
                    $df_witness_1_date_type = Helper::Preferences("default_" . $request->witness_1_id_type . "_end_date_type");
                    $var_witness_1_date = "witness_1_id_end_date";
                    if ($df_witness_1_date_type) {
                        $var_witness_1_date = "witness_1_id_end_date_hj";
                    }
                    $df_witness_2_date_type = Helper::Preferences("default_" . $request->witness_2_id_type . "_end_date_type");
                    $var_witness_2_date = "witness_2_id_end_date";
                    if ($df_witness_2_date_type) {
                        $var_witness_2_date = "witness_2_id_end_date_hj";
                    }

                    $validator = Validator::make($request->all(), [
                        'name' => 'required',
                        'phone' => 'required',
                        'id_type' => 'required',
                        'id_number' => 'required',
                        'date' => 'required',
                        'amount' => 'required',
                        'total' => 'required',
                        'instalments_amount' => 'required',
                        'instalments_count' => 'required',
                        'pay_method' => 'required',
                        'instalments_date' => 'required',
                    ]);
                    if ($validator->passes()) {
                        try {
                            if (!Helper::Preferences("default_period")) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                            }

                            if ($request->id_number == $request->guarantor_id_number) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.sponsorError')]]);
                            }
                            if ((($request->id_number == $request->witness_1_id_number || $request->witness_1_id_number == $request->guarantor_id_number) && $request->witness_1_id_number != "") || (($request->id_number == $request->witness_2_id_number || $request->witness_2_id_number == $request->guarantor_id_number) && $request->witness_2_id_number != "")) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.witnessError')]]);
                            }
                            if ($request->witness_2_id_number == $request->witness_1_id_number && $request->witness_1_id_number != "") {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.witness2Error')]]);
                            }

                            $Loan->cat_id = $request->cat_id;
                            $Loan->period_id = Helper::Preferences("default_period");
                            if ($request->person_id != "") {
                                $Loan->person_id = decrypt($request->person_id);
                            }
                            $Loan->id_number = $request->id_number;
                            $Loan->id_type = $request->id_type;
                            $Loan->id_date_type = ($df_end_date_type) ? 1 : 0;
                            if ($request->$var_id_end_date != "") {
                                $Loan->id_end_date = date('Y-m-d', strtotime($request->$var_id_end_date));
                            }
                            $Loan->name = $request->name;
                            $Loan->phone = $request->phone;
                            $Loan->email = $request->email;
                            $Loan->date = date('Y-m-d', strtotime($request->date));
                            $Loan->date_type = Helper::Preferences("date_type");
                            $Loan->amount = $request->amount;
                            $Loan->total = $request->total;
                            $Loan->instalments_count = $request->instalments_count;
                            $Loan->instalments_amount = $request->instalments_amount;
                            $Loan->instalments_last = $request->instalments_last;
                            $Loan->instalments_date = date('Y-m-d', strtotime($request->instalments_date));

                            $Loan->guarantor_name = $request->guarantor_name;
                            $Loan->guarantor_phone = $request->guarantor_phone;
                            $Loan->guarantor_id_number = $request->guarantor_id_number;
                            $Loan->guarantor_id_type = $request->guarantor_id_type;
                            $Loan->guarantor_id_date_type = ($df_guarantor_date_type) ? 1 : 0;
                            if ($request->$var_guarantor_date != "") {
                                $Loan->guarantor_id_end_date = date('Y-m-d', strtotime($request->$var_guarantor_date));
                            }
                            $Loan->guarantor_work_place = $request->guarantor_work_place;
                            $Loan->guarantor_work_phone = $request->guarantor_work_phone;

                            $Loan->witness_1_name = $request->witness_1_name;
                            $Loan->witness_1_phone = $request->witness_1_phone;
                            $Loan->witness_1_id_number = $request->witness_1_id_number;
                            $Loan->witness_1_id_type = $request->witness_1_id_type;
                            $Loan->witness_1_id_date_type = ($df_witness_1_date_type) ? 1 : 0;
                            if ($request->$var_witness_1_date != "") {
                                $Loan->witness_1_id_end_date = date('Y-m-d', strtotime($request->$var_witness_1_date));
                            }

                            $Loan->witness_2_name = $request->witness_2_name;
                            $Loan->witness_2_phone = $request->witness_2_phone;
                            $Loan->witness_2_id_number = $request->witness_2_id_number;
                            $Loan->witness_2_id_type = $request->witness_2_id_type;
                            $Loan->witness_2_id_date_type = ($df_witness_2_date_type) ? 1 : 0;
                            if ($request->$var_witness_2_date != "") {
                                $Loan->witness_2_id_end_date = date('Y-m-d', strtotime($request->$var_witness_2_date));
                            }

                            $Loan->pay_method = $request->pay_method;
                            $Loan->notes = $request->notes;
                            $Loan->updated_ip = \Request::ip();
                            $Loan->updated_by = Auth::user()->id;
                            $Loan->save();

                            if ($request->reinstall) {
                                // delete old installments
                                $Loan->installments()->delete();
                                // add installments
                                if ($Loan->instalments_count > 0) {
                                    $_date = date('Y-m-d', strtotime($Loan->instalments_date));
                                    for ($i = 1; $i <= $Loan->instalments_count; $i++) {
                                        if ($i > 1) {
                                            if ($Loan->pay_method > 0) {
                                                $_date = date('Y-m-d', strtotime("+" . $Loan->pay_method . " month", strtotime($_date)));
                                            } else {
                                                $_date = date('Y-m-d', strtotime("+12 month", strtotime($_date)));
                                            }
                                        }
                                        $Installment = new Installment;
                                        $Installment->loan_id = $Loan->id;
                                        $Installment->person_id = $Loan->person_id;
                                        $Installment->receipt_id = 0;
                                        $Installment->period_id = 0;
                                        if ($i == $Loan->instalments_count) {
                                            $Installment->amount = $Loan->instalments_last;
                                        } else {
                                            $Installment->amount = $Loan->instalments_amount;
                                        }
                                        $Installment->date = $_date;
                                        $Installment->date_type = Helper::Preferences("date_type");
                                        $Installment->pay_method = $Loan->pay_method;
                                        $Installment->paid_status = 0;
                                        $Installment->created_ip = \Request::ip();
                                        $Installment->created_by = Auth::user()->id;
                                        $Installment->save();
                                    }
                                }
                            }

                            // Log to history
                            History::Log(
                                $module = 7,
                                $cat = 701,
                                $window = 0,
                                $record_id = $Loan->id,
                                $operation = 3,
                                $title = $Loan->amount . "<br>" . @$Loan->name
                            );

                            return response()->json(['stat' => 'success', "id" => encrypt($Loan->id)]);
                        } catch (\Exception $e) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                        }
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function destroy($id = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                if (Roles::Check("loans_delete")) {
                    $Loan = Loan::find(decrypt($id));
                    if (!empty($Loan)) {
                        // delete attachments
                        $Attachments = Attachment::where('cat', 4)->where('cat_id', $Loan->id)->get();
                        foreach ($Attachments as $Attachment) {
                            if ($Attachment->file != "") {
                                if ($Attachment->remote && Helper::FileSystem() != "") {
                                    Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                } else {
                                    if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                        File::delete("uploads/attachments/" . $Attachment->file);
                                    }
                                }
                            }
                            $Attachment->shares()->delete();
                            $Attachment->comments()->delete();
                            $Attachment->alerts()->delete();
                            $Attachment->delete();
                        }
                        $Loan->comments()->delete();
                        $Loan->alerts()->delete();
                        $Loan->installments()->delete();
                        $Loan->delete();

                        // Log to history
                        History::Log(
                            $module = 7,
                            $cat = 701,
                            $window = 0,
                            $record_id = $Loan->id,
                            $operation = 4,
                            $title = $Loan->amount . "<br>" . $Loan->name
                        );

                        return json_encode(array("stat" => "success", "id" => $Loan->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function edit_status($id = '')
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loan_status_edit")) {
                if ($id != "") {
                    try {
                        $Loan = Loan::find(decrypt($id));
                        if (!empty($Loan)) {
                            return view("back.accounting.loans.edit.status", compact("Loan"));
                        }
                    } catch (\Exception $e) {
                    }
                }
            }
        }
        return "";
    }

    public function update_status(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("loan_status_edit")) {

                $Loan = Loan::find(decrypt($request->id));
                if (!empty($Loan)) {
                    try {
                        if ($request->person_status === "1") {
                            $person_status = 1;
                            $person_status_text = trans('backLang.approved');
                            $loan_status_title = "<span class=\"text-success\"><i class=\"fe fe-check-circle\"></i> " . $person_status_text . "</span>";
                        } elseif ($request->person_status === "-1") {
                            $person_status = -1;
                            $person_status_text = trans('backLang.notApproved');
                            $loan_status_title = "<span class=\"text-danger\"><i class=\"fe fe-x-circle\"></i> " . $person_status_text . "</span>";
                        } else {
                            $person_status = 0;
                            $person_status_text = trans('backLang.pending');
                            $loan_status_title = "<span class=\"text-info\"><i class=\"fe fe-clock\"></i> " . $person_status_text . "</span>";
                        }
                        if ($person_status === -1 && $request->reject_reason == "") {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.whatIsTheReasonForReject')]]);
                        } else {
                            $Loan->status = $person_status;
                            if ($person_status === -1) {
                                $Loan->reject_reason = $request->reject_reason;
                            } else {
                                $Loan->reject_reason = "";
                            }
                            $Loan->approved_ip = \Request::ip();
                            $Loan->approved_by = Auth::user()->id;
                            $Loan->approved_at = date('Y-m-d H:i:s');
                            $Loan->save();


                            // Log to history
                            History::Log(
                                $module = 7,
                                $cat = 701,
                                $window = 0,
                                $record_id = $Loan->id,
                                $operation = 3,
                                $title = trans('backLang.changeLoanStatus') . ": " . $loan_status_title . "<br>" . $Loan->reject_reason
                            );

                            return response()->json(['stat' => 'success', 'id' => encrypt($Loan->id)]);
                        }
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function print(Request $request)
    {
        if (Roles::Check("loans_print")) {

            $search_word = $request->input('search_word');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_cat_id = $request->input('search_cat_id');
            $search_status = $request->input('search_status');


            if (Roles::Check("view")) {
                $Loans = Loan::where("created_by", @Auth::user()->id);
            } else {
                $Loans = Loan::where("id", ">", 0);
            }
            if ($search_word != "") {
                $Loans = $Loans->where(function ($query) use ($search_word) {
                    $query->where("id", $search_word)
                        ->orwhere("name", "like", "%" . $search_word . "%")
                        ->orwhere("phone", $search_word)
                        ->orwhere("email", $search_word)
                        ->orwhere("id_number", $search_word)
                        ->orwhere("notes", "like", "%" . $search_word . "%")
                        ->orwhere("guarantor_name", "like", "%" . $search_word . "%")
                        ->orwhere("witness_1_name", "like", "%" . $search_word . "%")
                        ->orwhere("witness_2_name", "like", "%" . $search_word . "%");
                });
            }
            if ($search_cat_id > 0) {
                $Loans = $Loans->where("cat_id", $search_cat_id);
            }
            if ($search_user > 0) {
                $Loans = $Loans->where("created_by", $search_user);
            }
            if ($search_status != "") {
                $Loans = $Loans->where("status", $search_status);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Loans = $Loans->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Loans = $Loans->where('date', "<=", $search_end_date);
            }
            $Loans = $Loans->orderby("id", "asc")->get();

            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            return view("back.accounting.loans.print", compact("Loans", "request", "DefaultCurrency"));
        }
        return "";
    }

    public function export(Request $request)
    {
        if (Roles::Check("loans_export")) {
            return (new LoansExport($request))->download(trans('backLang.loans') . '.xlsx');
        }
        return "";
    }

    public function print_file($id = '')
    {
        if (Roles::Check("loan_print")) {
            if ($id != "") {
                try {
                    $Loan = Loan::find(decrypt($id));
                    if (!empty($Loan)) {
                        $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
                        return view("back.accounting.loans.view.print", compact("Loan", "DefaultCurrency"));
                    }
                } catch (\Exception $e) {
                }
            }
        }
        return "";
    }

    public function receipt_store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("loans_receipt")) {
                $validator = Validator::make($request->all(), [
                    'loan_id' => 'required',
                    'fund' => 'required',
                    'pay_method' => 'required',
                ]);

                if ($validator->passes()) {
                    try {
                        $Loan = Loan::find(decrypt($request->loan_id));
                        if (!empty($Loan)) {
                            if (!Helper::Preferences("default_period")) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                            }

                            $Receipt = new Receipt;
                            $Receipt->type = 0;
                            $Receipt->code = $request->code;
                            $Receipt->period_id = Helper::Preferences("default_period");
                            $Receipt->date = (Helper::Preferences("date_type")) ? Helper::DateView(date('Y-m-d'), 0, 1, 'Y-m-d') : date('Y-m-d');
                            $Receipt->date_type = Helper::Preferences("date_type");
                            $Receipt->amount_type = 0;
                            $Receipt->amount = $Loan->total;
                            $Receipt->currency_id = Helper::Preferences("default_currency");
                            $Receipt->cost = $Loan->total;
                            $Receipt->details = trans("backLang.loanPay") . " " . @$Loan->name . " - " . trans("backLang.loanNo") . " " . $Loan->id;
                            $Receipt->ledger_id = Helper::Preferences("loans_ledger");
                            $Receipt->cat_id = $request->cat_id;
                            $Receipt->bank_id = $request->fund;
                            $Receipt->method_id = $request->pay_method;
                            $Receipt->repeated = false;
                            $Receipt->posted = Helper::Preferences("posting_method");
                            $Receipt->created_ip = \Request::ip();
                            $Receipt->created_by = Auth::user()->id;
                            $Receipt->save();

                            //connect receipt
                            $Loan->receipt_id = $Receipt->id;
                            $Loan->save();

                            // delete old journals
                            $Receipt->journals()->delete();

                            // save new to journal
                            $Journal = new Journal;
                            $Journal->type = 0;
                            $Journal->period_id = $Receipt->period_id;
                            $Journal->receipt_id = $Receipt->id;
                            $Journal->center_id = $Receipt->center_id;
                            $Journal->date = $Receipt->date;
                            $Journal->date_type = $Receipt->date_type;
                            $Journal->currency_id = $Receipt->currency_id;
                            $Journal->details = $Receipt->details;
                            $Journal->posted = Helper::Preferences("posting_method");
                            $Journal->created_by = Auth::user()->id;

                            // From Ledger:
                            $Journal->ledger_id = $Receipt->ledger_id;
                            $Journal->dept = $Receipt->amount;
                            $Journal->cost_dept = $Receipt->cost;
                            $Journal->credit = 0;
                            $Journal->cost_credit = 0;
                            $Journal->save();
                            // To Ledger:
                            $SecondJournal = $Journal->replicate();
                            $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                            $SecondJournal->dept = 0;
                            $SecondJournal->cost_dept = 0;
                            $SecondJournal->credit = $Receipt->amount;
                            $SecondJournal->cost_credit = $Receipt->cost;
                            $SecondJournal->save();

                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = ($Receipt->type) ? 602 : 603,
                                $window = 0,
                                $record_id = $Receipt->id,
                                $operation = 2,
                                $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr($Receipt->details, 0, 150)
                            );

                            return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                        }
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
                return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function receipt_print($id = '')
    {
        if (Roles::Check("loans_receipt")) {
            if ($id != "") {
                try {
                    $Loan = Loan::find(decrypt($id));
                    if (!empty($Loan)) {
                        if (!empty($Loan->receipt)) {
                            echo "
                        <a class=\"btn btn-light mt-3 w-100\"
                                           href=\"" . route("AppReceiptPrint", encrypt($Loan->receipt->id)) . "\"
                                           data-toggle=\"tooltip\" target=\"_blank\"><i
                                                class=\"icon-printer\"></i> " . trans("backLang.printPayReceipt") . "</a>
                        ";
                        }
                    }
                } catch (\Exception $e) {
                }
            }
        }
        return "";
    }

    public function load_loan()
    {
        $Loan = Loan::where("created_by", Auth::user()->id)->orderby("id", "desc")->first();
        if (!empty($Loan)) {
            return redirect()->action("Accounting\LoanController@view", [encrypt($Loan->id), 1]);
        }
        return redirect()->route("Loans");
    }
}
