<?php

namespace App\Http\Controllers\Accounting;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Period;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;

class PeriodController extends Controller

{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_accounting", true);
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'check',
            1 => 'code',
            2 => 'start',
            3 => 'end',
            4 => 'status',
            5 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');


        if (Roles::Check("view")) {
            $Periods = Period::where("created_by", @Auth::user()->id);
        } else {
            $Periods = Period::where("id", ">", 0);
        }
        $totalData = $Periods->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Periods = $Periods->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Periods as $Period) {
                $nestedData['check'] = "<div class='text-center'><label class=\"custom-control custom-checkbox\"><input type=\"checkbox\" class=\"custom-control-input\" name=\"ids[]\" value=\"" . $Period->id . "\"><span class=\"custom-control-label\">&nbsp;</span></label></div>";
                $nestedData['code'] = "<div class='text-center'>" . $Period->code . "</div>";
                $nestedData['start'] = "<div class='text-center'>" . $Period->start . "</div>";
                $nestedData['end'] = "<div class='text-center'>" . $Period->end . "</div>";
                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Period->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";

                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
  <button class=\"btn btn-sm btn-info\" type='button' onclick=\"edit_period('" . encrypt($Period->id) . "')\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</button>
 ";
                }

                if (Roles::Check("delete")) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" type='button' onclick=\"delete_period('" . encrypt($Period->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $validator = Validator::make($request->all(), [
                'code' => 'required',
                'start' => 'required',
                'end' => 'required'
            ]);
            if ($validator->passes()) {
                try {
                    $Period = new Period;
                    $Period->code = $request->code;
                    $Period->start = date('Y-m-d', strtotime($request->start));
                    $Period->end = date('Y-m-d', strtotime($request->end));
                    $Period->date_type = Helper::Preferences("date_type");
                    $Period->status = true;
                    $Period->created_ip = \Request::ip();
                    $Period->created_by = Auth::user()->id;
                    $Period->save();

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6005,
                        $record_id = $Period->id,
                        $operation = 2,
                        $title = $Period->title_ar . "<br>" . $Period->title_en
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Period->id)]);
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Period = Period::find(decrypt($id));
                if (!empty($Period)) {
                    return view("back.accounting.preferences.periods.edit", compact("Period"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {

            $validator = Validator::make($request->all(), [
                'code' => 'required',
                'start' => 'required',
                'end' => 'required'
            ]);
            if ($validator->passes()) {
                try {
                    $Period = Period::find(decrypt($request->id));
                    if (!empty($Period)) {
                        $Period->code = $request->code;
                        $Period->start = $request->start;
                        $Period->end = $request->end;
                        $Period->updated_ip = \Request::ip();
                        $Period->updated_by = Auth::user()->id;
                        $Period->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6005,
                            $record_id = $Period->id,
                            $operation = 3,
                            $title = $Period->title_ar . "<br>" . $Period->title_en
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Period->id)]);
                    }
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function multi_update(Request $request)
    {

        if (Roles::Check("edit")) {
            if ($request->ids != "") {
                if ($request->action == "activate") {
                    $Periods = Period::wherein('id', $request->ids)->get();
                    foreach ($Periods as $Period) {
                        $Period->status = true;
                        $Period->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6005,
                            $record_id = $Period->id,
                            $operation = 9,
                            $title = $Period->title_ar . "<br>" . $Period->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "block") {

                    $Periods = Period::wherein('id', $request->ids)->get();
                    foreach ($Periods as $Period) {
                        $Period->status = false;
                        $Period->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6005,
                            $record_id = $Period->id,
                            $operation = 8,
                            $title = $Period->title_ar . "<br>" . $Period->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "delete") {
                    $Periods = Period::wherein('id', $request->ids)->get();
                    foreach ($Periods as $Period) {

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 600,
                            $window = 6005,
                            $record_id = $Period->id,
                            $operation = 4,
                            $title = $Period->title_ar . "<br>" . $Period->title_en
                        );

                        $Period->delete();
                    }

                    return response()->json(['stat' => 'success']);
                }

            } else {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.noRowsSelected')]]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Period = Period::find(decrypt($id));
                if (!empty($Period)) {

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6005,
                        $record_id = $Period->id,
                        $operation = 4,
                        $title = $Period->title_ar . "<br>" . $Period->title_en
                    );

                    $Period->delete();
                    return json_encode(array("stat" => "success", "id" => $Period->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
