<?php

namespace App\Http\Controllers\Accounting;

use App\Models\Bank;
use App\Models\Category;
use App\Models\Currency;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Receipt;
use App\Models\Period;
use Auth;
use Helper;
use File;
use Illuminate\Support\Facades\Storage;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;

class PostingController extends Controller
{
    private $uploadPath = "uploads/receipts/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        $Currencies = Currency::where("status", true)->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();
        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
        return view("back.accounting.posting.home", compact("Currencies", "UsersList", "Periods", "ledgers_array"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'check',
            1 => 'id',
            2 => 'date',
            3 => 'amount',
            4 => 'amount',
            5 => 'details',
            6 => 'posted',
            7 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_currency = $request->input('search_currency');
        $search_posted = $request->input('search_posted');
        $search_ledger = $request->input('search_ledger');

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", ">", 0);
        }
        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_posted != "") {
            $Receipts = $Receipts->where("posted", $search_posted);
        }
        if ($search_ledger > 0) {
            $receipt_ids = Journal::select("receipt_id")->where('ledger_id', $search_ledger);
            $Receipts = $Receipts->wherein("id", $receipt_ids);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_period > 0) {
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $totalData = $Receipts->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Receipts = $Receipts->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Receipts as $Receipt) {

                // details prepare
                $dept = "";
                $credit = "";
                $details_from = "";
                $details_from_count = 0;
                $details_from_title = trans("backLang.fromAccount") . ": ";
                $details_to = "";
                $details_to_count = 0;
                $details_to_title = trans("backLang.toAccount") . ": ";
                foreach ($Receipt->journals->where("dept", ">", 0) as $Journal) {
                    $details_from_count++;
                    $details_from .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $dept .= number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                    $credit .= "<br>";
                }
                foreach ($Receipt->journals->where("credit", ">", 0) as $Journal) {
                    $details_to_count++;
                    $details_to .= @$Journal->ledger->{"title_" . trans('backLang.boxCode')} . "<br>";
                    $credit .= number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . "<br>";
                }
                if ($details_from_count > 1) {
                    $details_from_title = trans("backLang.fromThoseMentioned") . ":<br>";
                    $dept = "<br>" . $dept;
                    $credit = "<br>" . $credit;
                }
                if ($details_to_count > 1) {
                    $details_to_title = trans("backLang.toThoseMentioned") . ":<br>";
                }
                $details = "<span class='text-info'>" . $details_from_title . "</span>" . $details_from . "<div class='ml-3'>" . "<span class='text-info'>" . $details_to_title . "</span>" . $details_to . "</div><div> " . $Receipt->details . "</div>";


                $nestedData['check'] = "<div class='text-center'><label class=\"custom-control custom-checkbox\"><input type=\"checkbox\" class=\"custom-control-input\" name=\"ids[]\" value=\"" . $Receipt->id . "\"><span class=\"custom-control-label\">&nbsp;</span></label></div>";
                $nestedData['id'] = "<div class='text-center'>" . $Receipt->id . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['dept'] = "<div class='text-center'>" . $dept . "</div>";
                $nestedData['credit'] = "<div class='text-center'>" . $credit . "</div>";
                $nestedData['details'] = $details;
                $nestedData['posted'] = "<div class='text-center'>" . (($Receipt->posted) ? "<span>" . trans("backLang.posted") . "</span>" : "<span class='text-danger'>" . trans("backLang.notPosted") . "</span>") . "</div>";

                $options = "<div class=\"text-center\">";

                if ($Receipt->posted) {
                    if (Roles::Check("accountant_unpost")) {
                        $options .= " <button type='button' class=\"btn btn-sm btn-light mb-2 width-100px\" onclick=\"unpost_row('" . encrypt($Receipt->id) . "')\"><i class=\"fe fe-rotate-cw\"> " . trans("backLang.postCancel") . " </i></button><br>";
                    }
                } else {
                    if (Roles::Check("accountant_post")) {
                        $options .= " <button type='button' class=\"btn btn-sm btn-secondary mb-2 width-100px\" onclick=\"post_row('" . encrypt($Receipt->id) . "')\"><i class=\"fe fe-corner-up-left\"> " . trans("backLang.postDo") . " </i></button><br>";
                    }
                }

                if (Roles::Check("accountant_print")) {
                    $options .= " <button class=\"btn btn-sm btn-light\" type='button' onclick=\"view_receipt('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.entryDetails") . "\"><i class=\"icon-book-open\"></i></button>";
                }
                if (Roles::Check("edit") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_edit")) {
                        $options .= " <a href=\"" . route("AppJournalsEdit", encrypt($Receipt->id)) . "\" class=\"btn btn-sm btn-info\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></a>";
                    }
                }
                if (Roles::Check("delete") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_delete")) {
                        $options .= " <button type='button' class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function post($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("accountant_post")) {
                $Receipt = Receipt::find(decrypt($id));
                if (!empty($Receipt)) {

                    // post related journals
                    foreach ($Receipt->journals as $Journal) {
                        $Journal->posted = true;
                        $Journal->save();
                    }

                    // post receipt
                    $Receipt->posted = true;
                    $Receipt->save();

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 607,
                        $window = 0,
                        $record_id = $Receipt->id,
                        $operation = 17,
                        $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr(strip_tags($Receipt->details), 0, 150)
                    );
                    return json_encode(array("stat" => "success", "id" => $Receipt->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function unpost($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("accountant_unpost")) {
                $Receipt = Receipt::find(decrypt($id));
                if (!empty($Receipt)) {

                    // post related journals
                    foreach ($Receipt->journals as $Journal) {
                        $Journal->posted = false;
                        $Journal->save();
                    }

                    // post receipt
                    $Receipt->posted = false;
                    $Receipt->save();

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 607,
                        $window = 0,
                        $record_id = $Receipt->id,
                        $operation = 18,
                        $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr(strip_tags($Receipt->details), 0, 150)
                    );
                    return json_encode(array("stat" => "success", "id" => $Receipt->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function multi(Request $request)
    {

        if ($request->ids != "") {
            if ($request->action == "post") {
                $Receipts = Receipt::wherein('id', $request->ids)->get();
                foreach ($Receipts as $Receipt) {
                    if (@$Receipt->period->status) {
                        // post related journals
                        foreach ($Receipt->journals as $Journal) {
                            $Journal->posted = true;
                            $Journal->save();
                        }

                        // post receipt
                        $Receipt->posted = true;
                        $Receipt->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 607,
                            $window = 0,
                            $record_id = $Receipt->id,
                            $operation = 17,
                            $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr(strip_tags($Receipt->details), 0, 150)
                        );
                    }
                }
                return response()->json(['stat' => 'success']);
            } elseif ($request->action == "unpost") {
                $Receipts = Receipt::wherein('id', $request->ids)->get();
                foreach ($Receipts as $Receipt) {
                    if (@$Receipt->period->status) {
                        // post related journals
                        foreach ($Receipt->journals as $Journal) {
                            $Journal->posted = false;
                            $Journal->save();
                        }

                        // post receipt
                        $Receipt->posted = false;
                        $Receipt->save();

                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = 607,
                            $window = 0,
                            $record_id = $Receipt->id,
                            $operation = 18,
                            $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr(strip_tags($Receipt->details), 0, 150)
                        );
                    }
                }
                return response()->json(['stat' => 'success']);
            } elseif ($request->action == "delete") {
                $Receipts = Receipt::wherein('id', $request->ids)->get();
                foreach ($Receipts as $Receipt) {
                    if (@$Receipt->period->status) {
                        try {
                            // delete related journals
                            $Receipt->journals()->delete();

                            //delete file
                            if ($Receipt->file != "") {
                                try {
                                    if ($Receipt->remote && Helper::FileSystem() != "") {
                                        Storage::disk(Helper::FileSystem())->delete($Receipt->file);
                                    } else {
                                        if (file_exists($this->uploadPath . $Receipt->file)) {
                                            File::delete($this->uploadPath . $Receipt->file);
                                        }
                                    }
                                } catch (\Exception $e) {

                                }
                            }
                            $Receipt->delete();

                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = ($Receipt->type) ? 602 : 603,
                                $window = 0,
                                $record_id = $Receipt->id,
                                $operation = 4,
                                $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . mb_substr(strip_tags($Receipt->details), 0, 150)
                            );
                        } catch (\Exception $e) {

                        }
                    }
                }
                return response()->json(['stat' => 'success']);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.noRowsSelected')]]);
    }
}
