<?php

namespace App\Http\Controllers\Accounting;

use App\Models\Bank;
use App\Models\Currency;
use App\Models\Journal;
use App\Models\Ledger;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App;
use App\Models\Receipt;
use App\Models\Period;
use App\Models\Category;
use Auth;
use Helper;
use File;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;
use App\Exports\ReceiptExport;
use App\Imports\ReceiptImport;
use Illuminate\Support\Facades\Storage;

class ReceiptController extends Controller
{
    private $uploadPath = "uploads/receipts/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check Device licence
    }

    public function index($stat = "revenues", $create = '')
    {
        $Currencies = Currency::where("status", true)->get();
        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        $Periods = Period::orderby("code", "asc")->get();

        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

        return view("back.accounting.receipts.home", compact("Currencies", "Funds", "UsersList", "stat", "Periods", "create", "ledgers_array"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'date',
            2 => 'amount',
            3 => 'details',
            4 => 'ledger_id',
            5 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $search_word = $request->input('search_word');
        $search_type = $request->input('search_type');
        $search_fund = $request->input('search_fund');
        $search_pay_method = $request->input('search_pay_method');
        $search_currency = $request->input('search_currency');
        $search_cat = $request->input('search_cat');
        $search_user = $request->input('search_user');
        $search_start = $request->input('search_start');
        $search_end = $request->input('search_end');
        $search_center_id = $request->input('search_center_id');
        $search_period = $request->input('search_period');
        $search_ledger = $request->input('search_ledger');

        if (Roles::Check("view")) {
            $Receipts = Receipt::where("created_by", @Auth::user()->id);
        } else {
            $Receipts = Receipt::where("id", '>', 0);
        }
        if ($search_type) {
            $Receipts = $Receipts->where("type", true);
        } else {
            $Receipts = $Receipts->where("type", false);
        }

        if ($search_word != "") {
            $Receipts = $Receipts->where(function ($query) use ($search_word) {
                $query->where("id", $search_word)
                    ->orwhere("details", "like", "%" . $search_word . "%")
                    ->orwhere("name", "like", "%" . $search_word . "%")
                    ->orwhere('code', $search_word);
            });
        }
        if ($search_fund > 0) {
            $Receipts = $Receipts->where("bank_id", $search_fund);
        }
        if ($search_period > 0) {
            $Receipts = $Receipts->where("period_id", $search_period);
        }
        if ($search_ledger > 0) {
            $Receipts = $Receipts->where("ledger_id", $search_ledger);
        }
        if ($search_pay_method > 0) {
            $Receipts = $Receipts->where("method_id", $search_pay_method);
        }
        if ($search_currency > 0) {
            $Receipts = $Receipts->where("currency_id", $search_currency);
        }
        if ($search_cat > 0) {
            $Receipts = $Receipts->where("cat_id", $search_cat);
        }
        if ($search_center_id > 0) {
            $Receipts = $Receipts->where("center_id", $search_center_id);
        }
        if ($search_user > 0) {
            $Receipts = $Receipts->where("created_by", $search_user);
        }
        if ($search_start != "") {
            $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
            $Receipts = $Receipts->where('date', ">=", $search_start_date);
        }
        if ($search_end != "") {
            $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
            $Receipts = $Receipts->where('date', "<=", $search_end_date);
        }

        $totalData = $Receipts->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Receipts = $Receipts->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->orderBy("id", "desc")
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Receipts as $Receipt) {
                $nestedData['id'] = "<div class='text-center'>" . $Receipt->id . "</div>";
                $nestedData['date'] = "<div class='text-center'>" . date(Helper::Preferences("date_format"), strtotime($Receipt->date)) . " " . ((@$Receipt->date_type) ? trans("backLang.H") : trans("backLang.M")) . "</div>";
                $nestedData['amount'] = "<div class='text-center'><strong>" . number_format($Receipt->amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Receipt->currency->symbol . "</strong></div>";
                $nestedData['details'] = $Receipt->details;
                $nestedData['ledger_id'] = "<span class=\"badge badge-light font-13\">" . @$Receipt->ledger->code . "</span> " . @$Receipt->ledger->{"title_" . trans("backLang.code")};

                $options = "<div class=\"text-center\">";
                if (Roles::Check("accountant_print")) {
                    $options .= " <a class=\"btn btn-sm btn-light\" href=\"" . route("AppReceiptPrint", encrypt($Receipt->id)) . "\" data-toggle=\"tooltip\" target='_blank' data-original-title=\"" . trans("backLang.print") . "\"><i class=\"icon-printer\"></i></a>";
                }
                if (Roles::Check("edit") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_edit")) {
                        $options .= " <button class=\"btn btn-sm btn-info\" onclick=\"edit_row('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></button>";
                    }
                }
                if (Roles::Check("delete") && @$Receipt->period->status) {
                    if (Roles::Check("accountant_delete")) {
                        $options .= " <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Receipt->id) . "')\" data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\"><i class=\"icon-trash\"></i></button>";
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if (Roles::Check("accountant_add")) {
                $validator = Validator::make($request->all(), [
                    'amount' => 'required',
                    'currency' => 'required',
                    'date' => 'required',
                    'pay_method' => 'required',
                    'fund' => 'required',
                    'ledger' => 'required',
                    'details' => 'required',
                    'type' => 'required'
                ]);
                if ($validator->passes()) {
                    try {

                        if (!Helper::Preferences("default_period")) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }
                        $CurrentPeriod = Period::find(Helper::Preferences("default_period"));
                        if (!@$CurrentPeriod->status) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.youNeedToDefineDefaultPeriod')]]);
                        }

                        // Start File Upload
                        $formFileName = "file";
                        $fileFinalRand = "";
                        $fileFinalExt = "";
                        $uploaded = false;
                        $path = "";
                        if ($request->$formFileName != "") {
                            $path = $this->uploadPath;
                            $fileFinalRand = time() . rand(1111, 9999);
                            $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                            $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);
                        }

                        $Receipt = new Receipt;
                        $Receipt->type = $request->type;
                        $Receipt->code = $request->code;
                        $Receipt->period_id = Helper::Preferences("default_period");
                        $Receipt->date = date('Y-m-d', strtotime($request->date));
                        $Receipt->date_type = Helper::Preferences("date_type");
                        $Receipt->amount_type = $request->type;
                        $Receipt->amount = $request->amount;
                        $Receipt->currency_id = $request->currency;
                        $Receipt->cost = Helper::CurrencyTransfer($request->amount, $request->currency, Helper::Preferences("default_currency"));
                        $Receipt->details = $request->details;
                        $Receipt->ledger_id = $request->ledger;
                        $Receipt->cat_id = $request->cat_id;
                        $Receipt->bank_id = $request->fund;
                        $Receipt->method_id = $request->pay_method;
                        $Receipt->center_id = $request->center_id;

                        if ($uploaded) {
                            $Receipt->file = $fileFinalRand;
                            $Receipt->ext = $fileFinalExt;
                            $Receipt->remote = 0;
                        }
                        if ($request->charges_status) {
                            $Receipt->bank_charges = true;
                            $Receipt->bank_charges_amount = $request->charges_amount;
                            $Receipt->bank_charges_details = $request->charges_details;
                        }
                        $Receipt->repeated = false;
                        $Receipt->posted = Helper::Preferences("posting_method");
                        $Receipt->created_ip = \Request::ip();
                        $Receipt->created_by = Auth::user()->id;
                        $Receipt->save();

                        // delete old journals
                        $Receipt->journals()->delete();

                        // save new to journal
                        $Journal = new Journal;
                        $Journal->type = 0;
                        $Journal->period_id = $Receipt->period_id;
                        $Journal->receipt_id = $Receipt->id;
                        $Journal->center_id = $Receipt->center_id;
                        $Journal->date = $Receipt->date;
                        $Journal->date_type = $Receipt->date_type;
                        $Journal->currency_id = $Receipt->currency_id;
                        $Journal->details = $Receipt->details;
                        $Journal->posted = Helper::Preferences("posting_method");
                        $Journal->created_by = Auth::user()->id;

                        if ($Receipt->type) {
                            // revenues
                            $ledger_id = $Receipt->ledger_id;
                            $Category = Category::find($Receipt->cat_id);
                            if (!empty($Category)) {
                                if (!empty($Category->ledger)) {
                                    $ledger_id = $Category->ledger->id;
                                }

                            }
                            // From Ledger:
                            $Journal->ledger_id = $ledger_id;
                            $Journal->credit = $Receipt->amount;
                            $Journal->cost_credit = $Receipt->cost;
                            $Journal->dept = 0;
                            $Journal->cost_dept = 0;
                            $Journal->save();
                            // To Ledger:
                            $SecondJournal = $Journal->replicate();
                            $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                            $SecondJournal->credit = 0;
                            $SecondJournal->cost_credit = 0;
                            $SecondJournal->dept = $Receipt->amount;
                            $SecondJournal->cost_dept = $Receipt->cost;
                            $SecondJournal->save();
                        } else {
                            // expenses
                            $ledger_id = $Receipt->ledger_id;
                            $Category = Category::find($Receipt->cat_id);
                            if (!empty($Category)) {
                                if (!empty($Category->ledger)) {
                                    $ledger_id = $Category->ledger->id;
                                }

                            }
                            // From Ledger:
                            $Journal->ledger_id = $ledger_id;
                            $Journal->dept = $Receipt->amount;
                            $Journal->cost_dept = $Receipt->cost;
                            $Journal->credit = 0;
                            $Journal->cost_credit = 0;
                            $Journal->save();
                            // To Ledger:
                            $SecondJournal = $Journal->replicate();
                            $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                            $SecondJournal->dept = 0;
                            $SecondJournal->cost_dept = 0;
                            $SecondJournal->credit = $Receipt->amount;
                            $SecondJournal->cost_credit = $Receipt->cost;
                            $SecondJournal->save();
                        }

                        //bank charges
                        if ($Receipt->bank_charges && !empty($Receipt->bank)) {
                            if ($Receipt->bank->charges_ledger_id > 0) {
                                $ThirdJournal = $Journal->replicate();
                                $ThirdJournal->details = $Receipt->bank_charges_details;
                                $ThirdJournal->ledger_id = $Receipt->bank->ledger_id;
                                $ThirdJournal->dept = 0;
                                $ThirdJournal->cost_dept = 0;
                                $ThirdJournal->credit = $Receipt->bank_charges_amount;
                                $ThirdJournal->cost_credit = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                $ThirdJournal->save();

                                $FourthJournal = $Journal->replicate();
                                $ThirdJournal->details = $Receipt->bank_charges_details;
                                $FourthJournal->ledger_id = $Receipt->bank->charges_ledger_id;
                                $FourthJournal->dept = $Receipt->bank_charges_amount;
                                $FourthJournal->cost_dept = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                $FourthJournal->credit = 0;
                                $FourthJournal->cost_credit = 0;
                                $FourthJournal->save();
                            }
                        }
                        // Log to history
                        History::Log(
                            $module = 6,
                            $cat = (@$Receipt->type) ? 602 : 603,
                            $window = 0,
                            $record_id = @$Receipt->id,
                            $operation = 2,
                            $title = @$Receipt->amount . " " . @$Receipt->currency->symbol . "<br>" . mb_substr(@$Receipt->details, 0, 150)
                        );

                        //Remote storage
                        if ($uploaded) {
                            if (Helper::FileSystem() != "") {
                                $filepath = $path . $fileFinalRand;
                                $localFile = File::get($filepath);
                                $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                                if ($Remote_store) {
                                    $Receipt->remote = "1";
                                    $Receipt->save();
                                    // Delete from local
                                    File::delete($filepath);
                                }
                            }
                        }

                        return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Receipt = Receipt::find(decrypt($id));
                    if (!empty($Receipt)) {
                        $Currencies = Currency::where("status", true)->get();
                        $Funds = Bank::where("status", true)->orderby("type", "desc")->get();
                        $UsersList = User::where("status", true)->orderby("name", "asc")->get();

                        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                        return view("back.accounting.receipts.edit", compact("Receipt", "Currencies", "Funds", "UsersList", "ledgers_array"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                $Receipt = Receipt::find(decrypt($request->id));
                if (!empty($Receipt)) {
                    $validator = Validator::make($request->all(), [
                        'amount' => 'required',
                        'currency' => 'required',
                        'date' => 'required',
                        'pay_method' => 'required',
                        'fund' => 'required',
                        'ledger' => 'required',
                        'details' => 'required'
                    ]);
                    if ($validator->passes()) {
                        try {

                            $CurrentPeriod = Period::find($Receipt->period_id);
                            if (!@$CurrentPeriod->status) {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.youCanNotUpdateIfPeriodClosed')]]);
                            }

                            // delete old photo
                            if ($request->file_delete == 1 || $request->file != "") {
                                if ($Receipt->file != "") {
                                    if ($Receipt->remote && Helper::FileSystem() != "") {
                                        Storage::disk(Helper::FileSystem())->delete($Receipt->file);
                                    } else {
                                        if (file_exists($this->uploadPath . $Receipt->file)) {
                                            File::delete($this->uploadPath . $Receipt->file);
                                        }
                                    }
                                }

                                $Receipt->file = "";
                                $Receipt->ext = "";
                                $Receipt->remote = 0;
                            }

                            // Start File Upload
                            $formFileName = "file";
                            $fileFinalRand = "";
                            $fileFinalExt = "";
                            $uploaded = false;
                            $path = "";
                            if ($request->$formFileName != "") {
                                $path = $this->uploadPath;
                                $fileFinalRand = time() . rand(1111, 9999);
                                $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                                $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);
                            }
                            $Receipt->code = $request->code;
                            $Receipt->date = date('Y-m-d', strtotime($request->date));
                            $Receipt->date_type = Helper::Preferences("date_type");
                            if ($Receipt->amount != $request->amount || $Receipt->currency_id != $request->currency) {
                                $Receipt->amount = $request->amount;
                                $Receipt->currency_id = $request->currency;
                                $Receipt->cost = Helper::CurrencyTransfer($request->amount, $request->currency, Helper::Preferences("default_currency"));
                            }
                            $Receipt->details = $request->details;
                            $Receipt->ledger_id = $request->ledger;
                            $Receipt->cat_id = $request->cat_id;
                            $Receipt->bank_id = $request->fund;
                            $Receipt->method_id = $request->pay_method;
                            $Receipt->center_id = $request->center_id;

                            if ($uploaded) {
                                $Receipt->file = $fileFinalRand;
                                $Receipt->ext = $fileFinalExt;
                                $Receipt->remote = 0;
                            }

                            if ($request->charges_status) {
                                $Receipt->bank_charges = true;
                                $Receipt->bank_charges_amount = $request->charges_amount;
                                $Receipt->bank_charges_details = $request->charges_details;
                            } else {
                                $Receipt->bank_charges = false;
                                $Receipt->bank_charges_amount = null;
                                $Receipt->bank_charges_details = null;
                            }

                            $Receipt->repeated = false;
                            $Receipt->posted = Helper::Preferences("posting_method");
                            $Receipt->created_ip = \Request::ip();
                            $Receipt->created_by = Auth::user()->id;
                            $Receipt->save();


                            // delete old journals
                            $Receipt->journals()->delete();

                            // save new to journal
                            $Journal = new Journal;
                            $Journal->type = 0;
                            $Journal->period_id = $Receipt->period_id;
                            $Journal->receipt_id = $Receipt->id;
                            $Journal->center_id = $Receipt->center_id;
                            $Journal->date = $Receipt->date;
                            $Journal->date_type = $Receipt->date_type;
                            $Journal->currency_id = $Receipt->currency_id;
                            $Journal->details = $Receipt->details;
                            $Journal->posted = Helper::Preferences("posting_method");
                            $Journal->created_by = Auth::user()->id;

                            if ($Receipt->type) {
                                // revenues
                                $ledger_id = $Receipt->ledger_id;
                                $Category = Category::find($Receipt->cat_id);
                                if (!empty($Category)) {
                                    if (!empty($Category->ledger)) {
                                        $ledger_id = $Category->ledger->id;
                                    }

                                }
                                // From Ledger:
                                $Journal->ledger_id = $ledger_id;
                                $Journal->credit = $Receipt->amount;
                                $Journal->cost_credit = $Receipt->cost;
                                $Journal->dept = 0;
                                $Journal->cost_dept = 0;
                                $Journal->save();
                                // To Ledger:
                                $SecondJournal = $Journal->replicate();
                                $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                                $SecondJournal->credit = 0;
                                $SecondJournal->cost_credit = 0;
                                $SecondJournal->dept = $Receipt->amount;
                                $SecondJournal->cost_dept = $Receipt->cost;
                                $SecondJournal->save();
                            } else {
                                // expenses
                                $ledger_id = $Receipt->ledger_id;
                                $Category = Category::find($Receipt->cat_id);
                                if (!empty($Category)) {
                                    if (!empty($Category->ledger)) {
                                        $ledger_id = $Category->ledger->id;
                                    }

                                }
                                // From Ledger:
                                $Journal->ledger_id = $ledger_id;
                                $Journal->dept = $Receipt->amount;
                                $Journal->cost_dept = $Receipt->cost;
                                $Journal->credit = 0;
                                $Journal->cost_credit = 0;
                                $Journal->save();
                                // To Ledger:
                                $SecondJournal = $Journal->replicate();
                                $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                                $SecondJournal->dept = 0;
                                $SecondJournal->cost_dept = 0;
                                $SecondJournal->credit = $Receipt->amount;
                                $SecondJournal->cost_credit = $Receipt->cost;
                                $SecondJournal->save();
                            }


                            //bank charges
                            if ($Receipt->bank_charges && !empty($Receipt->bank)) {
                                if ($Receipt->bank->charges_ledger_id > 0) {
                                    $ThirdJournal = $Journal->replicate();
                                    $ThirdJournal->details = $Receipt->bank_charges_details;
                                    $ThirdJournal->ledger_id = $Receipt->bank->ledger_id;
                                    $ThirdJournal->dept = 0;
                                    $ThirdJournal->cost_dept = 0;
                                    $ThirdJournal->credit = $Receipt->bank_charges_amount;
                                    $ThirdJournal->cost_credit = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                    $ThirdJournal->save();

                                    $FourthJournal = $Journal->replicate();
                                    $ThirdJournal->details = $Receipt->bank_charges_details;
                                    $FourthJournal->ledger_id = $Receipt->bank->charges_ledger_id;
                                    $FourthJournal->dept = $Receipt->bank_charges_amount;
                                    $FourthJournal->cost_dept = Helper::CurrencyTransfer($Receipt->bank_charges_amount, $Receipt->currency_id, Helper::Preferences("default_currency"));
                                    $FourthJournal->credit = 0;
                                    $FourthJournal->cost_credit = 0;
                                    $FourthJournal->save();
                                }
                            }

                            // Log to history
                            History::Log(
                                $module = 6,
                                $cat = (@$Receipt->type) ? 602 : 603,
                                $window = 0,
                                $record_id = @$Receipt->id,
                                $operation = 3,
                                $title = @$Receipt->amount . " " . @$Receipt->currency->symbol . "<br>" . mb_substr(@$Receipt->details, 0, 150)
                            );

                            //Remote storage
                            if ($uploaded) {
                                if (Helper::FileSystem() != "") {
                                    $filepath = $path . $fileFinalRand;
                                    $localFile = File::get($filepath);
                                    $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                                    if ($Remote_store) {
                                        $Receipt->remote = "1";
                                        $Receipt->save();
                                        // Delete from local
                                        File::delete($filepath);
                                    }
                                }
                            }

                            return response()->json(['stat' => 'success', "id" => encrypt($Receipt->id)]);
                        } catch (\Exception $e) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                        }
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Receipt = Receipt::find(decrypt($id));
                if (!empty($Receipt)) {

                    // delete related journals
                    $Receipt->journals()->delete();

                    //delete file
                    if ($Receipt->file != "") {
                        if ($Receipt->remote && Helper::FileSystem() != "") {
                            Storage::disk(Helper::FileSystem())->delete($Receipt->file);
                        } else {
                            if (file_exists($this->uploadPath . $Receipt->file)) {
                                File::delete($this->uploadPath . $Receipt->file);
                            }
                        }
                    }

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = (@$Receipt->type) ? 602 : 603,
                        $window = 0,
                        $record_id = @$Receipt->id,
                        $operation = 4,
                        $title = @$Receipt->amount . " " . @$Receipt->currency->symbol . "<br>" . mb_substr(@$Receipt->details, 0, 150)
                    );

                    $Receipt->delete();
                    return json_encode(array("stat" => "success", "id" => $Receipt->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function receipt($id = "")
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_edit")) {
                if ($id != "") {
                    $Receipt = Receipt::find(decrypt($id));
                    if (!empty($Receipt)) {
                        return view("back.accounting.receipts.receipt", compact("Receipt"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function print(Request $request)
    {
        if (Roles::Check("accountant_print")) {

            $search_word = $request->input('search_word');
            $search_type = $request->input('search_type');
            $search_fund = $request->input('search_fund');
            $search_pay_method = $request->input('search_pay_method');
            $search_currency = $request->input('search_currency');
            $search_cat = $request->input('search_cat');
            $search_user = $request->input('search_user');
            $search_start = $request->input('search_start');
            $search_end = $request->input('search_end');
            $search_center_id = $request->input('search_center_id');
            $search_period = $request->input('search_period');
            $search_ledger = $request->input('search_ledger');
            $Fund = [];
            $Currency = [];
            $Period = [];
            $Ledger = [];

            if (Roles::Check("view")) {
                $Receipts = Receipt::where("created_by", @Auth::user()->id);
            } else {
                $Receipts = Receipt::where("id", '>', 0);
            }
            if ($search_type) {
                $Receipts = $Receipts->where("type", true);
            } else {
                $Receipts = $Receipts->where("type", false);
            }

            if ($search_word != "") {
                $Receipts = $Receipts->where(function ($query) use ($search_word) {
                    $query->where("id", $search_word)
                        ->orwhere("details", "like", "%" . $search_word . "%")
                        ->orwhere("name", "like", "%" . $search_word . "%")
                        ->orwhere('code', $search_word);
                });
            }
            if ($search_fund > 0) {
                $Fund = Bank::find($search_fund);
                $Receipts = $Receipts->where("bank_id", $search_fund);
            }
            if ($search_period > 0) {
                $Period = Period::find($search_period);
                $Receipts = $Receipts->where("period_id", $search_period);
            }

            if ($search_ledger > 0) {
                $Ledger = Ledger::find($search_ledger);
                $Receipts = $Receipts->where("ledger_id", $search_ledger);
            }
            if ($search_pay_method > 0) {
                $Receipts = $Receipts->where("method_id", $search_pay_method);
            }
            if ($search_currency > 0) {
                $Currency = Currency::find($search_currency);
                $Receipts = $Receipts->where("currency_id", $search_currency);
            }
            if ($search_cat > 0) {
                $Receipts = $Receipts->where("cat_id", $search_cat);
            }
            if ($search_center_id > 0) {
                $Receipts = $Receipts->where("center_id", $search_center_id);
            }
            if ($search_user > 0) {
                $Receipts = $Receipts->where("created_by", $search_user);
            }
            if ($search_start != "") {
                $search_start_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_start)));
                $Receipts = $Receipts->where('date', ">=", $search_start_date);
            }
            if ($search_end != "") {
                $search_end_date = Helper::DatePrepare(date('Y-m-d', strtotime($search_end)));
                $Receipts = $Receipts->where('date', "<=", $search_end_date);
            }
            $Receipts = $Receipts->orderBy("date", "asc")->get();

            $DefaultCurrency = Currency::find(Helper::Preferences("default_currency"));
            return view("back.accounting.receipts.print", compact("Receipts", "request", "Fund", "Currency", "Period", "DefaultCurrency","Ledger"));
        }
        return "";
    }

    public function export(Request $request)
    {
        if (Roles::Check("accountant_export")) {
            $title = trans('backLang.expenses');
            if ($request->search_type) {
                $title = trans('backLang.revenues');
            }
            return (new ReceiptExport($request))->download($title . '.xlsx');
        }
        return "";
    }

    public function import(Request $request)
    {
        if (Roles::Check("edit")) {
            if (Roles::Check("accountant_import")) {
                if ($request->file != "" && $request->fund_id > 0) {
                    Excel::import(new ReceiptImport($request->fund_id), $request->file('file'));
                    return response()->json(['stat' => 'success']);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function download()
    {
        return Storage::disk('local')->download("xlsx/receipts.xlsx", "receipts" . ".xlsx");
    }

    public function file_preview($id = "")
    {
        if ($id != "") {
            $Receipt = Receipt::find(decrypt($id));
            if (!empty($Receipt)) {
                if ($Receipt->file != "") {
                    if ($Receipt->remote && Helper::FileSystem() != "") {
                        return Storage::disk(Helper::FileSystem())->response($Receipt->file);
                    } else {
                        $file_url = $this->uploadPath . $Receipt->file;
                        if (file_exists($file_url)) {
                            return response()->file($file_url);
                        }
                    }
                }
            }
        }
        return redirect()->route("NotFound");
    }

    public function file_download($id = "")
    {
        if ($id != "") {
            $Receipt = Receipt::find(decrypt($id));
            if (!empty($Receipt)) {
                if ($Receipt->file != "") {

                    if ($Receipt->remote && Helper::FileSystem() != "") {
                        return Storage::disk(Helper::FileSystem())->download($Receipt->file, urldecode($Receipt->file) . "." . $Receipt->ext);
                    } else {
                        $file_url = $this->uploadPath . $Receipt->file;
                        if (file_exists($file_url)) {
                            return response()->download($file_url, urldecode($Receipt->file) . "." . $Receipt->ext);
                        }
                    }
                }
            }
        }
        return redirect()->route("NotFound");
    }
}
