<?php

namespace App\Http\Controllers\Auth;

use Illuminate\Http\Request;
use App\Models\Authorize;
use App\Http\Controllers\Controller;
use App\Mail\AuthorizeDevice;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Redirect;
use Cookie;
use Helper;
use DeviceTrust;

class AuthorizeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        Helper::detectLang();
        $this->middleware('auth');
    }
    /**
     *  Validate the token for the Authorization.
     *
     * @param $token
     * @return \Illuminate\Http\Response
     */
    public function verify($token = null)
    {
        if (Authorize::validateToken($token)) {
            //Save to cookies for 10 years
            Cookie::queue("DkmshTksiWxnodPMs0w2C", encrypt($token), time() + (10 * 365 * 24 * 60 * 60));

            return Redirect::route('adminHome')->with([
                'doneMessage' => trans("backLang.authorizeDone"),
            ]);
        }
        return Redirect::route('login')->with([
            'errorMessage' => trans("backLang.authorizeFailed"),
        ]);
    }
    /**
     * Get the needed authorization credentials from the request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function resend(Request $request)
    {
        if (Authorize::inactive() && auth()->check()) {
            $authorize = Authorize::make()->resetAttempt();
            Mail::to($request->user())->send(new AuthorizeDevice($authorize));
            $authorize->increment('attempt');
            return view('auth.authorize');
        }
    }
}
