<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Alert;
use App\Models\User;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use Validator;
use Cats;
use App\Helpers\History;

class AlertController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function load(Request $request)
    {
        $cat = $request->cat;
        $cat_id = $request->cat_id;
        $cat_title = $request->cat_title;
        $alert_roles = [
            "create" => ($request->role_create == "true" || $request->role_create == "1") ? true : false,
            "edit" => ($request->role_edit == "true" || $request->role_edit == "1") ? true : false,
            "delete" => ($request->role_delete == "true" || $request->role_delete == "1") ? true : false,
        ];
        if ($cat_id != "") {
            $Alerts = Alert::where("cat", $cat)->where("cat_id", decrypt($cat_id))->orderby("id", "asc")->get();
        } else {
            $Alerts = Alert::where("cat", $cat)->where("cat_id", 0)->orderby("id", "asc")->get();
        }
        return view("back.preferences.alerts.list", compact("Alerts", "cat", "cat_id", "cat_title", "alert_roles"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $validator = Validator::make($request->all(), [
                'title' => 'required',
                'date' => 'required',
                'time' => 'required',
            ]);
            if ($validator->passes()) {

                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                $alert_system = $request->system;
                $alert_mail = $request->mail;
                $alert_sms = $request->sms;
                $alert_browser = $request->browser;
                $alert_date = date('Y-m-d H:i:s', strtotime($request->date . " " . $request->time));

                if ($alert_date > date('Y-m-d H:i:s')) {
                    if ($alert_system || $alert_mail || $alert_sms || $alert_browser) {
                        if ($cat_id >= 0) {
                            $Alert = new Alert;
                            $Alert->cat = $request->cat;
                            $Alert->cat_id = $cat_id;
                            $Alert->status = false;
                            $Alert->alert_date = $alert_date;
                            $Alert->title = $request->title;
                            $Alert->system = ($alert_system) ? 1 : 0;
                            $Alert->mail = ($alert_mail) ? 1 : 0;
                            $Alert->sms = ($alert_sms) ? 1 : 0;
                            $Alert->browser = ($alert_browser) ? 1 : 0;
                            $Alert->created_ip = \Request::ip();
                            $Alert->created_by = Auth::user()->id;
                            $Alert->save();

                            // Log to history
                            History::Log(
                                $module = @Cats::AlertsTypes()[$Alert->cat]['history']['module'],
                                $cat = @Cats::AlertsTypes()[$Alert->cat]['history']['cat'],
                                $window = @Cats::AlertsTypes()[$Alert->cat]['history']['window'],
                                $record_id = $Alert->id,
                                $operation = 2,
                                $title = $Alert->title."<br>".date('Y-m-d h:i a', strtotime($Alert->alert_date))
                            );

                        }
                        return response()->json(['stat' => 'success']);
                    } else {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.selectAlertChannels')]]);
                    }
                } else {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.alertDateNotSuitable')]]);
                }
            } else {
                return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  string $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Alert = Alert::find(decrypt($id));
                if (!empty($Alert)) {
                    return view("back.preferences.alerts.edit", compact("Alert"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $Alert = Alert::find(decrypt($request->id));
            if (!empty($Alert)) {
                if ($Alert->created_by == @Auth::user()->id) {
                    $validator = Validator::make($request->all(), [
                        'title' => 'required',
                        'date' => 'required',
                        'time' => 'required',
                    ]);
                    if ($validator->passes()) {

                        if ($request->cat_id != "") {
                            $cat_id = decrypt($request->cat_id);
                        } else {
                            $cat_id = 0;
                        }
                        $alert_system = $request->system;
                        $alert_mail = $request->mail;
                        $alert_sms = $request->sms;
                        $alert_browser = $request->browser;
                        $alert_date = date('Y-m-d H:i:s', strtotime($request->date . " " . $request->time));

                        if ($alert_date > date('Y-m-d H:i:s')) {
                            if ($alert_system || $alert_mail || $alert_sms || $alert_browser) {
                                if ($cat_id >= 0) {
                                    $Alert->status = false;
                                    $Alert->alert_date = $alert_date;
                                    $Alert->title = $request->title;
                                    $Alert->system = ($alert_system) ? 1 : 0;
                                    $Alert->mail = ($alert_mail) ? 1 : 0;
                                    $Alert->sms = ($alert_sms) ? 1 : 0;
                                    $Alert->browser = ($alert_browser) ? 1 : 0;
                                    $Alert->updated_ip = \Request::ip();
                                    $Alert->updated_by = Auth::user()->id;
                                    $Alert->save();

                                    // Log to history
                                    History::Log(
                                        $module = @Cats::AlertsTypes()[$Alert->cat]['history']['module'],
                                        $cat = @Cats::AlertsTypes()[$Alert->cat]['history']['cat'],
                                        $window = @Cats::AlertsTypes()[$Alert->cat]['history']['window'],
                                        $record_id = $Alert->id,
                                        $operation = 3,
                                        $title = $Alert->title."<br>".date('Y-m-d h:i a', strtotime($Alert->alert_date))
                                    );
                                }
                                return response()->json(['stat' => 'success']);
                            } else {
                                return response()->json(['stat' => 'error', 'error' => [trans('backLang.selectAlertChannels')]]);
                            }
                        } else {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.alertDateNotSuitable')]]);
                        }
                    } else {
                        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                    }
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Alert = Alert::find(decrypt($id));
                if (!empty($Alert)) {
                    if ($Alert->created_by == @Auth::user()->id) {

                        // Log to history
                        History::Log(
                            $module = @Cats::AlertsTypes()[$Alert->cat]['history']['module'],
                            $cat = @Cats::AlertsTypes()[$Alert->cat]['history']['cat'],
                            $window = @Cats::AlertsTypes()[$Alert->cat]['history']['window'],
                            $record_id = $Alert->id,
                            $operation = 4,
                            $title = $Alert->title."<br>".date('Y-m-d h:i a', strtotime($Alert->alert_date))
                        );

                        $Alert->delete();
                        return json_encode(array("stat" => "success", "id" => $Alert->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
