<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Attachment;
use App\Models\Share;
use App\Models\Category;
use Auth;
use Helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use Redirect;
use Cats;
use Roles;
use Validator;
use File;
use App\Helpers\History;

class AttachmentController extends Controller
{

    private $uploadPath = "uploads/attachments/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index($cat = "mine")
    {
        $mine = 0;
        if ($cat != "mine") {
            try {
                $cat = decrypt($cat);
            } catch (\Exception $e) {
                $cat = 0;
            }
        } else {
            $cat = 0;
            $mine = 1;
        }
        $cat_title = trans('backLang.myFiles');

        foreach (Cats::AttachmentTypes() as $key => $AttachmentType) {
            if ($key === $cat && $mine != 1) {
                $cat_title = $AttachmentType['title'];
            }
        }

        $attachments_roles = [
            "create" => Roles::Check("archive_files_upload"),
            "download" => Roles::Check("archive_files_download"),
            "edit" => Roles::Check("archive_files_edit"),
            "delete" => Roles::Check("archive_files_delete"),
            "multiple" => false,
            "folder_id" => 0,
        ];
        $cat_id = encrypt(Auth::user()->id);
        $UsersList = User::where("status", true)->orderby("name", "asc")->get();
        return view("back.preferences.attachments.home", compact("cat", "cat_id", "cat_title", "attachments_roles", "mine", "UsersList"));
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'created_at',
            1 => 'title',
            2 => 'created_by',
            3 => 'size',
            4 => 'options',
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $cat = $request->input('cat');
        $folder_id = $request->input('folder_id');
        $mine = $request->input('mine');
        $search_category = $request->input('category');
        $search_user_id = $request->input('user_id');
        $search_word = $request->input('search_word');
        $search_stat = $request->input('search_stat');
        $search_cat_id = $request->input('search_cat_id');

        try {
            $folder_id = decrypt($folder_id);
        } catch (\Exception $e) {
            $folder_id = 0;
        }
        try {
            $cat = decrypt($cat);
        } catch (\Exception $e) {
            $cat = 0;
        }
        try {
            if ($search_stat) {
                $cat_id = decrypt($search_cat_id);
            } else {
                $cat_id = 0;
            }
        } catch (\Exception $e) {
            $cat_id = 0;
        }

        if ($cat == 0 && $mine) {
            $cat_id = Auth::user()->id;
        }

        $allowed_Atypes = "";
        $allowed_Atypes_array = [];
        foreach (Cats::AttachmentTypes() as $key => $AttachmentType) {
            if (!$AttachmentType['hidden']) {
                if (Roles::Check($AttachmentType['role'])) {
                    $allowed_Atypes .= $key . ",";
                }
            }
        }
        if ($allowed_Atypes != "") {
            $allowed_Atypes = rtrim($allowed_Atypes, ",");
            $allowed_Atypes_array = explode(",", $allowed_Atypes);
        }

        $Attachments = Attachment::where("id", 0);
        if (in_array($cat, $allowed_Atypes_array) || $mine) {

            if (Roles::Check("view")) {
                $Attachments = Attachment::where("created_by", @Auth::user()->id)->where("cat", $cat)->where("folder_id", $folder_id);
            } else {
                $Attachments = Attachment::where("cat", $cat)->where("folder_id", $folder_id);
            }
            if ($cat_id < 1) {
                $cat_id = 0;
            }

            if ($cat_id > 0) {
                if ($cat != 0 || $mine || $search_stat) {
                    $Attachments_shared_ids = Share::select('cat_id')->where("cat", 1)->where("user_id", ($cat == 0) ? $cat_id : @Auth::user()->id)->get();
                    $Attachments = $Attachments->where(function ($query) use ($cat_id, $Attachments_shared_ids) {
                        $query->where('cat_id', $cat_id)->orWhereIn('id', $Attachments_shared_ids);
                    });
                }
            }
            if ($search_category > 0) {
                $Attachments = $Attachments->where("category_id", $search_category);
            }
            if ($search_user_id > 0) {
                $Attachments = $Attachments->where("created_by", $search_user_id);
            }
            if ($search_word != "") {
                $Attachments = $Attachments->where(function ($query) use ($search_word) {
                    $query->where('title', 'like', '%' . $search_word . '%')
                        ->orWhere('details', 'like', '%' . $search_word . '%');
                });
            }
        }
        $totalData = $Attachments->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Attachments = $Attachments->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Attachments as $Attachment) {
                if ($Attachment->folder) {
                    $file_url = "javascript:void(0)";
                    $file_click = "onclick=\"open_folder('" . encrypt($Attachment->id) . "')\"";
                    $icon = "<i class=\"fa fa-folder-o\" style=\"font-size: 40px\"></i>";
                } else {
                    $file_url = route("AppAttachmentsView", encrypt($Attachment->id));
                    $file_click = "";
                    if (Helper::IsPhoto($Attachment->ext) && !$Attachment->remote) {
                        $icon = "<img src=\"" . route("AppAttachmentsPreview", encrypt($Attachment->id)) . "\" style=\"max-height: 40px;max-width:60px\">";
                    } else {
                        $icon = Helper::GetIconFromExt($Attachment->ext, $size = 40);
                    }
                }
                $nestedData['folder'] = "<div class='text-center'><a href='" . $file_url . "' " . $file_click . ">" . $icon . "</a></div>";
                $download = "";
                if (!$Attachment->folder) {
                    if (Roles::Check("archive_files_download")) {
                        $download = "
 <a class=\"btn btn-sm btn-light pull-right\" href=\"" . route("AppAttachmentsDownload", encrypt($Attachment->id)) . "\"  data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.download") . "\" title=\"" . trans("backLang.download") . "\"><i class=\"fe fe-download\"></i></a>";
                    }
                }

                $nestedData['title'] = $download . "<a href='" . $file_url . "' " . $file_click . ">" . $Attachment->title . "</a><div class='small text-muted'>" . Helper::DateView($Attachment->created_at, 1) . "</div>";

                // shared
                $Attachment_shared = Share::where("cat", 1)->where("cat_id", $Attachment->id)->whereColumn('user_id', '!=', 'created_by')->get();
                $shared_users = "<div class=\"text-center avatar-list avatar-list-stacked ml-2 mr-2\">";
                if (!empty($Attachment->user)) {
                    $share_user_avatar = URL::to('uploads/users/profile.jpg');
                    if ($Attachment->user->photo != "") {
                        $share_user_avatar = URL::to('uploads/users/' . $Attachment->user->photo);
                    }
                    $shared_users .= "<img class=\"avatar avatar-sm\" src=\"" . $share_user_avatar . "\" data-toggle=\"tooltip\" title=\"" . $Attachment->user->name . "\" data-original-title=\"" . $Attachment->user->name . "\"/>";
                }
                $ci = 1;
                $total_ci = count($Attachment_shared);
                if (count($Attachment_shared) > 0) {
                    foreach ($Attachment_shared as $Attachment_share) {
                        if (!empty($Attachment_share->user)) {
                            if ($ci < 5) {
                                $share_user_avatar = URL::to('uploads/users/profile.jpg');
                                if ($Attachment_share->user->photo != "") {
                                    $share_user_avatar = URL::to('uploads/users/' . $Attachment_share->user->photo);
                                }
                                $shared_users .= "<img class=\"avatar avatar-sm\" src=\"" . $share_user_avatar . "\" data-toggle=\"tooltip\" title=\"" . $Attachment_share->user->name . "\" data-original-title=\"" . $Attachment_share->user->name . "\"/>";
                                $ci++;
                                $total_ci--;
                            }
                        }
                    }
                }
                if ($total_ci > 0) {
                    $shared_users .= "<span class=\"avatar avatar-sm text-light\">+" . $total_ci . "</span>";
                }
                $shared_users .= "</div>";

                $nestedData['created_by'] = $shared_users;


                if ($Attachment->folder) {
                    $nestedData['size'] = "<div class='text-center' dir='ltr'> - </div>";
                } else {
                    $nestedData['size'] = "<div class='text-center' dir='ltr'> " . Helper::formatFileSize($Attachment->size, 2) . " </div>";
                }

                $options = "<div class=\"text-center\">";

                if (Roles::Check("archive_files_edit")) {
                    if (Roles::Check("edit")) {
                        $options .= " <button class=\"btn btn-sm btn-light\" onclick=\"edit_row('" . encrypt($Attachment->id) . "')\"  data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.edit") . "\" title=\"" . trans("backLang.edit") . "\"><i class=\"icon-pencil\"></i></button>";
                    }
                }


                if (Roles::Check("archive_files_delete")) {
                    if (Roles::Check("delete")) {
                        $options .= " <button class=\"btn btn-sm btn-light\" onclick=\"delete_row('" . encrypt($Attachment->id) . "')\"  data-toggle=\"tooltip\" data-original-title=\"" . trans("backLang.delete") . "\" title=\"" . trans("backLang.delete") . "\"><i class=\"fe fe-trash\"></i></button>";
                    }
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $folder_tree = "";
        if ($folder_id > 0) {
            if (count(Helper::FolderTree($folder_id)) > 0) {
                $folder_tree = "<div class=\"border mt-3\">
                <nav aria-label=\"breadcrumb\">
                    <ol class=\"breadcrumb mb-0\">
                        <li class=\"breadcrumb-item\"><a href=\"javascript:void(0)\" onclick=\"open_folder('" . encrypt(0) . "')\"> <i class=\"fa fa-folder-o\"></i> " . trans("backLang.rootFolder") . "</a></li>";
                foreach (array_reverse(Helper::FolderTree($folder_id)) as $FatherFolder) {
                    $folder_tree .= "<li class=\"breadcrumb-item\"><a href=\"javascript:void(0)\" onclick=\"open_folder('" . encrypt($FatherFolder['id']) . "')\" class='text-info'><i class=\"fa fa-folder-o\"></i> " . $FatherFolder['title'] . "</a></li>
                        ";
                }
                $folder_tree .= "
                    </ol>
                </nav>
            </div>
";
            }
        }

        //total statics
        if (Roles::Check("view")) {
            $Attachments = Attachment::where("created_by", @Auth::user()->id)->wherein("cat", $allowed_Atypes_array);
        } else {
            $Attachments = Attachment::where("id", '>', 0)->wherein("cat", $allowed_Atypes_array);
        }
        $statics_total_count = $Attachments->count();
        $statics_total_size = $Attachments->sum('size');
        $total_statics = [
            "count" => $statics_total_count,
            "size" => Helper::formatFileSize($statics_total_size)
        ];
        //statics calc
        $statics = [];
        foreach ($allowed_Atypes_array as $AttachmentType) {
            //statics
            if (Roles::Check("view")) {
                $Attachments = Attachment::where("created_by", @Auth::user()->id)->where("cat", $AttachmentType);
            } else {
                $Attachments = Attachment::where("cat", $AttachmentType);
            }
            $s_count = $Attachments->count();
            $s_size = $Attachments->sum('size');
            $statics[] = [
                "type" => $AttachmentType,
                "count" => $s_count,
                "size" => Helper::formatFileSize($s_size),
                "percent" => ($statics_total_size > 0) ? round($s_size * 100 / $statics_total_size) : 0
            ];
        }
        //my statics
        if (Roles::Check("view")) {
            $Attachments = Attachment::where("created_by", @Auth::user()->id)->where("cat", 0)->where("cat_id", Auth::user()->id);
        } else {
            $Attachments = Attachment::where("cat", 0)->where("cat_id", Auth::user()->id);
        }
        if ($cat_id > 0) {
            $Attachments = $Attachments->where('cat_id', $cat_id);
        }
        $s_count = $Attachments->count();
        $s_size = $Attachments->sum('size');
        $my_statics = [
            "count" => $s_count,
            "size" => Helper::formatFileSize($s_size),
            "percent" => ($statics_total_size > 0) ? round($s_size * 100 / $statics_total_size) : 0
        ];


        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "tree" => $folder_tree,
            "statics" => $statics,
            "my_statics" => $my_statics,
            "total_statics" => $total_statics
        );

        return json_encode($json_data);

    }

    public function load(Request $request)
    {
        $cat = $request->cat;
        $cat_id = $request->cat_id;
        $folder_id = 0;
        if ($request->folder_id != "") {
            $folder_id = decrypt($request->folder_id);
        }
        $attachments_roles = [
            "create" => ($request->role_create == "true" || $request->role_create == "1") ? true : false,
            "download" => ($request->role_download == "true" || $request->role_download == "1") ? true : false,
            "edit" => ($request->role_edit == "true" || $request->role_edit == "1") ? true : false,
            "delete" => ($request->role_delete == "true" || $request->role_delete == "1") ? true : false,
            "multiple" => ($request->role_multiple == "true" || $request->role_multiple == "1") ? true : false,
            "folder_id" => $folder_id,
        ];
        if ($cat_id != "") {
            $Attachments = Attachment::where("cat", $cat)->where("cat_id", decrypt($cat_id))->where("folder_id", $folder_id)->orderby("id", "desc")->get();
        } else {
            $Attachments = Attachment::where("cat", $cat)->where("cat_id", 0)->where("folder_id", $folder_id)->orderby("id", "desc")->get();
        }
        $UsersList = User::where("status", true)->where("id", "!=", @Auth::user()->id)->orderby("name", "asc")->get();
        return view("back.preferences.attachments.list", compact("Attachments", "cat", "cat_id", "attachments_roles", "UsersList"));
    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                $folder_id = 0;
                if ($request->folder_id != "") {
                    $folder_id = decrypt($request->folder_id);
                }

                if ($cat_id >= 0) {
                    if ($request->file != "") {
                        $extensions = explode(",", Helper::Preferences("allowed_ext"));
                        $result = array($request->file('file')->getClientOriginalExtension());
                        if (!in_array(strtolower($result[0]), $extensions)) {
                            return response()->json(['stat' => 'error', 'error' => [$result[0] . " : " . trans('backLang.fileTypeNotAllowed')]]);
                        }
                    }

                    // Start of Upload Files
                    $formFileName = "file";
                    $path = $this->uploadPath;
                    $fileFinalTitle = basename($request->file($formFileName)->getClientOriginalName(),
                        '.' . $request->file($formFileName)->getClientOriginalExtension());
                    $fileFinalRand = time() . rand(1111, 9999);
                    $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                    $fileSize = $request->file($formFileName)->getSize();
                    $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);


                    if ($uploaded) {
                        $Attachment = new Attachment;
                        $Attachment->folder = 0;
                        $Attachment->folder_id = $folder_id;
                        $Attachment->file = $fileFinalRand;
                        $Attachment->ext = $fileFinalExt;
                        $Attachment->title = $request->title;
                        $Attachment->details = $request->details;
                        $Attachment->size = $fileSize;
                        $Attachment->remote = 0;
                        $Attachment->cat = $request->cat;
                        $Attachment->cat_id = $cat_id;
                        $Attachment->category_id = 0;
                        $Attachment->status = true;
                        $Attachment->hidden = false;
                        $Attachment->share = false;
                        $Attachment->created_ip = \Request::ip();
                        $Attachment->created_by = Auth::user()->id;
                        $Attachment->save();


                        //self Share
                        $Share = Share::where("cat", 1)->where("cat_id", $Attachment->id)->where("user_id", Auth::user()->id)->first();
                        if (empty($Share)) {
                            $Share = new Share;
                            $Share->cat = 1;
                            $Share->cat_id = $Attachment->id;
                            $Share->user_id = Auth::user()->id;
                            $Share->status = true;
                            $Share->created_ip = \Request::ip();
                            $Share->created_by = Auth::user()->id;
                            $Share->save();
                        }

                        // save sharing like father
                        if ($Attachment->folder_id > 0) {
                            $father_folder = Attachment::find($Attachment->folder_id);
                            if (!empty($father_folder)) {
                                if ($father_folder->share) {
                                    $FatherShares = Share::where("cat", 1)->where("cat_id", $father_folder->id)->get();
                                    if (count($FatherShares) > 0) {
                                        foreach ($FatherShares as $FatherShare) {
                                            $Share = new Share;
                                            $Share->cat = 1;
                                            $Share->cat_id = $Attachment->id;
                                            $Share->user_id = $FatherShare->user_id;
                                            $Share->status = true;
                                            $Share->created_ip = \Request::ip();
                                            $Share->created_by = Auth::user()->id;
                                            $Share->save();
                                        }
                                    }
                                }
                            }
                        }

                        //Remote storage
                        if (Helper::FileSystem() != "") {
                            $filepath = $path . $fileFinalRand;
                            $localFile = File::get($filepath);
                            $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                            if ($Remote_store) {
                                $Attachment->remote = "1";
                                $Attachment->save();
                                // Delete from local
                                File::delete($filepath);
                            }
                        }


                        // Log to history
                        History::Log(
                            $module = @Cats::AttachmentTypes()[$Attachment->cat]['history']['module'],
                            $cat = @Cats::AttachmentTypes()[$Attachment->cat]['history']['cat'],
                            $window = @Cats::AttachmentTypes()[$Attachment->cat]['history']['window'],
                            $record_id = $Attachment->id,
                            $operation = 2,
                            $title = $Attachment->title . "<br>" . Helper::formatFileSize($Attachment->size, 2)
                        );

                        return response()->json(['stat' => 'success', 'id' => encrypt($Attachment->id)]);
                    }
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function upload(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                $folder_id = 0;
                if ($request->folder_id != "") {
                    $folder_id = decrypt($request->folder_id);
                }

                if ($cat_id >= 0) {
                    if ($request->file != "") {
                        $extensions = explode(",", Helper::Preferences("allowed_ext"));
                        $result = array($request->file('file')->getClientOriginalExtension());
                        if (!in_array(strtolower($result[0]), $extensions)) {
                            return response()->json('error', 400);
                        }
                    }

                    // Start of Upload Files
                    $formFileName = "file";
                    $path = $this->uploadPath;
                    $fileFinalTitle = basename($request->file($formFileName)->getClientOriginalName(),
                        '.' . $request->file($formFileName)->getClientOriginalExtension());
                    $fileFinalRand = time() . rand(1111, 9999);
                    $fileFinalExt = strtolower($request->file($formFileName)->getClientOriginalExtension());
                    $fileSize = $request->file($formFileName)->getSize();
                    $uploaded = $request->file($formFileName)->move($path, $fileFinalRand);

                    if ($uploaded) {
                        $Attachment = new Attachment;
                        $Attachment->folder = 0;
                        $Attachment->folder_id = $folder_id;
                        $Attachment->file = $fileFinalRand;
                        $Attachment->ext = $fileFinalExt;
                        $Attachment->title = $fileFinalTitle;
                        $Attachment->details = "";
                        $Attachment->size = $fileSize;
                        $Attachment->remote = 0;
                        $Attachment->cat = $request->cat;
                        $Attachment->cat_id = $cat_id;
                        $Attachment->category_id = 0;
                        $Attachment->status = true;
                        $Attachment->hidden = false;
                        $Attachment->share = false;
                        $Attachment->created_ip = \Request::ip();
                        $Attachment->created_by = Auth::user()->id;
                        $Attachment->save();

                        //Share with others
                        if ($request->share_with != "") {
                            $share_with = $request->share_with;
                            if ($share_with > 0 && $Attachment->id >= 0) {
                                $Share = Share::where("cat", 1)->where("cat_id", $Attachment->id)->where("user_id", $share_with)->first();
                                if (empty($Share)) {
                                    $Share = new Share;
                                    $Share->cat = 1;
                                    $Share->cat_id = $Attachment->id;
                                    $Share->user_id = $share_with;
                                    $Share->status = true;
                                    $Share->created_ip = \Request::ip();
                                    $Share->created_by = Auth::user()->id;
                                    $Share->save();
                                }
                            }
                        }

                        //self Share
                        $Share = Share::where("cat", 1)->where("cat_id", $Attachment->id)->where("user_id", Auth::user()->id)->first();
                        if (empty($Share)) {
                            $Share = new Share;
                            $Share->cat = 1;
                            $Share->cat_id = $Attachment->id;
                            $Share->user_id = Auth::user()->id;
                            $Share->status = true;
                            $Share->created_ip = \Request::ip();
                            $Share->created_by = Auth::user()->id;
                            $Share->save();
                        }

                        if ($Attachment->folder_id > 0) {
                            $father_folder = Attachment::find($Attachment->folder_id);
                            if (!empty($father_folder)) {
                                if ($father_folder->share) {
                                    $FatherShares = Share::where("cat", 1)->where("cat_id", $father_folder->id)->get();
                                    if (count($FatherShares) > 0) {
                                        foreach ($FatherShares as $FatherShare) {
                                            $Share = new Share;
                                            $Share->cat = 1;
                                            $Share->cat_id = $Attachment->id;
                                            $Share->user_id = $FatherShare->user_id;
                                            $Share->status = true;
                                            $Share->created_ip = \Request::ip();
                                            $Share->created_by = Auth::user()->id;
                                            $Share->save();
                                        }
                                    }
                                }
                            }
                        }


                        //Remote storage
                        if (Helper::FileSystem() != "") {
                            $filepath = $path . $fileFinalRand;
                            $localFile = File::get($filepath);
                            $Remote_store = Storage::disk(Helper::FileSystem())->put($fileFinalRand, $localFile);
                            if ($Remote_store) {
                                $Attachment->remote = "1";
                                $Attachment->save();
                                // Delete from local
                                File::delete($filepath);
                            }
                        }

                        // Log to history
                        History::Log(
                            $module = @Cats::AttachmentTypes()[$Attachment->cat]['history']['module'],
                            $cat = @Cats::AttachmentTypes()[$Attachment->cat]['history']['cat'],
                            $window = @Cats::AttachmentTypes()[$Attachment->cat]['history']['window'],
                            $record_id = $Attachment->id,
                            $operation = 2,
                            $title = $Attachment->title . "<br>" . Helper::formatFileSize($Attachment->size, 2)
                        );

                        return response()->json(['success' => 200, 'id' => $Attachment->id]);

                    }
                }
            } catch (\Exception $e) {
                return response()->json('error', 400);
            }
        }
        return response()->json('error', 400);
    }

    public function folder(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }

                $folder_id = 0;
                if ($request->folder_id != "") {
                    $folder_id = decrypt($request->folder_id);
                }

                if ($cat_id >= 0) {
                    $validator = Validator::make($request->all(), [
                        'title' => 'required'
                    ]);
                    if ($validator->passes()) {
                        $Attachment = new Attachment;
                        $Attachment->folder = 1;
                        $Attachment->folder_id = $folder_id;
                        $Attachment->file = "";
                        $Attachment->ext = "";
                        $Attachment->title = $request->title;
                        $Attachment->details = $request->details;
                        $Attachment->size = 0;
                        $Attachment->remote = 0;
                        $Attachment->cat = $request->cat;
                        $Attachment->cat_id = $cat_id;
                        $Attachment->category_id = $request->category_id;
                        $Attachment->status = true;
                        $Attachment->hidden = false;
                        $Attachment->share = ($request->share) ? 1 : 0;
                        $Attachment->created_ip = \Request::ip();
                        $Attachment->created_by = Auth::user()->id;
                        $Attachment->save();

                        // save sharing
                        if ($request->user_ids != "") {
                            foreach ($request->user_ids as $user_id) {
                                $user_id = decrypt($user_id);
                                $Share = new Share;
                                $Share->cat = 1;
                                $Share->cat_id = $Attachment->id;
                                $Share->user_id = $user_id;
                                $Share->status = true;
                                $Share->created_ip = \Request::ip();
                                $Share->created_by = Auth::user()->id;
                                $Share->save();
                            }
                        }

                        // Log to history
                        History::Log(
                            $module = @Cats::AttachmentTypes()[$Attachment->cat]['history']['module'],
                            $cat = @Cats::AttachmentTypes()[$Attachment->cat]['history']['cat'],
                            $window = @Cats::AttachmentTypes()[$Attachment->cat]['history']['window'],
                            $record_id = $Attachment->id,
                            $operation = 2,
                            $title = $Attachment->title
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Attachment->id)]);
                    } else {
                        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                    }
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Attachment = Attachment::find(decrypt($id));
                if (!empty($Attachment)) {
                    if ($Attachment->created_by == Auth::user()->id) {

                        function run_delete($item, $path)
                        {
                            if ($item->file != "") {
                                if ($item->remote && Helper::FileSystem() != "") {
                                    Storage::disk(Helper::FileSystem())->delete($item->file);
                                } else {
                                    if (file_exists($path . $item->file)) {
                                        File::delete($path . $item->file);
                                    }
                                }
                            }
                            $item->shares()->delete();
                            $item->comments()->delete();
                            $item->alerts()->delete();
                            $item->delete();


                            // Log to history
                            History::Log(
                                $module = @Cats::AttachmentTypes()[$item->cat]['history']['module'],
                                $cat = @Cats::AttachmentTypes()[$item->cat]['history']['cat'],
                                $window = @Cats::AttachmentTypes()[$item->cat]['history']['window'],
                                $record_id = $item->id,
                                $operation = 4,
                                $title = $item->title . "<br>" . Helper::formatFileSize($item->size, 2)
                            );

                        }

                        // delete all sub
                        $FolderSubItems = Helper::FolderSubItems($Attachment->id);
                        if (count($FolderSubItems) > 0) {
                            foreach ($FolderSubItems as $FolderSubItem) {
                                run_delete(Attachment::find($FolderSubItem['id']), $this->uploadPath);
                            }
                        }

                        // Delete main folder
                        run_delete($Attachment, $this->uploadPath);
                    } else {
                        Share::where("cat", 1)->where("cat_id", $Attachment->id)->where("user_id", Auth::user()->id)->delete();
                    }
                    return json_encode(array("stat" => "success", "id" => $Attachment->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Attachment = Attachment::find(decrypt($id));
                if (!empty($Attachment)) {
                    $UsersList = User::where("status", true)->where("id", "!=", @Auth::user()->id)->orderby("name", "asc")->get();
                    $Shares = Share::where("cat", 1)->where("cat_id", $Attachment->id)->orderby("id", "asc")->get();
                    return view("back.preferences.attachments.edit", compact("Attachment", "Shares", "UsersList"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $Attachment = Attachment::find(decrypt($request->id));
            if (!empty($Attachment)) {
                $validator = Validator::make($request->all(), [
                    'title' => 'required'
                ]);
                if ($validator->passes()) {
                    $Attachment->category_id = $request->category_id;
                    $Attachment->title = $request->title;
                    $Attachment->details = $request->details;
                    $Attachment->share = ($request->share) ? 1 : 0;
                    $Attachment->updated_ip = \Request::ip();
                    $Attachment->updated_by = Auth::user()->id;
                    $Attachment->save();

                    if ($Attachment->created_by == Auth::user()->id) {
                        //Delete old shares
                        Share::where("cat", 1)->where("cat_id", $Attachment->id)->delete();
                        //save shares
                        if ($request->user_ids != "") {
                            foreach ($request->user_ids as $user_id) {
                                $user_id = decrypt($user_id);
                                if ($user_id > 0) {
                                    $Share = new Share;
                                    $Share->cat = 1;
                                    $Share->cat_id = $Attachment->id;
                                    $Share->user_id = $user_id;
                                    $Share->status = true;
                                    $Share->created_ip = \Request::ip();
                                    $Share->created_by = Auth::user()->id;
                                    $Share->save();
                                }
                            }
                        }

                        // run same share for all sub items
                        if ($Attachment->share && $Attachment->folder) {
                            $FolderSubItems = Helper::FolderSubItems($Attachment->id);
                            if (count($FolderSubItems) > 0) {
                                foreach ($FolderSubItems as $FolderSubItem) {
                                    //Delete old shares
                                    Share::where("cat", 1)->where("cat_id", $FolderSubItem['id'])->delete();
                                    //save shares
                                    if ($request->user_ids != "") {
                                        foreach ($request->user_ids as $user_id) {
                                            $user_id = decrypt($user_id);
                                            if ($user_id > 0) {
                                                $Share = new Share;
                                                $Share->cat = 1;
                                                $Share->cat_id = $FolderSubItem['id'];
                                                $Share->user_id = $user_id;
                                                $Share->status = true;
                                                $Share->created_ip = \Request::ip();
                                                $Share->created_by = Auth::user()->id;
                                                $Share->save();
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }


                    // Log to history
                    History::Log(
                        $module = @Cats::AttachmentTypes()[$Attachment->cat]['history']['module'],
                        $cat = @Cats::AttachmentTypes()[$Attachment->cat]['history']['cat'],
                        $window = @Cats::AttachmentTypes()[$Attachment->cat]['history']['window'],
                        $record_id = $Attachment->id,
                        $operation = 3,
                        $title = $Attachment->title . "<br>" . Helper::formatFileSize($Attachment->size, 2)
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Attachment->id)]);
                } else {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function view($id = "")
    {
        if ($id != "") {
            $Attachment = Attachment::where('folder', false)->find(decrypt($id));
            if (!empty($Attachment)) {
                $Attachment_is_shared = Share::where("cat", 1)->where("cat_id", $Attachment->id)->where("user_id", @Auth::user()->id)->first();

                if ($Attachment->created_by == @Auth::user()->id || !empty($Attachment_is_shared) || ($Attachment->cat = 1 && Roles::Check("profile_tab_files"))) {
                    return view("back.preferences.attachments.view", compact("Attachment"));
                } else {
                    return redirect()->route("AppAttachments");
                }
            }
        }
        return redirect()->route("NotFound");
    }

    public function preview($id = "")
    {
        if ($id != "") {
            $Attachment = Attachment::find(decrypt($id));
            if (!empty($Attachment)) {
                if ($Attachment->file != "") {
                    if ($Attachment->remote && Helper::FileSystem() != "") {
                        return Storage::disk(Helper::FileSystem())->response($Attachment->file);
                    } else {
                        $file_url = $this->uploadPath . $Attachment->file;
                        if (file_exists($file_url)) {
                            return response()->file($file_url);
                        }
                    }
                }
            }
        }
        return redirect()->route("NotFound");
    }

    public function download($id = "")
    {
        if ($id != "") {
            $Attachment = Attachment::find(decrypt($id));
            if (!empty($Attachment)) {
                if ($Attachment->file != "") {

                    // Log to history
                    History::Log(
                        $module = @Cats::AttachmentTypes()[$Attachment->cat]['history']['module'],
                        $cat = @Cats::AttachmentTypes()[$Attachment->cat]['history']['cat'],
                        $window = @Cats::AttachmentTypes()[$Attachment->cat]['history']['window'],
                        $record_id = $Attachment->id,
                        $operation = 7,
                        $title = $Attachment->title . "<br>" . Helper::formatFileSize($Attachment->size, 2)
                    );

                    if ($Attachment->remote && Helper::FileSystem() != "") {
                        return Storage::disk(Helper::FileSystem())->download($Attachment->file, urldecode($Attachment->title) . "." . $Attachment->ext);
                    } else {
                        $file_url = $this->uploadPath . $Attachment->file;
                        if (file_exists($file_url)) {
                            return response()->download($file_url, urldecode($Attachment->title) . "." . $Attachment->ext);
                        }
                    }
                }
            }
        }
        return redirect()->route("NotFound");
    }
}
