<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Models\Preference;
use App\Http\Requests;
use App\Http\Controllers\Controller;
use Auth;
use Cookie;
use File;
use Helper;
use Roles;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Redirect;
use DeviceTrust;
use App\Helpers\History;
use App\Helpers\Backup;

class BackupController extends Controller

{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        //Helper::windowPermissionCheck("app_settings");
        // confirm password
        Helper::confirmPassword("AppBackup");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $Preferences = Preference::find(1);
        return view("back.preferences.backup", compact("Preferences"));
    }

    public function backup_update(Request $request)
    {
        //
        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->mysql_path = $request->mysql_path;
            $Preference->mysqldump_path = $request->mysqldump_path;
            $Preference->backup_type = $request->backup_type;
            $Preference->backup_ftp = $request->backup_ftp;

            $Preference->save();

            // Update .env file
            $env_update = Helper::changeEnv([
                'MYSQL_PATH' => $request->mysql_path,
                'MYSQLDUMP_PATH' => $request->mysqldump_path,
                'BACKUP_TYPE' => $request->backup_type
            ]);

            // Log to history
            History::Log(
                $module = 1,
                $cat = 111,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = $Preference->mysql_path . "<br>" . $Preference->mysqldump_path
            );

            return redirect()->action('Preferences\BackupController@index')
                ->with('doneMessage', trans('backLang.saveDone'));
        }
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'title',
            1 => 'file',
            2 => 'created_at',
            3 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');


        try {
            $Backups = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];
        } catch (\Exception $e) {
            $Backups = [];
        }

        $totalData = count($Backups);
        $totalFiltered = $totalData;
        if ($dir == "asc") {
            $Backups = collect($Backups)->sortBy($order)->toArray();
        } else {
            $Backups = collect($Backups)->sortBy($order)->reverse()->toArray();
        }

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Backups as $Backup) {
                $nestedData['title'] = "<button class=\"btn btn-sm btn-icon on-default m-r-5 button-edit pull-right\" onclick=\"edit_row('" . encrypt($Backup->file) . "','" . $Backup->title . "')\" title='" . trans("backLang.edit") . "'><i class=\"icon-pencil\" aria-hidden=\"true\"></i></button>" . $Backup->title;
                $nestedData['file'] = $Backup->file;
                $nestedData['created_at'] = "<div class='text-center'> " . Helper::DateView($Backup->created_at, 1) . " </div>";
                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
<button class=\"btn btn-sm btn-primary\"  onclick=\"restore_row('" . encrypt($Backup->file) . "')\">
<i class=\"icon-clock\"></i> " . trans('backLang.restore') . "
</button>";
                }

                if (Roles::Check("add")) {
                    $options .= "
<a class=\"btn btn-sm btn-info\" href=\"" . route("AppBackupDownload", ["file" => encrypt($Backup->file)]) . "\">
<i class=\"icon-cloud-download\"></i> " . trans('backLang.download') . "
</a>";
                }

                if (Roles::Check("delete")) {
                    $options .= "
<button class=\"btn btn-sm btn-danger\"  onclick=\"delete_row('" . encrypt($Backup->file) . "')\">
<i class=\"icon-trash\"></i> " . trans('backLang.delete') . "
</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;
                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function upload()
    {
        return view("backEnd.app.settings.backup.upload");
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $Backup_file = 'backup_' . date('m_d_Y_h_i_s_a') . ".sql";
            $remote = 0;
            //Generate Backup
            if (Backup::dumpDatabase($Backup_file)) {
                //Save to FTP
                if (Helper::Preferences("backup_ftp") && Helper::FileSystem() != "") {
                    $localFile = Storage::disk('local')->get("backups/" . $Backup_file);
                    $Remote_store = Storage::disk(Helper::FileSystem())->put("backups/" . $Backup_file, $localFile);
                    if ($Remote_store) {
                        $remote = 1;
                        // Delete from local
                        Storage::disk('local')->delete("backups/" . $Backup_file);
                    }
                }

                // storage location is project_root/storage/app/backup.json file.
                $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                $fileData = [
                    "title" => $request->title,
                    "file" => $Backup_file,
                    "remote" => $remote,
                    "created_by" => Auth::user()->id,
                    "created_ip" => \Request::ip(),
                    "created_at" => date('Y-m-d H:i:s'),
                ];

                array_push($backup_list, $fileData);

                Storage::disk('local')->put('backup.json', json_encode($backup_list));

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 111,
                    $window = 0,
                    $record_id = 0,
                    $operation = 12,
                    $title = $request->title
                );
            } else {
                return json_encode(array("stat" => "error"));
            }
        } catch (\Exception $e) {
            return json_encode(array("stat" => "error"));
        }

        return json_encode(array("stat" => "success", "file" => $Backup_file, "download_url" => route("AppBackupDownload", ["file" => encrypt($Backup_file)])));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        try {
            // storage location is project_root/storage/app/backup.json file.
            $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

            $i = 0;
            $new_backup_list = [];
            foreach ($backup_list as $backup_row) {
                //check the property of every element
                if (decrypt($request->file) == $backup_row->file) {
                    $new_bu = [
                        "title" => $request->title,
                        "file" => $backup_row->file,
                        "remote" => $backup_row->remote,
                        "created_by" => $backup_row->created_by,
                        "created_ip" => $backup_row->created_ip,
                        "created_at" => $backup_row->created_at,
                    ];
                    array_push($new_backup_list, $new_bu);
                } else {
                    $new_bu = [
                        "title" => $backup_row->title,
                        "file" => $backup_row->file,
                        "remote" => $backup_row->remote,
                        "created_by" => $backup_row->created_by,
                        "created_ip" => $backup_row->created_ip,
                        "created_at" => $backup_row->created_at,
                    ];
                    array_push($new_backup_list, $new_bu);
                }
                $i++;
            }
            Storage::disk('local')->put('backup.json', json_encode($new_backup_list));

            // Log to history
            History::Log(
                $module = 1,
                $cat = 111,
                $window = 0,
                $record_id = 0,
                $operation = 3,
                $title = $request->title
            );

        } catch (\Exception $e) {
            return json_encode(array("stat" => "error"));
        }

        return json_encode(array("stat" => "success"));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function download($file)
    {
        if (Roles::Check("add")) {
            try {
                // storage location is project_root/storage/app/backup.json file.
                $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                $new_backup_list = [];
                $file_name = "";
                foreach ($backup_list as $backup_row) {
                    //check the property of every element
                    if (decrypt($file) == $backup_row->file) {
                        $file_name = $backup_row->file;
                        if ($backup_row->remote && Helper::FileSystem() != "") {
                            return Storage::disk(Helper::FileSystem())->download("backups/" . $backup_row->file, $backup_row->file);
                        } else {
                            return Storage::disk('local')->download("backups/" . $backup_row->file, $backup_row->file);
                        }

                    }
                }
                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 111,
                    $window = 0,
                    $record_id = 0,
                    $operation = 7,
                    $title = $file_name
                );

            } catch (\Exception $e) {

            }
        }
        return redirect()->action('Preferences\BackupController@index');
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function restore($file = "")
    {
        if (Roles::Check("edit")) {
            try {
                $file_name = decrypt($file);
                if (Backup::restoreDatabase($file_name)) {
                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 111,
                        $window = 0,
                        $record_id = 0,
                        $operation = 13,
                        $title = $file_name
                    );
                } else {
                    return json_encode(array("stat" => "error"));
                }
            } catch (\Exception $e) {
                return json_encode(array("stat" => "error"));
            }
        }
        return json_encode(array("stat" => "success"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function restore_from_file(Request $request)
    {
        if (Roles::Check("edit")) {
            $this->validate($request, [
                'file' => 'required'
            ]);

            $formFileName = "file";
            $Backup_file = "";
            $file_saved = 0;
            if ($request->$formFileName != "") {
                $Backup_file = 'backup_' . date('m_d_Y_h_i_s_a') . '.' . $request->file($formFileName)->getClientOriginalExtension();
                $file_saved = $request->file->storeAs('backups', $Backup_file);
            }
            if ($file_saved != "") {

                if ($request->save_file) {
                    $remote = 0;

                    //Save to FTP
                    if (Helper::Preferences("backup_ftp") && Helper::FileSystem() != "") {
                        $localFile = Storage::disk('local')->get("backups/" . $Backup_file);
                        $Remote_store = Storage::disk(Helper::FileSystem())->put("backups/" . $Backup_file, $localFile);
                        if ($Remote_store) {
                            $remote = 1;
                            // Delete from local
                            Storage::disk('local')->delete("backups/" . $Backup_file);
                        }
                    }

                    // storage location is project_root/storage/app/backup.json file.
                    $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                    $fileData = [
                        "title" => $Backup_file,
                        "file" => $Backup_file,
                        "remote" => $remote,
                        "created_by" => Auth::user()->id,
                        "created_ip" => \Request::ip(),
                        "created_at" => date('Y-m-d H:i:s'),
                    ];

                    array_push($backup_list, $fileData);

                    Storage::disk('local')->put('backup.json', json_encode($backup_list));

                }

                //Run restore
                if (Backup::restoreDatabase($Backup_file)) {
                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 111,
                        $window = 0,
                        $record_id = 0,
                        $operation = 13,
                        $title = $Backup_file
                    );

                    if (!$request->save_file) {
                        Storage::disk('local')->delete("backups/" . $Backup_file);
                    }
                    return redirect()->action('Preferences\BackupController@index')->with('doneMessage', trans('backLang.BackupRestored'));
                } else {
                    return redirect()->action('Preferences\BackupController@index')->with('errorMessage', trans('backLang.error'));
                }
            }
        }
        return redirect()->action('Preferences\BackupController@index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($file = "")
    {
        if (Roles::Check("delete")) {
            try {
                // storage location is project_root/storage/app/backup.json file.
                $backup_list = Storage::disk('local')->exists('backup.json') ? json_decode(Storage::disk('local')->get('backup.json')) : [];

                $i = 0;
                $new_backup_list = [];
                $backup_file_name = "";
                foreach ($backup_list as $backup_row) {
                    //check the property of every element
                    if (decrypt($file) == $backup_row->file) {

                        $backup_file_name = $backup_row->title . " " . $backup_row->file;
                        unset($backup_list[$i]);

                        if ($backup_row->remote && Helper::FileSystem() != "") {
                            Storage::disk(Helper::FileSystem())->delete("backups/" . $backup_row->file);
                        } else {
                            Storage::disk('local')->delete("backups/" . $backup_row->file);
                        }

                    } else {
                        $new_bu = [
                            "title" => $backup_row->title,
                            "file" => $backup_row->file,
                            "remote" => $backup_row->remote,
                            "created_by" => $backup_row->created_by,
                            "created_ip" => $backup_row->created_ip,
                            "created_at" => $backup_row->created_at,
                        ];
                        array_push($new_backup_list, $new_bu);
                    }
                    $i++;
                }
                Storage::disk('local')->put('backup.json', json_encode($new_backup_list));

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 111,
                    $window = 0,
                    $record_id = 0,
                    $operation = 4,
                    $title = $backup_file_name
                );

            } catch (\Exception $e) {
                return json_encode(array("stat" => "error"));
            }
        }

        return json_encode(array("stat" => "success"));
    }

}
