<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Ledger;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;

class CategoryController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // confirm password
        Helper::confirmPassword("AppCategories");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($cat = 0)
    {
        return view("back.preferences.categories.list", compact("cat"));
    }

    /**
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function list(Request $request)
    {
        $columns = array(
            0 => 'check',
            1 => 'order',
            2 => 'title_' . trans("backLang.code"),
            3 => 'ledger_id',
            4 => 'status',
            5 => 'father_id',
            6 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $cat = $request->input('cat');

        if (Roles::Check("view")) {
            $Categories = Category::where("created_by", @Auth::user()->id)->where("cat", $cat);
        } else {
            $Categories = Category::where("cat", $cat);
        }
        $totalData = $Categories->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Categories = $Categories->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Categories as $Category) {
                $nestedData['check'] = "<div class='text-center'><label class=\"custom-control custom-checkbox\"><input type=\"checkbox\" class=\"custom-control-input\" name=\"ids[]\" value=\"" . $Category->id . "\"><span class=\"custom-control-label\">&nbsp;</span></label></div>";
                $nestedData['order'] = "<div class='text-center'>
<input class=\"form-control\" name=\"row_ids[]\" type=\"hidden\" value=\"" . $Category->id . "\">
<input type='number' name='row_" . $Category->id . "' min='0' class='form-control form-order' value='" . $Category->order . "'></div>";

                $nestedData['title'] = "<span class='rounded' style='display:inline-block;padding:5px 10px;" . (($Category->font_color != "") ? "color:" . $Category->font_color : "") . "" . (($Category->bg_color != "") ? ";background:" . $Category->bg_color : "") . "'>" . $Category->{"title_" . trans("backLang.code")} . "</span>";

                $nestedData['ledger_id'] = "<div class='text-center'>" . @$Category->ledger->code . " - " . @$Category->ledger->{"title_" . trans("backLang.code")} . "</div>";
                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Category->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";
                $nestedData['father_id'] = (!empty($Category->parent)) ? $Category->parent->{"title_" . trans("backLang.code")} : "-";
                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
  <button class=\"btn btn-sm btn-info\" type='button' onclick=\"edit_row('" . encrypt($Category->id) . "')\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</button>
 ";
                }

                if (Roles::Check("delete")) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" type='button' onclick=\"delete_row('" . encrypt($Category->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    /**
     *
     * @param string $id
     * @return \Illuminate\Http\Response
     */
    public function create($cat = 0)
    {
        if (Roles::Check("add")) {
            if ($cat > 0) {
                foreach (Cats::CategoriesTypes() as $key => $CategoryType) {
                    if ($key == $cat) {
                        $CategoryType_id = $key;
                        if (Roles::Check("view")) {
                            $Categories = Category::where("created_by", @Auth::user()->id)->where("cat", $cat)->get();
                        } else {
                            $Categories = Category::where("cat", $cat)->get();
                        }

                        $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                        $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);
                        return view("back.preferences.categories.create", compact("CategoryType", "Categories", "CategoryType_id", "ledgers_array", "key"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $validator = Validator::make($request->all(), [
                'cat' => 'required'
            ]);
            if ($validator->passes()) {
                try {

                    $next_row_order = Category::where('cat', $request->cat)->where('father_id', $request->father_id)->max('order');
                    if ($next_row_order < 1) {
                        $next_row_order = 1;
                    } else {
                        $next_row_order++;
                    }

                    $Category = new Category;
                    $Category->cat = $request->cat;
                    $Category->order = $next_row_order;
                    $Category->father_id = $request->father_id;
                    $Category->title_ar = $request->title_ar;
                    $Category->title_en = $request->title_en;
                    $Category->bg_color = $request->bg_color;
                    $Category->font_color = $request->font_color;
                    $Category->status = true;
                    $Category->ledger_id = ($request->ledger_id > 0) ? $request->ledger_id : 0;
                    if (@$request->start != "") {
                        $Category->start = date('Y-m-d', strtotime($request->start));
                    } else {
                        $Category->start = null;
                    }
                    if (@$request->end != "") {
                        $Category->end = date('Y-m-d', strtotime($request->end));
                    } else {
                        $Category->end = null;
                    }
                    $Category->created_ip = \Request::ip();
                    $Category->created_by = Auth::user()->id;
                    $Category->save();


                    // Log to history
                    History::Log(
                        $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                        $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                        $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                        $record_id = $Category->id,
                        $operation = 2,
                        $title = $Category->title_ar . "<br>" . $Category->title_en
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Category->id)]);
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }


    /**
     *
     * @param string $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Category = Category::find(decrypt($id));
                if (!empty($Category)) {
                    if (Roles::Check("view")) {
                        $Categories = Category::where("created_by", @Auth::user()->id)->where("cat", $Category->cat)->get();
                    } else {
                        $Categories = Category::where("cat", $Category->cat)->get();
                    }
                    foreach (Cats::CategoriesTypes() as $key => $CategoryType) {
                        if ($key == $Category->cat) {
                            $CategoryType_id = $key;

                            $Ledgers = Ledger::where("id", ">", 0)->orderby("code", "asc")->get();
                            $ledgers_array = Helper::createItemArray($Ledgers, NULL, 0);

                            return view("back.preferences.categories.edit", compact("CategoryType", "Categories", "Category", "CategoryType_id", "ledgers_array", "key"));
                        }
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }


    /**
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (Roles::Check("edit")) {

            $validator = Validator::make($request->all(), [
                'cat' => 'required'
            ]);
            if ($validator->passes()) {
                try {
                    $Category = Category::find(decrypt($request->id));
                    if (!empty($Category)) {
                        $Category->father_id = $request->father_id;
                        $Category->title_ar = $request->title_ar;
                        $Category->title_en = $request->title_en;
                        $Category->bg_color = $request->bg_color;
                        $Category->font_color = $request->font_color;
                        $Category->status = true;
                        $Category->ledger_id = ($request->ledger_id > 0) ? $request->ledger_id : 0;
                        if (@$request->start != "") {
                            $Category->start = date('Y-m-d', strtotime($request->start));
                        } else {
                            $Category->start = null;
                        }
                        if (@$request->end != "") {
                            $Category->end = date('Y-m-d', strtotime($request->end));
                        } else {
                            $Category->end = null;
                        }
                        $Category->updated_ip = \Request::ip();
                        $Category->updated_by = Auth::user()->id;
                        $Category->save();

                        // Log to history
                        History::Log(
                            $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                            $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                            $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                            $record_id = $Category->id,
                            $operation = 3,
                            $title = $Category->title_ar . "<br>" . $Category->title_en
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($Category->id)]);
                    }
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
    }

    /**
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function multi_update(Request $request)
    {

        if (Roles::Check("edit")) {
            if ($request->action == "save") {
                foreach ($request->row_ids as $rowId) {
                    $Category = Category::find($rowId);
                    if (!empty($Category)) {
                        $row_no_val = "row_" . $rowId;
                        $Category->order = $request->$row_no_val;
                        $Category->save();

                        // Log to history
                        History::Log(
                            $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                            $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                            $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                            $record_id = $Category->id,
                            $operation = 3,
                            $title = $Category->title_ar . "<br>" . $Category->title_en
                        );

                    }
                }
                return response()->json(['stat' => 'success']);
            }
            if ($request->ids != "") {
                if ($request->action == "activate") {
                    $Categories = Category::wherein('id', $request->ids)->get();
                    foreach ($Categories as $Category) {
                        $Category->status = true;
                        $Category->save();

                        // Log to history
                        History::Log(
                            $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                            $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                            $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                            $record_id = $Category->id,
                            $operation = 9,
                            $title = $Category->title_ar . "<br>" . $Category->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "block") {

                    $Categories = Category::wherein('id', $request->ids)->get();
                    foreach ($Categories as $Category) {
                        $Category->status = false;
                        $Category->save();

                        // Log to history
                        History::Log(
                            $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                            $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                            $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                            $record_id = $Category->id,
                            $operation = 8,
                            $title = $Category->title_ar . "<br>" . $Category->title_en
                        );
                    }

                    return response()->json(['stat' => 'success']);
                } elseif ($request->action == "delete") {
                    $Categories = Category::wherein('id', $request->ids)->get();
                    foreach ($Categories as $Category) {

                        // Log to history
                        History::Log(
                            $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                            $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                            $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                            $record_id = $Category->id,
                            $operation = 4,
                            $title = $Category->title_ar . "<br>" . $Category->title_en
                        );

                        $Category->delete();
                    }

                    return response()->json(['stat' => 'success']);
                }

            } else {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.noRowsSelected')]]);
            }

        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Category = Category::find(decrypt($id));
                if (!empty($Category)) {

                    // Log to history
                    History::Log(
                        $module = @Cats::CategoriesTypes()[$Category->cat]['history']['module'],
                        $cat = @Cats::CategoriesTypes()[$Category->cat]['history']['cat'],
                        $window = @Cats::CategoriesTypes()[$Category->cat]['history']['window'],
                        $record_id = $Category->id,
                        $operation = 4,
                        $title = $Category->title_ar . "<br>" . $Category->title_en
                    );

                    $Category->delete();
                    return json_encode(array("stat" => "success", "id" => $Category->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
