<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Chat;
use App\Models\Attachment;
use Auth;
use Helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Events\ChatSentEvent;
use App\Events\ChatReadEvent;
use Cache;
use File;

class ChatController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        //Role Check
        Roles::Check("role_chat", true);
    }

    public function index()
    {
        $UsersList = User::where("status", true)->where("id", "!=", Auth::user()->id)->orderby("name", "asc")->get();
        $latest_chat = Chat::where("sender_id", Auth::user()->id)->orderby('id', "desc")->first();
        $receiver = [];
        $LatestMessages = [];
        if (!empty($latest_chat)) {
            if (!empty($latest_chat->receiver)) {
                $receiver = $latest_chat->receiver;
            }
            $LatestMessages = Chat::where("sender_id", Auth::user()->id)->where("receiver_id", $receiver->id)->orderby('id', "desc")->get();
        } else {
            $FirstUser = User::where("status", true)->where("id", "!=", Auth::user()->id)->orderby("name", "asc")->first();
            if (!empty($FirstUser)) {
                $receiver = $FirstUser;
            }
        }
        return view("back.preferences.chat.home", compact("UsersList", "LatestMessages", "receiver"));
    }

    /**
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function messages($id = 0)
    {
        $Messages = [];
        if ($id > 0) {
            $Messages = Chat::where([["sender_id", Auth::user()->id], ["receiver_id", $id], ['sender_del', false]])->orwhere([["sender_id", $id], ["receiver_id", Auth::user()->id], ['receiver_del', false]])->orderby('id', "asc")->get();
            if (count($Messages) > 0) {

                $Messages = $Messages->map(function ($Message) {
                    $s4ds_current_date = date('Y-m-d', $_SERVER['REQUEST_TIME']);
                    $day_mm = date('Y-m-d', strtotime($Message->created_at));
                    if ($day_mm == $s4ds_current_date) {
                        $newMessage_date = date('h:i A', strtotime($Message->created_at));
                    } else {
                        $newMessage_date = Helper::DateView($Message->created_at, 1);
                    }
                    $new = false;
                    if ($Message->receiver_id == Auth::user()->id && !$Message->read_status) {
                        $Message->read_status = true;
                        $Message->read_at = now();
                        $Message->save();
                        $new = true;
                        if (Helper::CheckPusherStatus()) {
                            broadcast(new ChatReadEvent(Auth::user(), $Message))->toOthers();
                        }
                    }
                    $attachments = [];
                    if (count($Message->attachments) > 0) {
                        foreach ($Message->attachments as $Attachment) {
                            $Attachment_Icon = Helper::GetIconFromExt($Attachment->ext, $size = 60);
                            if (Helper::CheckPusherStatus()) {
                                if (Helper::IsPhoto($Attachment->ext) && !$Attachment->remote) {
                                    $Attachment_Icon = "<img src=\"" . route("AppAttachmentsPreview", encrypt($Attachment->id)) . "\">";
                                }
                            }
                            $attachments[] = [
                                "id" => $Attachment->id,
                                "title" => $Attachment->title . "." . $Attachment->ext,
                                "size" => Helper::formatFileSize($Attachment->size, 2),
                                "icon" => $Attachment_Icon,
                                "url" => route("AppAttachmentsView", encrypt($Attachment->id)),
                            ];
                        }
                    }

                    return [
                        'id' => $Message->id,
                        'sender_id' => $Message->sender_id,
                        'receiver_id' => $Message->receiver_id,
                        'message' => nl2br($Message->message),
                        'created_at' => $newMessage_date,
                        'read_status' => $Message->read_status,
                        'new' => $new,
                        'attachments' => $attachments
                    ];
                });
            }
        }
        return $Messages;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $receiver_id = $request->receiver_id;
            $receiver_user = User::find($receiver_id);
            if (!empty($receiver_user) && $request->message != "") {
                $sender_id = Auth::user()->id;
                $Chat = new Chat;
                $Chat->receiver_id = $receiver_id;
                $Chat->sender_id = $sender_id;
                $Chat->message = $request->message;
                $Chat->read_status = false;
                $Chat->sender_del = false;
                $Chat->receiver_del = false;
                $Chat->created_ip = \Request::ip();
                $Chat->save();

                $newMessage = [
                    'id' => $Chat->id,
                    'sender_id' => $Chat->sender_id,
                    'receiver_id' => $Chat->receiver_id,
                    'message' => nl2br($Chat->message),
                    'created_at' => date('h:i A', strtotime($Chat->created_at)),
                    'read_status' => $Chat->read_status,
                    'new' => false,
                    'attachments' => []
                ];
                if (Helper::CheckPusherStatus()) {
                    broadcast(new ChatSentEvent(Auth::user(), $newMessage))->toOthers();
                }


                return response()->json(['stat' => 'success', 'id' => $Chat->id]);
            }

        } catch (\Exception $e) {
            return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
        }

        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function users($id = 0)
    {
        $Users = User::where("status", true)->where("id", "!=", Auth::user()->id)->orderby('name', "asc")->get();
        $rec_status = ((Cache::has('user-is-online-' . $id)) ? 1 : 0);
        if (count($Users) > 0) {
            $Users = $Users->map(function ($User) {
                $this_user_avatar = URL::to('uploads/users/profile.jpg');
                if ($User->photo != "") {
                    $this_user_avatar = URL::to('uploads/users/' . $User->photo);
                }
                $u_status = ((Cache::has('user-is-online-' . $User->id)) ? 1 : 0);
                $alerts = Chat::where("sender_id", $User->id)->where("receiver_id", Auth::user()->id)->where("read_status", false)->count();
                if ($alerts == 0) {
                    $alerts = "";
                }
                return [
                    'id' => $User->id,
                    'name' => $User->name,
                    'job' => $User->job,
                    'online' => $u_status,
                    'avatar' => $this_user_avatar,
                    'alerts' => $alerts,
                ];
            });

        };
        return response()->json(['users' => $Users, 'rec_status' => $rec_status]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function read($message_id, $sender_id)
    {
        $Chat = Chat::where("receiver_id", Auth::user()->id)->where("sender_id", $sender_id)->find($message_id);
        if (!empty($Chat)) {
            $Chat->read_status = true;
            $Chat->read_at = now();
            $Chat->save();
            if (Helper::CheckPusherStatus()) {
                broadcast(new ChatReadEvent(Auth::user(), $Chat))->toOthers();
            }
            return response()->json(['stat' => 'success']);
        }
        return response()->json(['stat' => 'error']);
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Messages = Chat::where([["sender_id", Auth::user()->id], ["receiver_id", $id]])->orwhere([["sender_id", $id], ["receiver_id", Auth::user()->id]])->orderby('id', "asc")->get();
                if (count($Messages) > 0) {
                    foreach ($Messages as $Message) {
                        if ($Message->sender_id == Auth::user()->id) {
                            $Message->sender_del = true;
                        } else {
                            $Message->receiver_del = true;
                        }
                        if ($Message->receiver_del && $Message->sender_del) {

                            // delete attachments
                            $Attachments = Attachment::where('cat', 2)->where('cat_id', $Message->id)->get();
                            foreach ($Attachments as $Attachment) {
                                if ($Attachment->file != "") {
                                    if ($Attachment->remote && Helper::FileSystem() != "") {
                                        Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                    } else {
                                        if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                            File::delete("uploads/attachments/" . $Attachment->file);
                                        }
                                    }
                                }
                                $Attachment->shares()->delete();
                                $Attachment->comments()->delete();
                                $Attachment->alerts()->delete();
                                $Attachment->delete();
                            }

                            $Message->delete();
                        } else {
                            $Message->save();
                        }
                    }
                }
                return json_encode(array("stat" => "success"));
            }
        }
        return json_encode(array("stat" => "error"));
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function attach(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->details != "" || $request->uploaded_files != "") {
                    $sender_id = Auth::user()->id;
                    $Chat = new Chat;
                    $Chat->receiver_id = $request->receiver_id;
                    $Chat->sender_id = $sender_id;
                    $Chat->message = $request->details;
                    $Chat->read_status = false;
                    $Chat->sender_del = false;
                    $Chat->receiver_del = false;
                    $Chat->created_ip = \Request::ip();
                    $Chat->save();

                    $attachments = [];
                    if ($request->uploaded_files != "") {
                        $uploaded_files = explode(",", $request->uploaded_files);
                        foreach ($uploaded_files as $uploaded_id) {
                            if ($uploaded_id > 0) {
                                $Attachment = Attachment::where('created_by', $sender_id)->find($uploaded_id);
                                if (!empty($Attachment)) {
                                    $Attachment->cat_id = $Chat->id;
                                    $Attachment->save();

                                    $Attachment_Icon = Helper::GetIconFromExt($Attachment->ext, $size = 60);
                                    if (Helper::CheckPusherStatus()) {
                                        if (Helper::IsPhoto($Attachment->ext) && !$Attachment->remote) {
                                            $Attachment_Icon = "<img src=\"" . route("AppAttachmentsPreview", encrypt($Attachment->id)) . "\">";
                                        }
                                    }
                                    $attachments[] = [
                                        "id" => $Attachment->id,
                                        "title" => $Attachment->title . "." . $Attachment->ext,
                                        "size" => Helper::formatFileSize($Attachment->size, 2),
                                        "icon" => $Attachment_Icon,
                                        "url" => route("AppAttachmentsView", encrypt($Attachment->id)),
                                    ];
                                }
                            }
                        }
                    }
                    $newMessage = [
                        'id' => $Chat->id,
                        'sender_id' => $Chat->sender_id,
                        'receiver_id' => $Chat->receiver_id,
                        'message' => nl2br($Chat->message),
                        'created_at' => date('h:i A', strtotime($Chat->created_at)),
                        'read_status' => $Chat->read_status,
                        'new' => false,
                        'attachments' => $attachments
                    ];

                    if (Helper::CheckPusherStatus()) {
                        broadcast(new ChatSentEvent(Auth::user(), $newMessage));
                    }

                    return response()->json(['stat' => 'success', "id" => encrypt($Chat->id)]);
                } else {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.selectFilesFirst')]]);
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }
}
