<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Comment;
use Auth;
use Helper;
use Illuminate\Http\Request;
use URL;
use Redirect;
use Cats;
use Roles;
use Validator;
use App\Helpers\History;

class CommentController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }


    /**
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function load(Request $request)
    {
        $cat = $request->cat;
        $cat_id = $request->cat_id;
        $comments_roles = [
            "create" => ($request->role_create == "true" || $request->role_create == "1") ? true : false,
            "delete" => ($request->role_delete == "true" || $request->role_delete == "1") ? true : false,
        ];
        if ($cat_id != "") {
            $Comments = Comment::where("cat", $cat)->where("cat_id", decrypt($cat_id))->orderby("id", "asc")->get();
        } else {
            $Comments = Comment::where("cat", $cat)->where("cat_id", 0)->orderby("id", "asc")->get();
        }
        return view("back.preferences.comments.list", compact("Comments", "cat", "cat_id", "comments_roles"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                if ($request->comment != "" && $cat_id >= 0) {
                    $Comment = new Comment;
                    $Comment->father_id = 0;
                    $Comment->comment = $request->comment;
                    $Comment->cat = $request->cat;
                    $Comment->cat_id = $cat_id;
                    $Comment->status = true;
                    $Comment->hidden = false;
                    $Comment->created_ip = \Request::ip();
                    $Comment->created_by = Auth::user()->id;
                    $Comment->save();


                    // Log to history
                    History::Log(
                        $module = @Cats::CommentTypes()[$Comment->cat]['history']['module'],
                        $cat = @Cats::CommentTypes()[$Comment->cat]['history']['cat'],
                        $window = @Cats::CommentTypes()[$Comment->cat]['history']['window'],
                        $record_id = $Comment->id,
                        $operation = 15,
                        $title = mb_substr(strip_tags($Comment->comment), 0, 190)
                    );


                    return response()->json(['stat' => 'success', 'id' => encrypt($Comment->id)]);
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Comment = Comment::find(decrypt($id));
                if (!empty($Comment)) {
                    if (Roles::Check("delete_users_comments") || $Comment->created_by == @Auth::user()->id) {

                        // Log to history
                        History::Log(
                            $module = @Cats::CommentTypes()[$Comment->cat]['history']['module'],
                            $cat = @Cats::CommentTypes()[$Comment->cat]['history']['cat'],
                            $window = @Cats::CommentTypes()[$Comment->cat]['history']['window'],
                            $record_id = $Comment->id,
                            $operation = 4,
                            $title = mb_substr(strip_tags($Comment->comment), 0, 190)
                        );

                        $Comment->delete();
                        return json_encode(array("stat" => "success", "id" => $Comment->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
