<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Event;
use App\Models\Category;
use App\Models\Share;
use App\Models\Attachment;
use Auth;
use Helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use Redirect;
use Cats;
use Roles;
use Validator;
use File;
use App\Helpers\History;

class EventController extends Controller
{
    private $uploadPath = "uploads/events/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  string $cat
     * @return \Illuminate\Http\Response
     */
    public function index($cat = "mine")
    {
        $mine = 0;
        if ($cat != "mine") {
            try {
                $cat = decrypt($cat);
            } catch (\Exception $e) {
                $cat = 0;
            }
        } else {
            $cat = 0;
            $mine = 1;
        }

        $events_roles = [
            "create" => Roles::Check("calendar_add"),
            "edit" => Roles::Check("calendar_edit"),
            "delete" => Roles::Check("calendar_delete"),
            "share" => Roles::Check("calendar_share"),
        ];
        $cat_id = encrypt(Auth::user()->id);
        return view("back.preferences.events.home", compact("cat", "cat_id", "events_roles", "mine"));
    }

    /**
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function load(Request $request)
    {
        $cat = $request->cat;
        $cat_id = $request->cat_id;
        $events_roles = [
            "create" => ($request->role_create == "true" || $request->role_create == "1") ? true : false,
            "edit" => ($request->role_edit == "true" || $request->role_edit == "1") ? true : false,
            "delete" => ($request->role_delete == "true" || $request->role_delete == "1") ? true : false,
            "calendar_height" => $request->calendar_height,
            "default_view" => $request->default_view,
            "month_view" => ($request->month_view == "true" || $request->month_view == "1") ? true : false,
            "day_view" => ($request->day_view == "true" || $request->day_view == "1") ? true : false,
            "week_view" => ($request->week_view == "true" || $request->week_view == "1") ? true : false,
            "list_view" => ($request->list_view == "true" || $request->list_view == "1") ? true : false,
        ];
        return view("back.preferences.events.list", compact("cat", "cat_id", "events_roles"));
    }

    /**
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function list(Request $request)
    {
        $cat = $request->cat;
        $cat_id = decrypt($request->cat_id);
        $from_start = date('Y-m-d H:i:s', strtotime($request->start));
        $to_end = date('Y-m-d H:i:s', strtotime($request->end));

        $Events = Event::where("cat", $cat)->where("start_date", '>=', $from_start)->where("end_date", '<=', $to_end);

        if ($cat_id < 1) {
            $cat_id = 0;
        }
        $Events_shared_ids = Share::select('cat_id')->where("cat", 2)->where("user_id", ($cat == 0) ? $cat_id : @Auth::user()->id)->get();
        $Events = $Events->where(function ($query) use ($cat_id, $Events_shared_ids) {
            $query->where('cat_id', $cat_id)->orWhereIn('id', $Events_shared_ids);
        });

        $Events = $Events->orderby("id", "asc")->get();
        $Events_array = [];
        if (count($Events) > 0) {
            foreach ($Events as $Event) {
                $bg_color = "";
                $text_color = "";
                $event_url = route("AppEventsView", encrypt($Event->id));
                $event_category = Category::find($Event->category_id);
                if (!empty($event_category)) {
                    $bg_color = $event_category->bg_color;
                    $text_color = $event_category->font_color;
                }
                if ($Event->all_day) {
                    $Events_array[] = [
                        "title" => $Event->title . ((@$Event->user->id != Auth::user()->id) ? "\n" . trans("backLang.createdBy") . ": " . @$Event->user->name : ""),
                        "start" => date('Y-m-d', strtotime($Event->start_date)),
                        "end" => date('Y-m-d', strtotime($Event->end_date . " +1 day")),
                        "allDay" => true,
                        "url" => $event_url,
                        "color" => $bg_color,
                        "textColor" => $text_color
                    ];
                } else {
                    $Events_array[] = [
                        "title" => $Event->title . ((@$Event->user->id != Auth::user()->id) ? "\n" . @$Event->user->name : ""),
                        "start" => date('Y-m-d H:i:s', strtotime($Event->start_date)),
                        "end" => date('Y-m-d H:i:s', strtotime($Event->end_date)),
                        "url" => $event_url,
                        "color" => $bg_color,
                        "textColor" => $text_color
                    ];
                }
            }
        }
        return response()->json($Events_array);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                $validator = Validator::make($request->all(), [
                    'photo' => 'mimes:png,jpeg,jpg,gif|max:5000',
                    'title' => 'required',
                    'start_date' => 'required'
                ]);
                if ($validator->passes()) {
                    if ($cat_id >= 0) {
                        $allday = ($request->allday) ? 1 : 0;
                        $start_date = $request->start_date;
                        $start_time = $request->start_time;
                        if ($start_time == "" || $allday) {
                            $start_time = "00:00:00";
                        }
                        $end_date = $request->end_date;
                        $end_time = $request->end_time;
                        if ($end_time == "" || $allday) {
                            $end_time = "00:00:00";
                        }
                        $start = date('Y-m-d H:i:s', strtotime("$start_date $start_time"));
                        $end = date('Y-m-d H:i:s', strtotime("$end_date $end_time"));


                        $formFileName = "photo";
                        $fileFinalName = "";
                        if ($request->$formFileName != "") {
                            $fileFinalName = time() . rand(1111, 9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                            $path = $this->uploadPath;
                            $request->file($formFileName)->move($path, $fileFinalName);
                        }

                        $Event = new Event;
                        $Event->title = $request->title;
                        $Event->details = $request->details;
                        if ($fileFinalName != "") {
                            $Event->photo = $fileFinalName;
                        }
                        $Event->start_date = $start;
                        $Event->end_date = $end;
                        $Event->all_day = $allday;
                        $Event->category_id = $request->category_id;
                        $Event->cat = $request->cat;
                        $Event->cat_id = $cat_id;
                        $Event->status = true;
                        $Event->hidden = false;
                        $Event->sent = false;
                        $Event->created_ip = \Request::ip();
                        $Event->created_by = Auth::user()->id;
                        $Event->save();

                        // Log to history
                        History::Log(
                            $module = 2,
                            $cat = 0,
                            $window = 0,
                            $record_id = $Event->id,
                            $operation = 2,
                            $title = $Event->title . "<br>" . (($Event->all_day) ? date('Y-m-d', strtotime($Event->start_date)) : date('Y-m-d h:i a', strtotime($Event->start_date)))
                        );

                        return response()->json(['stat' => 'success', 'id' => encrypt($Event->id)]);
                    }
                } else {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function view($id = "")
    {
        if ($id != "") {
            $Event = Event::find(decrypt($id));
            if (!empty($Event)) {
                $Events_is_shared = Share::where("cat", 2)->where("cat_id", $Event->id)->where("user_id", @Auth::user()->id)->first();

                if ($Event->created_by == @Auth::user()->id || !empty($Events_is_shared) || ($Event->cat = 1 && Roles::Check("profile_tab_events"))) {
                    return view("back.preferences.events.view", compact("Event"));
                } else {
                    return redirect()->route("AppEvents");
                }
            }
        }
        return redirect()->route("NotFound");
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  string $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Event = Event::find(decrypt($id));
                if (!empty($Event)) {
                    return view("back.preferences.events.edit", compact("Event"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $Event = Event::find(decrypt($request->id));
            if (!empty($Event)) {
                $validator = Validator::make($request->all(), [
                    'photo' => 'mimes:png,jpeg,jpg,gif|max:5000',
                    'title' => 'required',
                    'start_date' => 'required'
                ]);
                if ($validator->passes()) {


                    // delete old photo
                    if ($request->photo_delete == 1) {
                        // Delete photo
                        if ($Event->photo != "") {
                            if (file_exists($this->uploadPath . $Event->photo)) {
                                File::delete($this->uploadPath . $Event->photo);
                            }
                        }
                        $Event->photo = "";
                    }

                    $formFileName = "photo";
                    $fileFinalName = "";
                    if ($request->$formFileName != "") {
                        // Delete photo
                        if ($Event->photo != "") {
                            if (file_exists($this->uploadPath . $Event->photo)) {
                                File::delete($this->uploadPath . $Event->photo);
                            }
                        }
                        $fileFinalName = time() . rand(1111, 9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                        $path = $this->uploadPath;
                        $request->file($formFileName)->move($path, $fileFinalName);
                    }

                    $allday = ($request->allday) ? 1 : 0;
                    $start_date = $request->start_date;
                    $start_time = $request->start_time;
                    if ($start_time == "" || $allday) {
                        $start_time = "00:00:00";
                    }
                    $end_date = $request->end_date;
                    $end_time = $request->end_time;
                    if ($end_time == "" || $allday) {
                        $end_time = "00:00:00";
                    }
                    $start = date('Y-m-d H:i:s', strtotime("$start_date $start_time"));
                    $end = date('Y-m-d H:i:s', strtotime("$end_date $end_time"));


                    $Event->title = $request->title;
                    $Event->details = $request->details;
                    if ($fileFinalName != "") {
                        $Event->photo = $fileFinalName;
                    }
                    $Event->start_date = $start;
                    $Event->end_date = $end;
                    $Event->all_day = $allday;
                    $Event->category_id = $request->category_id;
                    $Event->updated_ip = \Request::ip();
                    $Event->updated_by = Auth::user()->id;
                    $Event->save();

                    // Log to history
                    History::Log(
                        $module = 2,
                        $cat = 0,
                        $window = 0,
                        $record_id = $Event->id,
                        $operation = 3,
                        $title = $Event->title . "<br>" . (($Event->all_day) ? date('Y-m-d', strtotime($Event->start_date)) : date('Y-m-d h:i a', strtotime($Event->start_date)))
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Event->id)]);
                } else {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Event = Event::find(decrypt($id));
                if (!empty($Event)) {
                    if ($Event->photo != "") {
                        if (file_exists($this->uploadPath . $Event->photo)) {
                            File::delete($this->uploadPath . $Event->photo);
                        }
                    }
                    $Attachments = Attachment::where('cat', 1)->where('cat_id', $Event->id)->get();
                    foreach ($Attachments as $Attachment) {
                        if ($Attachment->file != "") {
                            if ($Attachment->remote && Helper::FileSystem() != "") {
                                Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                            } else {
                                if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                    File::delete("uploads/attachments/" . $Attachment->file);
                                }
                            }
                        }
                        $Attachment->shares()->delete();
                        $Attachment->comments()->delete();
                        $Attachment->alerts()->delete();
                        $Attachment->delete();
                    }

                    $Event->shares()->delete();
                    $Event->comments()->delete();
                    $Event->alerts()->delete();
                    $Event->delete();

                    // Log to history
                    History::Log(
                        $module = 2,
                        $cat = 0,
                        $window = 0,
                        $record_id = $Event->id,
                        $operation = 4,
                        $title = $Event->title . "<br>" . (($Event->all_day) ? date('Y-m-d', strtotime($Event->start_date)) : date('Y-m-d h:i a', strtotime($Event->start_date)))
                    );

                    return json_encode(array("stat" => "success", "id" => $Event->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
