<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Models\Event;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\Page;
use Auth;
use Helper;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use Redirect;
use Cats;
use Roles;
use Validator;
use File;
use App\Helpers\History;

class PageController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_text_pages", true);
        // confirm password
        Helper::confirmPassword("AppPages");
    }

    public function index()
    {
        foreach (Cats::TextPages() as $key => $df_page) {
            $Page = Page::find($key);
            if (empty($Page)) {
                $Page = new Page;
                $Page->id = $key;
                $Page->title_ar = $df_page['title_ar'];
                $Page->title_en = $df_page['title_en'];
                $Page->status = true;
                $Page->visits = 0;
                $Page->created_ip = \Request::ip();
                $Page->created_by = Auth::user()->id;
                $Page->save();
            }
        }
        return view("back.preferences.pages.list");
    }

    public function list(Request $request)
    {
        $columns = array(
            0 => 'id',
            1 => 'title_' . trans("backLang.code"),
            2 => 'visits',
            3 => 'status',
            4 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');


        if (Roles::Check("view")) {
            $Pages = Page::where("created_by", @Auth::user()->id);
        } else {
            $Pages = Page::where("id", "!=", 0);
        }
        $totalData = $Pages->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Pages = $Pages->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Pages as $Page) {
                $nestedData['id'] = "<div class='text-center'>" . $Page->id . "</div>";
                $nestedData['title'] = $Page->{"title_" . trans("backLang.code")};

                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Page->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['visits'] = "<div class='text-center'>" . $Page->visits . "</div>";
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";
                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
  <button class=\"btn btn-sm btn-info\" type='button' onclick=\"edit_row('" . encrypt($Page->id) . "')\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</button>
 ";
                }
                $is_df = false;
                foreach (Cats::TextPages() as $key => $df_page) {
                    if ($key == $Page->id) {
                        $is_df = true;
                    }
                }
                if (Roles::Check("delete") && !$is_df) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" type='button' onclick=\"delete_row('" . encrypt($Page->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                $Page = new Page;
                $Page->title_ar = $request->title_ar;
                $Page->title_en = $request->title_en;
                $Page->details_ar = $request->details_ar;
                $Page->details_en = $request->details_en;
                $Page->status = true;
                $Page->visits = 0;
                $Page->created_ip = \Request::ip();
                $Page->created_by = Auth::user()->id;
                $Page->save();

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 112,
                    $window = 0,
                    $record_id = $Page->id,
                    $operation = 2,
                    $title = $Page->title_ar . "<br>" . $Page->title_en
                );

                return response()->json(['stat' => 'success', 'id' => encrypt($Page->id)]);

            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $Page = Page::find(decrypt($id));
                if (!empty($Page)) {
                    return view("back.preferences.pages.edit", compact("Page"));
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $Page = Page::find(decrypt($request->id));
            if (!empty($Page)) {

                $Page->title_ar = $request->title_ar;
                $Page->title_en = $request->title_en;
                $Page->details_ar = $request->details_ar;
                $Page->details_en = $request->details_en;
                $Page->status = ($request->status) ? 1 : 0;
                $Page->updated_ip = \Request::ip();
                $Page->updated_by = Auth::user()->id;
                $Page->save();

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 112,
                    $window = 0,
                    $record_id = $Page->id,
                    $operation = 3,
                    $title = $Page->title_ar . "<br>" . $Page->title_en
                );

                return response()->json(['stat' => 'success', "id" => encrypt($Page->id)]);

            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Page = Page::find(decrypt($id));
                if (!empty($Page)) {
                    $Page->delete();

                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 112,
                        $window = 0,
                        $record_id = $Page->id,
                        $operation = 4,
                        $title = $Page->title_ar . "<br>" . $Page->title_en
                    );

                    return json_encode(array("stat" => "success", "id" => $Page->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
