<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Preference;
use App\Models\Authorize;
use Auth;
use Cookie;
use Helper;
use Cats;
use Notifications;
use Roles;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use GeoIP;
use App\Helpers\History;

class PreferenceController extends Controller

{
    private $uploadPath = "uploads/preferences/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        // confirm password
        Helper::confirmPassword("appSettings");

        $uploads_folder_size = 0;
        try {
            $uploadsPath = rtrim(DeviceTrust::settings("PUBLIC_PATH")) . '/uploads';
            $io = popen('/usr/bin/du -sk ' . $uploadsPath, 'r');
            $uploads_folder_size = fgets($io, 4096);
            $uploads_folder_size = substr($uploads_folder_size, 0, strpos($uploads_folder_size, "\t"));
            if ($uploads_folder_size > 0) {
                $uploads_folder_size = round($uploads_folder_size / 1024, 3);
            }
            pclose($io);
        } catch (\Exception $e) {

        }
        if ($uploads_folder_size == "") {
            $uploads_folder_size = 0;
        }

        return view("back.preferences.home", compact("uploads_folder_size"));
    }

    public function general_settings()
    {
        //Role Check
        Roles::Check("app_preferences_general", true);

        // confirm password
        Helper::confirmPassword("appGeneralSettings");

        $Preferences = Preference::find(1);

        if ($Preferences->more_settings != "") {
            try {
                $more_settings = json_decode($Preferences->more_settings);
            } catch (\Exception $e) {
                $more_settings = [];
            }
        }
        foreach (Cats::AdditionalGeneralSettings() as $AdditionalGeneralSetting) {
            $Preferences->$AdditionalGeneralSetting = @$more_settings->$AdditionalGeneralSetting;
        }

        return view("back.preferences.general", compact("Preferences"));
    }

    public function general_settings_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_general", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            // Start of Upload Files
            $formFileName = "app_logo_en";
            $fileFinalName = "";
            if ($request->$formFileName != "") {
                // Delete a app_logo_en photo
                if ($Preference->app_logo_en != "") {
                    File::delete($this->uploadPath . $Preference->app_logo_en);
                }

                $fileFinalName = time() . rand(1111,
                        9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                $path = $this->uploadPath;
                $request->file($formFileName)->move($path, $fileFinalName);
            }
            $formFileNameAr = "app_logo_ar";
            $fileFinalNameAr = "";
            if ($request->$formFileNameAr != "") {
                // Delete a app_logo_ar photo
                if ($Preference->app_logo_ar != "") {
                    File::delete($this->uploadPath . $Preference->app_logo_ar);
                }

                $fileFinalNameAr = time() . rand(1111,
                        9999) . '.' . $request->file($formFileNameAr)->getClientOriginalExtension();
                $path = $this->uploadPath;
                $request->file($formFileNameAr)->move($path, $fileFinalNameAr);
            }

            $Preference->app_title_ar = $request->app_title_ar;
            $Preference->app_title_en = $request->app_title_en;

            if ($request->app_logo_ar_delete == 1) {
                // Delete photo
                if ($Preference->app_logo_ar != "") {
                    File::delete($this->uploadPath . $Preference->app_logo_ar);
                }
                $Preference->app_logo_ar = "";
            }
            if ($request->app_logo_en_delete == 1) {
                // Delete photo
                if ($Preference->app_logo_en != "") {
                    File::delete($this->uploadPath . $Preference->app_logo_en);
                }
                $Preference->app_logo_en = "";
            }

            if ($fileFinalName != "") {
                $Preference->app_logo_en = $fileFinalName;
            }

            if ($fileFinalNameAr != "") {
                $Preference->app_logo_ar = $fileFinalNameAr;
            }
            $Preference->reports_load = 0;//$request->reports_load;
            $Preference->allowed_ext = $request->allowed_ext;
            $Preference->allowed_types = $request->allowed_types;
            $Preference->allowed_size = $request->allowed_size;

            // more settings
            $more_settings = [];
            foreach (Cats::AdditionalGeneralSettings() as $AdditionalGeneralSetting) {
                if ($request->exists($AdditionalGeneralSetting)) {
                    $more_settings[$AdditionalGeneralSetting] = $request->$AdditionalGeneralSetting;
                } else {
                    $more_settings[$AdditionalGeneralSetting] = Helper::Preferences($AdditionalGeneralSetting);
                }
            }
            $more_settings = json_encode($more_settings);
            $Preference->more_settings = $more_settings;
            // \more settings

            $Preference->updated_by = Auth::user()->id;
            $Preference->save();

            // Log to history
            History::Log(
                $module = 1,
                $cat = 101,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = $Preference->app_title_ar
            );

            return redirect()->action('Preferences\PreferenceController@general_settings')
                ->with('doneMessage', trans('backLang.saveDone'));
        }

    }

    public function languages()
    {
        //Role Check
        Roles::Check("app_preferences_language", true);

        // confirm password
        Helper::confirmPassword("appLanguagesSettings");

        $Preferences = Preference::find(1);
        return view("back.preferences.languages", compact("Preferences"));
    }

    public function languages_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_language", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->languages_ar_status = ($request->languages_ar_status) ? 1 : 0;
            $Preference->languages_en_status = ($request->languages_en_status) ? 1 : 0;
            $Preference->languages_by_default = $request->languages_by_default;
            $Preference->timezone = $request->timezone;
            $Preference->ar_box_status = ($request->ar_box_status) ? 1 : 0;
            $Preference->en_box_status = ($request->en_box_status) ? 1 : 0;


            $Preference->date_type = $request->date_type;
            $Preference->date_format = $request->date_format;
            $Preference->hijri_adjustment = $request->hijri_adjustment;

            $Preference->save();

            // Update .env file
            $env_update = Helper::changeEnv([
                'DEFAULT_LANGUAGE' => $request->languages_by_default,
                'TIMEZONE' => $request->timezone,
                'HIJRI_ADJUSTMENT' => $request->hijri_adjustment,
            ]);

            // Log to history
            History::Log(
                $module = 1,
                $cat = 109,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@languages')
                ->with('doneMessage', trans('backLang.saveDone'));
        }
    }

    public function mail_smtp_check(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        if ($request->mail_driver == "smtp" && $request->mail_host != "" && $request->mail_port != "") {
            try {
                function server_parse($socket, $expected_response)
                {
                    $server_response = '';
                    while (substr($server_response, 3, 1) != ' ') {
                        if (!($server_response = fgets($socket, 256))) {
                            return 'Error while fetching server response codes';
                        }
                    }

                    if (!(substr($server_response, 0, 3) == $expected_response)) {
                        return $server_response;
                    }
                }

                //Connect to the host on the specified port
                $smtpServer = $request->mail_host;
                $username = $request->mail_username;
                $password = $request->mail_password;
                $port = $request->mail_port;
                $timeout = 20;
                $output = "";

                $socket = fsockopen($smtpServer, $port, $errno, $errstr, $timeout);
                if (!$socket) {
                    return json_encode(array("stat" => "error", "error" => "$errstr ($errno)"));
                } else {

                    server_parse($socket, '220');

                    fwrite($socket, 'EHLO ' . $smtpServer . "\r\n");
                    $output .= server_parse($socket, '250');
                    if ($output != "") {
                        $output .= "<br>";
                    }
                    fwrite($socket, 'AUTH LOGIN' . "\r\n");
                    $output .= server_parse($socket, '334');
                    if ($output != "") {
                        $output .= "<br>";
                    }
                    fwrite($socket, base64_encode($username) . "\r\n");
                    $output .= server_parse($socket, '334');
                    if ($output != "") {
                        $output .= "<br>";
                    }
                    fwrite($socket, base64_encode($password) . "\r\n");
                    $output .= server_parse($socket, '235');

                    if ($output == "") {
                        return json_encode(array("stat" => "success"));
                    } else {
                        return json_encode(array("stat" => "error", "error" => $output));
                    }
                }
            } catch (\Exception $e) {
                return json_encode(array("stat" => "error", "error" => "$errstr ($errno)"));
            }
        }
        return json_encode(array("stat" => "error", "error" => "Failed .. no data to connect"));
    }

    public function mail_test(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->mail_driver = $request->mail_driver;
            $Preference->mail_host = $request->mail_host;
            $Preference->mail_port = $request->mail_port;
            $Preference->mail_username = $request->mail_username;
            $Preference->mail_password = $request->mail_password;
            $Preference->mail_encryption = $request->mail_encryption;
            $Preference->mail_from_address = $request->mail_from_address;
            $Preference->mail_from_name = $request->mail_from_name;
            $Preference->mail_test = $request->mail_test;
            $Preference->save();

            if ($request->mail_driver == "smtp" && $request->mail_host != "" && $request->mail_port != "") {
                try {
                    $email_subject = "Test Mail From " . $Preference->mail_from_name;
                    $email_body = "This is a Test Mail \r\n
Mail Driver: " . $Preference->mail_driver . "
Mail Host: " . $Preference->mail_host . "
Mail Port: " . $Preference->mail_port . "
Mail Username: " . $Preference->mail_username . "
Email from: " . $Preference->mail_from_address . "
Email to: " . $Preference->mail_test . "
";
                    $to_email = $Preference->mail_test;
                    $to_name = "";
                    $from_email = $Preference->mail_from_address;
                    $from_name = $Preference->mail_from_name;
                    Mail::send([], [], function ($message) use ($email_subject, $email_body, $to_email, $to_name, $from_email, $from_name) {
                        $message->from($from_email, $from_name)
                            ->to($to_email, $to_name)
                            ->subject($email_subject)
                            ->setBody($email_body);
                    });
                    return json_encode(array("stat" => "success"));
                } catch (\Exception $e) {
                    return json_encode(array("stat" => "error"));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function sms_balance()
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        return Helper::SMS_Balance();
    }

    public function ftp()
    {
        //Role Check
        Roles::Check("app_preferences_ftp", true);

        // confirm password
        Helper::confirmPassword("appFtpSettings");

        $Preferences = Preference::find(1);
        return view("back.preferences.ftp", compact("Preferences"));
    }

    public function ftp_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_ftp", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->ftp_status = ($request->ftp_status) ? 1 : 0;
            $Preference->ftp_host = $request->ftp_host;
            $Preference->ftp_username = $request->ftp_username;
            $Preference->ftp_password = $request->ftp_password;
            $Preference->ftp_port = $request->ftp_port;
            $Preference->ftp_root = $request->ftp_root;
            $Preference->ftp_passive = $request->ftp_passive;
            $Preference->ftp_ssl = $request->ftp_ssl;
            $Preference->ftp_timeout = $request->ftp_timeout;
            $Preference->dropbox_app_secret = $request->dropbox_app_secret;
            $Preference->dropbox_access_token = $request->dropbox_access_token;
            $Preference->save();

            // Update .env file
            $env_update = Helper::changeEnv([
                'FTP_HOST' => $request->ftp_host,
                'FTP_USERNAME' => $request->ftp_username,
                'FTP_PASSWORD' => $request->ftp_password,
                'FTP_PORT' => $request->ftp_port,
                'FTP_ROOT' => $request->ftp_root,
                'FTP_PASSIVE' => $request->ftp_passive,
                'FTP_SSL' => $request->ftp_ssl,
                'FTP_TIMEOUT' => $request->ftp_timeout,
                'DROPBOX_TOKEN' => $request->dropbox_access_token
            ]);

            // Log to history
            History::Log(
                $module = 1,
                $cat = 106,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@ftp')->with('doneMessage', trans('backLang.saveDone'));
        }
    }

    public function ftp_test(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_ftp", true);

        function checkFtp($host, $username, $password, $port = 21, $timeout = 10)
        {
            $con = ftp_connect($host, $port, $timeout);
            if (false === $con) {
                throw new \Exception('Unable to connect to FTP Server.');
            }
            $loggedIn = ftp_login($con, $username, $password);
            ftp_close($con);
            if (true === $loggedIn) {
                return true;
            } else {
                throw new \Exception('Unable to log in.');
            }
        }

        $host = $request->ftp_host;
        $username = $request->ftp_username;
        $password = $request->ftp_password;
        $port = $request->ftp_port;
        $timeout = $request->ftp_timeout;
        try {
            $result = checkFtp($host, $username, $password, $port, $timeout);
        } catch (\Exception $e) {
            //$result = $e->getMessage();
            $result = 'Unable to connect to FTP Server.';
        }
        if ($result === true) {
            return json_encode(array("stat" => "success"));
        }
        return json_encode(array("stat" => "error", "error" => $result));
    }

    public function security()
    {
        //Role Check
        Roles::Check("app_preferences_security", true);

        // confirm password
        Helper::confirmPassword("appSecurity");

        $Preferences = Preference::find(1);
        return view("back.preferences.security", compact("Preferences"));
    }

    public function security_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_security", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $old_device_auth = $Preference->device_auth;
            $Preference->session_status = ($request->session_status) ? 1 : 0;
            $Preference->session_period = $request->session_period;
            $Preference->session_timer = $request->session_timer;
            $Preference->multi_login = ($request->multi_login) ? 1 : 0;
            $Preference->device_auth = $request->device_auth;

            $Preference->updated_by = Auth::user()->id;
            $Preference->save();

            // Log to history
            History::Log(
                $module = 1,
                $cat = 104,
                $window = 1040,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            if ($old_device_auth == 0 && $Preference->device_auth > 0) {
                $location = "unknown";
                $visitor_ip = $_SERVER['REMOTE_ADDR'];
                try {
                    $loc = GeoIP($visitor_ip);
                    $location = $loc['country'] . ", " . $loc['city'];
                } catch (\Exception $e) {
                    $location = "unknown";
                }

                // trust current device
                $AuthorizeDevice = new Authorize;
                $AuthorizeDevice->user_id = Auth::user()->id;
                $AuthorizeDevice->authorized = true;
                $AuthorizeDevice->token = uniqid() . time();
                $AuthorizeDevice->ip_address = $visitor_ip;
                $AuthorizeDevice->browser = Helper::getBrowser();
                $AuthorizeDevice->os = Helper::getOS();
                $AuthorizeDevice->location = $location;
                $AuthorizeDevice->device_type = Helper::detectDevice();
                $AuthorizeDevice->device_name = @gethostbyaddr($_SERVER['REMOTE_ADDR']);
                $AuthorizeDevice->attempt = 0;
                $AuthorizeDevice->authorized_at = date('Y-m-d H:i:s');
                $AuthorizeDevice->save();
            }

            return redirect()->action('Preferences\PreferenceController@security')
                ->with('doneMessage', trans('backLang.saveDone'));
        }

    }

    public function security_ips_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_security", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->block_ips = $request->block_ips;
            $Preference->updated_by = Auth::user()->id;
            $Preference->save();

            // Log to history
            History::Log(
                $module = 1,
                $cat = 104,
                $window = 1042,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@security')
                ->with('doneMessage', trans('backLang.saveDone'));
        }

    }

    public function security_devices(Request $request)
    {
        $columns = array(
            0 => 'created_at',
            1 => 'browser',
            2 => 'location',
            3 => 'user_id',
            4 => 'authorized',
            5 => 'options'
        );

        $title_var = "title_" . trans('backLang.boxCode');
        $title_var2 = "title_" . trans('backLang.boxCodeOther');

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $user_id = $request->input('user_id');
        $find_q = $request->input('find_q');

        if ($user_id > 0) {
            $Authorizes = Authorize::where("user_id", $user_id);
        } else {
            $Authorizes = Authorize::where("id", '>', 0);
        }
        if ($find_q != "") {
            $Authorizes = $Authorizes->where(function ($query) use ($find_q) {
                $query->where('browser', 'like', '%' . $find_q . '%')->orWhere('os', 'like', '%' . $find_q . '%')->orWhere('location', 'like', '%' . $find_q . '%')->orWhere('device_type', 'like', '%' . $find_q . '%')->orWhere('device_name', 'like', '%' . $find_q . '%');
            });
        }

        $totalData = $Authorizes->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Authorizes = $Authorizes->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            $total = 0;
            foreach ($Authorizes as $Authorize) {

                $nestedData['created_at'] = "<div class='text-center' dir='ltr'>" . Helper::DateView($Authorize->created_at, 0) . "<br>" . date("h:i A", strtotime($Authorize->created_at)) . "</div>";
                $device_details = "";
                if ($Authorize->device_type == "computer") {
                    $device_details .= "<div class='pull-right text-muted'><i class='fa fa-4x fa-desktop'></i></div>";
                } elseif ($Authorize->device_type == "tablet") {
                    $device_details .= "<div class='pull-right text-muted'><i class='fa fa-4x fa-tablet'></i></div>";
                } elseif ($Authorize->device_type == "mobile") {
                    $device_details .= "<div class='pull-right text-muted'><i class='fa fa-4x fa-mobile'></i></div>";
                } else {
                    $device_details .= "<div class='pull-right text-muted'><i class='fa fa-4x fa-question-circle'></i></div>";
                }
                $device_details .= "<div>" . $Authorize->device_name . "</div>";
                $device_details .= "<div>" . $Authorize->os . "</div>";
                $device_details .= "<div>" . $Authorize->browser . " <i class=\"browser pull-right browser-" . strtolower($Authorize->browser) . "\"></i></div>";
                $nestedData['browser'] = "<div>" . $device_details . "</div>";
                $nestedData['location'] = "<div class='text-center'>" . $Authorize->location . "</div>";
                $stat = "<div class='badge badge-danger pt-2 pl-2 pr-2 pb-2'><i class='icon-close'></i> " . trans("backLang.notAuthorized") . "</div>";
                if ($Authorize->authorized) {
                    $stat = "<div class='badge badge-success pt-2 pl-2 pr-2 pb-2'><i class='icon-check'></i> " . trans("backLang.authorized") . "</div>";
                }
                $nestedData['authorized'] = "<div class='text-center'>" . $stat . "</div>";
                if (!empty($Authorize->user)) {
                    $nestedData['user_id'] = "<div class='text-center'>" . $Authorize->user->name . "</div>";
                } else {
                    $nestedData['user_id'] = "<div class='text-center'> - </div>";
                }
                $options = "<div class=\"text-center\">";
                if (!$Authorize->authorized) {
                    $options .= "<button class=\"btn btn-info\" title='" . trans("backLang.acceptAuthorized") . "' data-toggle=\"modal\" onclick=\"setToAcceptDevice('" . $Authorize->id . "')\"><i class=\"icon-pencil\"></i></button>";
                }
                if (Roles::Check("delete")) {
                    $options .= "
 <button class=\"btn btn-danger\" onclick=\"delete_row('" . $Authorize->id . "')\"  title='" . trans("backLang.delete") . "'><i class=\"icon-trash\"></i></button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function security_device_destroy($id = 0)
    {
        //Role Check
        Roles::Check("app_preferences_security", true);

        if ($id > 0) {
            if (Roles::Check("delete")) {
                $Authorize = Authorize::find($id);
                if (!empty($Authorize)) {
                    $Authorize->delete();

                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 104,
                        $window = 1041,
                        $record_id = $Authorize->id,
                        $operation = 4,
                        $title = $Authorize->ip_address . "<br>" . $Authorize->browser . ", " . $Authorize->os . ", " . $Authorize->device_type . ", " . $Authorize->device_name
                    );

                    return json_encode(array("stat" => "success", "id" => $id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function security_device_accept($id = 0)
    {
        //Role Check
        Roles::Check("app_preferences_security", true);

        if ($id > 0) {
            $Authorize = Authorize::find($id);
            if (!empty($Authorize)) {
                $Authorize->authorized = true;
                $Authorize->authorized_at = date('Y-m-d H:i:s');
                $Authorize->save();

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 104,
                    $window = 1041,
                    $record_id = $Authorize->id,
                    $operation = 14,
                    $title = $Authorize->ip_address . "<br>" . $Authorize->browser . ", " . $Authorize->os . ", " . $Authorize->device_type . ", " . $Authorize->device_name
                );

                return json_encode(array("stat" => "success", "id" => $id));
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function notifications()
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        // confirm password
        Helper::confirmPassword("appNotificationsSettings");

        $Preferences = Preference::find(1);
        return view("back.preferences.notifications.settings", compact("Preferences"));
    }

    public function notifications_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $notification_cases = [];
            foreach (Notifications::Cases() as $key => $NotificationCase) {
                $notification_cases[$key] = [
                    "system" => ($request->{"case_" . $key . "_system"}) ? "1" : "0",
                    "mail" => ($request->{"case_" . $key . "_mail"}) ? "1" : "0",
                    "sms" => ($request->{"case_" . $key . "_sms"}) ? "1" : "0",
                    "browser" => ($request->{"case_" . $key . "_browser"}) ? "1" : "0"
                ];
            }
            $notification_cases = json_encode($notification_cases);
            $Preference->notification_cases = $notification_cases;

            $Preference->mail_driver = $request->mail_driver;
            $Preference->mail_host = $request->mail_host;
            $Preference->mail_port = $request->mail_port;
            $Preference->mail_username = $request->mail_username;
            $Preference->mail_password = $request->mail_password;
            $Preference->mail_encryption = $request->mail_encryption;
            $Preference->mail_from_address = $request->mail_from_address;
            $Preference->mail_from_name = $request->mail_from_name;
            $Preference->mail_queue = $request->mail_queue;

            $Preference->sms_status = ($request->sms_status) ? 1 : 0;
            $Preference->sms_username = $request->sms_username;
            $Preference->sms_password = $request->sms_password;
            $Preference->sms_sender = $request->sms_sender;
            $Preference->one_signal_queue = $request->sms_queue;

            $Preference->one_signal_status = ($request->one_signal_status) ? 1 : 0;
            $Preference->one_signal_app_id = $request->one_signal_app_id;
            $Preference->one_signal_api_key = $request->one_signal_api_key;
            $Preference->one_signal_auth_key = $request->one_signal_auth_key;
            $Preference->one_signal_queue = $request->one_signal_queue;

            $Preference->pusher_status = ($request->pusher_status) ? 1 : 0;
            $Preference->pusher_app_id = $request->pusher_app_id;
            $Preference->pusher_app_key = $request->pusher_app_key;
            $Preference->pusher_app_secret = $request->pusher_app_secret;
            $Preference->pusher_cluster = $request->pusher_cluster;

            $Preference->save();


            // Update .env file
            $env_update = Helper::changeEnv([
                'BROADCAST_DRIVER' => ($request->pusher_status && $request->pusher_app_id != "") ? "pusher" : "log",
                'PUSHER_APP_ID' => $request->pusher_app_id,
                'PUSHER_APP_KEY' => $request->pusher_app_key,
                'PUSHER_APP_SECRET' => $request->pusher_app_secret,
                'PUSHER_APP_CLUSTER' => $request->pusher_cluster,
                'ONE_SIGNAL_STATUS' => $request->one_signal_status,
                'ONE_SIGNAL_APP_ID' => $request->one_signal_app_id,
                'ONE_SIGNAL_API_KEY' => $request->one_signal_api_key,
                'ONE_SIGNAL_AUTH_KEY' => $request->one_signal_auth_key,
                'QUEUE_MESSAGE_STATUS' => $request->one_signal_queue,
                'QUEUE_SMS_STATUS' => $request->sms_queue,
                'MAIL_DRIVER' => $request->mail_driver,
                'MAIL_HOST' => $request->mail_host,
                'MAIL_PORT' => $request->mail_port,
                'MAIL_USERNAME' => $request->mail_username,
                'MAIL_PASSWORD' => $request->mail_password,
                'MAIL_ENCRYPTION' => $request->mail_encryption,
                'MAIL_FROM_ADDRESS' => $request->mail_from_address,
                'MAIL_FROM_NAME' => $request->mail_from_name,
                'QUEUE_MAIL_STATUS' => $request->mail_queue,
            ]);


            // Log to history
            History::Log(
                $module = 1,
                $cat = 107,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@notifications')
                ->with('doneMessage', trans('backLang.saveDone'));
        }

    }

    public function cron()
    {
        //Role Check
        Roles::Check("app_preferences_cron", true);

        // confirm password
        Helper::confirmPassword("appCronSettings");

        $Preferences = Preference::find(1);
        return view("back.preferences.cron", compact("Preferences"));
    }

    public function cron_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_notifications", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $cron_settings = [
                "notifications" => [
                    "status" => ($request->del_notifications) ? 1 : 0,
                    "days" => $request->del_notifications_days
                ],
                "activities" => [
                    "status" => ($request->del_activities) ? 1 : 0,
                    "days" => $request->del_activities_days
                ],
                "messages" => [
                    "status" => ($request->del_messages) ? 1 : 0,
                    "days" => $request->del_messages_days
                ],
                "events" => [
                    "status" => ($request->del_events) ? 1 : 0,
                    "days" => $request->del_events_days
                ],
                "alerts" => [
                    "status" => ($request->del_alerts) ? 1 : 0,
                    "days" => $request->del_alerts_days
                ],
                "attachments" => [
                    "status" => ($request->del_attachments) ? 1 : 0,
                    "days" => $request->del_attachments_days
                ],
                "backups" => [
                    "status" => ($request->del_backups) ? 1 : 0,
                    "days" => $request->del_backups_days
                ],
                "chats" => [
                    "status" => ($request->del_chats) ? 1 : 0,
                    "days" => $request->del_chats_days
                ],
                "currencies" => [
                    "status" => ($request->update_currencies) ? 1 : 0,
                    "hours" => $request->update_currencies_hours
                ],
            ];

            $cron_settings = json_encode($cron_settings);
            $Preference->cron_settings = $cron_settings;

            $Preference->save();

            // Log to history
            History::Log(
                $module = 1,
                $cat = 110,
                $window = 1100,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@cron')
                ->with('doneMessage', trans('backLang.saveDone'));
        }

    }

    public function print()
    {
        //Role Check
        Roles::Check("app_preferences_print", true);

        // confirm password
        Helper::confirmPassword("appPrintSettings");

        $Preferences = Preference::find(1);
        return view("back.preferences.print", compact("Preferences"));
    }

    public function print_update(Request $request)
    {
        //Role Check
        Roles::Check("app_preferences_print", true);

        $Preference = Preference::find(1);
        if (!empty($Preference)) {

            $Preference->print_header_ar = $request->print_header_ar;
            $Preference->print_header_en = $request->print_header_en;
            $Preference->print_footer_ar = $request->print_footer_ar;
            $Preference->print_footer_en = $request->print_footer_en;
            $Preference->save();

            // Log to history
            History::Log(
                $module = 1,
                $cat = 113,
                $window = 0,
                $record_id = $Preference->id,
                $operation = 3,
                $title = ""
            );

            return redirect()->action('Preferences\PreferenceController@print')
                ->with('doneMessage', trans('backLang.saveDone'));
        }
    }
}
