<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Role;
use App\Models\User;
use App\Models\Message;
use App\Models\Activity;
use App\Models\Notice;
use App\Models\country;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Notifications;
use Validator;
use Hash;
use App\Helpers\History;
use App\Helpers\AppSender;

class ProfileController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }

    public function index()
    {
        $User = User::find(Auth::user()->id);
        if (!empty($User)) {
            $RolesList = Role::all();
            $SelfProfile = true;
            return view("back.preferences.users.profile", compact("User", "RolesList", "SelfProfile"));
        }
        return redirect()->route('NotFound');
    }

    public function update(Request $request)
    {
        if (Roles::Check("profile_update")) {
            $User = User::find(decrypt($request->id));
            if (!empty($User)) {
                $validator = Validator::make($request->all(), [
                    'photo' => 'mimes:png,jpeg,jpg,gif|max:5000',
                    'name' => 'required',
                    'email' => 'required|unique:users,email,' . $User->id,
                    'country' => 'required',
                    'phone' => 'required|numeric'
                ]);
                if ($validator->passes()) {
                    try {
                        $phone_fc = mb_substr($request->phone, 0, 1, 'utf-8');
                        if ($phone_fc == "0") {
                            $u_phone = mb_substr($request->phone, 1);
                        } else {
                            $u_phone = $request->phone;
                        }
                        $phone_full = $u_phone;
                        $phone_code = "";
                        $Country = country::find($request->country);
                        if (!empty($Country)) {
                            $phone_full = $Country->tel . $u_phone;
                            $phone_code = $Country->tel;
                        }

                        //check full phone
                        $User_count = User::where("phone_full", $phone_full)->where('id', "!=", $User->id)->count();
                        if ($User_count > 0) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.phoneIsUsed')]]);
                        }

                        // delete old photo
                        if ($request->photo_delete == 1) {
                            // Delete photo
                            if ($User->photo != "") {
                                if (file_exists($this->uploadPath . $User->photo)) {
                                    File::delete($this->uploadPath . $User->photo);
                                }
                            }
                            $User->photo = "";
                        }

                        $formFileName = "photo";
                        $fileFinalName = "";
                        if ($request->$formFileName != "") {
                            // Delete photo
                            if ($User->photo != "") {
                                if (file_exists($this->uploadPath . $User->photo)) {
                                    File::delete($this->uploadPath . $User->photo);
                                }
                            }

                            $fileFinalName = time() . rand(1111, 9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                            $path = $this->uploadPath;
                            $request->file($formFileName)->move($path, $fileFinalName);
                        }

                        $User->name = $request->name;
                        $User->job = $request->job;
                        $User->email = $request->email;
                        if ($request->password != "") {
                            $User->password = bcrypt($request->password);
                        }
                        $User->country_id = $request->country;
                        $User->phone_code = $phone_code;
                        $User->phone = $u_phone;
                        $User->phone_full = $phone_full;
                        if ($fileFinalName != "") {
                            $User->photo = $fileFinalName;
                        }
                        $User->updated_ip = \Request::ip();
                        $User->updated_by = Auth::user()->id;
                        $User->save();

                        // Log to history
                        History::Log(
                            $module = 1,
                            $cat = 103,
                            $window = 0,
                            $record_id = $User->id,
                            $operation = 2,
                            $title = $User->name . "<br>" . $User->email
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($User->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
                return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
            }
        }
    }

    public function messages(Request $request)
    {
        $columns = array(
            0 => 'created_at',
            1 => 'created_by',
            2 => 'status',
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $to_user_id = $request->input('to_user_id');
        $to_customer_id = $request->input('to_customer_id');
        $cat = $request->input('cat');
        $cat_id = $request->input('cat_id');
        $self_profile = $request->input('self_profile');

        $Messages = Message::where("id", '>', 0);
        if ($to_user_id != "") {
            $user_id = decrypt($to_user_id);
            $Messages = $Messages->where("to_user_id", $user_id);
        }
        if ($to_customer_id != "") {
            $customer_id = decrypt($to_customer_id);
            $Messages = $Messages->where("to_customer_id", $customer_id);
        }
        if ($cat > -1 && is_numeric($cat)) {
            $Messages = $Messages->where("cat", $cat);
        }
        if ($cat_id > 0 && is_numeric($cat_id)) {
            $Messages = $Messages->where("cat_id", $cat_id);
        }
        if (!Roles::Check("profile_hidden_messages")) {
            $Messages = $Messages->where("hidden", false);
        }

        $totalData = $Messages->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Messages = $Messages->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Messages as $Message) {

                $message_title = "";
                $message_icon = "";
                $message_details = "";
                if ($Message->type == 2) {
                    $message_title = $Message->details;
                    $message_icon = "<a href=\"javascript:void(0)\" onclick=\"message_view('" . encrypt($Message->id) . "')\"><i class='icon-screen-desktop fa-2x pull-right'></i></a>";
                    $message_details = "";
                } elseif ($Message->type == 1) {
                    $message_title = $Message->details;
                    $message_icon = "<a href=\"javascript:void(0)\" onclick=\"message_view('" . encrypt($Message->id) . "')\"><i class='icon-speech fa-2x pull-right'></i></a>";
                    $message_details = "<div><small>" . trans('backLang.phone') . ": <span dir='ltr'>+" . $Message->to_phone . "</span></small></div>";
                } elseif ($Message->type == 0) {
                    $message_title = $Message->title;
                    $message_icon = "<a href=\"javascript:void(0)\" onclick=\"message_view('" . encrypt($Message->id) . "')\"><i class='icon-envelope-letter fa-2x pull-right'></i></a>";
                    $message_details = "<div><small>" . trans('backLang.email') . ": " . $Message->to_email . "</small></div>";
                }


                $nestedData['created_at'] = $message_icon . "<small>" . Helper::DateView($Message->created_at, 1) . "</small><br><a href=\"javascript:void(0)\" onclick=\"message_view('" . encrypt($Message->id) . "')\">" . $message_title . "</a>" . $message_details;
                if (!empty($Message->user)) {
                    $nestedData['created_by'] = "<div class='text-center'>" . $Message->user->name . "</div>";
                } else {
                    $nestedData['created_by'] = "<div class='text-center'> - </div>";
                }

                $stat = "<i class='icon-clock text-muted mb-2' title='" . trans('backLang.messageSendPending') . "'></i>";
                $hidden = "";
                if ($Message->status === 2) {
                    $stat = "<i class='icon-close text-muted mb-2' title='" . trans('backLang.messageSendFailed') . "'></i>";
                } elseif ($Message->status === 1) {
                    $stat = "<i class='icon-check text-success mb-2' title='" . trans('backLang.messageSendDone') . "'></i>";
                }
                if ($Message->hidden && !@$self_profile) {
                    $hidden = "<br><i class='fa fa-eye-slash text-muted' title='" . trans('backLang.messageISHiddenFromProfile') . "'></i>";
                }

                $nestedData['status'] = "<div class='text-center'>" . $stat . $hidden . "</div>";

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    public function message_send(Request $request)
    {
        if ($request->user != "") {
            $User = User::find(decrypt($request->user));
            if (!empty($User)) {
                if (in_array($request->stat, ["notification", "sms", "mail", "browser"])) {
                    switch ($request->stat) {
                        case ($request->stat == "notification"):
                            // Start of Send
                            $Send = AppSender::ToSystem(
                                0,
                                [
                                    "type" => 0,
                                    "subject" => $request->subject,
                                    "message" => $request->details,
                                    "url" => $request->link,
                                    "icon" => "fe fe-bell",
                                ],
                                $User->id,
                                0,
                                $User->id
                            );
                            // End of Send Email
                            if ($Send) {
                                return response()->json(['stat' => 'success']);
                            }
                            break;
                        case ($request->stat == "mail"):
                            // Start of Send Email
                            $Send = AppSender::ToMail(
                                $request->email,
                                $User->name,
                                1,
                                [
                                    "subject" => $request->subject,
                                    "message" => $request->details,
                                    "to_user_name" => $User->name,
                                    "from_user_name" => Auth::user()->name,
                                ],
                                $User->id,
                                0,
                                $User->id,
                                false
                            );
                            // End of Send Email
                            if ($Send) {
                                return response()->json(['stat' => 'success']);
                            }
                            break;
                        case ($request->stat == "sms"):
                            // Start of Send SMS
                            $Send = AppSender::ToSMS(
                                $request->phone,
                                0,
                                [
                                    "message" => $request->details,
                                ],
                                $User->id,
                                0,
                                $User->id,
                                false
                            );
                            // End of Send SMS
                            if ($Send) {
                                return response()->json(['stat' => 'success']);
                            }
                            break;
                        case ($request->stat == "browser"):
                            // Start of Send to browser
                            $Send = AppSender::ToBrowser(
                                0,
                                [
                                    "message" => $request->details,
                                    "url" => $request->link,
                                ],
                                $User->id,
                                0,
                                $User->id,
                                false
                            );
                            // End of Send to browser
                            if ($Send) {
                                return response()->json(['stat' => 'success']);
                            }
                            break;
                    }
                }
            }
        }

        // Log to history
        History::Log(
            $module = 1,
            $cat = 103,
            $window = ($request->stat == "browser") ? 1032 : 1031,
            $record_id = $User->id,
            $operation = 2,
            $title = $request->subject
        );
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    public function message_view($id = "")
    {
        if ($id != "") {
            $Message = Message::find(decrypt($id));
            if (!empty($Message)) {
                $message_title = "";
                $message_details = "";
                if ($Message->type == 2) {
                    $message_title = "";
                    if ($Message->url != "") {
                        $message_details = "<div><small>" . trans('backLang.url') . ": <a href='" . $Message->url . "' target='_blank'>" . $Message->url . "</a></small></div>";
                    }
                } elseif ($Message->type == 1) {
                    $message_title = "";
                    $message_details = "<div><small>" . trans('backLang.phone') . ": <span dir='ltr'>+" . $Message->to_phone . "</span></small></div>";
                } elseif ($Message->type == 0) {
                    $message_title = "<div><strong>" . $Message->title . "</strong></div>";
                    $message_details = "<div><small>" . trans('backLang.email') . ": " . $Message->to_email . "</small></div>";
                }

                $message_view = "<div style='padding: 10px !important;'>" . $message_title . $message_details . "</div><div class=\"modal-body border-top bg-white\">" . $Message->details . "</div>";
                $message_view .= "<div class=\"modal-footer\">
<div class='modal-footer-extra'>
<small>" . trans('backLang.createdBy') . ": " . @$Message->user->name . "</small><br>
<small>" . trans('backLang.sendDate') . ": " . Helper::DateView($Message->created_at, 1) . "</small>
</div>
";
                if (Roles::Check("delete")) {
                    $message_view .= "<button type=\"button\" class=\"btn btn-sm btn-light mb-4\" onclick=\"message_delete('" . encrypt($Message->id) . "')\">
                        &nbsp; <i
                            class=\"fe fe-trash\"></i> " . trans('backLang.deleteMessage') . " &nbsp;
                    </button>
</div>";
                } else {
                    $message_view .= "<button type=\"button\" class=\"btn btn-sm btn-light mb-4\" data-dismiss=\"modal\">
                        &nbsp; <i
                            class=\"fe fe-x\"></i> " . trans('backLang.close') . " &nbsp;
                    </button>
</div>";
                }
                return $message_view;
            }
        }

        return "<div class='alert alert-danger'>" . trans('backLang.erorr') . "</div>";
    }

    public function message_destroy($id = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Message = Message::find(decrypt($id));
                if (!empty($Message)) {
                    $Message->delete();
                    return json_encode(array("stat" => "success", "id" => $Message->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function notifications(Request $request)
    {
        $columns = array(
            0 => 'created_at'
        );

        $title_var = "title_" . trans('backLang.boxCode');
        $title_var2 = "title_" . trans('backLang.boxCodeOther');
        $details_var = "details_" . trans('backLang.boxCode');
        $details_var2 = "details_" . trans('backLang.boxCodeOther');

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $to_user_id = $request->input('to_user_id');
        $to_customer_id = $request->input('to_customer_id');
        $cat = $request->input('cat');
        $cat_id = $request->input('cat_id');

        $Notices = Notice::where("id", '>', 0);
        $new_Notices = Notice::where("id", '>', 0);
        if ($to_user_id != "") {
            $user_id = decrypt($to_user_id);
            $Notices = $Notices->where("to_user_id", $user_id);
            $new_Notices = $new_Notices->where("to_user_id", $user_id);
        }
        if ($to_customer_id != "") {
            $customer_id = decrypt($to_customer_id);
            $Notices = $Notices->where("to_customer_id", $customer_id);
            $new_Notices = $new_Notices->where("to_customer_id", $customer_id);
        }
        if ($cat > -1 && is_numeric($cat)) {
            $Notices = $Notices->where("cat", $cat);
            $new_Notices = $new_Notices->where("cat", $cat);
        }
        if ($cat_id > 0 && is_numeric($cat_id)) {
            $Notices = $Notices->where("cat_id", $cat_id);
            $new_Notices = $new_Notices->where("cat_id", $cat_id);
        }

        $new_notifications_count = $new_Notices->where("status", false)->count();

        $totalData = $Notices->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Notices = $Notices->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Notices as $Notice) {

                if ($Notice->$title_var != "") {
                    $message_title = $Notice->$title_var;
                } else {
                    $message_title = $Notice->$title_var2;
                }
                if ($Notice->$details_var != "") {
                    $message_details = strip_tags($Notice->$details_var);
                } else {
                    $message_details = strip_tags($Notice->$details_var2);
                }
                if (strlen($message_details) > 100) {
                    $message_details = mb_substr($message_details, 0, 100) . "...";
                }

                $stat = "<div class='badge badge-danger pt-2 pl-2 pr-2 pb-2 pull-right'><i class='icon-clock mb-2'></i> " . trans('backLang.notificationNew') . "</div>";
                $text_class = "text-danger";
                if ($Notice->status === 1) {
                    $stat = "<div class='badge bg-light-gray pt-2 pl-2 pr-2 pb-2 pull-right'><i class='icon-check mb-2'></i> " . trans('backLang.notificationRead') . "</div>";
                    $text_class = "";
                }
                $message_icon = "<i class='fe fe-bell fa-3x ml-2'></i>";
                if ($Notice->icon != "") {
                    $message_icon = "<i class='" . $Notice->icon . " fa-3x ml-2'></i>";
                }
                $created_by = "";
                if (!empty($Notice->user)) {
                    $created_by = "<div><small><i class='icon-user'></i> " . $Notice->user->name . "</small></div>";
                }
                $nestedData['created_at'] = "<div class='pointer text-muted' onclick=\"notification_view('" . encrypt($Notice->id) . "')\"><table class='table mb-0 border-0'><tr><td class='text-center width-50px'>" . $message_icon . "</td><td>" . $stat . "<small>" . Helper::DateView($Notice->created_at, 1) . "</small><br><strong class='" . $text_class . "'>" . $message_title . "</strong><br>" . $message_details . $created_by . "</td></tr></table></div>";


                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data,
            "new_notifications_count" => $new_notifications_count
        );

        return json_encode($json_data);

    }

    public function notification_view($id = "")
    {
        if ($id != "") {
            $Notice = Notice::find(decrypt($id));
            if (!empty($Notice)) {
                if (@Auth::user()->id == $Notice->to_user_id) {
                    $Notice->status = 1;
                    $Notice->save();
                }

                $title_var = "title_" . trans('backLang.boxCode');
                $title_var2 = "title_" . trans('backLang.boxCodeOther');
                $details_var = "details_" . trans('backLang.boxCode');
                $details_var2 = "details_" . trans('backLang.boxCodeOther');

                if ($Notice->$title_var != "") {
                    $message_title = $Notice->$title_var;
                } else {
                    $message_title = $Notice->$title_var2;
                }
                if ($Notice->$details_var != "") {
                    $message_details = $Notice->$details_var;
                } else {
                    $message_details = $Notice->$details_var2;
                }
                $message_title = "<div><strong>" . $message_title . "</strong></div>";
                $message_url = "";
                if ($Notice->url != "") {
                    $message_url = "<div class='mt-2'><a href='" . $Notice->url . "' class='btn btn-primary' dir='ltr'>" . trans("backLang.viewDetails") . "</a></div>";
                }

                $notification_view = "<div style='padding: 10px !important;'>" . $message_title . "</div><div class=\"modal-body border-top bg-white\">" . $message_details . $message_url . "</div>";
                $notification_view .= "<div class=\"modal-footer\">
<div class='modal-footer-extra'>
<small>" . trans('backLang.createdBy') . ": " . @$Notice->user->name . "</small><br>
<small>" . trans('backLang.sendDate') . ": " . Helper::DateView($Notice->created_at, 1) . "</small>
</div>
";
                if (Roles::Check("delete")) {
                    $notification_view .= "<button type=\"button\" class=\"btn btn-sm btn-light mb-4\" onclick=\"notification_delete('" . encrypt($Notice->id) . "')\">
                        &nbsp; <i
                            class=\"fe fe-trash\"></i> " . trans('backLang.notificationDelete') . " &nbsp;
                    </button>
</div>";
                } else {
                    $notification_view .= "<button type=\"button\" class=\"btn btn-sm btn-light mb-4\" data-dismiss=\"modal\">
                        &nbsp; <i
                            class=\"fe fe-x\"></i> " . trans('backLang.close') . " &nbsp;
                    </button>
</div>";
                }
                return $notification_view;
            }
        }

        return "<div class='alert alert-danger'>" . trans('backLang.erorr') . "</div>";
    }

    public function notification_destroy($id = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Notice = Notice::find(decrypt($id));
                if (!empty($Notice)) {
                    $Notice->delete();
                    return json_encode(array("stat" => "success", "id" => $Notice->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    public function theme(Request $request)
    {
        $User = Auth::user();
        if (!empty($User)) {
            if ($request->stat == "color") {
                $User->theme_style = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "sidebar") {
                $User->theme_sidebar = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "navbar") {
                $User->theme_navbar = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "header") {
                $User->theme_header = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "menu") {
                $User->theme_menu = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "preload") {
                $User->theme_preload = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
            if ($request->stat == "sounds") {
                $User->alert_sounds = ($request->action) ? 1 : 0;
                $User->timestamps = false;
                $User->save();
            }
        }
        return json_encode(array("stat" => "success"));
    }

    public function pass($RedirectRouteName = "")
    {
        if ($RedirectRouteName == "") {
            $RedirectRouteName = "appSettings";
        }
        return view('back.preferences.pass', compact("RedirectRouteName"));
    }

    public function confirm(Request $request)
    {
        $this->validate($request, [
            'password' => 'required'
        ]);

        if (Hash::check($request->password, auth()->user()->password)) {
            //continue
            \Session()->put('app_pass_confirm', "true");
            return json_encode(array("stat" => "success"));
        }
        return json_encode(array("stat" => "error"));
    }

    public function notices()
    {

        $title_var = "title_" . trans('backLang.boxCode');
        $title_var2 = "title_" . trans('backLang.boxCodeOther');
        $details_var = "details_" . trans('backLang.boxCode');
        $details_var2 = "details_" . trans('backLang.boxCodeOther');

        $notifications = [];
        if (@Auth::check()) {
            $notificats = Notice::where("to_user_id", Auth::user()->id)->orderby("created_at", "desc")->limit(15)->get();
            foreach ($notificats as $Notice) {

                if ($Notice->$title_var != "") {
                    $message_title = $Notice->$title_var;
                } else {
                    $message_title = $Notice->$title_var2;
                }
                if ($Notice->$details_var != "") {
                    $message_details = strip_tags($Notice->$details_var);
                } else {
                    $message_details = strip_tags($Notice->$details_var2);
                }
                if (strlen($message_details) > 50) {
                    $message_details = mb_substr($message_details, 0, 50) . "..";
                }
                $s4ds_current_date = date('Y-m-d', $_SERVER['REQUEST_TIME']);
                $day_mm = date('Y-m-d', strtotime($Notice->created_at));
                if ($day_mm == $s4ds_current_date) {
                    $newMessage_date = date('h:i A', strtotime($Notice->created_at));
                } else {
                    $newMessage_date = Helper::DateView($Notice->created_at, 1);
                }

                $notifications[] = [
                    "id" => encrypt($Notice->id),
                    "title" => $message_title,
                    "details" => $message_details,
                    "icon" => ($Notice->icon != "") ? $Notice->icon : "fe fe-bell",
                    "created_at" => $newMessage_date,
                    "status" => $Notice->status,
                ];
            }
        }

        return json_encode(array(
            "chats" => Notifications::ChatAlerts(),
            "notifications_count" => Notifications::unreadCount(),
            "notifications" => $notifications
        ));
    }

    public function activities(Request $request)
    {
        $columns = array(
            0 => 'created_at',
            1 => 'window',
            2 => 'title',
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        $user_id = $request->input('user_id');
        $cat = $request->input('cat');
        $cat_id = $request->input('cat_id');
        $self_profile = $request->input('self_profile');
        $operation = $request->input('operation');
        $module = $request->input('module');
        $created_at = $request->input('created_at');

        $Activities = Activity::where("id", '>', 0);
        if ($user_id != "") {
            $user_id = decrypt($user_id);
            $Activities = $Activities->where("created_by", $user_id);
        }
        if ($cat > -1 && is_numeric($cat)) {
            $Activities = $Activities->where("cat", $cat);
        }
        if ($cat_id > 0 && is_numeric($cat_id)) {
            $Activities = $Activities->where("record_id", $cat_id);
        }
        if ($operation > 0) {
            $Activities = $Activities->where("operation", $operation);
        }
        if ($module > 0) {
            $Activities = $Activities->where("module", $module);
        }
        if ($created_at != "") {
            $start_date = date("Y-m-d H:i:s", strtotime($created_at . " 00:00:00"));
            $end_date = date("Y-m-d H:i:s", strtotime($created_at . " 23:59:59"));
            $Activities = $Activities->where("created_at", ">=", $start_date)->where("created_at", "<=", $end_date);
        }

        $totalData = $Activities->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Activities = $Activities->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Activities as $Activity) {
                $Activity_By = "";
                if (!empty($Activity->user)) {
                    $Activity_By = $Activity->user->name;
                } else if (!empty($Activity->customer)) {
                    $Activity_By = $Activity->customer->name_full;
                }
                $Activity_user = "<div><small>" . trans("backLang.createdBy") . ": <strong>" . $Activity_By . "</strong></small></div>";
                $operation = @History::Operations()[$Activity->operation]['title'];
                $icon = @History::Operations()[$Activity->operation]['icon'];
                if ($icon != "") {
                    $icon = "<i class='$icon'></i>";
                }
                $module = @History::Modules()[$Activity->module]['title'];
                $cat = @History::Modules()[$Activity->module]['cats'][$Activity->cat]['title'];
                if ($cat != "") {
                    $cat = " &rsaquo;&rsaquo; " . $cat;
                }
                $window = @History::Modules()[$Activity->module]['cats'][$Activity->cat]['windows'][$Activity->window];
                if ($window != "") {
                    $window = " &rsaquo;&rsaquo; " . $window;
                }

                $nestedData['created_at'] = "<small>" . Helper::DateView($Activity->created_at, 1) . "<br>" . $Activity->created_ip . "</small>";
                $nestedData['window'] = $icon . " <small><strong>" . $operation . "</strong><br>" . $module . " " . $cat . " " . $window . "</small>";
                $nestedData['title'] = "<small>" . $Activity->title . "</small>".$Activity_user;
                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

}
