<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Role;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use App\Helpers\History;

class RoleController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_roles", true);
        // confirm password
        Helper::confirmPassword("AppRoles");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view("back.preferences.roles.list");
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function list(Request $request)
    {
        $columns = array(
            0 => 'name',
            1 => 'created_by',
            2 => 'created_at',
            3 => 'status',
            4 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (Roles::Check("view")) {
            $Roles = Role::where("created_by", @Auth::user()->id);
        } else {
            $Roles = Role::where("id", '>', 0);
        }
        $totalData = $Roles->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Roles = $Roles->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Roles as $Role) {
                $nestedData['name'] = $Role->name;
                if (!empty($Role->user)) {
                    $nestedData['created_by'] = "<div class='text-center'>" . $Role->user->name . "</div>";
                } else {
                    $nestedData['created_by'] = "<div class='text-center'> - </div>";
                }
                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($Role->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['created_at'] = "<div class='text-center'> " . Helper::DateView($Role->created_at, 1) . " </div>";
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";
                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
 <a class=\"btn btn-sm btn-info\" href=\"" . route("AppRolesEdit", encrypt($Role->id)) . "\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</a>";
                }

                if (Roles::Check("add")) {
                    $options .= "
 <button class=\"btn btn-sm btn-primary\" onclick=\"copy_row('" . encrypt($Role->id) . "')\"><i class=\"fe fe-copy\"></i> " . trans("backLang.copy") . "</button>";
                }

                if (Roles::Check("delete") && $Role->id != 1) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($Role->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            if ($request->title != "") {
                if ($request->from_id != "") {
                    $OldRoles = Role::find(decrypt($request->from_id));
                    if (!empty($OldRoles)) {
                        $Roles = $OldRoles->replicate();
                        $Roles->name = $request->title;
                        $Roles->created_ip = \Request::ip();
                        $Roles->created_by = Auth::user()->id;
                        $Roles->save();

                        // Log to history
                        History::Log(
                            $module = 1,
                            $cat = 102,
                            $window = 0,
                            $record_id = $Roles->id,
                            $operation = 2,
                            $title = $Roles->name
                        );

                        return json_encode(array("stat" => "success", "id" => encrypt($Roles->id)));
                    }
                } else {
                    $Roles = new Role;
                    $Roles->name = $request->title;
                    $Roles->status = true;
                    $Roles->created_ip = \Request::ip();
                    $Roles->created_by = Auth::user()->id;
                    $Roles->save();

                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 102,
                        $window = 0,
                        $record_id = $Roles->id,
                        $operation = 2,
                        $title = $Roles->name
                    );

                    return json_encode(array("stat" => "success", "id" => encrypt($Roles->id)));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  string $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $ERoles = Role::find(decrypt($id));
                if (!empty($ERoles)) {
                    $Roles = [];
                    if ($ERoles->roles != "") {
                        try {
                            $Roles = json_decode($ERoles->roles);
                        } catch (\Exception $e) {

                        }
                    }
                    return view("back.preferences.roles.edit", compact("ERoles", "Roles"));
                }
            }
        }
        return redirect()->action('Preferences\RoleController@index');
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        //
        if (Roles::Check("edit")) {
            $Role = Role::find(decrypt($request->id));
            if (!empty($Role)) {

                $this->validate($request, [
                    'name' => 'required'
                ]);

                $roles = [];
                foreach (Roles::List() as $AppRoleGroup) {
                    if ($AppRoleGroup['group_check']) {
                        $roles[$AppRoleGroup['group_role']] = ($request->{$AppRoleGroup['group_role']}) ? "1" : "0";
                    }
                    foreach ($AppRoleGroup['group_roles'] as $AppRole) {
                        $roles[$AppRole['role']] = ($request->{$AppRole['role']}) ? "1" : "0";
                    }
                }
                $roles_string = json_encode($roles);
                $Role->name = $request->name;
                $Role->add = ($request->add) ? "1" : "0";
                $Role->edit = ($request->edit) ? "1" : "0";
                $Role->delete = ($request->delete) ? "1" : "0";
                $Role->view = ($request->view) ? "1" : "0";
                $Role->roles = $roles_string;
                $Role->status = ($request->status) ? "1" : "0";
                $Role->updated_ip = \Request::ip();
                $Role->updated_by = Auth::user()->id;
                $Role->save();

                // Log to history
                History::Log(
                    $module = 1,
                    $cat = 102,
                    $window = 0,
                    $record_id = $Role->id,
                    $operation = 3,
                    $title = $Role->name
                );

                return redirect()->action('Preferences\RoleController@edit', encrypt($Role->id))->with('doneMessage',
                    trans('backLang.saveDone'));
            }
        }
        return redirect()->action('Preferences\RoleController@index');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0)
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Role = Role::find(decrypt($id));
                if (!empty($Role) && $Role->id != 1) {
                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 102,
                        $window = 0,
                        $record_id = $Role->id,
                        $operation = 4,
                        $title = $Role->name
                    );

                    $Role->delete();
                    return json_encode(array("stat" => "success", "id" => $Role->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
