<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Share;
use App\Models\User;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use Validator;
use Cats;
use App\Helpers\History;
use App\Helpers\AppSender;

class ShareController extends Controller

{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function load(Request $request)
    {
        $cat = $request->cat;
        $cat_id = $request->cat_id;
        $share_roles = [
            "create" => ($request->role_create == "true" || $request->role_create == "1") ? true : false,
            "delete" => ($request->role_delete == "true" || $request->role_delete == "1") ? true : false,
            "title" => $request->title,
            "url" => $request->url,
        ];
        if ($cat_id != "") {
            $Shares = Share::where("cat", $cat)->where("cat_id", decrypt($cat_id))->orderby("id", "asc")->get();
            $Shared_with = Share::select('user_id')->where("cat", $cat)->where("cat_id", decrypt($cat_id))->get();
        } else {
            $Shares = Share::where("cat", $cat)->where("cat_id", 0)->orderby("id", "asc")->get();
            $Shared_with = Share::select('user_id')->where("cat", $cat)->where("cat_id", 0)->get();
        }
        $UsersList = User::where("status", true)->wherenotin("id", $Shared_with)->orderby("name", "asc")->get();
        return view("back.preferences.share.list", compact("Shares", "cat", "cat_id", "share_roles", "UsersList"));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {
            try {
                if ($request->cat_id != "") {
                    $cat_id = decrypt($request->cat_id);
                } else {
                    $cat_id = 0;
                }
                if ($request->user_ids != "") {
                    foreach ($request->user_ids as $user_id) {
                        $user_id = decrypt($user_id);
                        if ($user_id > 0 && $cat_id >= 0) {
                            $Share = Share::where("cat", $request->cat)->where("cat_id", $cat_id)->where("user_id", $user_id)->first();
                            if (empty($Share)) {
                                $Share = new Share;
                                $Share->cat = $request->cat;
                                $Share->cat_id = $cat_id;
                                $Share->user_id = $user_id;
                                $Share->status = true;
                                $Share->created_ip = \Request::ip();
                                $Share->created_by = Auth::user()->id;
                                $Share->save();

                                // Log to history
                                History::Log(
                                    $_module = @Cats::ShareTypes()[$Share->cat]['history']['module'],
                                    $_cat = @Cats::ShareTypes()[$Share->cat]['history']['cat'],
                                    $_window = @Cats::ShareTypes()[$Share->cat]['history']['window'],
                                    $_record_id = $Share->id,
                                    $_operation = 16,
                                    $_title = trans("backLang.shareWith")." ".@$Share->user->name
                                );

                                $User = $Share->user;
                                if(!empty($User)) {
                                    // Send Notification
                                    AppSender::Notify(
                                        2,
                                        [
                                            "type" => 0,
                                            "to_user_id" => $User->id,
                                            "to_email" => $User->email,
                                            "to_name" => $User->name,
                                            "to_phone" => $User->phone_full,

                                            "title" => $request->title,
                                            "to_user_name" => $User->name,
                                            "from_user_name" => @Auth::user()->name,
                                            "url" => $request->url,
                                            "icon" => "fe fe-git-merge",
                                        ],
                                        0,
                                        $User->id
                                    );
                                }

                            }
                        }
                    }
                    return response()->json(['stat' => 'success']);
                }
            } catch (\Exception $e) {
                return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = 0,$title = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $Share = Share::find(decrypt($id));
                if (!empty($Share)) {
                    if ($Share->created_by == @Auth::user()->id || $Share->user_id == @Auth::user()->id) {

                        // Log to history
                        History::Log(
                            $_module = @Cats::ShareTypes()[$Share->cat]['history']['module'],
                            $_cat = @Cats::ShareTypes()[$Share->cat]['history']['cat'],
                            $_window = @Cats::ShareTypes()[$Share->cat]['history']['window'],
                            $_record_id = $Share->id,
                            $_operation = 4,
                            $_title = trans("backLang.shareWith")." ".@$Share->user->name
                        );


                        $User = $Share->user;
                        if(!empty($User)) {
                            // Send Notification
                            AppSender::Notify(
                                3,
                                [
                                    "type" => 0,
                                    "to_user_id" => $User->id,
                                    "to_email" => $User->email,
                                    "to_name" => $User->name,
                                    "to_phone" => $User->phone_full,

                                    "title" => $title,
                                    "to_user_name" => $User->name,
                                    "from_user_name" => @Auth::user()->name,
                                    "url" => "",
                                    "icon" => "fe fe-eye-off",
                                ],
                                0,
                                $User->id
                            );
                        }

                        $Share->delete();
                        return json_encode(array("stat" => "success", "id" => $Share->id));
                    }
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }
}
