<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Template;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Cats;
use Roles;
use Notifications;
use Validator;
use App\Helpers\History;

class TemplateController extends Controller
{
    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_templates", true);
        // confirm password
        Helper::confirmPassword("AppTemplates");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($cat = 3, $stat = 0)
    {
        return view("back.preferences.templates.list", compact("cat", "stat"));
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function load($cat = '', $stat = '')
    {
        if ($stat > 0 && $cat != "") {
            $Template = Template::where('cat', $cat)->where('stat', $stat)->first();
            if (!empty($Template)) {
                return view("back.preferences.templates.edit", compact("Template"));
            } else {
                foreach (Notifications::Templates() as $key => $Temp) {
                    if ($key == $stat) {
                        // save to db
                        $Template = new Template;
                        $Template->cat = $cat;
                        $Template->stat = $key;
                        $Template->status = true;
                        $Template->save();
                        return view("back.preferences.templates.edit", compact("Template"));
                    }
                }
            }
        }
        echo "<div class='alert alert-danger'>" . trans("backLang.erorr") . "</div>";
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $Template = Template::find(decrypt($request->id));
            if (!empty($Template)) {
                try {

                    $Template->subject = $request->subject;
                    $Template->content = $request->details;
                    $Template->url = $request->url;
                    $Template->css = $request->css;
                    $Template->status = 1;
                    $Template->updated_ip = \Request::ip();
                    $Template->updated_by = Auth::user()->id;
                    $Template->save();


                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 108,
                        $window = ($Template->cat == 2) ? 1082 : (($Template->cat == 1) ? 1081 : 1080),
                        $record_id = $Template->id,
                        $operation = 3,
                        $title = $Template->subject
                    );

                    return response()->json(['stat' => 'success', "id" => encrypt($Template->id)]);
                } catch (\Exception $e) {
                    return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                }

            }
        }
    }
}
