<?php

namespace App\Http\Controllers\Preferences;

use App;
use App\Http\Controllers\Controller;
use App\Models\Role;
use App\Models\User;
use App\Models\country;
use App\Models\Event;
use App\Models\Attachment;
use App\Models\Message;
use App\Models\Chat;
use Auth;
use Helper;
use File;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use URL;
use DeviceTrust;
use Excel;
use Mail;
use Redirect;
use Roles;
use Validator;
use App\Helpers\History;
use App\Helpers\AppSender;


class UserController extends Controller
{

    private $uploadPath = "uploads/users/";

    public function __construct()
    {
        // Define Language
        Helper::detectLang();
        // check page permission
        Roles::Check("app_preferences_users", true);
        // confirm password
        Helper::confirmPassword("AppUsers");
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $RolesList = Role::all();
        $Total_User_count = User::count();
        return view("back.preferences.users.list", compact("RolesList", "Total_User_count"));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function list(Request $request)
    {
        $columns = array(
            0 => 'name',
            1 => 'email',
            2 => 'role_id',
            3 => 'status',
            4 => 'options'
        );

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');

        if (Roles::Check("view")) {
            $Users = User::where("created_by", @Auth::user()->id);
        } else {
            $Users = User::where("id", '>', 0);
        }
        $totalData = $Users->count();
        $totalFiltered = $totalData;
        //order, paginate
        $Users = $Users->offset($start)
            ->limit($limit)
            ->orderBy($order, $dir)
            ->get();

        $data = array();
        if ($totalFiltered > 0) {
            foreach ($Users as $User) {
                if ($User->photo != "") {
                    $u_photo = URL::to('uploads/users/' . $User->photo);

                } else {
                    $u_photo = URL::to('uploads/users/profile.jpg');
                }
                $user_photo = "<div class='pull-left mr-3'><img src=\"" . $u_photo . "\" class='avatar avatar-lg' style='background:#fff'></div>";

                $nestedData['name'] = "<a href='" . route("AppUsersEdit", encrypt($User->id)) . "'>" . $user_photo . $User->name . "<br>" . $User->job . "</a>";
                $flag = "";
                if (!empty($User->country)) {
                    $flag = "<i class=\"flag flag-" . mb_strtolower($User->country->code) . "\" title='" . $User->country->{"title_" . trans('backLang.boxCode')} . "'></i>";
                }
                $nestedData['email'] = $User->email . "<br>" . $flag . " &nbsp;<span dir='ltr'>" . "+" . $User->phone_full . "</span>";
                $nestedData['role_id'] = "<div class='text-center'><span class=\"tag tag-default\">" . @$User->roles->name . "</span></div>";

                $stat = "<i class='fa fa-times text-danger'></i>";
                if ($User->status) {
                    $stat = "<i class='fa fa-check text-info'></i>";
                }
                $nestedData['status'] = "<div class='text-center'>" . $stat . "</div>";
                $options = "<div class=\"text-center\">";

                if (Roles::Check("edit")) {
                    $options .= "
 <a class=\"btn btn-sm btn-info\" href=\"" . route("AppUsersEdit", encrypt($User->id)) . "\"><i class=\"icon-pencil\"></i> " . trans("backLang.edit") . "</a>";
                }

                if (Roles::Check("delete") && $User->id != 1) {
                    $options .= "
 <button class=\"btn btn-sm btn-danger\" onclick=\"delete_row('" . encrypt($User->id) . "')\"><i class=\"icon-trash\"></i> " . trans("backLang.delete") . "</button>";
                }
                $options .= "</div>";
                $nestedData['options'] = $options;

                $data[] = $nestedData;
            }
        }

        $json_data = array(
            "draw" => intval($request->input('draw')),
            "recordsTotal" => intval($totalData),
            "recordsFiltered" => intval($totalFiltered),
            "data" => $data
        );

        return json_encode($json_data);

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (Roles::Check("add")) {

            $Total_User_count = User::count();
            if (DeviceTrust::UsersLimit() == 0 || DeviceTrust::UsersLimit() > $Total_User_count) {

                $validator = Validator::make($request->all(), [
                    'photo' => 'mimes:png,jpeg,jpg,gif|max:5000',
                    'name' => 'required',
                    'email' => 'required|email|unique:users',
                    'password' => 'required',
                    'country' => 'required',
                    'phone' => 'required|numeric'
                ]);
                if ($validator->passes()) {
                    try {
                        $phone_fc = mb_substr($request->phone, 0, 1, 'utf-8');
                        if ($phone_fc == "0") {
                            $u_phone = mb_substr($request->phone, 1);
                        } else {
                            $u_phone = $request->phone;
                        }
                        $phone_full = $u_phone;
                        $phone_code = "";
                        $Country = country::find($request->country);
                        if (!empty($Country)) {
                            $phone_full = $Country->tel . $u_phone;
                            $phone_code = $Country->tel;
                        }

                        //check full phone
                        $User_count = User::where("phone_full", $phone_full)->count();
                        if ($User_count > 0) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.phoneIsUsed')]]);
                        }

                        $formFileName = "photo";
                        $fileFinalName = "";
                        if ($request->$formFileName != "") {
                            $fileFinalName = time() . rand(1111, 9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                            $path = $this->uploadPath;
                            $request->file($formFileName)->move($path, $fileFinalName);
                        }

                        $User = new User;
                        $User->name = $request->name;
                        $User->job = $request->job;
                        $User->email = $request->email;
                        $User->password = bcrypt($request->password);
                        $User->role_id = $request->role_id;
                        $User->country_id = $request->country;
                        $User->phone_code = $phone_code;
                        $User->phone = $u_phone;
                        $User->phone_full = $phone_full;
                        if ($fileFinalName != "") {
                            $User->photo = $fileFinalName;
                        }
                        $User->status = true;
                        $User->created_ip = \Request::ip();
                        $User->created_by = Auth::user()->id;
                        $User->save();

                        // Log to history
                        History::Log(
                            $module = 1,
                            $cat = 103,
                            $window = 0,
                            $record_id = $User->id,
                            $operation = 2,
                            $title = $User->name . "<br>" . $User->email
                        );

                        // Send Notification
                        AppSender::Notify(
                            1,
                            [
                                "type" => 0,
                                "to_user_id" => $User->id,
                                "to_email" => $User->email,
                                "to_name" => $User->name,
                                "to_phone" => $User->phone_full,

                                "name" => $User->name,
                                "job" => $User->job,
                                "email" => $User->email,
                                "phone" => "+" . $User->phone_full,
                                "password" => $request->password,
                                "country" => @$User->country->title_ar,
                                "date" => Helper::DateView($User->created_at, 1),
                                "ip" => $User->created_ip,
                                "url" => route("AppUserProfile"),
                                "icon" => "fe fe-bell",
                            ],
                            0,
                            $User->id
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($User->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                } else {
                    return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
                }
            }
        }
        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  string $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = "")
    {
        if (Roles::Check("edit")) {
            if ($id != "") {
                $User = User::find(decrypt($id));
                if (!empty($User)) {
                    $RolesList = Role::all();
                    $SelfProfile = false;
                    return view("back.preferences.users.profile", compact("User", "RolesList", "SelfProfile"));
                }
            }
        }
        return redirect()->action('Preferences\UserController@index');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (Roles::Check("edit")) {
            $User = User::find(decrypt($request->id));
            if (!empty($User)) {
                $validator = Validator::make($request->all(), [
                    'photo' => 'mimes:png,jpeg,jpg,gif|max:5000',
                    'name' => 'required',
                    'email' => 'required|unique:users,email,' . $User->id,
                    'country' => 'required',
                    'phone' => 'required|numeric'
                ]);
                if ($validator->passes()) {
                    try {
                        $phone_fc = mb_substr($request->phone, 0, 1, 'utf-8');
                        if ($phone_fc == "0") {
                            $u_phone = mb_substr($request->phone, 1);
                        } else {
                            $u_phone = $request->phone;
                        }
                        $phone_full = $u_phone;
                        $phone_code = "";
                        $Country = country::find($request->country);
                        if (!empty($Country)) {
                            $phone_full = $Country->tel . $u_phone;
                            $phone_code = $Country->tel;
                        }

                        //check full phone
                        $User_count = User::where("phone_full", $phone_full)->where('id', "!=", $User->id)->count();
                        if ($User_count > 0) {
                            return response()->json(['stat' => 'error', 'error' => [trans('backLang.phoneIsUsed')]]);
                        }

                        // delete old photo
                        if ($request->photo_delete == 1) {
                            // Delete photo
                            if ($User->photo != "") {
                                if (file_exists($this->uploadPath . $User->photo)) {
                                    File::delete($this->uploadPath . $User->photo);
                                }
                            }
                            $User->photo = "";
                        }

                        $formFileName = "photo";
                        $fileFinalName = "";
                        if ($request->$formFileName != "") {
                            // Delete photo
                            if ($User->photo != "") {
                                if (file_exists($this->uploadPath . $User->photo)) {
                                    File::delete($this->uploadPath . $User->photo);
                                }
                            }

                            $fileFinalName = time() . rand(1111, 9999) . '.' . $request->file($formFileName)->getClientOriginalExtension();
                            $path = $this->uploadPath;
                            $request->file($formFileName)->move($path, $fileFinalName);
                        }

                        $User->name = $request->name;
                        $User->job = $request->job;
                        $User->email = $request->email;
                        if ($request->password != "") {
                            $User->password = bcrypt($request->password);
                        }

                        $User->role_id = $request->role_id;
                        $User->country_id = $request->country;
                        $User->phone_code = $phone_code;
                        $User->phone = $u_phone;
                        $User->phone_full = $phone_full;
                        if ($fileFinalName != "") {
                            $User->photo = $fileFinalName;
                        }

                        $User->status = ($request->status) ? 1 : 0;
                        $User->updated_ip = \Request::ip();
                        $User->updated_by = Auth::user()->id;
                        $User->save();

                        // Log to history
                        History::Log(
                            $module = 1,
                            $cat = 103,
                            $window = 0,
                            $record_id = $User->id,
                            $operation = 3,
                            $title = $User->name . "<br>" . $User->email
                        );

                        return response()->json(['stat' => 'success', "id" => encrypt($User->id)]);
                    } catch (\Exception $e) {
                        return response()->json(['stat' => 'error', 'error' => [trans('backLang.erorr')]]);
                    }
                }
                return response()->json(['stat' => 'error', 'error' => $validator->errors()->all()]);
            }
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function destroy($id = "")
    {
        // Check role
        if ($id != "") {
            if (Roles::Check("delete")) {
                $User = User::find(decrypt($id));
                if (!empty($User) && $User->id != 1) {
                    if ($User->photo != "") {
                        if (file_exists($this->uploadPath . $User->photo)) {
                            File::delete($this->uploadPath . $User->photo);
                        }
                    }

                    // delete attachments
                    $Attachments = Attachment::where('cat', 0)->where('cat_id', $User->id)->get();
                    foreach ($Attachments as $Attachment) {
                        if ($Attachment->file != "") {
                            if ($Attachment->remote && Helper::FileSystem() != "") {
                                Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                            } else {
                                if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                    File::delete("uploads/attachments/" . $Attachment->file);
                                }
                            }
                        }
                        $Attachment->shares()->delete();
                        $Attachment->comments()->delete();
                        $Attachment->alerts()->delete();
                        $Attachment->delete();
                    }

                    // delete events
                    $Events = Event::where('cat', 0)->where('cat_id', $User->id)->get();
                    foreach ($Events as $Event) {
                        if ($Event->photo != "") {
                            if (file_exists("uploads/events/" . $Event->photo)) {
                                File::delete("uploads/events/" . $Event->photo);
                            }
                        }
                        $Attachments = Attachment::where('cat', 1)->where('cat_id', $Event->id)->get();
                        foreach ($Attachments as $Attachment) {
                            if ($Attachment->file != "") {
                                if ($Attachment->remote && Helper::FileSystem() != "") {
                                    Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                } else {
                                    if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                        File::delete("uploads/attachments/" . $Attachment->file);
                                    }
                                }
                            }
                            $Attachment->shares()->delete();
                            $Attachment->comments()->delete();
                            $Attachment->alerts()->delete();
                            $Attachment->delete();
                        }

                        $Event->shares()->delete();
                        $Event->comments()->delete();
                        $Event->alerts()->delete();
                        $Event->delete();
                    }

                    // delete chat
                    $Chats = Chat::where('sender_id', $User->id)->orWhere('receiver_id', $User->id)->get();
                    foreach ($Chats as $Chat) {
                        $Attachments = Attachment::where('cat', 2)->where('cat_id', $Chat->id)->get();
                        foreach ($Attachments as $Attachment) {
                            if ($Attachment->file != "") {
                                if ($Attachment->remote && Helper::FileSystem() != "") {
                                    Storage::disk(Helper::FileSystem())->delete($Attachment->file);
                                } else {
                                    if (file_exists("uploads/attachments/" . $Attachment->file)) {
                                        File::delete("uploads/attachments/" . $Attachment->file);
                                    }
                                }
                            }
                            $Attachment->shares()->delete();
                            $Attachment->comments()->delete();
                            $Attachment->alerts()->delete();
                            $Attachment->delete();
                        }
                        $Chat->delete();
                    }

                    $User->comments()->delete();
                    $User->messages()->delete();
                    $User->delete();

                    // Log to history
                    History::Log(
                        $module = 1,
                        $cat = 103,
                        $window = 0,
                        $record_id = $User->id,
                        $operation = 4,
                        $title = $User->name . "<br>" . $User->email
                    );

                    return json_encode(array("stat" => "success", "id" => $User->id));
                }
            }
        }
        return json_encode(array("stat" => "error"));
    }

}
