<?php

namespace App\Http\Middleware;

use Closure;

use App\Models\Authorize;
use App\Mail\AuthorizeDevice as AuthorizeMail;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Request;
use Illuminate\Support\Carbon;
use GeoIP;
use Helper;
use Cookie;
use Auth;

class AuthorizeDevice
{
    /**
     * @var \App\Models\Authorize
     */
    private $authorize;

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  \Closure $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        // Device authorize
        if (Helper::Preferences("device_auth") > 0) {

            $cookie_saved = Cookie::get('DkmshTksiWxnodPMs0w2C');
            $current_ip = Request::ip();
            $authorized = false;
            if ($cookie_saved != "") {
                $cookie_saved = decrypt($cookie_saved);
            }
            $Authorize = Authorize::where([['token', $cookie_saved], ['authorized', true], ['authorized_at', '<', Carbon::tomorrow()], ['ip_address', '!=', $current_ip], ['user_id', Auth::user()->id]])->first();
            if (!empty($Authorize)) {
                $Authorize->ip_address = $current_ip;
                $Authorize->save();
                $authorized = true;
            } else {
                $Authorize2 = Authorize::where([['ip_address', $current_ip], ['authorized', true], ['authorized_at', '<', Carbon::tomorrow()], ['user_id', Auth::user()->id]])->first();
                if (!empty($Authorize2)) {
                    //Save to cookies for 10 years
                    Cookie::queue("DkmshTksiWxnodPMs0w2C", encrypt($Authorize2->token), time() + (10 * 365 * 24 * 60 * 60));
                    $authorized = true;
                }
            }

            if (!$authorized) {
                if (Authorize::inactive() && auth()->check()) {
                    $this->authorize = Authorize::make();
                    if ($this->authorize->noAttempt()) {
                        if (Helper::Preferences("device_auth") == 1) {
                            Mail::to($request->user())
                                ->send(new AuthorizeMail($this->authorize));
                        }else{
                            $this->authorize->browser = Helper::getBrowser();
                            $this->authorize->token = uniqid() . time();
                            $visitor_ip = $_SERVER['REMOTE_ADDR'];
                            try {
                                $loc = GeoIP($visitor_ip);
                                $location = $loc['country'].", ".$loc['city'];
                            }catch (\Exception $e){
                                $location = "unknown";
                            }
                            $this->authorize->location = $location;

                            $Device_type = Helper::detectDevice();
                            $this->authorize->device_type = $Device_type;
                            $this->authorize->device_name =  @gethostbyaddr($_SERVER['REMOTE_ADDR']);
                            $this->authorize->os = Helper::getOS();
                            $this->authorize->save();
                        }
                        $this->authorize->increment('attempt');
                    }
                    return response()->view('auth.authorize');
                }
            }

        }

        // once login from the same account
        if (!Helper::Preferences("multi_login")) {
            $User = Auth::user();
            $logged_there = false;
            if (!empty($User)) {
                $user_log_session = Cookie::get('user_log_session');
                if ($user_log_session != $User->log_session) {
                    // logged from another device
                    $logged_there = true;
                }
            }
            if($logged_there){
                if (Auth::check()) {
                    Auth::logout();
                }
                return response()->view('auth.out');
            }
        }
        return $next($request);
    }

    /**
     * Determines if the authorize attempt is timed out.
     *
     * @return bool
     */
    private function timeout()
    {
        $waiting = $this->authorize->created_at->addMinutes(120);
        if (now() >= $waiting) {
            return true;
        }
        return false;
    }
}
