<?php

namespace App\Imports;

use App\Models\Category;
use App\Models\country;
use App\Models\Currency;
use App\Helpers\History;
use App\Models\Journal;
use App\Models\Receipt;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Auth;
use Helper;


class ReceiptImport implements ToCollection
{
    protected $fund_id;

    public function __construct($fund_id)
    {
        $this->fund_id = $fund_id;
    }


    public function collection(Collection $rows)
    {
        try {
            $i = 0;
            foreach ($rows as $row) {
                if ($i > 0) {
                    $currency_id = 0;
                    $Currency = Currency::where("code", trim($row[3]))->first();
                    if (!empty($Currency)) {
                        $currency_id = $Currency->id;
                    }

                    $Receipt = new Receipt;
                    $Receipt->type = $this->ParseType($row[0]);
                    $Receipt->code = trim($row[10]);
                    $Receipt->period_id = Helper::Preferences("default_period");
                    $Receipt->date = date('Y-m-d', strtotime(trim($row[1])));
                    $Receipt->date_type = Helper::Preferences("date_type");
                    $Receipt->amount_type = $this->ParseType($row[0]);
                    $Receipt->amount = trim($row[2]);
                    $Receipt->currency_id = $currency_id;
                    $Receipt->cost = Helper::CurrencyTransfer(trim($row[2]), $currency_id, Helper::Preferences("default_currency"));
                    $Receipt->details = trim($row[7]);
                    $Receipt->ledger_id = trim($row[5]);
                    $Receipt->cat_id = $this->ParseCat($row[8], ($this->ParseType($row[0])) ? 8 : 9);
                    $Receipt->bank_id = $this->fund_id;
                    $Receipt->method_id = $this->ParseCat($row[4], 10);
                    $Receipt->center_id = $this->ParseCat($row[9], 11);
                    $Receipt->file = "";
                    $Receipt->repeated = false;
                    $Receipt->posted = Helper::Preferences("posting_method");
                    $Receipt->created_ip = \Request::ip();
                    $Receipt->created_by = Auth::user()->id;
                    $Receipt->save();

                    // delete old journals
                    $Receipt->journals()->delete();

                    // save new to journal
                    $Journal = new Journal;
                    $Journal->type = 0;
                    $Journal->period_id = $Receipt->period_id;
                    $Journal->receipt_id = $Receipt->id;
                    $Journal->center_id = $Receipt->center_id;
                    $Journal->date = $Receipt->date;
                    $Journal->date_type = $Receipt->date_type;
                    $Journal->currency_id = $Receipt->currency_id;
                    $Journal->details = $Receipt->details;
                    $Journal->posted = Helper::Preferences("posting_method");
                    $Journal->created_by = Auth::user()->id;

                    if ($Receipt->type) {
                        // revenues
                        $ledger_id = Helper::Preferences("revenues_ledger");
                        $Category = Category::find($Receipt->cat_id);
                        if (!empty($Category)) {
                            if (!empty($Category->ledger)) {
                                $ledger_id = $Category->ledger->id;
                            }

                        }
                        // From Ledger:
                        $Journal->ledger_id = $ledger_id;
                        $Journal->credit = $Receipt->amount;
                        $Journal->cost_credit = $Receipt->cost;
                        $Journal->dept = 0;
                        $Journal->cost_dept = 0;
                        $Journal->save();
                        // To Ledger:
                        $SecondJournal = $Journal->replicate();
                        $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                        $SecondJournal->credit = 0;
                        $SecondJournal->cost_credit = 0;
                        $SecondJournal->dept = $Receipt->amount;
                        $SecondJournal->cost_dept = $Receipt->cost;
                        $SecondJournal->save();
                    } else {
                        // expenses
                        $ledger_id = Helper::Preferences("expenses_ledger");
                        $Category = Category::find($Receipt->cat_id);
                        if (!empty($Category)) {
                            if (!empty($Category->ledger)) {
                                $ledger_id = $Category->ledger->id;
                            }

                        }
                        // From Ledger:
                        $Journal->ledger_id = $ledger_id;
                        $Journal->dept = $Receipt->amount;
                        $Journal->cost_dept = $Receipt->cost;
                        $Journal->credit = 0;
                        $Journal->cost_credit = 0;
                        $Journal->save();
                        // To Ledger:
                        $SecondJournal = $Journal->replicate();
                        $SecondJournal->ledger_id = @$Receipt->bank->ledger_id;
                        $SecondJournal->dept = 0;
                        $SecondJournal->cost_dept = 0;
                        $SecondJournal->credit = $Receipt->amount;
                        $SecondJournal->cost_credit = $Receipt->cost;
                        $SecondJournal->save();
                    }

                    // Log to history
                    History::Log(
                        $module = 6,
                        $cat = 600,
                        $window = 6002,
                        $record_id = $Receipt->id,
                        $operation = 2,
                        $title = $Receipt->amount . " " . $Receipt->currency->symbol . "<br>" . substr($Receipt->details, 0, 150)
                    );
                }
                $i++;
            }
        } catch (\Exception $e) {

        }
    }

    public function ParseType($type_text)
    {
        switch (trim($type_text)) {
            case "سند قبض":
                $group = 1;
                break;
            default:
                $group = 0;
        }
        return $group;
    }

    public function ParseCat($cat_text, $cat)
    {
        if (trim($cat_text) != "") {
            $Category = Category::where("title_ar", trim($cat_text))->where("cat", $cat)->first();
            if (empty($Category)) {
                $next_row_order = Category::where('cat', $cat)->max('order');
                if ($next_row_order < 1) {
                    $next_row_order = 1;
                } else {
                    $next_row_order++;
                }

                $Category = new Category;
                $Category->cat = $cat;
                $Category->order = $next_row_order;
                $Category->father_id = 0;
                $Category->title_ar = trim($cat_text);
                $Category->title_en = trim($cat_text);
                $Category->bg_color = "";
                $Category->font_color = "";
                $Category->status = true;
                $Category->created_ip = \Request::ip();
                $Category->created_by = Auth::user()->id;
                $Category->save();
            }
            if (!empty($Category)) {
                return $Category->id;
            }
        }
        return 0;
    }

}
