<?php

namespace App\Jobs;

use App\Message;
use Helper;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use OneSignal;

class SendMessage implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected $details;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($details)
    {
        $this->details = $details;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        if (Helper::CheckOneSignalStatus()) {
            $Message = Message::find($this->details['message_id']);
            if (!empty($Message)) {
                // push one signal notification
                if (Helper::Preferences("one_signal_status") && Helper::Preferences("one_signal_app_id") != "") {
                    $to_user_id = $Message->to_user_id;
                    $attached_url = null;
                    if (@$this->details['attached_url'] != "") {
                        $attached_url = $this->details['attached_url'];
                    }
                    try {
                        OneSignal::sendNotificationUsingTags($this->details['message'], array(["field" => "tag", "key" => "userId", "relation" => "=", "value" => "$to_user_id"]), $attached_url, $data = null, $buttons = null, $schedule = null);
                        $Message->status = true;
                        $Message->save();

                    } catch (\Exception $e) {

                        $Message->status = 2;
                        $Message->save();

                        return false;
                    }
                }
            }
        }
    }
}
