<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;

class User extends Authenticatable
{
    use HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'job',
        'email',
        'password',
        'photo',
        'role_id',
        'status',
        'type',
        'country_id',
        'phone_code',
        'phone',
        'phone_full',
        'connect_email',
        'connect_password',
        'provider_id',
        'provider',
        'access_token',
        'email_token',
        'phone_token',
        'phone_verify',
        'email_verify',
        'read_status',
        'created_ip',
        'created_by',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token', 'login_count', 'alert_sounds', 'welcome_box', 'welcome_box_status',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function roles()
    {

        return $this->belongsTo('App\Models\Role', 'role_id');
    }

    public function country()
    {

        return $this->belongsTo('App\Models\country', 'country_id');
    }

    public function deletedBy()
    {
        return $this->belongsTo('App\Models\User', 'deleted_by');
    }

    public function sentChats()
    {
        return $this->hasMany('App\Models\Chat', 'sender_id');
    }

    public function receivedChats()
    {
        return $this->hasMany('App\Models\Chat', 'receiver_id');
    }

    public function attachments()
    {
        return $this->hasMany('App\Models\Attachment', 'cat_id')->where("cat", 0);
    }

    public function messages()
    {
        return $this->hasMany('App\Models\Message', 'to_user_id');
    }

    public function comments()
    {
        return $this->hasMany('App\Models\Comment', 'cat_id')->where("cat", 0);
    }
}

