@extends('back.layouts.master')
<?php
$page_title = trans('backLang.openingBalances');
$add_title = trans('backLang.addOpeningBalance');
$edit_title = trans('backLang.editOpeningBalance');
$add_link = route("AppBalancesCreate");
if (@$stat == "reconciliation") {
    $page_title = trans('backLang.reconciliation');
    $add_title = trans('backLang.addReconciliation');
    $edit_title = trans('backLang.editReconciliation');
    $add_link = route("AppReconciliationCreate");
}
?>
@section('title', $page_title)
@section("content")
    <div class="section-body pt-3">
        <div class="container-fluid">

            <div class="card card-collapsed" id="search_card">
                <div class="card-header">
                    <h3 class="card-title"><i class='icon-wallet'></i> {!! trans('backLang.accountant') !!} <i
                            class='fe fe-chevron-{{ trans("backLang.right") }}'></i><span> {!! $page_title !!}</span>
                    </h3>
                    <div class="card-options">
                        <a href="#" class="card-options-collapse" data-toggle="card-collapse"><span
                                class="icon-magnifier" data-toggle="tooltip" id="search_card_icon"
                                title="{{ trans('backLang.search') }}"></span></a>
                    </div>
                </div>
                <div class="card-body">
                    @include('back.accounting.balances.search')
                </div>
            </div>

            <div class="row clearfix">
                <div class="col-sm-12">
                    <div class="card">
                        <div class="card-header">
                            @if (Roles::Check("add"))
                                <a href="{{ $add_link }}" class="btn btn-primary"><i
                                        class="fe fe-plus mb-2"></i> {{ $add_title }}</a>
                            @endif
                            <div class="card-options">
                                @if (Roles::Check("accountant_print"))
                                    <a href="javascript:void(0)" data-toggle="tooltip"
                                       id="_print_btn"
                                       data-original-title="{{ trans("backLang.print") }}">
                                        <i
                                            class="icon-printer"></i>
                                    </a>
                                @endif
                                @if (Roles::Check("accountant_export"))
                                    <a href="javascript:void(0)" data-toggle="tooltip"
                                       id="_export_btn"
                                       data-original-title="{{ trans("backLang.export") }}">
                                        <i
                                            class="icon-cloud-download"></i>
                                    </a>
                                @endif
                            </div>
                        </div>
                        <div class="card-body">
                            <div class="table-responsive table-nowrap mt-3">
                                <table
                                    class="table table-hover js-basic-example dataTable table_custom border-style spacing5"
                                    id="receipts_table">
                                    <thead>
                                    <th>{{ trans('backLang.date') }}</th>
                                    <th>{{ trans('backLang.ledger') }}</th>
                                    <th>{{ trans('backLang.debtor') }}</th>
                                    <th>{{ trans('backLang.creditor') }}</th>
                                    <th>{{ trans('backLang.desc') }}</th>
                                    <th class="text-center">{{ trans('backLang.options') }}</th>
                                    </thead>
                                </table>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection

@section("page-script")
    <script type="text/javascript">
        var table_name = "#receipts_table";
        var table_paginate = "{{ DeviceTrust::settings('BACKEND_PAGINATION') }}";
        $(document).ready(function () {
            var dataTable = $(table_name).DataTable({
                "processing": true,
                "serverSide": true,
                "searching": false,
                "responsive": true,
                "pageLength": table_paginate,
                "ajax": {
                    "url": "{{ route('AppBalancesList') }}",
                    "dataType": "json",
                    "type": "POST",
                    "data": function (data) {
                        data._token = "{{csrf_token()}}";
                        data.search_word = $("#search_word").val();
                        data.search_currency = $("#search_currency").val();
                        data.search_user = $("#search_user").val();
                        data.search_start = $("#search_start").val();
                        data.search_end = $("#search_end").val();
                        data.search_type = $("#search_type").val();
                        data.search_center_id = $("#search_center_id").val();
                        data.search_posted = $("#search_posted").val();
                        data.search_ledger = $("#search_ledger").val();
                        data.search_period = $("#search_period").val();
                    }

                },
                "dom": '<"dataTables_wrapper"<"row"<"col-sm-12 col-md-9"i><"col-sm-12 col-md-3"l>><"row"<"col-sm-12 col-md-12"r>><"row"t><"row"<"w-p100"p>>>',
                "fnDrawCallback": function () {
                    if ($(table_name + '_paginate .paginate_button').length > 3) {
                        $(table_name + '_paginate')[0].style.display = "block";
                    } else {
                        $(table_name + '_paginate')[0].style.display = "none";
                    }
                    $('[data-toggle="tooltip"]').tooltip();
                },
                @if(trans('backLang.boxCode') =="ar")
                "language": {
                    {!! trans("backLang.dataTablesTranslation") !!}
                },
                @endif
                "columns": [
                    {"data": "date"},
                    {"data": "ledger_id"},
                    {"data": "dept"},
                    {"data": "credit"},
                    {"data": "details"},
                    {"data": "options", "orderable": false}
                ],
                "order": [[0, "desc"]]

            });
            $.fn.dataTable.ext.errMode = 'none';


            $("#journals_search_form").submit(function (evt) {
                if ($("#search_submit_stat").val() == "") {
                    evt.preventDefault();
                    $(table_name).DataTable().ajax.reload();
                }
            });

            $('#search_card_icon').click(function () {
                $("#search_card").toggleClass("bg-light");
            });

            @if (Roles::Check("accountant_export"))
            $("#_export_btn").on('click', function () {
                $("#search_submit_stat").val("export");
                $("#journals_search_form").attr('action', '{{ route("AppBalancesExport") }}');
                $("#journals_search_form").attr('target', '_blank');
                $("#journals_search_form").submit();
                $("#journals_search_form").attr('action', '{{ route("AppJournals") }}');
                $("#search_submit_stat").val("");
                $("#journals_search_form").attr('target', '');
            });
            @endif
            @if (Roles::Check("accountant_print"))
            $("#_print_btn").on('click', function () {
                $("#search_submit_stat").val("print");
                $("#journals_search_form").attr('action', '{{ route("AppBalancesPrint") }}');
                $("#journals_search_form").attr('target', '_blank');
                $("#journals_search_form").submit();
                $("#journals_search_form").attr('action', '{{ route("AppJournals") }}');
                $("#search_submit_stat").val("");
                $("#journals_search_form").attr('target', '');
            });
            @endif

            @if(Helper::Preferences("date_type"))
            $("#search_start").hijriDatePicker({
                locale: "{{ (trans("backLang.code")=="ar")?"ar-SA":"en-US" }}",
                format: "YYYY/MM/DD",
                hijriFormat: "iYYYY/iMM/iDD",
                showSwitcher: false,
                allowInputToggle: false,
                useCurrent: true,
                isRTL: false,
                keepOpen: false,
                hijri: true,
                debug: false,
                showClear: false,
                showTodayButton: false,
                showClose: false,
                icons: {
                    time: '<i class="fa fa-clock-o"></i>',
                    date: '<i class="fa fa-calendar"></i>',
                    up: '<i class="fa fa-chevron-up"></i>',
                    down: '<i class="fa fa-chevron-down"></i>',
                    previous: '<i class="fa fa-chevron-right"></i>',
                    next: '<i class="fa fa-chevron-left"></i>',
                    today: '<i class="fa fa-screenshot"></i>',
                    clear: '<i class="fa fa-trash"></i>',
                    close: '<i class="fa fa-remove"></i>'
                }
            });
            $("#search_end").hijriDatePicker({
                locale: "{{ (trans("backLang.code")=="ar")?"ar-SA":"en-US" }}",
                format: "YYYY/MM/DD",
                hijriFormat: "iYYYY/iMM/iDD",
                showSwitcher: false,
                allowInputToggle: false,
                useCurrent: true,
                isRTL: false,
                keepOpen: false,
                hijri: true,
                debug: false,
                showClear: false,
                showTodayButton: false,
                showClose: false,
                icons: {
                    time: '<i class="fa fa-clock-o"></i>',
                    date: '<i class="fa fa-calendar"></i>',
                    up: '<i class="fa fa-chevron-up"></i>',
                    down: '<i class="fa fa-chevron-down"></i>',
                    previous: '<i class="fa fa-chevron-right"></i>',
                    next: '<i class="fa fa-chevron-left"></i>',
                    today: '<i class="fa fa-screenshot"></i>',
                    clear: '<i class="fa fa-trash"></i>',
                    close: '<i class="fa fa-remove"></i>'
                }
            });
            @else
            $('.datepicker').datepicker({
                format: "yyyy/mm/dd",
                language: 'ar'
            });
            @endif


            $('.list-items').selectize({
                searchField: 'search-text',
                render: {
                    option: function (data, escape) {
                        return '<div>' +
                            '<span class="title">' + escape(data.text) + '</span>' +
                            '</div>';
                    },
                    item: function (data, escape) {
                        return '<div>' +
                            escape(data.text) +
                            '</div>';
                    }
                }
            });

            $('.custom-list').selectize({
                searchField: 'search-text',
                render: {
                    option: function (data, escape) {
                        return '<div>' +
                            '<span class="image"><img src="' + data.image + '" alt=""></span>' +
                            '<span class="title">' + escape(data.text) + '</span>' +
                            '<span class="code">' + escape(data.code) + '</span>' +
                            '</div>';
                    },
                    item: function (data, escape) {
                        return '<div>' +
                            '<span class="image"><img src="' + data.image + '" alt=""></span>' +
                            escape(data.text) + '</div>';
                    }
                }
            });
        });

        function delete_row(rid) {
            ajax_delete_row("<?php echo route("AppBalancesDestroy"); ?>/" + rid, '{{ trans("backLang.confirmationDeleteMsg") }}', '{{ trans("backLang.delete") }}', '{{ trans("backLang.cancel") }}', '{{ trans("backLang.deleteDone") }}', '{{ trans("backLang.erorr") }}', '{{ trans("backLang.close") }}', table_name, false);
        }
    </script>
@endsection
