@extends('back.layouts.master')
@section('title', trans('backLang.accountantTree'))
@section("content")
    <div class="section-body pt-3">
        <div class="container-fluid">
            <nav aria-label="breadcrumb">
                <ol class="breadcrumb">
                    <li class="breadcrumb-item" aria-current="page"><i class='icon-wallet'></i>  {{ trans('backLang.accountant') }}</li>
                    <li class="breadcrumb-item active" aria-current="page">{{ trans('backLang.accountantTree') }}</li>
                </ol>
            </nav>

            <div class="row clearfix">
                <div class="col-sm-12">
                    <div class="card">
                        <div class="card-header">
                            @if (Roles::Check("add") && Roles::Check("accountant_add"))
                                <button type="button" class="btn btn-primary create_ledger_btn"><i
                                        class="fe fe-plus mb-2"></i> {{ trans('backLang.addNewLedger') }}</button>
                            @else
                                <h3 class="card-title">{{ trans('backLang.accountantTree') }}</h3>
                            @endif

                            <div class="card-options">
                                @if (Roles::Check("accountant_print"))
                                    <a href="{{ route("AppLedgersPrint") }}" target="_blank" data-toggle="tooltip"
                                       id="_print_btn"
                                       data-original-title="{{ trans("backLang.print") }}">
                                        <i
                                            class="icon-printer"></i>
                                    </a>
                                @endif
                                @if (Roles::Check("add"))
                                    @if (Roles::Check("accountant_import"))
                                        <a href="javascript:void(0)" data-toggle="tooltip"
                                           id="_import_btn"
                                           data-original-title="{{ trans("backLang.importFromFile") }}">
                                            <i
                                                class="icon-login"></i>
                                        </a>
                                    @endif
                                @endif
                                @if (Roles::Check("accountant_export"))
                                    <a href="{{ route("AppLedgersExport") }}" target="_blank" data-toggle="tooltip"
                                       id="_export_btn"
                                       data-original-title="{{ trans("backLang.export") }}">
                                        <i
                                            class="icon-cloud-download"></i>
                                    </a>
                                @endif
                            </div>
                        </div>
                        <div class="card-body pa-0" id="ledgers_table_container">
                        </div>
                    </div>
                </div>
            </div>

        </div>
    </div>
@endsection
@section("popup")
    @if (Roles::Check("add"))
        @if (Roles::Check("accountant_add"))
            <div class="modal fade" id="create_ledger" tabindex="-1" role="dialog" aria-hidden="true">
                <div class="modal-dialog modal-lg" role="document">
                    <div class="modal-content">
                        {{Form::open(['route'=>['AppLedgersStore'],'method'=>'POST','id'=>'create_ledger_form'])}}
                        <div class="modal-header">
                            <h5 class="modal-title"><i
                                    class="icon-plus mb-2"></i> {{ trans('backLang.addNewLedger') }}</h5>
                            <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                                <span aria-hidden="true">&times;</span>
                            </button>
                        </div>
                        <div class="modal-body pa-0">
                            <div class="alert alert-danger m-3  displaynone" id="create_ledger_errors">
                                <ul></ul>
                            </div>
                            <div class="form-fields">
                                <div class="text-center">
                                    <div class="loader" style="display: inline-block"></div>
                                    <div>
                                        <small class="text-muted">{{ trans("backLang.loading") }}</small>
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="modal-footer">
                            <button type="button" class="btn btn-light" data-dismiss="modal"><i
                                    class="fe fe-x"></i> {{ trans('backLang.cancel') }}
                            </button>
                            <button type="submit" id="create_ledger_submit" class="btn btn-primary">
                                &nbsp; <i
                                    class="fe fe-save"></i> {{ trans('backLang.save') }} &nbsp;
                            </button>
                        </div>
                        {{Form::close()}}
                    </div>
                </div>
            </div>
            @if (Roles::Check("accountant_import"))
                <div class="modal fade" id="_import_ledgers" tabindex="-1" role="dialog" aria-hidden="true">
                    <div class="modal-dialog" role="document">
                        <div class="modal-content">
                            {{Form::open(['route'=>['AppLedgersImportFromFile'],'method'=>'POST','id'=>'_import_ledgers_form' , 'files' => true])}}
                            <div class="modal-header">
                                <h5 class="modal-title"><i
                                        class="icon-book-open mb-2"></i> {{ trans('backLang.importFromFile') }}</h5>
                                <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                                    <span aria-hidden="true">&times;</span>
                                </button>
                            </div>
                            <div class="modal-body m-0 p-4">
                                <div class="alert alert-danger displaynone m-3" id="_import_ledgers_errors">
                                    <ul></ul>
                                </div>
                                <div class="row">
                                    <div class="col-sm-12">
                                        <div
                                            class="alert alert-info">{{ trans("backLang.oldLedgersWillBeRemoved") }}</div>
                                        <div class="form-group">
                                            <input type="file" required name="file" id="xls_file" data-height="120"
                                                   data-allowed-file-extensions="xls xlsx">
                                            <small>
                                                <i class="icon-info"></i> {!!  trans('backLang.xlsTypes') !!}
                                            </small>
                                        </div>
                                    </div>
                                </div>
                            </div>
                            <div class="modal-footer">
                                <div class="form-group modal-footer-extra">
                                    <a href="{{ route("AppLedgersDownload") }}"><i
                                            class="fe fe-download"></i> {{ trans("backLang.downloadExcelFileExample") }}
                                    </a>
                                </div>
                                <button type="button" class="btn btn-light" data-dismiss="modal"><i
                                        class="fe fe-x"></i> {{ trans('backLang.cancel') }}
                                </button>
                                <button type="submit" id="_import_ledgers_submit" class="btn btn-primary">
                                    &nbsp; <i
                                        class="fe fe-save"></i> {{ trans('backLang.importData') }} &nbsp;
                                </button>
                            </div>
                            {{Form::close()}}
                        </div>
                    </div>
                </div>
            @endif
        @endif
    @endif

    @if (Roles::Check("edit"))
        @if (Roles::Check("accountant_edit"))
            <div class="modal fade" id="edit_ledger" tabindex="-1" role="dialog" aria-hidden="true">
                <div class="modal-dialog modal-lg" role="document">
                    <div class="modal-content">
                        {{Form::open(['route'=>['AppLedgersUpdate'],'method'=>'POST','id'=>'edit_ledger_form'])}}
                        <div class="modal-header">
                            <h5 class="modal-title"><i
                                    class="icon-pencil mb-2"></i> {{ trans('backLang.editLedger') }}</h5>
                            <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                                <span aria-hidden="true">&times;</span>
                            </button>
                        </div>
                        <div class="modal-body pa-0">
                            <div class="alert alert-danger m-3 displaynone" id="edit_ledger_errors">
                                <ul></ul>
                            </div>
                            <div class="form-fields">
                                <div class="text-center">
                                    <div class="loader" style="display: inline-block"></div>
                                    <div>
                                        <small class="text-muted">{{ trans("backLang.loading") }}</small>
                                    </div>
                                </div>
                            </div>
                        </div>
                        <div class="modal-footer">
                            <button type="button" class="btn btn-light" data-dismiss="modal"><i
                                    class="fe fe-x"></i> {{ trans('backLang.cancel') }}
                            </button>
                            <button type="submit" id="edit_ledger_submit" class="btn btn-primary">
                                &nbsp; <i
                                    class="fe fe-save"></i> {{ trans('backLang.save') }} &nbsp;
                            </button>
                        </div>
                        {{Form::close()}}
                    </div>
                </div>
            </div>
        @endif
    @endif
@endsection
@push("after-scripts")
    <script src="{{ asset('back/custom/js/jquery-simple-tree-table.js') }}"></script>
@endpush
@section("page-script")
    <script type="text/javascript">
        $(document).ready(function () {
            $("#create_ledger").on("hidden.bs.modal", function () {
                $("#create_ledger_errors").hide();
                $('#create_ledger').find('.modal-body .form-fields').html("<div class=\"text-center p-4\"><div class=\"loader\" style=\"display: inline-block\"></div><br><small class=\"text-muted\">{{ trans("backLang.loading") }}</small></div>");
            });

            $(".create_ledger_btn").on("click", function () {
                $("#create_ledger_submit").prop('disabled', true);
                $.get("{{ route("AppLedgersCreate") }}", function (data) {
                    $('#create_ledger').find('.modal-body .form-fields').html(data);
                    $("#create_ledger_submit").prop('disabled', false);
                });
                $('#create_ledger').modal('show');
            });

            ajax_post_form("create_ledger", '{{ trans("backLang.save") }}', 'fe fe-save', '{!! trans("backLang.saveDone") !!}', '{!! trans("backLang.close") !!}', true, "", false, false, true, function () {
                reload_table();
            });

            ajax_post_form("edit_ledger", '{{ trans("backLang.save") }}', 'fe fe-save', '{!! trans("backLang.saveDone") !!}', '{!! trans("backLang.close") !!}', true, "", false, false, false, function () {
                reload_table();
            });

            $("#_import_btn").on('click', function () {
                $('#_import_ledgers').modal('show');
            });
            var dropify_settings = {
                messages: {
                    'default': '{{ trans("backLang.dragAndDropFileHere") }}',
                    'replace': '{{ trans("backLang.dragAndDropFileToReplace") }}',
                    'remove': '{{ trans("backLang.delete") }}',
                    'error': '{{ trans("backLang.errorSomethingWrong") }}',
                }
            };
            var drEvent = $('#xls_file').dropify(dropify_settings);

            ajax_post_form("_import_ledgers", '{{ trans("backLang.importData") }}', 'fe fe-save', '{!! trans("backLang.saveDone") !!}', '{!! trans("backLang.close") !!}', true, "", false, false, true, function () {
                reload_table();
            });

            //load on page start
            reload_table();
        });

        function reload_table() {
            $('#ledgers_table_container').html("<div class=\"text-center p-4\"><div class=\"loader\" style=\"display: inline-block\"></div><br><small class=\"text-muted\">{{ trans("backLang.loading") }}</small></div>");
            $.get("{{ route("AppLedgersList") }}", function (data) {
                $('#ledgers_table_container').html(data);
            });
        }

        function edit_row(rid) {
            $("#edit_ledger_submit").prop('disabled', true);
            $.get("{{ route("AppLedgersEdit") }}/" + rid, function (data) {
                $('#edit_ledger').find('.modal-body .form-fields').html(data);
                $("#edit_ledger_submit").prop('disabled', false);
            });
            $('#edit_ledger').modal('show');
        }

        function delete_row(mid) {
            ajax_delete_row("<?php echo route("AppLedgersDestroy"); ?>/" + mid, '{{ trans("backLang.confirmationDeleteMsg") }}', '{{ trans("backLang.deleteMessage") }}', '{{ trans("backLang.cancel") }}', '{{ trans("backLang.deleteDone") }}', '{{ trans("backLang.erorr") }}', '{{ trans("backLang.close") }}', "", false, false, function () {
                reload_table();
            });
        }
    </script>
@endsection
