@extends('back.layouts.print')
@section('title', trans('backLang.loans'))
@section("content")
    <style>
        @media print {
            @page {
                size: landscape
            }
        }
    </style>
    <div class="container">
        {!! Helper::PrintHeaderAndFooter("")["header"] !!}
        <table class="responsive-table center-table">
            <caption>
                <h1>{!! trans('backLang.loans') !!}</h1>
                @if($request->search_word !="")
                    <label>{{ trans('backLang.searchFor') }}</label>
                    <strong>{!! $request->search_word !!}</strong>
                @endif
                @if($request->search_cat_id !="")
                    <label>{{ trans('backLang.tribe') }}</label>
                    <strong>{!! Helper::Category($request->search_cat_id) !!}</strong>
                @endif

                @if($request->search_status !="")
                    <?php
                    if ($request->search_status == 1) {
                        $loan_status_title = trans('backLang.approved');
                    } elseif ($request->search_status == -1) {
                        $loan_status_title = trans('backLang.notApproved');
                    } else {
                        $loan_status_title = trans('backLang.pending');
                    }
                    ?>
                    <label>{{ trans('backLang.loanStatus') }}</label>
                    <strong>{!! $loan_status_title !!}</strong>
                @endif

                @if($request->search_start !="")
                    <label>{{ trans('backLang.fromDate') }}</label>
                    <strong>{!! $request->search_start !!}</strong>
                @endif
                @if($request->search_end !="")
                    <label>{{ trans('backLang.toDate') }}</label>
                    <strong>{!! $request->search_end !!}</strong>
                @endif
                @if($request->search_user !="")
                    <label>{{ trans('backLang.user') }}</label>
                    <strong>{!! Helper::usernameById($request->search_user) !!}</strong>
                @endif
            </caption>
            <thead>
            <th>{{ trans('backLang.recordId') }}</th>
            <th>{{ trans('backLang.date') }}</th>
            <th>{{ trans('backLang.beneficiaryName') }}</th>
            <th>{{ trans('backLang.loansCategory') }}</th>
            <th>{{ trans('backLang.amount') }}</th>
            <th>{{ trans('backLang.loanInstalmentCount') }}</th>
            <th>{{ trans('backLang.paid') }}</th>
            <th>{{ trans('backLang.unpaid') }}</th>
            <th>{{ trans('backLang.status') }}</th>
            </thead>
            <tbody>
            @if(count($Loans) >0)
                @foreach($Loans as $Loan)
                    <?php
                    if ($Loan->status == 1) {
                        $loan_status_title = "<span class=\"text-success\"><i class=\"fe fe-check-circle\"></i> " . trans('backLang.approved') . "</span>";
                    } elseif ($Loan->status == -1) {
                        $loan_status_title = "<span class=\"text-danger\"><i class=\"fe fe-x-circle\"></i> " . trans('backLang.notApproved') . "</span>";
                    } else {
                        $loan_status_title = "<span class=\"text-info\"><i class=\"fe fe-clock\"></i> " . trans('backLang.pending') . "</span>";
                    }
                    $paid_amount = $Loan->installments()->where("paid_status", true)->sum("amount");
                    $unpaid = $Loan->total - $paid_amount;
                    ?>
                    <tr>
                        <td>{{ $Loan->id }}</td>
                        <td>{{ date(Helper::Preferences("date_format"), strtotime($Loan->date)) . " " . ((@$Loan->date_type) ? trans("backLang.H") : trans("backLang.M")) }}</td>
                        <td>{{ $Loan->name }}</td>
                        <td>{!! Helper::Category($Loan->cat_id) !!}</td>
                        <td>{{ number_format($Loan->total, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol }}</td>
                        <td>{{ $Loan->instalments_count }}</td>
                        <td>{{ (($paid_amount >0)? number_format($paid_amount, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol:"-") }}</td>
                        <td>{{ (number_format($unpaid, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$DefaultCurrency->symbol) }}</td>
                        <td>{!! $loan_status_title !!}</td>
                    </tr>
                @endforeach
            @else
                <tr>
                    <td colspan="9">
                        <small>{{ trans("backLang.noData") }}</small>
                    </td>
                </tr>
            @endif
            </tbody>
        </table>
        {!! Helper::PrintHeaderAndFooter("")["footer"] !!}
    </div>
@endsection
