@extends('back.layouts.print')
@section('title', trans('backLang.fundsAndBanksReport'))
@section("content")
    <style>
        @media print {
            @page {
                size: landscape;
            }
        }
    </style>
    <div class="container">
        {!! Helper::PrintHeaderAndFooter("")["header"] !!}
        <table class="responsive-table center-table" id="ledgers_table">
            <caption>
                <h1>{!! trans('backLang.fundsAndBanksReport') !!}</h1>

                @if(!empty($Period))
                    <label>{{ trans('backLang.accountantPeriod') }}</label>
                    <strong>{{ $Period->code }}</strong>
                @endif
                @if(!empty($Ledger))
                    <label>{{ trans('backLang.ledger') }}</label>
                    <strong>{{ $Ledger->{"title_".trans("backLang.code")} }}</strong>
                @endif
                @if($request->search_word !="")
                    <label>{{ trans('backLang.searchFor') }}</label>
                    <strong>{!! $request->search_word !!}</strong>
                @endif
                @if(!empty($Currency))
                    <label>{{ trans('backLang.currency') }}</label>
                    <strong>{{ $Currency->{"title_".trans("backLang.code")} }}</strong>
                @endif
                @if($request->search_start !="")
                    <label>{{ trans('backLang.fromDate') }}</label>
                    <strong>{!! $request->search_start !!}</strong>
                @endif
                @if($request->search_end !="")
                    <label>{{ trans('backLang.toDate') }}</label>
                    <strong>{!! $request->search_end !!}</strong>
                @endif
                @if($request->search_center_id !="")
                    <label>{{ trans('backLang.costPosition') }}</label>
                    <strong>{!! Helper::Category($request->search_center_id) !!}</strong>
                @endif
                @if($request->search_user !="")
                    <label>{{ trans('backLang.user') }}</label>
                    <strong>{!! Helper::usernameById($request->search_user) !!}</strong>
                @endif
            </caption>
            <thead>
            <th>{{ trans('backLang.ledgerType') }}</th>
            <th>{{ trans('backLang.debtor') }}</th>
            <th>{{ trans('backLang.creditor') }}</th>
            <th>{{ trans('backLang.balance') }} ( {{ @$DefaultCurrency->symbol }} )</th>
            <th>{{ trans('backLang.desc') }}</th>
            <th>{{ trans('backLang.date') }}</th>
            <th>{{ trans('backLang.entryNo') }}</th>
            <th>{{ trans('backLang.dailyEntry') }}</th>
            <th>{{ trans('backLang.fileNo') }}</th>
            </thead>
            <tbody>
            <?php
            $i = 0;
            $balance = 0;
            $total_of_dept = 0;
            $total_of_credit = 0;
            ?>
            @if(count($Journals) >0)
                @foreach($Journals as $Journal)
                    <?php
                    if ($i == 0) {
                        $balance = round($sum_of_dept - $sum_of_credit, 2);
                        $total_of_dept += $sum_of_dept;
                        $total_of_credit += $sum_of_credit;
                    } else {
                        if ($Journal->cost_dept > 0) {
                            $balance = round($balance + $Journal->cost_dept, 2);
                            $total_of_dept += $Journal->cost_dept;
                        }
                        if ($Journal->cost_credit > 0) {
                            $balance = round($balance - $Journal->cost_credit, 2);
                            $total_of_credit += $Journal->cost_credit;
                        }
                    }
                    $cur_details = "";
                    if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                        if ($Journal->dept > 0) {
                            $cur_details = " ( " . $Journal->dept . " " . @$Journal->currency->symbol . " )";
                        } else {
                            $cur_details = " ( " . $Journal->credit . " " . @$Journal->currency->symbol . " )";
                        }
                    }
                    $type = trans("backLang.dailyEntry");
                    if ($Journal->type == 2) {
                        $type = trans("backLang.reconciliationEntry");
                    } elseif ($Journal->type == 1) {
                        $type = trans("backLang.startBalance");
                    } else {
                        if (@$Journal->receipt->type == 1) {
                            $type = trans("backLang.cashReceipt");
                        } elseif (@$Journal->receipt->type == 0) {
                            $type = trans("backLang.payReceipt");
                        }
                    }
                    ?>
                    <tr>
                        <td>{{ $type }}</td>
                        <td>{{ (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") }}</td>
                        <td>{{ (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") }}</td>
                        <td>{{ number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) }}</td>
                        <td>{{ $Journal->details . $cur_details }}</td>
                        <td>{!! (date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M"))) !!}</td>
                        <td>{{ $Journal->id }}</td>
                        <td>{{ @$Journal->receipt->id }}</td>
                        <td>{{ @$Journal->receipt->code }}</td>
                    </tr>
                    @php($i++)
                @endforeach
            @else
                <tr>
                    <td colspan="8">
                        <small>{{ trans("backLang.noData") }}</small>
                    </td>
                </tr>
            @endif
            </tbody>
        </table>
        <br>
        <table class="responsive-table center-table" id="ledgers_table">
            <thead>
            <tr>
                <th>{{ trans('backLang.debtor') }}
                </th>
                <th>{{ trans('backLang.creditor') }}
                </th>
                <th>{{ trans('backLang.balance') }}</th>
            </tr>
            </thead>
            <tbody>
            <tr>
                <td>{{ number_format($total_of_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) }} {{ @$DefaultCurrency->symbol }}</td>
                <td>{{ number_format($total_of_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) }} {{ @$DefaultCurrency->symbol }}</td>
                <td><strong>{{ number_format($balance, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) }} {{ @$DefaultCurrency->symbol }}</strong></td>
            </tr>
            </tbody>
        </table>

        {!! Helper::PrintHeaderAndFooter("")["footer"] !!}
    </div>
@endsection
