@extends('back.layouts.print')
@section('title', trans('backLang.masterReport'))
@section("content")
    <style>
        @media print {
            @page {
                size: landscape;
            }
        }
    </style>
    <div class="container">
        {!! Helper::PrintHeaderAndFooter("")["header"] !!}
        <table class="responsive-table center-table" id="ledgers_table">
            <caption>
                <h1>{!! trans('backLang.masterReport') !!}</h1>

                @if(!empty($Period))
                    <label>{{ trans('backLang.accountantPeriod') }}</label>
                    <strong>{{ $Period->code }}</strong>
                @endif
                @if(!empty($Ledger))
                    <label>{{ trans('backLang.ledger') }}</label>
                    <strong>{{ $Ledger->{"title_".trans("backLang.code")} }}</strong>
                @endif
                @if($request->search_word !="")
                    <label>{{ trans('backLang.searchFor') }}</label>
                    <strong>{!! $request->search_word !!}</strong>
                @endif
                @if(!empty($Currency))
                    <label>{{ trans('backLang.currency') }}</label>
                    <strong>{{ $Currency->{"title_".trans("backLang.code")} }}</strong>
                @endif
                @if($request->search_start !="")
                    <label>{{ trans('backLang.fromDate') }}</label>
                    <strong>{!! $request->search_start !!}</strong>
                @endif
                @if($request->search_end !="")
                    <label>{{ trans('backLang.toDate') }}</label>
                    <strong>{!! $request->search_end !!}</strong>
                @endif
                @if($request->search_center_id !="")
                    <label>{{ trans('backLang.costPosition') }}</label>
                    <strong>{!! Helper::Category($request->search_center_id) !!}</strong>
                @endif
                @if($request->search_user !="")
                    <label>{{ trans('backLang.user') }}</label>
                    <strong>{!! Helper::usernameById($request->search_user) !!}</strong>
                @endif
            </caption>
            <thead>
            <th>{{ trans('backLang.entryNo') }}</th>
            <th>{{ trans('backLang.ledgerType') }}</th>
            <th>{{ trans('backLang.date') }}</th>
            <th>{{ trans('backLang.debtor') }}</th>
            <th>{{ trans('backLang.creditor') }}</th>
            <th>{{ trans('backLang.ledger') }}</th>
            <th>{{ trans('backLang.desc') }}</th>
            <th>{{ trans('backLang.costPosition') }}</th>
            <th>{{ trans('backLang.dailyEntry') }}</th>
            <th>{{ trans('backLang.fileNo') }}</th>
            </thead>
            <tbody>
            @if(count($Journals) >0)
                @foreach($Journals as $Journal)
                    <?php
                    $cur_details = "";
                    if ($Journal->currency_id != Helper::Preferences("default_currency")) {
                        if ($Journal->dept > 0) {
                            $cur_details = " ( " . number_format($Journal->dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                        } else {
                            $cur_details = " ( " . number_format($Journal->credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) . " " . @$Journal->currency->symbol . " )";
                        }
                    }
                    $type = trans("backLang.dailyEntry");
                    if ($Journal->type == 2) {
                        $type = trans("backLang.reconciliationEntry");
                    } elseif ($Journal->type == 1) {
                        $type = trans("backLang.startBalance");
                    } else {
                        if (@$Journal->receipt->type == 1) {
                            $type = trans("backLang.cashReceipt");
                        } elseif (@$Journal->receipt->type == 0) {
                            $type = trans("backLang.payReceipt");
                        }
                    }
                    ?>
                    <tr>
                        <td>{{ $Journal->id }}</td>
                        <td>{{ $type }}</td>
                        <td>{!! (date(Helper::Preferences("date_format"), strtotime($Journal->date)) . " " . ((@$Journal->date_type) ? trans("backLang.H") : trans("backLang.M"))) !!}</td>
                        <td>{{ (($Journal->dept > 0) ? number_format($Journal->cost_dept, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") }}</td>
                        <td>{{ (($Journal->credit > 0) ? number_format($Journal->cost_credit, Helper::Preferences("decimal_round"), Helper::Preferences("decimal_symbol"), Helper::Preferences("thousand_symbol")) : "") }}</td>
                        <td>{{ @$Journal->ledger->code." - ".@$Journal->ledger->{"title_" . trans('backLang.boxCode')} }}</td>
                        <td>{{ $Journal->details . $cur_details }}</td>
                        <td>{{ @$Journal->center->{"title_" . trans('backLang.boxCode')} }}</td>
                        <td>{{ @$Journal->receipt->id }}</td>
                        <td>{{ @$Journal->receipt->code }}</td>
                    </tr>
                @endforeach
            @else
                <tr>
                    <td colspan="9">
                        <small>{{ trans("backLang.noData") }}</small>
                    </td>
                </tr>
            @endif
            </tbody>
        </table>

        {!! Helper::PrintHeaderAndFooter("")["footer"] !!}
    </div>
@endsection
