@extends('back.layouts.master')
@section('title', trans('backLang.textPages'))
@section("content")
    <div class="section-body pt-3">
        <div class="container-fluid">
            <nav aria-label="breadcrumb">
                <ol class="breadcrumb">
                    <li class="breadcrumb-item" aria-current="page"><a
                            href="{{ route("appSettings") }}">{{ trans('backLang.settings') }}</a></li>
                    <li class="breadcrumb-item active"
                        aria-current="page">{{ trans('backLang.textPages') }}</li>
                </ol>
            </nav>

            <div class="row clearfix">
                <div class="col-sm-12">
                    <div class="card">
                        <div class="card-header">
                            @if (Roles::Check("add"))
                                <button type="button" class="btn btn-primary" data-toggle="modal"
                                        data-target="#create_page"><i
                                        class="fe fe-plus mb-2"></i> {{ trans('backLang.addNewPage') }}</button>
                            @else
                                <h3 class="card-title">{{ trans('backLang.users') }}</h3>
                            @endif
                        </div>
                        <div class="card-body">
                            <div class="table-responsive mt-3">
                                <table
                                    class="table table-hover js-basic-example dataTable table_custom border-style spacing5"
                                    id="pages_table">
                                    <thead>
                                    <th>#</th>
                                    <th>{{ trans('backLang.title') }}</th>
                                    <th>{{ trans('backLang.visits') }}</th>
                                    <th>{{ trans('backLang.status') }}</th>
                                    <th class="text-center">{{ trans('backLang.options') }}</th>
                                    </thead>
                                </table>
                            </div>
                            @include('back.preferences.pages.create')
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>


    <div class="modal fade" id="edit_page" tabindex="-1" role="dialog" aria-hidden="true">
        <div class="modal-dialog modal-lg" role="document">
            <div class="modal-content">
                {{Form::open(['route'=>['AppPagesUpdate'],'method'=>'POST','id'=>'edit_page_form'])}}
                <div class="modal-header">
                    <h5 class="modal-title"><i
                            class="icon-pencil mb-2"></i> {{ trans('backLang.editPage') }}</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body">
                    <div class="alert alert-danger displaynone" id="edit_page_errors">
                        <ul></ul>
                    </div>
                    <div class="form-fields">
                        <div class="text-center">
                            <div class="loader" style="display: inline-block"></div>
                            <div>
                                <small class="text-muted">{{ trans("backLang.loading") }}</small>
                            </div>
                        </div>
                    </div>
                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-light" data-dismiss="modal"><i
                            class="fe fe-x"></i> {{ trans('backLang.cancel') }}
                    </button>
                    <button type="submit" id="edit_page_submit" class="btn btn-primary">
                        &nbsp; <i
                            class="fe fe-save"></i> {{ trans('backLang.save') }} &nbsp;
                    </button>
                </div>
                {{Form::close()}}
            </div>
        </div>
    </div>

@endsection
@push("after-styles")
    <link rel="stylesheet" href="{{ asset('back/plugins/bootstrap-colorpicker/css/bootstrap-colorpicker.css') }}">
@endpush
@push("after-scripts")
    <script src="{{ asset('back/plugins/bootstrap-colorpicker/js/bootstrap-colorpicker.js') }}"></script>
@endpush
@section("page-script")
    <script>

        var table_paginate = "{{ DeviceTrust::settings('BACKEND_PAGINATION') }}";
        var table_name = "#pages_table";

        $(document).ready(function () {
            $(table_name).DataTable({
                "processing": true,
                "serverSide": true,
                "searching": false,
                "responsive": true,
                "pageLength": table_paginate,
                "ajax": {
                    "url": "{{ route('AppPagesList') }}",
                    "dataType": "json",
                    "type": "POST",
                    "data": function (data) {
                        data._token = "{{csrf_token()}}";
                    }

                },
                "dom": '<"dataTables_wrapper"<"row"<"col-sm-12 col-md-9"i><"col-sm-12 col-md-3"l>><"row"<"col-sm-12 col-md-12"r>><"row"t><"row"<"w-p100"p>>>',
                "fnDrawCallback": function () {
                    if ($(table_name + '_paginate .paginate_button').length > 3) {
                        $(table_name + '_paginate')[0].style.display = "block";
                    } else {
                        $(table_name + '_paginate')[0].style.display = "none";
                    }
                },
                @if(trans('backLang.boxCode') =="ar")
                "language": {
                    {!! trans("backLang.dataTablesTranslation") !!}
                },
                @endif
                "columns": [
                    {"width": "5%","data": "id"},
                    {"data": "title"},
                    {"width": "5%","data": "visits"},
                    {"width": "5%","data": "status"},
                    {"width": "10%", "data": "options", "orderable": false}
                ],
                "order": [[0, "asc"]]

            });
            $.fn.dataTable.ext.errMode = 'none';

        });

        $(document).ready(function () {
            $('.summer').summernote({
                lang: 'ar-AR',
                height: 250,
                dialogsInBody: true,
                toolbar: [
                    ['style', ['style']],
                    ['font', ['bold', 'underline', 'clear']],
                    ['fontname', ['fontname','height']],
                    ['color', ['color']],
                    ['para', [ 'ol', 'paragraph']],
                    ['table', ['table']],
                    ['insert', ['link', 'picture', 'video', 'hr']],
                    ['view', ['fullscreen', 'codeview']],
                    ['help', ['help']]
                ],
                lang: (app_preferences["lang"] == "ar") ? "ar-AR" : "en-EN"
            });
            $(document).on("show.bs.modal", '.modal', function (event) {
                var zIndex = 100000 + (10 * $(".modal:visible").length);
                $(this).css("z-index", zIndex);
                setTimeout(function () {
                    $(".modal-backdrop").not(".modal-stack").first().css("z-index", zIndex - 1).addClass("modal-stack");
                }, 0);
            }).on("hidden.bs.modal", '.modal', function (event) {
                $(".modal:visible").length && $("body").addClass("modal-open");
            });
            $(document).on('inserted.bs.tooltip', function (event) {
                var zIndex = 100000 + (10 * $(".modal:visible").length);
                var tooltipId = $(event.target).attr("aria-describedby");
                $("#" + tooltipId).css("z-index", zIndex);
            });
            $(document).on('inserted.bs.popover', function (event) {
                var zIndex = 100000 + (10 * $(".modal:visible").length);
                var popoverId = $(event.target).attr("aria-describedby");
                $("#" + popoverId).css("z-index", zIndex);
            });

            // store page
            ajax_post_form("create_page", '{{ trans("backLang.add") }}', 'fe fe-save', '{!! trans("backLang.addDone") !!}', '{!! trans("backLang.close") !!}', true, table_name , false, false, true, function () {
                $("#details_ar").summernote("reset");
                $("#details_en").summernote("reset");
            });

            // update page
            ajax_post_form("edit_page", '{{ trans("backLang.save") }}', 'fe fe-save', '{!! trans("backLang.saveDone") !!}', '{!! trans("backLang.close") !!}', true, table_name , false, false, true);

            $("#edit_page").on("hidden.bs.modal", function () {
                $("#edit_page_errors").hide();
                $('#edit_page').find('.modal-body .form-fields').html("<div class=\"text-center\"><div class=\"loader\" style=\"display: inline-block\"></div><br><small class=\"text-muted\">{{ trans("backLang.loading") }}</small></div>");
            });
        });

        function edit_row(rid) {
            $("#edit_page_submit").prop('disabled', true);
            $.get("{{ route("AppPagesEdit") }}/" + rid, function (data) {
                $('#edit_page').find('.modal-body .form-fields').html(data);
                $("#edit_page_submit").prop('disabled', false);
            });
            $('#edit_page').modal('show');
        }

        // delete category
        function delete_row(rid) {
            ajax_delete_row("<?php echo route("AppPagesDestroy"); ?>/" + rid, '{{ trans("backLang.confirmationDeleteMsg") }}', '{{ trans("backLang.delete") }}', '{{ trans("backLang.cancel") }}', '{{ trans("backLang.deleteDone") }}', '{{ trans("backLang.erorr") }}', '{{ trans("backLang.close") }}', table_name, false);
        }

    </script>
@endsection
