@extends('back.layouts.master')
@section('title', trans('backLang.roles'))
@section("content")
    <div class="section-body pt-3">
        <div class="container-fluid">
            <nav aria-label="breadcrumb">
                <ol class="breadcrumb">
                    <li class="breadcrumb-item" aria-current="page"><a
                            href="{{ route("appSettings") }}">{{ trans('backLang.settings') }}</a></li>
                    <li class="breadcrumb-item active" aria-current="page">{{ trans('backLang.roles') }}</li>
                </ol>
            </nav>

            <div class="row clearfix">
                <div class="col-sm-12">
                    <div class="card">
                        <div class="card-header">
                            @if (Roles::Check("add"))
                                <button type="button" id="create_role_btn" class="btn btn-primary"><i
                                        class="fe fe-plus mb-2"></i> {{ trans('backLang.newRole') }}</button>
                            @endif
                        </div>
                        <div class="card-body">
                            {{Form::open(['route'=>['AppRolesStore'],'method'=>'POST','id'=>'create_role_form'])}}
                            <input type="hidden" name="title" id="role_title" value="">
                            <input type="hidden" name="from_id" id="from_id" value="">
                            {{Form::close()}}

                            <div class="table-responsive mt-3">
                                <table
                                    class="table table-hover js-basic-example dataTable table_custom border-style spacing5"
                                    id="roles_table">
                                    <thead>
                                    <th>{{ trans('backLang.roleName') }}</th>
                                    <th>{{ trans('backLang.createdBy') }}</th>
                                    <th>{{ trans('backLang.createdAt') }}</th>
                                    <th>{{ trans('backLang.status') }}</th>
                                    <th class="text-center">{{ trans('backLang.options') }}</th>
                                    </thead>
                                </table>
                            </div>

                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection
@push("after-styles")
    <link rel="stylesheet" href="{{ asset('back/plugins/multi-select/css/multi-select.css') }}">
@endpush
@push("after-scripts")
    <script src="{{ asset('back/plugins/bootstrap-multiselect/bootstrap-multiselect.js') }}"></script>
@endpush
@section("page-script")
    <script type="text/javascript">

        var table_name = "#roles_table";
        var table_paginate = "{{ DeviceTrust::settings('BACKEND_PAGINATION') }}";

        $(document).ready(function () {
            $('#create_role_btn').click(function () {
                swal({
                    title: "{{ trans("backLang.newRole") }}",
                    text: "{{ trans("backLang.roleNameConfirm") }}",
                    type: "input",
                    showCancelButton: true,
                    closeOnConfirm: false,
                    animation: "slide-from-top",
                    inputPlaceholder: "{{ trans("backLang.roleName") }}",
                    inputValue: "",
                    confirmButtonText: "{{ trans("backLang.add") }}",
                    cancelButtonText: "{{ trans("backLang.cancel") }}",
                    showLoaderOnConfirm: true,
                }, function (inputValue) {
                    if (inputValue === false) return false;
                    if (inputValue === "") {
                        swal.showInputError("{{ trans("backLang.roleNameConfirm") }}");
                        return false
                    }
                    $("#role_title").val(inputValue);
                    $("#from_id").val('');
                    var formData = $("#create_role_form").serialize();
                    $.ajax({
                        type: "POST",
                        url: "<?php echo route("AppRolesStore"); ?>",
                        data: formData,
                        success: function (result) {
                            var obj_result = jQuery.parseJSON(result);
                            if (obj_result.stat == 'success' && obj_result.id != "") {
                                location.href = '{{ route("AppRolesEdit") }}/' + obj_result.id;
                            } else {
                                swal({
                                    title: "<span class='text-danger'>{{ trans("backLang.erorr") }}</span>",
                                    text: "",
                                    html: true,
                                    type: "error",
                                    confirmButtonText: "{{ trans("backLang.close") }}",
                                    confirmButtonColor: "#acacac",
                                });
                            }
                        }
                    });

                });
            });

            var dataTable = $(table_name).DataTable({
                "processing": true,
                "serverSide": true,
                "searching": false,
                "responsive": true,
                "pageLength": table_paginate,
                "ajax": {
                    "url": "{{ route('AppRolesList') }}",
                    "dataType": "json",
                    "type": "POST",
                    "data": function (data) {
                        data._token = "{{csrf_token()}}";
                    }

                },
                "dom": '<"dataTables_wrapper"<"row"<"col-sm-12 col-md-9"i><"col-sm-12 col-md-3"l>><"row"<"col-sm-12 col-md-12"r>><"row"t><"row"<"w-p100"p>>>',
                "fnDrawCallback": function () {
                    if ($(table_name + '_paginate .paginate_button').length > 3) {
                        $(table_name + '_paginate')[0].style.display = "block";
                    } else {
                        $(table_name + '_paginate')[0].style.display = "none";
                    }
                },
                @if(trans('backLang.boxCode') =="ar")
                "language": {
                    {!! trans("backLang.dataTablesTranslation") !!}
                },
                @endif
                "columns": [
                    {"data": "name"},
                    {"data": "created_by"},
                    {"data": "created_at"},
                    {"data": "status"},
                    {"data": "options", "orderable": false}
                ],
                "order": [[2, "asc"]]

            });
            $.fn.dataTable.ext.errMode = 'none';
        });

        function copy_row(rid) {
            swal({
                title: "<i class='fe fe-copy'></i> {{ trans("backLang.copy") }}",
                text: "{{ trans("backLang.roleNameConfirm") }}",
                type: "input",
                showCancelButton: true,
                closeOnConfirm: false,
                animation: "slide-from-top",
                inputPlaceholder: "{{ trans("backLang.roleName") }}",
                inputValue: "",
                confirmButtonText: "{{ trans("backLang.add") }}",
                cancelButtonText: "{{ trans("backLang.cancel") }}",
                showLoaderOnConfirm: true,
                html: true,
            }, function (inputValue) {
                if (inputValue === false) return false;
                if (inputValue === "") {
                    swal.showInputError("{{ trans("backLang.roleNameConfirm") }}");
                    return false
                }
                $("#role_title").val(inputValue);
                $("#from_id").val(rid);
                var formData = $("#create_role_form").serialize();
                $.ajax({
                    type: "POST",
                    url: "<?php echo route("AppRolesStore"); ?>",
                    data: formData,
                    success: function (result) {
                        var obj_result = jQuery.parseJSON(result);
                        if (obj_result.stat == 'success' && obj_result.id != "") {
                            location.href = '{{ route("AppRolesEdit") }}/' + obj_result.id;
                        } else {
                            swal({
                                title: "<span class='text-danger'>{{ trans("backLang.erorr") }}</span>",
                                text: "",
                                html: true,
                                type: "error",
                                confirmButtonText: "{{ trans("backLang.close") }}",
                                confirmButtonColor: "#acacac",
                            });
                        }
                    }
                });

            });
        }

        // delete user
        function delete_row(rid) {
            ajax_delete_row("<?php echo route("AppRolesDestroy"); ?>/" + rid, '{{ trans("backLang.confirmationDeleteMsg") }}', '{{ trans("backLang.delete") }}', '{{ trans("backLang.cancel") }}', '{{ trans("backLang.deleteDone") }}', '{{ trans("backLang.erorr") }}', '{{ trans("backLang.close") }}', table_name, false);
        }
    </script>
@endsection
