@extends('back.layouts.master')
@section('title', trans('backLang.users'))
@section("content")
    <div class="section-body pt-3">
        <div class="container-fluid">
            <nav aria-label="breadcrumb">
                <ol class="breadcrumb">
                    <li class="breadcrumb-item" aria-current="page"><a
                            href="{{ route("appSettings") }}">{{ trans('backLang.settings') }}</a></li>
                    <li class="breadcrumb-item active" aria-current="page">{{ trans('backLang.users') }}</li>
                </ol>
            </nav>

            <div class="row clearfix">
                <div class="col-sm-12">
                    <div class="card">
                        <div class="card-header">
                            @if (Roles::Check("add")  && (DeviceTrust::UsersLimit() == 0 || DeviceTrust::UsersLimit() > $Total_User_count))
                                <button type="button" class="btn btn-primary" data-toggle="modal"
                                        data-target="#create_user"><i
                                        class="fe fe-plus mb-2"></i> {{ trans('backLang.newUser') }}</button>
                            @else
                                <h3 class="card-title">{{ trans('backLang.users') }}</h3>
                            @endif
                        </div>
                        <div class="card-body">
                            <div class="table-responsive mt-3">
                                <table
                                    class="table table-hover js-basic-example dataTable table_custom border-style spacing5"
                                    id="users_table">
                                    <thead>
                                    <th>{{ trans('backLang.fullName') }}</th>
                                    <th>{{ trans('backLang.contactInfo') }}</th>
                                    <th>{{ trans('backLang.roles') }}</th>
                                    <th>{{ trans('backLang.status') }}</th>
                                    <th class="text-center">{{ trans('backLang.options') }}</th>
                                    </thead>
                                </table>
                            </div>
                            @include('back.preferences.users.create')
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection
@section("page-script")
    <script type="text/javascript">

        var table_name = "#users_table";
        var table_paginate = "{{ DeviceTrust::settings('BACKEND_PAGINATION') }}";

        $(document).ready(function () {
            $('.countries-list').selectize({
                searchField: 'search-text',
                render: {
                    option: function (data, escape) {
                        return '<div>' +
                            '<span class="image"><img src="' + data.image + '" alt=""></span>' +
                            '<span class="title">' + escape(data.text) + '</span>' +
                            '<span class="code">' + escape(data.code) + '</span>' +
                            '</div>';
                    },
                    item: function (data, escape) {
                        return '<div>' +
                            '<span class="image"><img src="' + data.image + '" alt=""></span>' +
                            escape(data.text) +
                            escape(data.code) +
                            '</div>';
                    }
                }
            });

            var dropify_settings = {
                messages: {
                    'default': '{{ trans("backLang.dragAndDropFileHere") }}',
                    'replace': '{{ trans("backLang.dragAndDropFileToReplace") }}',
                    'remove': '{{ trans("backLang.delete") }}',
                    'error': '{{ trans("backLang.errorSomethingWrong") }}',
                }
            };

            var drEvent = $('#photo').dropify(dropify_settings);
            drEvent.on('dropify.beforeClear', function (event, element) {
                $("#photo_delete").val('1');
            });

            $('.selectize-input>input').attr('autocomplete', 'none');

            var dataTable = $(table_name).DataTable({
                "processing": true,
                "serverSide": true,
                "searching": false,
                "responsive": true,
                "pageLength": table_paginate,
                "ajax": {
                    "url": "{{ route('AppUsersList') }}",
                    "dataType": "json",
                    "type": "POST",
                    "data": function (data) {
                        data._token = "{{csrf_token()}}";
                    }

                },
                "dom": '<"dataTables_wrapper"<"row"<"col-sm-12 col-md-9"i><"col-sm-12 col-md-3"l>><"row"<"col-sm-12 col-md-12"r>><"row"t><"row"<"w-p100"p>>>',
                "fnDrawCallback": function () {
                    if ($(table_name + '_paginate .paginate_button').length > 3) {
                        $(table_name + '_paginate')[0].style.display = "block";
                    } else {
                        $(table_name + '_paginate')[0].style.display = "none";
                    }
                },
                @if(trans('backLang.boxCode') =="ar")
                "language": {
                    {!! trans("backLang.dataTablesTranslation") !!}
                },
                @endif
                "columns": [
                    {"data": "name"},
                    {"data": "email"},
                    {"data": "role_id"},
                    {"data": "status"},
                    {"data": "options", "orderable": false}
                ],
                "order": [[0, "asc"]]

            });
            $.fn.dataTable.ext.errMode = 'none';


            // store user
            ajax_post_form("create_user", '{{ trans("backLang.add") }}', 'fe fe-save', '{!! trans("backLang.addDone") !!}', '{!! trans("backLang.close") !!}', true, table_name, false, false, true);

        });

        // delete user
        function delete_row(rid) {
            ajax_delete_row("<?php echo route("AppUsersDestroy"); ?>/" + rid, '{{ trans("backLang.confirmationDeleteMsg") }}', '{{ trans("backLang.delete") }}', '{{ trans("backLang.cancel") }}', '{{ trans("backLang.deleteDone") }}', '{{ trans("backLang.erorr") }}', '{{ trans("backLang.close") }}', table_name, false);
        }

    </script>
@endsection
