<?php

use App\Helpers\DeviceTrust;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Accounting\LedgerController;
use App\Http\Controllers\Accounting\AccountingPreferencesController;
use App\Http\Controllers\Accounting\PeriodController;
use App\Http\Controllers\Accounting\CurrencyController;
use App\Http\Controllers\Accounting\BankController;
use App\Http\Controllers\Accounting\JournalController;
use App\Http\Controllers\Accounting\BalanceController;
use App\Http\Controllers\Accounting\PostingController;
use App\Http\Controllers\Accounting\BankBalanceController;
use App\Http\Controllers\Accounting\AccountingHomeController;
use App\Http\Controllers\Accounting\LoanController;
use App\Http\Controllers\Accounting\InstallmentController;
use App\Http\Controllers\Accounting\CheckPrinterController;
use App\Http\Controllers\Accounting\ReceiptController;
use App\Http\Controllers\Accounting\AccountingReportsController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/
Route::Group(['prefix' => DeviceTrust::env('BACKEND_PATH'), 'middleware' => ['authorize', 'auth', 'ipcheck']], function () {


    // Ledgers
    Route::get('/accounting/ledgers', [LedgerController::class, 'index'])->name('AppLedgers');
    Route::get('/accounting/ledgers/list', [LedgerController::class, 'list'])->name('AppLedgersList');
    Route::get('/accounting/ledgers/create', [LedgerController::class, 'create'])->name('AppLedgersCreate');
    Route::post('/accounting/ledgers/store', [LedgerController::class, 'store'])->name('AppLedgersStore');
    Route::get('/accounting/ledgers/edit/{id?}', [LedgerController::class, 'edit'])->name('AppLedgersEdit');
    Route::post('/accounting/ledgers/update', [LedgerController::class, 'update'])->name('AppLedgersUpdate');
    Route::get('/accounting/ledgers/destroy/{id?}', [LedgerController::class, 'destroy'])->name('AppLedgersDestroy');
    Route::post('/accounting/ledgers/list', [LedgerController::class, 'list'])->name('AppLedgersList');
    Route::get('/accounting/ledgers/print', [LedgerController::class, 'print'])->name('AppLedgersPrint');
    Route::get('/accounting/ledgers/export', [LedgerController::class, 'export'])->name('AppLedgersExport');
    Route::post('/accounting/ledgers/import', [LedgerController::class, 'import'])->name('AppLedgersImportFromFile');
    Route::get('/accounting/ledgers/download', [LedgerController::class, 'download'])->name('AppLedgersDownload');

    // Accounting settings
    Route::get('/accounting/settings', [AccountingPreferencesController::class, 'index'])->name('appAccountingSettings');
    Route::get('/accounting/settings-load/{whid?}', [AccountingPreferencesController::class, 'load'])->name('appAccountingSettingsLoad');
    Route::post('/accounting/settings', [AccountingPreferencesController::class, 'update'])->name('appAccountingSettingsUpdate');
    Route::post('/accounting/close', [AccountingPreferencesController::class, 'close'])->name('appAccountingPeriodClose');

    // Periods
    Route::post('/accounting/settings/periods/store', [PeriodController::class, 'store'])->name('AppPeriodsStore');
    Route::get('/accounting/settings/periods/edit/{id?}', [PeriodController::class, 'edit'])->name('AppPeriodsEdit');
    Route::post('/accounting/settings/periods/update', [PeriodController::class, 'update'])->name('AppPeriodsUpdate');
    Route::get('/accounting/settings/periods/destroy/{id?}', [PeriodController::class, 'destroy'])->name('AppPeriodsDestroy');
    Route::post('/accounting/settings/periods/save', [PeriodController::class, 'multi_update'])->name('AppPeriodsMultiUpdate');
    Route::post('/accounting/settings/periods/list', [PeriodController::class, 'list'])->name('AppPeriodsList');

    // Currencies
    Route::post('/accounting/settings/currencies/store', [CurrencyController::class, 'store'])->name('AppCurrenciesStore');
    Route::get('/accounting/settings/currencies/edit/{id?}', [CurrencyController::class, 'edit'])->name('AppCurrenciesEdit');
    Route::post('/accounting/settings/currencies/update', [CurrencyController::class, 'update'])->name('AppCurrenciesUpdate');
    Route::get('/accounting/settings/currencies/destroy/{id?}', [CurrencyController::class, 'destroy'])->name('AppCurrenciesDestroy');
    Route::post('/accounting/settings/currencies/save', [CurrencyController::class, 'multi_update'])->name('AppCurrenciesMultiUpdate');
    Route::post('/accounting/settings/currencies/list', [CurrencyController::class, 'list'])->name('AppCurrenciesList');

    // Banks
    Route::post('/accounting/settings/banks/store', [BankController::class, 'store'])->name('AppBanksStore');
    Route::get('/accounting/settings/banks/edit/{id?}', [BankController::class, 'edit'])->name('AppBanksEdit');
    Route::post('/accounting/settings/banks/update', [BankController::class, 'update'])->name('AppBanksUpdate');
    Route::get('/accounting/settings/banks/destroy/{id?}', [BankController::class, 'destroy'])->name('AppBanksDestroy');
    Route::post('/accounting/settings/banks/save', [BankController::class, 'multi_update'])->name('AppBanksMultiUpdate');
    Route::post('/accounting/settings/banks/list', [BankController::class, 'list'])->name('AppBanksList');

    // journal
    Route::get('/accounting/journals', [JournalController::class, 'index'])->name('AppJournals');
    Route::post('/accounting/journals/list', [JournalController::class, 'list'])->name('AppJournalsList');
    Route::get('/accounting/journals/create', [JournalController::class, 'create'])->name('AppJournalsCreate');
    Route::post('/accounting/journals/store', [JournalController::class, 'store'])->name('AppJournalsStore');
    Route::get('/accounting/journals/edit/{id?}', [JournalController::class, 'edit'])->name('AppJournalsEdit');
    Route::post('/accounting/journals/update', [JournalController::class, 'update'])->name('AppJournalsUpdate');
    Route::post('/accounting/journals/print', [JournalController::class, 'print'])->name('AppJournalsPrint');
    Route::post('/accounting/journals/export', [JournalController::class, 'export'])->name('AppJournalsExport');
    Route::get('/accounting/journals/receipt/{id?}', [JournalController::class, 'receipt'])->name('AppJournalReceipt');
    Route::get('/accounting/journals/ledgers', [JournalController::class, 'ledgers'])->name('AppJournalLedgers');

    //balances
    Route::get('/accounting/balances', [BalanceController::class, 'opening'])->name('AppOpeningBalances');
    Route::get('/accounting/reconciliation', [BalanceController::class, 'reconciliation'])->name('AppReconciliation');
    Route::post('/accounting/balances/list', [BalanceController::class, 'list'])->name('AppBalancesList');
    Route::get('/accounting/balances/create', [BalanceController::class, 'balances_create'])->name('AppBalancesCreate');
    Route::get('/accounting/reconciliation/create', [BalanceController::class, 'reconciliation_create'])->name('AppReconciliationCreate');
    Route::post('/accounting/balances/store', [BalanceController::class, 'store'])->name('AppBalancesStore');
    Route::get('/accounting/balances/edit/{id?}', [BalanceController::class, 'balances_edit'])->name('AppBalancesEdit');
    Route::get('/accounting/reconciliation/edit/{id?}', [BalanceController::class, 'reconciliation_edit'])->name('AppReconciliationEdit');
    Route::post('/accounting/balances/update', [BalanceController::class, 'update'])->name('AppBalancesUpdate');
    Route::get('/accounting/balances/destroy/{id?}', [BalanceController::class, 'destroy'])->name('AppBalancesDestroy');
    Route::post('/accounting/balances/print', [BalanceController::class, 'print'])->name('AppBalancesPrint');
    Route::get('/accounting/balance/print/{id?}', [BalanceController::class, 'receipt'])->name('AppBalancePrint');
    Route::post('/accounting/balances/export', [BalanceController::class, 'export'])->name('AppBalancesExport');

    //Posting
    Route::get('/accounting/posting', [PostingController::class, 'index'])->name('AppAccountantPosting');
    Route::post('/accounting/posting/list', [PostingController::class, 'list'])->name('AppAccountantPostingList');
    Route::get('/accounting/posting/post/{id?}', [PostingController::class, 'post'])->name('AppAccountantPost');
    Route::get('/accounting/posting/cancel/{id?}', [PostingController::class, 'unpost'])->name('AppAccountantUnPost');
    Route::post('/accounting/posting/multi', [PostingController::class, 'multi'])->name('AppAccountantPostingMulti');

    //bank balance
    Route::get('/accounting/bank', [BankBalanceController::class, 'bank'])->name('AppBankBalance');
    Route::get('/accounting/safe', [BankBalanceController::class, 'safe'])->name('AppSafeBalance');
    Route::post('/accounting/bank/list', [BankBalanceController::class, 'list'])->name('AppBankBalanceList');
    Route::post('/accounting/bank/export', [BankBalanceController::class, 'export'])->name('AppBankBalanceExport');
    Route::post('/accounting/bank/print', [BankBalanceController::class, 'print'])->name('AppBankBalancePrint');
    Route::post('/accounting/bank/transfer', [BankBalanceController::class, 'transfer'])->name('AppBankBalanceTransfer');
    Route::get('/accounting/bank/balance/{id?}', [BankBalanceController::class, 'balance'])->name('AppBankBalanceCheck');

    // home
    Route::get('/accounting/home/bank', [AccountingHomeController::class, 'bank'])->name('AppAccountingHomeBank');
    Route::get('/accounting/home/summary', [AccountingHomeController::class, 'summary'])->name('AppAccountingHomeSummary');
    Route::get('/accounting/home/cash/{stat?}', [AccountingHomeController::class, 'cash'])->name('AppAccountingHomeCashFlow');

    // Loans
    Route::get('/accounting/loans', [LoanController::class, 'index'])->name('Loans');
    Route::get('/accounting/loan/load', [LoanController::class, 'load_loan'])->name('LoansLoadLast');
    Route::post('/accounting/loans/list', [LoanController::class, 'list'])->name('LoansList');
    Route::get('/accounting/loans/view/{id}/{pay?}', [LoanController::class, 'view'])->name('LoansView');
    Route::get('/accounting/loan/print/{id?}', [LoanController::class, 'print_file'])->name('LoanPrintFile');
    Route::post('/accounting/loans/export', [LoanController::class, 'export'])->name('LoansExport');
    Route::post('/accounting/loans/print', [LoanController::class, 'print'])->name('LoansPrint');
    Route::get('/accounting/loans/create/{id?}', [LoanController::class, 'create'])->name('LoansCreate');
    Route::post('/accounting/loans/store', [LoanController::class, 'store'])->name('LoansStore');
    Route::get('/accounting/loans/edit/{id?}', [LoanController::class, 'edit'])->name('LoansEdit');
    Route::post('/accounting/loans/update', [LoanController::class, 'update'])->name('LoansUpdate');
    Route::get('/accounting/loans/destroy/{id?}', [LoanController::class, 'destroy'])->name('LoansDestroy');
    Route::post('/accounting/loans/update/status', [LoanController::class, 'update_status'])->name('LoansUpdateStatus');
    Route::get('/accounting/loans/edit/status/{id?}', [LoanController::class, 'edit_status'])->name('LoansEditStatus');
    Route::post('/accounting/loans/receipt/store', [LoanController::class, 'receipt_store'])->name('LoansReceiptStore');
    Route::get('/accounting/loans/receipt/print/{id?}', [LoanController::class, 'receipt_print'])->name('LoansReceiptPrint');

    //installments
    Route::post('/accounting/loans/installments/list', [InstallmentController::class, 'list'])->name('LoansInstallmentsList');
    Route::get('/accounting/loans/installments/destroy/{id?}', [InstallmentController::class, 'destroy'])->name('LoansInstallmentsDestroy');
    Route::get('/accounting/loans/installments/edit/{id?}', [InstallmentController::class, 'edit'])->name('LoansInstallmentsEdit');
    Route::post('/accounting/loans/installments/update', [InstallmentController::class, 'update'])->name('LoansInstallmentsUpdate');
    Route::post('/accounting/loans/installments/create', [InstallmentController::class, 'create'])->name('LoansInstallmentsCreate');


    //Checks Printing
    Route::get('/accounting/check-printer', [CheckPrinterController::class, 'index'])->name('appChecksPrint');

    //Revenues & Expenses
    Route::post('/accounting/receipts/list', [ReceiptController::class, 'list'])->name('AppReceiptsList');
    Route::post('/accounting/receipts/store', [ReceiptController::class, 'store'])->name('AppReceiptsStore');
    Route::get('/accounting/receipts/edit/{id?}', [ReceiptController::class, 'edit'])->name('AppReceiptsEdit');
    Route::get('/accounting/receipts/destroy/{id?}', [ReceiptController::class, 'destroy'])->name('AppReceiptsDestroy');
    Route::post('/accounting/receipts/update', [ReceiptController::class, 'update'])->name('AppReceiptsUpdate');
    Route::get('/accounting/receipt/print/{id?}', [ReceiptController::class, 'receipt'])->name('AppReceiptPrint');
    Route::post('/accounting/receipts/export', [ReceiptController::class, 'export'])->name('AppReceiptsExport');
    Route::post('/accounting/receipts/print', [ReceiptController::class, 'print'])->name('AppReceiptsPrint');
    Route::post('/accounting/receipts/import', [ReceiptController::class, 'import'])->name('AppReceiptsImportFromFile');
    Route::get('/accounting/receipts/download', [ReceiptController::class, 'download'])->name('AppReceiptsDownload');
    Route::get('/accounting/file/preview/{id?}', [ReceiptController::class, 'file_preview'])->name('AppReceiptsFilePreview');
    Route::get('/accounting/file/download/{id?}', [ReceiptController::class, 'file_download'])->name('AppReceiptsFileDownload');
    Route::get('/accounting/{stat}/{create?}', [ReceiptController::class, 'index'])->name('AppReceipts');

    //reports
    Route::get('/reports/revenues', [AccountingReportsController::class, 'revenues'])->name('AppRevenuesReport');
    Route::get('/reports/expenses', [AccountingReportsController::class, 'expenses'])->name('AppExpensesReport');
    Route::post('/reports/receipts/list', [AccountingReportsController::class, 'receipts_list'])->name('AppReportsReceiptsList');
    Route::get('/reports/receipt/{id?}', [AccountingReportsController::class, 'receipt'])->name('AppReceiptPreview');

    Route::get('/reports/profit', [AccountingReportsController::class, 'profit'])->name('AppProfitReport');
    Route::post('/reports/profit/list', [AccountingReportsController::class, 'profit_list'])->name('AppProfitReportList');
    Route::post('/reports/profit/export', [AccountingReportsController::class, 'profit_export'])->name('AppProfitReportExport');

    Route::get('/reports/statement', [AccountingReportsController::class, 'statement'])->name('AppStatementReport');
    Route::post('/reports/statement/list', [AccountingReportsController::class, 'statement_list'])->name('AppStatementReportList');
    Route::post('/reports/statement/export', [AccountingReportsController::class, 'statement_export'])->name('AppStatementReportExport');
    Route::post('/reports/statement/print', [AccountingReportsController::class, 'statement_print'])->name('AppStatementReportPrint');

    Route::get('/reports/ledger', [AccountingReportsController::class, 'ledger'])->name('AppLedgerReport');
    Route::post('/reports/ledger/list', [AccountingReportsController::class, 'ledger_list'])->name('AppLedgerReportList');
    Route::post('/reports/ledger/export', [AccountingReportsController::class, 'ledger_export'])->name('AppLedgerReportExport');
    Route::post('/reports/ledger/print', [AccountingReportsController::class, 'ledger_print'])->name('AppLedgerReportPrint');

    Route::get('/reports/income', [AccountingReportsController::class, 'income'])->name('AppIncomeReport');
    Route::post('/reports/income/list', [AccountingReportsController::class, 'income_list'])->name('AppIncomeReportList');

    Route::get('/reports/cash', [AccountingReportsController::class, 'cash'])->name('AppCashFlowReport');
    Route::post('/reports/cash/list', [AccountingReportsController::class, 'cash_list'])->name('AppCashFlowReportList');

    Route::get('/reports/trial', [AccountingReportsController::class, 'trial'])->name('AppTrialReport');
    Route::post('/reports/trial/list', [AccountingReportsController::class, 'trial_list'])->name('AppTrialReportList');

    Route::get('/reports/financial', [AccountingReportsController::class, 'financial'])->name('AppFinancialReport');
    Route::post('/reports/financial/list', [AccountingReportsController::class, 'financial_list'])->name('AppFinancialReportList');

    Route::get('/reports/entries', [AccountingReportsController::class, 'entries'])->name('AppEntriesReport');
    Route::post('/reports/entries/list', [AccountingReportsController::class, 'entries_list'])->name('AppEntriesReportList');

    Route::get('/reports/banks', [AccountingReportsController::class, 'banks'])->name('AppBanksReport');
    Route::post('/reports/banks/list', [AccountingReportsController::class, 'banks_list'])->name('AppBanksReportList');
    Route::post('/reports/banks/export', [AccountingReportsController::class, 'banks_export'])->name('AppBanksReportExport');
    Route::post('/reports/banks/print', [AccountingReportsController::class, 'banks_print'])->name('AppBanksReportPrint');

    Route::get('/reports/positions', [AccountingReportsController::class, 'positions'])->name('AppPositionsReport');
    Route::post('/reports/positions/list', [AccountingReportsController::class, 'positions_list'])->name('AppPositionsReportList');
    Route::post('/reports/positions/export', [AccountingReportsController::class, 'positions_export'])->name('AppPositionsReportExport');
    Route::post('/reports/positions/print', [AccountingReportsController::class, 'positions_print'])->name('AppPositionsReportPrint');

    Route::get('/reports/incomelist', [AccountingReportsController::class, 'incomelist'])->name('AppIncomeListReport');
    Route::post('/reports/incomelist/list', [AccountingReportsController::class, 'incomelist_list'])->name('AppIncomeListReportList');

    Route::get('/reports/balancesheet', [AccountingReportsController::class, 'balance'])->name('AppBalanceSheetReport');
    Route::post('/reports/balancesheet/list', [AccountingReportsController::class, 'balance_list'])->name('AppBalanceSheetReportList');

    Route::get('/reports/installments', [AccountingReportsController::class, 'installments'])->name('AppInstallmentsReport');
    Route::post('/reports/installments/list', [AccountingReportsController::class, 'installments_list'])->name('AppInstallmentsReportList');
    Route::post('/reports/installments/export', [AccountingReportsController::class, 'installments_export'])->name('AppInstallmentsReportExport');
    Route::post('/reports/installments/print', [AccountingReportsController::class, 'installments_print'])->name('AppInstallmentsReportPrint');

    //currencies exchange
    Route::post('/currencies-exchange', [AccountingHomeController::class, 'currencies_exchange'])->name('AppCurrencyExchange');

    // Home
    Route::get('/accounting/home', [AccountingHomeController::class, 'index'])->name('AppAccountingHome');

});
