<?php

use App\Helpers\DeviceTrust;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Preferences\PreferenceController;
use App\Http\Controllers\Preferences\ProfileController;
use App\Http\Controllers\Preferences\BackupController;
use App\Http\Controllers\Preferences\TemplateController;
use App\Http\Controllers\Preferences\CategoryController;
use App\Http\Controllers\Preferences\PageController;
use App\Http\Controllers\Preferences\RoleController;
use App\Http\Controllers\Preferences\UserController;
use App\Http\Controllers\Preferences\CommentController;
use App\Http\Controllers\Preferences\ShareController;
use App\Http\Controllers\Preferences\AlertController;
use App\Http\Controllers\Preferences\AttachmentController;
use App\Http\Controllers\Preferences\EventController;
use App\Http\Controllers\Preferences\ChatController;
use App\Http\Controllers\Preferences\ReportsController;
use App\Http\Controllers\Preferences\LanguageController;
use App\Http\Middleware\LanguageSwitcher;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

// Language Route
Route::post('/lang', [LanguageController::class, 'index'])->middleware(LanguageSwitcher::class)->name('lang');
// For Language direct URL link
Route::get('/lang/{lang}', [LanguageController::class, 'change'])->middleware(LanguageSwitcher::class)->name('langChange');
// .. End of Language Route

Route::get('/logout', function () {
    try {
        if (\Auth::check()) {
            $expiresAt = \Carbon\Carbon::now();
            Cache::put('user-is-online-' . Auth::user()->id, true, $expiresAt);
            Auth::logout();
        }
    } catch (\Exception $e) {

    }
    return redirect('/');
})->name('logout');

Route::get('/relog', function () {
    try {
        if (\Auth::check()) {
            Auth::logout();
        }
    } catch (\Exception $e) {

    }
    return redirect()->route("adminHome");
})->name('Relogin');


Route::Group(['prefix' => DeviceTrust::env('BACKEND_PATH'), 'middleware' => ['authorize', 'auth', 'ipcheck']], function () {
    // No Permission
    Route::get('/403', function () {
        return view('back.errors.503');
    })->name('NoPermission');

    // Not Found
    Route::get('/404', function () {
        return view('back.errors.404');
    })->name('NotFound');

    // preferences
    Route::get('/preferences', [PreferenceController::class, 'index'])->name('appSettings');
    Route::get('/preferences/general', [PreferenceController::class, 'general_settings'])->name('appGeneralSettings');
    Route::post('/preferences/general', [PreferenceController::class, 'general_settings_update'])->name('appGeneralSettingsUpdate');
    Route::get('/preferences/devices', [PreferenceController::class, 'devices_authorize'])->name('appDevicesAuthorize');
    Route::get('/preferences/languages', [PreferenceController::class, 'languages'])->name('appLanguagesSettings');
    Route::post('/preferences/languages', [PreferenceController::class, 'languages_update'])->name('appLanguagesSettingsUpdate');
    Route::post('/preferences/mail/smtp', [PreferenceController::class, 'mail_smtp_check'])->name('appMailSMTPCheck');
    Route::post('/preferences/mail/test', [PreferenceController::class, 'mail_test'])->name('appMailTest');
    Route::get('/preferences/sms/balance', [PreferenceController::class, 'sms_balance'])->name('appSMSBalance');
    Route::get('/preferences/ftp', [PreferenceController::class, 'ftp'])->name('appFtpSettings');
    Route::post('/preferences/ftp', [PreferenceController::class, 'ftp_update'])->name('appFtpSettingsUpdate');
    Route::post('/preferences/ftp/test', [PreferenceController::class, 'ftp_test'])->name('appFTPTest');
    Route::get('/preferences/notifications', [PreferenceController::class, 'notifications'])->name('appNotificationsSettings');
    Route::post('/preferences/notifications', [PreferenceController::class, 'notifications_update'])->name('appNotificationsSettingsUpdate');

    Route::get('/preferences/print', [PreferenceController::class, 'print'])->name('appPrintSettings');
    Route::post('/preferences/print', [PreferenceController::class, 'print_update'])->name('appPrintSettingsUpdate');

    Route::get('/preferences/cron', [PreferenceController::class, 'cron'])->name('appCronSettings');
    Route::post('/preferences/cron', [PreferenceController::class, 'cron_update'])->name('appCronSettingsUpdate');

    Route::get('/preferences/security', [PreferenceController::class, 'security'])->name('appSecurity');
    Route::post('/preferences/security', [PreferenceController::class, 'security_update'])->name('appSecurityUpdate');
    Route::post('/preferences/security/blocking', [PreferenceController::class, 'security_ips_update'])->name('appSecurityIPsUpdate');
    Route::get('/preferences/security/devices/destroy/{id?}', [PreferenceController::class, 'security_device_destroy'])->name('appSecurityDestroyDevices');
    Route::get('/preferences/security/devices/accept/{id?}', [PreferenceController::class, 'security_device_accept'])->name('appSecurityAcceptDevices');
    Route::post('/preferences/security/devices/list', [PreferenceController::class, 'security_devices'])->name('appSecurityDevices');

    // Settings Pass
    Route::get('preferences/password/{RedirectRouteName?}', [ProfileController::class, 'pass'])->name('AppPassConfirmBox');
    Route::post('preferences/password/confirm', [ProfileController::class, 'confirm'])->name('AppPassConfirm');

    // Backup
    Route::get('/preferences/backup', [BackupController::class, 'index'])->name('AppBackup');
    Route::post('/preferences/backup/list', [BackupController::class, 'list'])->name('AppBackupList');
    Route::post('/preferences/backup/store', [BackupController::class, 'store'])->name('AppBackupStore');
    Route::post('/preferences/backup/restore', [BackupController::class, 'restore_from_file'])->name('AppBackupRestoreFromFile');
    Route::post('/preferences/backup/update', [BackupController::class, 'update'])->name('AppBackupUpdate');
    Route::get('/preferences/backup/download/{file}', [BackupController::class, 'download'])->name('AppBackupDownload');
    Route::get('/preferences/backup/restore/{file?}', [BackupController::class, 'restore'])->name('AppBackupRestore');
    Route::get('/preferences/backup/destroy/{file?}', [BackupController::class, 'destroy'])->name('AppBackupDestroy');
    Route::post('/preferences/backup/settings', [BackupController::class, 'backup_update'])->name('appBackupSettingsUpdate');

    // Templates
    Route::get('/preferences/templates/{cat?}/{stat?}', [TemplateController::class, 'index'])->name('AppTemplates');
    Route::get('/preferences/templates/load/{cat?}/{stat?}', [TemplateController::class, 'load'])->name('AppTemplatesLoad');
    Route::post('/preferences/templates/update', [TemplateController::class, 'update'])->name('AppTemplatesUpdate');

    // Categories
    Route::get('/preferences/categories/{cat?}', [CategoryController::class, 'index'])->name('AppCategories');
    Route::get('/preferences/categories/create/{cat?}', [CategoryController::class, 'create'])->name('AppCategoriesCreate');
    Route::post('/preferences/categories/store', [CategoryController::class, 'store'])->name('AppCategoriesStore');
    Route::get('/preferences/categories/edit/{id?}', [CategoryController::class, 'edit'])->name('AppCategoriesEdit');
    Route::post('/preferences/categories/update', [CategoryController::class, 'update'])->name('AppCategoriesUpdate');
    Route::get('/preferences/categories/destroy/{id?}', [CategoryController::class, 'destroy'])->name('AppCategoriesDestroy');
    Route::post('/preferences/categories/save', [CategoryController::class, 'multi_update'])->name('AppCategoriesMultiUpdate');
    Route::post('/preferences/categories/list', [CategoryController::class, 'list'])->name('AppCategoriesList');

    // Pages
    Route::post('/preferences/pages/list', [PageController::class, 'list'])->name('AppPagesList');
    Route::post('/preferences/pages/store', [PageController::class, 'store'])->name('AppPagesStore');
    Route::get('/preferences/pages/destroy/{id?}', [PageController::class, 'destroy'])->name('AppPagesDestroy');
    Route::get('/preferences/pages/edit/{id?}', [PageController::class, 'edit'])->name('AppPagesEdit');
    Route::post('/preferences/pages/update', [PageController::class, 'update'])->name('AppPagesUpdate');
    Route::get('/preferences/pages', [PageController::class, 'index'])->name('AppPages');

    // roles
    Route::get('/preferences/roles', [RoleController::class, 'index'])->name('AppRoles');
    Route::post('/preferences/roles/store', [RoleController::class, 'store'])->name('AppRolesStore');
    Route::get('/preferences/roles/edit/{id?}', [RoleController::class, 'edit'])->name('AppRolesEdit');
    Route::post('/preferences/roles/update', [RoleController::class, 'update'])->name('AppRolesUpdate');
    Route::get('/preferences/roles/destroy/{id?}', [RoleController::class, 'destroy'])->name('AppRolesDestroy');
    Route::post('/preferences/roles/list', [RoleController::class, 'list'])->name('AppRolesList');

    // Users
    Route::get('/preferences/users', [UserController::class, 'index'])->name('AppUsers');
    Route::get('/preferences/users/create/', [UserController::class, 'create'])->name('AppUsersCreate');
    Route::post('/preferences/users/store', [UserController::class, 'store'])->name('AppUsersStore');
    Route::get('/preferences/users/edit/{id?}', [UserController::class, 'edit'])->name('AppUsersEdit');
    Route::post('/preferences/users/update', [UserController::class, 'update'])->name('AppUsersUpdate');
    Route::get('/preferences/users/destroy/{id?}', [UserController::class, 'destroy'])->name('AppUsersDestroy');
    Route::post('/preferences/users/list', [UserController::class, 'list'])->name('AppUsersList');

    // Profile
    Route::get('/profile', [ProfileController::class, 'index'])->name('AppUserProfile');
    Route::post('/profile/update', [ProfileController::class, 'update'])->name('AppUserProfileUpdate');
    Route::post('/profile/send', [ProfileController::class, 'message_send'])->name('AppUserProfileSend');
    Route::post('/theme-update', [ProfileController::class, 'theme'])->name('theme');

    // Activities
    Route::post('/activities', [ProfileController::class, 'activities'])->name('AppUserActivities');

    // Messages
    Route::post('/messages', [ProfileController::class, 'messages'])->name('AppMessages');
    Route::get('/messages/view/{id?}', [ProfileController::class, 'message_view'])->name('AppMessagesView');
    Route::get('/messages/delete/{id?}', [ProfileController::class, 'message_destroy'])->name('AppMessagesDelete');

    // Notifications
    Route::post('/notifications', [ProfileController::class, 'notifications'])->name('AppNotifications');
    Route::get('/notifications/view/{id?}', [ProfileController::class, 'notification_view'])->name('AppNotificationsView');
    Route::get('/notifications/delete/{id?}', [ProfileController::class, 'notification_destroy'])->name('AppNotificationsDelete');

    // Comments
    Route::post('/comments', [CommentController::class, 'load'])->name('AppComments');
    Route::post('/comments/store', [CommentController::class, 'store'])->name('AppCommentsStore');
    Route::get('/comments/destroy/{id?}', [CommentController::class, 'destroy'])->name('AppCommentsDestroy');

    // Share
    Route::post('/share', [ShareController::class, 'load'])->name('AppShares');
    Route::post('/share/store', [ShareController::class, 'store'])->name('AppSharesStore');
    Route::get('/share/destroy/{id?}/{title?}', [ShareController::class, 'destroy'])->name('AppSharesDestroy');

    // Alerts
    Route::post('/alerts', [AlertController::class, 'load'])->name('AppAlerts');
    Route::post('/alerts/store', [AlertController::class, 'store'])->name('AppAlertsStore');
    Route::get('/alerts/edit/{id?}', [AlertController::class, 'edit'])->name('AppAlertsEdit');
    Route::post('/alerts/update', [AlertController::class, 'update'])->name('AppAlertsUpdate');
    Route::get('/alerts/destroy/{id?}', [AlertController::class, 'destroy'])->name('AppAlertsDestroy');

    // Attachments
    Route::post('/attachments/list', [AttachmentController::class, 'list'])->name('AppAttachmentsList');
    Route::post('/attachments/load', [AttachmentController::class, 'load'])->name('AppAttachmentsLoad');
    Route::post('/attachments/upload', [AttachmentController::class, 'upload'])->name('AppAttachmentsUpload');
    Route::get('/attachments/destroy/{id?}', [AttachmentController::class, 'destroy'])->name('AppAttachmentsDestroy');
    Route::get('/attachments/preview/{id?}', [AttachmentController::class, 'preview'])->name('AppAttachmentsPreview');
    Route::get('/attachments/download/{id?}', [AttachmentController::class, 'download'])->name('AppAttachmentsDownload');
    Route::get('/attachments/edit/{id?}', [AttachmentController::class, 'edit'])->name('AppAttachmentsEdit');
    Route::post('/attachments/update', [AttachmentController::class, 'update'])->name('AppAttachmentsUpdate');
    Route::post('/attachments/folder', [AttachmentController::class, 'folder'])->name('AppAttachmentsFolder');
    Route::post('/attachments/store', [AttachmentController::class, 'store'])->name('AppAttachmentsStore');
    Route::get('/attachments/view/{id?}', [AttachmentController::class, 'view'])->name('AppAttachmentsView');
    Route::get('/attachments/{cat?}', [AttachmentController::class, 'index'])->name('AppAttachments');

    // Events
    Route::post('/events/list', [EventController::class, 'list'])->name('AppEventsList');
    Route::post('/events/load', [EventController::class, 'load'])->name('AppEventsLoad');
    Route::post('/events/store', [EventController::class, 'store'])->name('AppEventsStore');
    Route::get('/events/destroy/{id?}', [EventController::class, 'destroy'])->name('AppEventsDestroy');
    Route::get('/events/edit/{id?}', [EventController::class, 'edit'])->name('AppEventsEdit');
    Route::post('/events/update', [EventController::class, 'update'])->name('AppEventsUpdate');
    Route::get('/events/view/{id?}', [EventController::class, 'view'])->name('AppEventsView');
    Route::get('/events/{cat?}', [EventController::class, 'index'])->name('AppEvents');

    // Chat
    Route::get('/chat', [ChatController::class, 'index'])->name('AppChat');
    Route::get('/chat/messages/{id?}', [ChatController::class, 'messages'])->name('AppChatList');
    Route::get('/chat/users/{id?}', [ChatController::class, 'users'])->name('AppChatList');
    Route::get('/chat/read/{mid?}/{uid?}', [ChatController::class, 'read'])->name('AppChatMessageRead');
    Route::post('/chat/store', [ChatController::class, 'store'])->name('AppChatStore');
    Route::post('/chat/attach', [ChatController::class, 'attach'])->name('AppChatAttach');
    Route::get('/chat/destroy/{id?}', [ChatController::class, 'destroy'])->name('AppChatDestroy');


    // notifications
    Route::get('/notices', [ProfileController::class, 'notices'])->name('AppNotices');

    // Reports
    Route::get('/reports', [ReportsController::class, 'index'])->name('AppReports');

    // Clear Cache
    Route::get('/cache-clear', function () {
        Artisan::call('cache:clear');
        Artisan::call('view:clear');
        return redirect()->back()->with('doneMessage', trans('backLang.cashClearDone'));
    })->name('cacheClear');


});
// Cron
Route::get('/app-queue-listen', function () {
    Artisan::call('queue:work');
});
Route::get('/app-schedule-run', function () {
    Artisan::call('schedule:run');
});
// .. End of Backend Routes
